﻿using System;
using System.Linq;
using System.Reflection;
using Autofac;
using Autofac.Builder;
using Autofac.Features.Scanning;
using GestionConcesionarios.Infraestructura.Helpers;
using Module = Autofac.Module;

namespace GestionConcesionarios.Aplicacion.Core.Container
{
  public class ApplicationModule : Module
  {
    #region Atributos

    private readonly string[] _namespacesToScan;
    private readonly Assembly[] _assembliesToScan;

    #endregion

    #region Metodos Publicos

    public ApplicationModule(string[] namespacesToScan,
                             Assembly[] assembliesToScan)
    {
      if (namespacesToScan == null) throw new ArgumentNullException("namespacesToScan");
      if (assembliesToScan == null) throw new ArgumentNullException("assembliesToScan");

      _namespacesToScan = namespacesToScan;
      _assembliesToScan = assembliesToScan;
    }

    #endregion

    #region Metodos Protegidos

    protected override void Load(ContainerBuilder builder)
    {
      foreach (Assembly assembly in _assembliesToScan)
      {
        //Servicios de aplicacion
        builder.RegisterAssemblyTypes(assembly).Where(l => IsNamespaceToScan(l.Namespace) && l.ImplementsInterface(typeof(IAplicacionService))).AsImplementedInterfaces().AsSelf();
      }
    }

    #endregion

    #region Metodos Privados

    private bool IsNamespaceToScan(string @namespace)
    {
      return @namespace != null && _namespacesToScan.Any(@namespace.Contains);
    }

    #endregion
  }
}