﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los articulos
  /// </summary>
  public interface IGestionArticuloService
  {
    /// <summary>
    /// Añadir un nuevo articulo
    /// </summary>
    /// <param name="articulo">Articulo a añadir</param>
    void AddArticulo(Articulo articulo);

    /// <summary>
    /// Añadir un nuevo articulo
    /// </summary>    
    /// <param name="nombre">Nombre del artículo</param>
    /// <param name="descripcion">Descripcion del artículo.</param>
    /// <param name="idProveedor">Id proveedor del artículo.</param>
    /// <param name="idCategoriaArticulo">Id categoria articulo.</param>
    /// <param name="costeActualPorUnidad">Coste actual por unidad.</param>
    /// <param name="costeAnteriorPorUnidad">Coste anterior por unidad.</param>
    void AddArticulo(string nombre, string descripcion, Guid? idProveedor, string idCategoriaArticulo,
                     decimal? costeActualPorUnidad, decimal? costeAnteriorPorUnidad);

    /// <summary>
    /// Modificar un articulo existente
    /// </summary>
    /// <param name="articulo">Articulo con cambios a modificar</param>
    void ChangeArticulo(Articulo articulo);

    /// <summary>
    /// Modificar un articulo existente
    /// </summary>
    /// <param name="idGuid">Identificador único del articulo a modificar</param>
    /// <param name="nombre">Nombre del artículo</param>
    /// <param name="descripcion">Descripcion del artículo.</param>
    /// <param name="idProveedor">Id proveedor del artículo.</param>
    /// <param name="idCategoriaArticulo">Id categoria articulo.</param>
    /// <param name="costeActualPorUnidad">Coste actual por unidad.</param>
    /// <param name="costeAnteriorPorUnidad">Coste anterior por unidad.</param>
    void ChangeArticulo(Guid idGuid, string nombre, string descripcion, Guid? idProveedor, string idCategoriaArticulo,
                        decimal? costeActualPorUnidad, decimal? costeAnteriorPorUnidad);

    /// <summary>
    /// Borrar un articulo existente 
    /// </summary>
    /// <param name="articulo">Articulo a eliminar</param>
    void RemoveArticulo(Articulo articulo);

    /// <summary>
    /// Borrar un articulo existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del articulo a eliminar</param>
    void RemoveArticulo(Guid idGuid);

    /// <summary>
    /// Buscar articulo por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del articulo</param>
    /// <returns>Lista de articulos si existen o null si no existen articulos con el valor id igual a <paramref name="id"/></returns>
    List<Articulo> BuscarArticuloPorId(Guid? id);

    /// <summary>
    /// Buscar articulo por el valor de la propiedad Nombre
    /// </summary>
    /// <param name="nombre">Especificación de nombre del articulo</param>
    /// <returns>Lista de articulos si existen o null si no existen articulos con el valor nombre igual a <paramref name="nombre"/></returns>
    List<Articulo> BuscarArticuloPorNombre(string nombre);
  }
}