﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulosAlmacen
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los articulos Almacen
  /// </summary>
  public interface IGestionArticuloAlmacenService
  {
    /// <summary>
    /// Añadir un nuevo articuloAlmacen
    /// </summary>
    /// <param name="articuloAlmacen">ArticuloAlmacen a añadir</param>
    void AddArticuloAlmacen(ArticuloAlmacen articuloAlmacen);

    /// <summary>
    /// Añadir un nuevo articulo al almacen.
    /// </summary>
    /// <param name="idArticuloAlmacen">Id de la entrada del articulo en el almacen.</param>
    /// <param name="idEmpleado">Id empleado.</param>
    /// <param name="fechaEntrada">Fecha entrada del artículo en el almacén.</param>
    /// <param name="stockMinimo">Stock mínimo.</param>
    /// <param name="stockMaximo">Stock máximo.</param>
    /// <param name="stockActual">Stock actual.</param>
    /// <param name="ubicacion">Ubicación del artículo en el almacén.</param>
    /// <param name="idArticulo">Id del Artículo</param>
    /// <param name="codigoTaller">Código del taller donde está el almacén.</param>
    void AddArticuloAlmacen(Guid idArticuloAlmacen, Guid? idEmpleado, DateTime? fechaEntrada, decimal? stockMinimo,
                            decimal? stockMaximo, decimal? stockActual, string ubicacion, Guid idArticulo, string codigoTaller);
    /// <summary>
    /// Modificar un articuloAlmacen existente
    /// </summary>
    /// <param name="articuloAlmacen">ArticuloAlmacen con cambios a modificar</param>
    void ChangeArticuloAlmacen(ArticuloAlmacen articuloAlmacen);

    /// <summary>
    /// Modificar un articuloAlmacen existente
    /// </summary>
    /// <param name="idArticuloAlmacen">Id de la entrada del articulo en el almacen.</param>
    /// <param name="idEmpleado">Id empleado.</param>
    /// <param name="fechaEntrada">Fecha entrada del artículo en el almacén.</param>
    /// <param name="stockMinimo">Stock mínimo.</param>
    /// <param name="stockMaximo">Stock máximo.</param>
    /// <param name="stockActual">Stock actual.</param>
    /// <param name="ubicacion">Ubicación del artículo en el almacén.</param>
    /// <param name="idArticulo">Id del Artículo</param>
    /// <param name="codigoTaller">Código del taller donde está el almacén.</param>
    void ChangeArticuloAlmacen(Guid idArticuloAlmacen, Guid? idEmpleado, DateTime? fechaEntrada, decimal? stockMinimo,
                               decimal? stockMaximo, decimal? stockActual, string ubicacion, Guid idArticulo,
                               string codigoTaller);

    /// <summary>
    /// Borrar un articuloAlmacen existente 
    /// </summary>
    /// <param name="articuloAlmacen">ArticuloAlmacen a eliminar</param>
    void RemoveArticuloAlmacen(ArticuloAlmacen articuloAlmacen);

    /// <summary>
    /// Borrar un articuloAlmacen existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del articuloAlmacen a eliminar</param>
    void RemoveArticuloAlmacen(Guid idGuid);

    /// <summary>
    /// Buscar articuloAlmacen por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del articuloAlmacen</param>
    /// <returns>Lista de articuloAlmacens si existen o null si no existen articuloAlmacens con el valor id igual a <paramref name="id"/></returns>
    List<ArticuloAlmacen> BuscarArticuloAlmacenPorId(Guid? id);

    /// <summary>
    /// Buscar articuloAlmacen por el valor de la propiedad IdArticulo
    /// </summary>
    /// <param name="idArticulo">Especificación de Id del articulo en la entrada del almacén</param>
    /// <returns>Lista de articulos en Almacen si existen o null si no existen articulos en Almacen con el valor idArticulo igual a <paramref name="idArticulo"/></returns>
    List<ArticuloAlmacen> BuscarArticuloAlmacenPorIdArticulo(Guid? idArticulo);
  }
}