﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.CategoriasArticulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionCategoriasArticulos
{
  /// <summary>
  /// Implementación de IGestionCategoriasArticuloService
  /// </summary>
  public class GestionCategoriasArticuloService : IAplicacionService, IGestionCategoriasArticuloService
  {
    #region Atributos

    readonly ICategoriaArticuloRepository _categoriaArticuloRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="categoriaArticuloRepository">Dependencia al repositorio de categorias articulo, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionCategoriasArticuloService(ICategoriaArticuloRepository categoriaArticuloRepository)
    {
      if (categoriaArticuloRepository == null) throw new ArgumentNullException("categoriaArticuloRepository");

      _categoriaArticuloRepository = categoriaArticuloRepository;
    }

    #endregion

    #region Miembros de IGestionCategoriasArticuloService

    /// <summary>
    /// <see cref="IGestionCategoriasArticuloService"/>
    /// </summary>
    /// <returns><see cref="CategoriaArticulo"/></returns>
    public List<CategoriaArticulo> BuscarTodasLasCategoriasArticulo()
    {
      return _categoriaArticuloRepository.BuscarTodasLasCategoriasArticulo();
    }
    #endregion
  }
}
