﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los clientes
  /// </summary>
  public interface IGestionClienteService
  {
    /// <summary>
    /// Añadir nuevo cliente
    /// </summary>
    /// <param name="cliente">Cliente a añadir</param>
    void AddCliente(Cliente cliente);

    /// <summary>
    /// Añade un nuevo cliente con los valores pasados por parámetros
    /// </summary>
    /// <param name="idGuid">Id GUID del nuevo cliente.</param>
    /// <param name="nombre">Nombre del nuevo cliente.</param>
    /// <param name="primerApellido">Primer apellido del nuevo cliente.</param>
    /// <param name="segundoApellido">Segundo apellido del nuevo cliente.</param>
    /// <param name="correoElectronico">Correo electronico del nuevo cliente.</param>
    /// <param name="cuentaBancaria">Cuenta bancaria del nuevo cliente.</param>
    /// <param name="direccion">Direccion del nuevo cliente.</param>
    /// <param name="listaTelefonos">Lista telefonos del nuevo cliente.</param>
    /// <param name="nifCif">Nif/cif del nuevo cliente.</param>
    /// <param name="nombreComercial">Nombre comercial del nuevo cliente.</param>
    /// <param name="observaciones">Observaciones del nuevo cliente.</param>
    /// <param name="formaDePago">Forma de pago del nuevo cliente.</param>
    /// <param name="idCodigoPostal">Id codigo postal del nuevo cliente.</param>
    void AddCliente(Guid idGuid, string nombre, string primerApellido, string segundoApellido, string correoElectronico,
                    string cuentaBancaria, string direccion, string listaTelefonos, string nifCif,
                    string nombreComercial, string observaciones,
                    int? formaDePago, int? idCodigoPostal);

    /// <summary>
    /// Modificar un cliente existente
    /// </summary>
    /// <param name="cliente">Cliente con cambios a modificar</param>
    void ChangeCliente(Cliente cliente);

    /// <summary>
    /// Modificar el cliente existente con Id idGuid con los valores pasados por parámetro
    /// </summary>
    /// <param name="idGuid">Id GUID del cliente a modificar.</param>
    /// <param name="nombre">Nombre del cliente.</param>
    /// <param name="primerApellido">Primer apellido del cliente.</param>
    /// <param name="segundoApellido">Segundo apellido del cliente.</param>
    /// <param name="correoElectronico">Correo electronico del cliente.</param>
    /// <param name="cuentaBancaria">Cuenta bancaria del cliente.</param>
    /// <param name="direccion">Direccion del cliente.</param>
    /// <param name="listaTelefonos">Lista telefonos del cliente.</param>
    /// <param name="nifCif">Nif/cif del cliente.</param>
    /// <param name="nombreComercial">Nombre comercial del cliente.</param>
    /// <param name="observaciones">Observaciones del cliente.</param>
    /// <param name="formaDePago">Forma de pago del cliente.</param>
    /// <param name="idCodigoPostal">Id codigo postal del cliente.</param>
    void ChangeCliente(Guid idGuid, string nombre, string primerApellido, string segundoApellido,
                       string correoElectronico,
                       string cuentaBancaria, string direccion, string listaTelefonos, string nifCif,
                       string nombreComercial, string observaciones, 
                       int? formaDePago, int? idCodigoPostal);

    /// <summary>
    /// Borrar un cliente existente
    /// </summary>
    /// <param name="cliente">Cliente a eliminar</param>
    void RemoveCliente(Cliente cliente);

    /// <summary>
    /// Borrar un cliente existente por su Id
    /// </summary>
    /// <param name="idGuid">Id del Cliente a eliminar</param>
    void RemoveCliente(Guid idGuid);

    /// <summary>
    /// Buscar cliente por el valor de las propiedades nombre, primer y segundo apellido
    /// </summary>
    /// <param name="nombre">Especificación de nombre del cliente</param>
    /// <param name="primerApellido">Especificación de primer apellido del cliente</param>
    /// <param name="segundoApellido">Especificación de segundo apellido del cliente</param>
    /// <returns>Lista de clientes si existen o null si no existen clientes con los valores nombre, primer y segundo apellido iguales a <paramref name="nombre"/>, <paramref name="primerApellido"/> y <paramref name="segundoApellido"/></returns>
    List<Cliente> BuscarCliente(string nombre, string primerApellido, string segundoApellido);

    /// <summary>
    /// Buscar cliente por el valor de la propiedad nifCif
    /// </summary>
    /// <param name="nifCif">Especificación de nifCif del cliente</param>
    /// <returns>Lista de clientes si existen o null si no existen clientes con el valor nifCif igual a <paramref name="nifCif"/></returns>
    List<Cliente> BuscarClientePorNifCif(string nifCif);

    /// <summary>
    /// Buscar cliente por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del cliente</param>
    /// <returns>Lista de clientes si existen o null si no existen clientes con el valor id igual a <paramref name="id"/></returns>
    List<Cliente> BuscarClientePorId(Guid? id);
  }
}
