﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.CodigosPostales;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.CodigosPostales
{
  /// <summary>
  /// Implementación de IGestionCodigoPostalService
  /// </summary>
  public class GestionCodigoPostalService : IAplicacionService, IGestionCodigoPostalService
  {
    #region Atributos

    readonly ICodigoPostalRepository _codigoPostalRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="codigoPostalRepository">Dependencia al repositorio de codigos postales, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionCodigoPostalService(ICodigoPostalRepository codigoPostalRepository)
    {
      if (codigoPostalRepository == null) throw new ArgumentNullException("codigoPostalRepository");

      _codigoPostalRepository = codigoPostalRepository;
    }

    #endregion

    #region Miembros de IGestionCodigoPostalService

    /// <summary>
    /// <see cref="IGestionCodigoPostalService"/>
    /// </summary>
    /// <returns><see cref="CodigoPostal"/></returns>
    public List<CodigoPostal> BuscarTodosLosCodigosPostales()
    {
      return _codigoPostalRepository.BuscarCodigoPostal();
    }
    #endregion
  }
}
