﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.ContenidosReparaciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionContenidosReparaciones
{
  /// <summary>
  /// Implementación de IGestionContenidoReparacionService
  /// </summary>
  public class GestionContenidoReparacionService : IAplicacionService, IGestionContenidoReparacionService
  {
    #region Atributos

    private readonly IContenidoReparacionRepository _contenidoReparacionRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="contenidoReparacionRepository">Dependencia al repositorio de contenidoReparacions, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionContenidoReparacionService(IContenidoReparacionRepository contenidoReparacionRepository)
    {
      if (contenidoReparacionRepository == null) throw new ArgumentNullException("contenidoReparacionRepository");

      _contenidoReparacionRepository = contenidoReparacionRepository;
    }

    #endregion

    #region Miembros de IGestionContenidoReparacionService

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="contenidoReparacion"><see cref="IGestionContenidoReparacionService"/></param>
    public void AddContenidoReparacion(ContenidoReparacion contenidoReparacion)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _contenidoReparacionRepository.UnitOfWork;

      _contenidoReparacionRepository.Add(contenidoReparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionContenidoReparacionService"/></param>    
    public void AddContenidoReparacion(Guid idGuid)
    {
      var contenidoReparacion = new ContenidoReparacion
                {
                  Id = idGuid
                };

      AddContenidoReparacion(contenidoReparacion);
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    public void AddContenidoReparacion()
    {
      throw new NotImplementedException();
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionContenidoReparacionService"/></param>    
    public void ChangeContenidoReparacion(Guid idGuid)
    {
      var contenidoReparacion = BuscarContenidoReparacionPorId(idGuid).FirstOrDefault();
      if (contenidoReparacion != null)
      {
        // TODO: Falta poner propiedades a actualizar	
        //cliente.Nombre = nombre;

        ChangeContenidoReparacion(contenidoReparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="contenidoReparacion"><see cref="IGestionContenidoReparacionService"/></param>
    public void ChangeContenidoReparacion(ContenidoReparacion contenidoReparacion)
    {
      if (contenidoReparacion == null) throw new ArgumentNullException("contenidoReparacion");

      var unitOfWork = _contenidoReparacionRepository.UnitOfWork;

      _contenidoReparacionRepository.Modify(contenidoReparacion);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionContenidoReparacionService"/></param>
    public void RemoveContenidoReparacion(Guid idGuid)
    {
      var contenidoReparacion = BuscarContenidoReparacionPorId(idGuid).FirstOrDefault();
      if (contenidoReparacion != null)
      {
        RemoveContenidoReparacion(contenidoReparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="contenidoReparacion"><see cref="IGestionContenidoReparacionService"/></param>
    public void RemoveContenidoReparacion(ContenidoReparacion contenidoReparacion)
    {
      if (contenidoReparacion == null) throw new ArgumentNullException("contenidoReparacion");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _contenidoReparacionRepository.UnitOfWork;

      _contenidoReparacionRepository.Remove(contenidoReparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionContenidoReparacionService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionContenidoReparacionService"/> </param>
    /// <returns><see cref="IGestionContenidoReparacionService"/></returns>
    public List<ContenidoReparacion> BuscarContenidoReparacionPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del contenidoReparacion en el repositorio
      var spec = new IdContenidoReparacionSpecification(id);

      return _contenidoReparacionRepository.BuscarContenidoReparacion(spec);
    }

    #endregion
  }
}