﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los empleados
  /// </summary>
  public interface IGestionEmpleadoService
  {
    /// <summary>
    /// Añadir un nuevo empleado
    /// </summary>
    /// <param name="empleado">Empleado a añadir</param>
    void AddEmpleado(Empleado empleado);

    /// <summary>
    /// Añadir un nuevo empleado
    /// </summary>    
    void AddEmpleado();

    /// <summary>
    /// Modificar un empleado existente
    /// </summary>
    /// <param name="empleado">Empleado con cambios a modificar</param>
    void ChangeEmpleado(Empleado empleado);

    /// <summary>
    /// Modificar un empleado existente
    /// </summary>
    /// <param name="idGuid">Identificador único del empleado a modificar </param>    
    void ChangeEmpleado(Guid idGuid);

    /// <summary>
    /// Borrar un empleado existente 
    /// </summary>
    /// <param name="empleado">Empleado a eliminar</param>
    void RemoveEmpleado(Empleado empleado);

    /// <summary>
    /// Borrar un empleado existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del empleado a eliminar</param>
    void RemoveEmpleado(Guid idGuid);

    /// <summary>
    /// Buscar empleado por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del empleado</param>
    /// <returns>Lista de empleados si existen o null si no existen empleados con el valor id igual a <paramref name="id"/></returns>
    List<Empleado> BuscarEmpleadoPorId(Guid? id);

    /// <summary>
    /// Encontrar todos los empleados
    /// </summary>
    /// <returns>Todos los empleados</returns>
    List<Empleado> BuscarTodosLosEmpleados();
  }
}