﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionManosDeObra
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de las mano De Obra
  /// </summary>
  public interface IGestionManoDeObraService
  {
    /// <summary>
    /// Añadir una nueva manoDeObra
    /// </summary>
    /// <param name="manoDeObra">ManoDeObra a añadir</param>
    void AddManoDeObra(ManoDeObra manoDeObra);

    /// <summary>
    /// Añadir una nueva manoDeObra
    /// </summary>
    /// <param name="idGuid">Identificador único de la mano de obra a añadir </param>
    /// <param name="descripcion">Descripción de la mano de obra a añadir</param>
    /// <param name="duracion">Duración  de la mano de obra a añadir</param>
    /// <param name="precioPorHora">Precio/Hora  de la mano de obra a añadir</param>
    void AddManoDeObra(Guid idGuid, string descripcion, decimal? duracion, decimal? precioPorHora);

    /// <summary>
    /// Modificar una manoDeObra existente
    /// </summary>
    /// <param name="manoDeObra">ManoDeObra con cambios a modificar</param>
    void ChangeManoDeObra(ManoDeObra manoDeObra);

    /// <summary>
    /// Modificar una manoDeObra existente
    /// </summary>
    /// <param name="idGuid">Identificador único de la manoDeObra a modificar </param>
    /// <param name="descripcion">Descripción de la mano de obra a modificar</param>
    /// <param name="duracion">Duración  de la mano de obra a modificar</param>
    /// <param name="precioPorHora">Precio/Hora  de la mano de obra a modificar</param>
    void ChangeManoDeObra(Guid idGuid, string descripcion, decimal? duracion, decimal? precioPorHora);

    /// <summary>
    /// Borrar una manoDeObra existente 
    /// </summary>
    /// <param name="manoDeObra">ManoDeObra a eliminar</param>
    void RemoveManoDeObra(ManoDeObra manoDeObra);

    /// <summary>
    /// Borrar una manoDeObra existente 
    /// </summary>
    /// <param name="idGuid">Identificador único de la manoDeObra a eliminar</param>
    void RemoveManoDeObra(Guid idGuid);

    /// <summary>
    /// Buscar manoDeObra por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id de la manoDeObra</param>
    /// <returns>Lista de manoDeObras si existen o null si no existen manoDeObras con el valor id igual a <paramref name="id"/></returns>
    List<ManoDeObra> BuscarManoDeObraPorId(Guid? id);

    /// <summary>
    /// Buscar manoDeObra por el valor de la propiedad Descripcion
    /// </summary>
    /// <param name="descripcion">Especificación de Descripcion de la manoDeObra</param>
    /// <returns>Lista de manoDeObras si existen o null si no existen manoDeObras con el valor descripcion igual a <paramref name="descripcion"/></returns>
    List<ManoDeObra> BuscarManoDeObraPorDescripcion(string descripcion);
  }
}