﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Marcas;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas
{
  /// <summary>
  /// Implementación de IGestionMarcaService
  /// </summary>
  public class GestionMarcaService : IAplicacionService, IGestionMarcaService
  {
    #region Atributos

    readonly IMarcaRepository _marcaRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="marcaRepository">Dependencia al repositorio de marcas, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionMarcaService(IMarcaRepository marcaRepository)
    {
      if (marcaRepository == null) throw new ArgumentNullException("marcaRepository");

      _marcaRepository = marcaRepository;
    }

    #endregion

    #region Miembros de IGestionMarcaService

    /// <summary>
    /// <see cref="IGestionMarcaService"/>
    /// </summary>
    /// <returns><see cref="Marca"/></returns>
    public List<Marca> BuscarTodasLasMarcas()
    {
      return _marcaRepository.BuscarMarcas();
    }
    #endregion
  }
}
