﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Modelos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos
{
  /// <summary>
  /// Implementación de IGestionModeloService
  /// </summary>
  public class GestionModeloService : IAplicacionService, IGestionModeloService
  {
    #region Atributos

    private readonly IModeloRepository _modeloRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="modeloRepository">Dependencia al repositorio de modelos, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionModeloService(IModeloRepository modeloRepository)
    {
      if (modeloRepository == null) throw new ArgumentNullException("modeloRepository");

      _modeloRepository = modeloRepository;
    }

    #endregion

    #region Miembros de IGestionModeloService

    /// <summary>
    /// <see cref="IGestionModeloService"/>
    /// </summary>
    /// <param name="idMarca"><see cref="IGestionModeloService"/> </param>
    /// <returns><see cref="IGestionModeloService"/></returns>
    public List<Modelo> BuscarListaModeloPorIdMarca(string idMarca)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new IdMarcaModeloSpecification(idMarca);

      return _modeloRepository.BuscarModelo(spec);
    }

    /// <summary>
    /// <see cref="IGestionModeloService"/>
    /// </summary>
    /// <returns><see cref="Modelo"/></returns>
    public IList<Modelo> BuscarTodosLosModelos()
    {
      return _modeloRepository.BuscarModelos();
    }

    #endregion
  }
}
