﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de las ordenes de Reparacion
  /// </summary>
  public interface IGestionOrdenDeReparacionService
  {
    /// <summary>
    /// Añadir una nueva orden Reparacion
    /// </summary>
    /// <param name="ordenReparacion">OrdenReparacion a añadir</param>
    void AddOrdenReparacion(OrdenReparacion ordenReparacion);

    /// <summary>
    /// Añadir una nueva orden Reparacion
    /// </summary>
    /// <param name="idGuid">Identificador único de la ordenReparacion a añadir </param>
    /// <param name="numeroOrden">Numero de orden de la ordenReparacion a añadir </param>
    /// <param name="fechaHoraEntrega">FechaHora de entrega de la ordenReparacion a añadir </param>
    /// <param name="estado">Estado de la ordenReparacion a añadir </param>
    /// <param name="idPersonalAsignado">Personal asignado a la ordenReparacion a añadir </param>
    /// <param name="subTotal">SubTotal de la ordenReparacion a añadir </param>
    /// <param name="importeIva">ImporteIva de la ordenReparacion a añadir </param>
    /// <param name="diagnostico">Diagnóstico de la ordenReparacion a añadir </param>
    /// <param name="idRecepcion">Id Recepcion de la ordenReparacion a añadir </param>
    /// <param name="idVehiculo">Id vehículo de la ordenReparacion a añadir </param>
    /// <param name="codigoTaller">Codigo taller de la ordenReparacion a añadir </param>
    void AddOrdenReparacion(Guid idGuid, string numeroOrden, DateTime? fechaHoraEntrega, int? estado, Guid? idPersonalAsignado, decimal? subTotal, decimal? importeIva, string diagnostico, Guid? idRecepcion, Guid? idVehiculo, string codigoTaller);

    /// <summary>
    /// Modificar una orden Reparacion existente
    /// </summary>
    /// <param name="ordenReparacion">OrdenReparacion con cambios a modificar</param>
    void ChangeOrdenReparacion(OrdenReparacion ordenReparacion);

    /// <summary>
    /// Modificar una orden Reparacion existente
    /// </summary>
    /// <param name="idGuid">Identificador único de la ordenReparacion a modificar </param>
    /// <param name="numeroOrden">Numero de orden de la ordenReparacion a modificar </param>
    /// <param name="fechaHoraEntrega">FechaHora de entrega de la ordenReparacion a modificar </param>
    /// <param name="estado">Estado de la ordenReparacion a modificar </param>
    /// <param name="idPersonalAsignado">Personal asignado a la ordenReparacion a modificar </param>
    /// <param name="subTotal">SubTotal de la ordenReparacion a modificar </param>
    /// <param name="importeIva">ImporteIva de la ordenReparacion a modificar </param>
    /// <param name="diagnostico">Diagnóstico de la ordenReparacion a modificar </param>
    /// <param name="idRecepcion">Id Recepcion de la ordenReparacion a modificar </param>
    /// <param name="idVehiculo">Id vehículo de la ordenReparacion a modificar </param>
    /// <param name="codigoTaller">Codigo taller de la ordenReparacion a modificar </param>
    void ChangeOrdenReparacion(Guid idGuid, string numeroOrden, DateTime? fechaHoraEntrega, int? estado, Guid? idPersonalAsignado, decimal? subTotal, decimal? importeIva, string diagnostico, Guid? idRecepcion, Guid? idVehiculo, string codigoTaller);

    /// <summary>
    /// Borrar una ordenReparacion existente 
    /// </summary>
    /// <param name="ordenReparacion">OrdenReparacion a eliminar</param>
    void RemoveOrdenReparacion(OrdenReparacion ordenReparacion);

    /// <summary>
    /// Borrar una ordenReparacion existente 
    /// </summary>
    /// <param name="idGuid">Identificador único de la ordenReparacion a eliminar</param>
    void RemoveOrdenReparacion(Guid idGuid);

    /// <summary>
    /// Buscar ordenReparacion por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id de la ordenReparacion</param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor id igual a <paramref name="id"/></returns>
    List<OrdenReparacion> BuscarOrdenReparacionPorId(Guid? id);

    /// <summary>
    /// Buscar ordenReparacion por el valor de la propiedad numeroOrden
    /// </summary>
    /// <param name="numeroOrden">Especificación de numero de orden de la ordenReparacion</param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor numeroOrden igual a <paramref name="numeroOrden"/></returns>
    List<OrdenReparacion> BuscarOrdenDeReparacionPorNumeroOrden(string numeroOrden);

    /// <summary>
    /// Buscar ordenReparacion por el valor de la propiedad matricula
    /// </summary>
    /// <param name="matricula">Especificación de matricula de la ordenReparacion</param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor matricula igual a <paramref name="matricula"/></returns>
    List<OrdenReparacion> BuscarOrdenDeReparacionPorMatricula(string matricula);

    /// <summary>
    /// Buscar ordenReparacion por el valor de la propiedad nombreCliente
    /// </summary>
    /// <param name="nombreCliente">Especificación de nombreCliente de la ordenReparacion</param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor nombreCliente igual a <paramref name="nombreCliente"/></returns>
    List<OrdenReparacion> BuscarOrdenDeReparacionPorNombreCliente(string nombreCliente);

    /// <summary>
    /// Buscar ordenReparacion por el valor de las propiedades idEmpleado y estado
    /// </summary>
    /// <param name="idEmpleado">Especificación de idEmpleado de la ordenReparacion</param>
    /// <param name="estado">Especificación de estado de la ordenReparacion </param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor idEmpleado igual a <paramref name="idEmpleado"/> y el valor estado igual a <paramref name="estado"/></returns>
    List<OrdenReparacion> BuscarPorEmpleadoYEstado(Guid? idEmpleado, int estado);

    /// <summary>
    /// Buscar ordenReparacion por el valor de la propiedad recepcionId
    /// </summary>
    /// <param name="recepcionId">Especificación de recepcionId de la ordenReparacion</param>
    /// <returns>Lista de ordenes de Reparacion si existen o null si no existen ordenes de Reparacion con el valor recepcionId igual a <paramref name="recepcionId"/></returns>
    List<OrdenReparacion> BuscarOrdenReparacionPorRecepcionId(Guid recepcionId);
  }
}