﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetes
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los paquetes
  /// </summary>
  public interface IGestionPaqueteService
  {
    /// <summary>
    /// Añadir un nuevo paquete
    /// </summary>
    /// <param name="paquete">Paquete a añadir</param>
    void AddPaquete(Paquete paquete);

    /// <summary>
    /// Añadir un nuevo paquete
    /// </summary>    
    void AddPaquete(Guid idArticulo, Guid idManoDeObra, Guid idPaqueteDeServicio);

    /// <summary>
    /// Modificar un paquete existente
    /// </summary>
    /// <param name="paquete">Paquete con cambios a modificar</param>
    void ChangePaquete(Paquete paquete);

    /// <summary>
    /// Modificar un paquete existente
    /// </summary>
    /// <param name="idArticulo">Identificador único del articulo del paquete a modificar </param>    
    /// <param name="idManoDeObra">Identificador único de la mano de obra del paquete a modificar </param>
    /// <param name="idPaqueteDeServicio">Identificador único del paquete de servicio del paquete a modificar </param>
    void ChangePaquete(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio);

    /// <summary>
    /// Borrar un paquete existente 
    /// </summary>
    /// <param name="paquete">Paquete a eliminar</param>
    void RemovePaquete(Paquete paquete);

    /// <summary>
    /// Borrar un paquete existente 
    /// </summary>
    /// <param name="idArticulo">Identificador único del articulo del paquete a eliminar </param>    
    /// <param name="idManoDeObra">Identificador único de la mano de obra del paquete a eliminar </param>
    /// <param name="idPaqueteDeServicio">Identificador único del paquete de servicio del paquete a eliminar </param>
    void RemovePaquete(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio);

    /// <summary>
    /// Buscar paquete por el valor de la propiedad Id
    /// </summary>
    /// <param name="idArticulo">Especificación de Id del articulo del paquete</param>
    /// <param name="idManoDeObra">Especificación de Id de la mano de obra del paquete</param>
    /// <param name="idPaqueteDeServicio">Especificación de Id del paquete de servicio del paquete</param>
    /// <returns>Lista de paquetes si existen o null si no existen paquetes</returns>
    List<Paquete> BuscarPaquetePorId(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio);
  }
}