﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los paquetes De Servicio
  /// </summary>
  public interface IGestionPaqueteDeServicioService
  {
    /// <summary>
    /// Añadir un nuevo paqueteDeServicio
    /// </summary>
    /// <param name="paqueteDeServicio">PaqueteDeServicio a añadir</param>
    void AddPaqueteDeServicio(PaqueteDeServicio paqueteDeServicio);

    /// <summary>
    /// Añadir un nuevo paqueteDeServicio
    /// </summary>
    /// <param name="idGuid">Identificador único del paqueteDeServicio a añadir </param>    
    /// <param name="descripcion">Descripción del paqueteDeServicio a añadir </param>
    void AddPaqueteDeServicio(Guid idGuid, string descripcion);

    /// <summary>
    /// Modificar un paqueteDeServicio existente
    /// </summary>
    /// <param name="paqueteDeServicio">PaqueteDeServicio con cambios a modificar</param>
    void ChangePaqueteDeServicio(PaqueteDeServicio paqueteDeServicio);

    /// <summary>
    /// Modificar un paqueteDeServicio existente
    /// </summary>
    /// <param name="idGuid">Identificador único del paqueteDeServicio a modificar </param>    
    /// <param name="descripcion">Descripción del paqueteDeServicio a modificar </param>
    void ChangePaqueteDeServicio(Guid idGuid, string descripcion);

    /// <summary>
    /// Borrar un paqueteDeServicio existente 
    /// </summary>
    /// <param name="paqueteDeServicio">PaqueteDeServicio a eliminar</param>
    void RemovePaqueteDeServicio(PaqueteDeServicio paqueteDeServicio);

    /// <summary>
    /// Borrar un paqueteDeServicio existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del paqueteDeServicio a eliminar</param>
    void RemovePaqueteDeServicio(Guid idGuid);

    /// <summary>
    /// Buscar paqueteDeServicio por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del paqueteDeServicio</param>
    /// <returns>Lista de paqueteDeServicios si existen o null si no existen paqueteDeServicios con el valor id igual a <paramref name="id"/></returns>
    List<PaqueteDeServicio> BuscarPaqueteDeServicioPorId(Guid? id);

    /// <summary>
    /// Buscar paqueteDeServicio por el valor de la propiedad Descripcion
    /// </summary>
    /// <param name="descripcion">Especificación de descripcion del paqueteDeServicio</param>
    /// <returns>Lista de paqueteDeServicios si existen o null si no existen paqueteDeServicios con el valor descripcion igual a <paramref name="descripcion"/></returns>
    List<PaqueteDeServicio> BuscarPaqueteDeServicioPorDescripcion(string descripcion);
  }
}