﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Poblaciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPoblaciones
{
  /// <summary>
  /// Implementación de IGestionPoblacionService
  /// </summary>
  public class GestionPoblacionService : IAplicacionService, IGestionPoblacionService
  {
    #region Atributos

    private readonly IPoblacionRepository _poblacionRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="poblacionRepository">Dependencia al repositorio de poblaciones, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionPoblacionService(IPoblacionRepository poblacionRepository)
    {
      if (poblacionRepository == null) throw new ArgumentNullException("poblacionRepository");

      _poblacionRepository = poblacionRepository;
    }

    #endregion

    #region Miembros de IGestionPoblacionService

    /// <summary>
    /// <see cref="IGestionPoblacionService"/>
    /// </summary>
    /// <param name="idProvincia"><see cref="IGestionPoblacionService"/> </param>
    /// <returns><see cref="IGestionPoblacionService"/></returns>
    public List<Poblacion> BuscarListaPoblacionPorIdProvincia(string idProvincia)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new IdProvinciaPoblacionSpecification(idProvincia);

      return _poblacionRepository.BuscarPoblacion(spec);
    }
    #endregion
  }
}
