﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPresupuestos
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los presupuestos
  /// </summary>
  public interface IGestionPresupuestoService
  {
    /// <summary>
    /// Añadir un nuevo presupuesto
    /// </summary>
    /// <param name="presupuesto">Presupuesto a añadir</param>
    void AddPresupuesto(Presupuesto presupuesto);

    /// <summary>
    /// Añadir un nuevo presupuesto
    /// </summary>    
    void AddPresupuesto(Guid idPresupuesto, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller);

    /// <summary>
    /// Modificar un presupuesto existente
    /// </summary>
    /// <param name="presupuesto">Presupuesto con cambios a modificar</param>
    void ChangePresupuesto(Presupuesto presupuesto);

    /// <summary>
    /// Modificar un presupuesto existente
    /// </summary>
    void ChangePresupuesto(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller);

    /// <summary>
    /// Borrar un presupuesto existente 
    /// </summary>
    /// <param name="presupuesto">Presupuesto a eliminar</param>
    void RemovePresupuesto(Presupuesto presupuesto);

    /// <summary>
    /// Borrar un presupuesto existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del presupuesto a eliminar</param>
    void RemovePresupuesto(Guid idGuid);

    /// <summary>
    /// Buscar presupuesto por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del presupuesto</param>
    /// <returns>Lista de presupuestos si existen o null si no existen presupuestos con el valor id igual a <paramref name="id"/></returns>
    List<Presupuesto> BuscarPresupuestoPorId(Guid? id);

    /// <summary>
    /// Buscar presupuesto por numero.
    /// </summary>
    /// <param name="numero">Especificación de número de presupuesto.</param>
    /// <returns>Lista de presupuestos si existen o null si no existen presupuestos con el valor numero igual a <paramref name="numero"/></returns>
    List<Presupuesto> BuscarPresupuestoPorNumero(string numero);

    /// <summary>
    /// Buscar presupuesto por cliente y fecha.
    /// </summary>
    /// <param name="cliente">Especificación de cliente del presupuesto.</param>
    /// <param name="fecha">Especificación de fecha a partir de la cual se realiza la búsqueda.</param>
    /// <returns>Lista de presupuestos si existen o null si no existen presupuestos que coincidan con los criterios anteriores</returns>
    List<Presupuesto> BuscarPresupuestoPorClienteYFecha(Guid? cliente, DateTime? fecha);
    
  }
}