﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los proveedores
  /// </summary>
  public interface IGestionProveedorService
  {
    /// <summary>
    /// Añadir un nuevo proveedor
    /// </summary>
    /// <param name="proveedor">Proveedor a añadir</param>
    void AddProveedor(Proveedor proveedor);

    /// <summary>
    /// Añadir un nuevo proveedor
    /// </summary>    
    /// <param name="idGuid">Identificador único del proveedor a añadir </param>
    /// <param name="codigo">Código del proveedor a añadir </param>
    /// <param name="razonSocial">Razon social del proveedor a añadir </param>
    /// <param name="direccion">Direccion del proveedor a añadir </param>
    /// <param name="idCodigoPostal">Id del código postal del proveedor a añadir </param>
    /// <param name="cifDni">Cif/Dni del proveedor a añadir </param>
    /// <param name="telefonos">Teléfonos del proveedor a añadir </param>
    /// <param name="observaciones">Observaciones del proveedor a añadir </param>
    /// <param name="correoElectronico">Correo electrónico del proveedor a añadir </param>
    /// <param name="paginaWeb">Página web del proveedor a añadir </param>    
    void AddProveedor(Guid idGuid, string codigo, string razonSocial, string direccion, int? idCodigoPostal,
                      string cifDni, string telefonos, string observaciones, string correoElectronico,
                      string paginaWeb);

    /// <summary>
    /// Modificar un proveedor existente
    /// </summary>
    /// <param name="proveedor">Proveedor con cambios a modificar</param>
    void ChangeProveedor(Proveedor proveedor);

    /// <summary>
    /// Modificar un proveedor existente
    /// </summary>
    /// <param name="idGuid">Identificador único del proveedor a modificar </param>  
    /// <param name="codigo">Código del proveedor a modificar </param>
    /// <param name="razonSocial">Razon social del proveedor a modificar </param>
    /// <param name="direccion">Direccion del proveedor a modificar </param>
    /// <param name="idCodigoPostal">Id del código postal del proveedor a modificar </param>
    /// <param name="cifDni">Cif/Dni del proveedor a modificar </param>
    /// <param name="telefonos">Teléfonos del proveedor a modificar </param>
    /// <param name="observaciones">Observaciones del proveedor a modificar </param>
    /// <param name="correoElectronico">Correo electrónico del proveedor a modificar </param>
    /// <param name="paginaWeb">Página web del proveedor a modificar </param>    
    void ChangeProveedor(Guid idGuid, string codigo, string razonSocial, string direccion, int? idCodigoPostal,
                         string cifDni, string telefonos, string observaciones, string correoElectronico,
                         string paginaWeb);

    /// <summary>
    /// Borrar un proveedor existente 
    /// </summary>
    /// <param name="proveedor">Proveedor a eliminar</param>
    void RemoveProveedor(Proveedor proveedor);

    /// <summary>
    /// Borrar un proveedor existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del proveedor a eliminar</param>
    void RemoveProveedor(Guid idGuid);

    /// <summary>
    /// Buscar proveedor por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del proveedor</param>
    /// <returns>Lista de proveedors si existen o null si no existen proveedores con el valor id igual a <paramref name="id"/></returns>
    List<Proveedor> BuscarProveedorPorId(Guid? id);

    /// <summary>
    /// Buscar todos los proveedores.
    /// </summary>
    /// <returns>Lista de todos los proveedores que existan</returns>
    List<Proveedor> BuscarTodosLosProveedores();

    /// <summary>
    /// Buscar proveedor por el valor de la propiedad Codigo
    /// </summary>
    /// <param name="codigo">Especificación de Codigo del proveedor</param>
    /// <returns>Lista de proveedors si existen o null si no existen proveedores con el valor codigo igual a <paramref name="codigo"/></returns>
    List<Proveedor> BuscarProveedorPorCodigo(string codigo);

    /// <summary>
    /// Buscar proveedor por el valor de la propiedad CifDni
    /// </summary>
    /// <param name="cifDni">Especificación de CifDni del proveedor</param>
    /// <returns>Lista de proveedors si existen o null si no existen proveedores con el valor cifDni igual a <paramref name="cifDni"/></returns>
    List<Proveedor> BuscarProveedorPorCifDni(string cifDni);

    /// <summary>
    /// Buscar proveedor por el valor de la propiedad RazonSocial
    /// </summary>
    /// <param name="razonSocial">Especificación de razón social del proveedor</param>
    /// <returns>Lista de proveedors si existen o null si no existen proveedores con el valor RazonSocial igual a <paramref name="razonSocial"/></returns>
    List<Proveedor> BuscarProveedorPorRazonSocial(string razonSocial);
  }
}