﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Recepciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionRecepciones
{
  /// <summary>
  /// Implementación de IGestionRecepcionService
  /// </summary>
  public class GestionRecepcionVehiculoService : IAplicacionService, IGestionRecepcionVehiculoService
  {
    #region Atributos

    private readonly IRecepcionRepository _recepcionRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="recepcionRepository">Dependencia al repositorio de recepciones, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionRecepcionVehiculoService(IRecepcionRepository recepcionRepository)
    {
      if (recepcionRepository == null) throw new ArgumentNullException("recepcionRepository");

      _recepcionRepository = recepcionRepository;
    }

    #endregion

    #region Miembros de IGestionRecepcionService

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="recepcion"><see cref="IGestionRecepcionVehiculoService"/></param>
    public void AddRecepcion(Recepcion recepcion)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _recepcionRepository.UnitOfWork;

      _recepcionRepository.Add(recepcion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionRecepcionVehiculoService"/></param>
    /// <param name="fechaHoraRecepcion"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="observacionesSobreElCliente"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="nombreContactoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="telefonoContactoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="subTotal"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="importeIva"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="kilometraje"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="prediagnostico"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="contenidoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="combustible"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="fechaHoraEntrega"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="observaciones"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idCliente"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idVehiculo"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionRecepcionVehiculoService"/> </param>    
    public void AddRecepcion(Guid idGuid, DateTime? fechaHoraRecepcion, string observacionesSobreElCliente,
                             string nombreContactoAdicional, string telefonoContactoAdicional, decimal? subTotal,
                             decimal? importeIva, decimal? kilometraje, string prediagnostico, string contenidoAdicional,
                             int? combustible, DateTime? fechaHoraEntrega, string observaciones, Guid? idCliente,
                             Guid? idVehiculo, Guid? idEmpleado, string codigoTaller)
    {
      var recepcion = new Recepcion
                {
                  Id = idGuid,
                  FechaHoraRecepcion = fechaHoraRecepcion,
                  ObservacionesSobreElCliente = observacionesSobreElCliente,
                  NombreContactoAdicional = nombreContactoAdicional,
                  TelefonoContactoAdicional = telefonoContactoAdicional,
                  SubTotal = subTotal,
                  ImporteIva = importeIva,
                  Kilometraje = kilometraje,
                  Prediagnostico = prediagnostico,
                  ContenidoAdicional = contenidoAdicional,
                  Combustible = combustible,
                  FechaHoraEntrega = fechaHoraEntrega,
                  Observaciones = observaciones,
                  IdCliente = idCliente,
                  IdVehiculo = idVehiculo,
                  IdEmpleado = idEmpleado,
                  CodigoTaller = codigoTaller
                };

      AddRecepcion(recepcion);
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionRecepcionVehiculoService"/></param>    
    /// <param name="fechaHoraRecepcion"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="observacionesSobreElCliente"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="nombreContactoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="telefonoContactoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="subTotal"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="importeIva"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="kilometraje"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="prediagnostico"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="contenidoAdicional"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="combustible"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="fechaHoraEntrega"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="observaciones"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idCliente"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idVehiculo"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionRecepcionVehiculoService"/> </param>
    public void ChangeRecepcion(Guid idGuid, DateTime? fechaHoraRecepcion, string observacionesSobreElCliente,
                             string nombreContactoAdicional, string telefonoContactoAdicional, decimal? subTotal,
                             decimal? importeIva, decimal? kilometraje, string prediagnostico, string contenidoAdicional,
                             int? combustible, DateTime? fechaHoraEntrega, string observaciones, Guid? idCliente,
                             Guid? idVehiculo, Guid? idEmpleado, string codigoTaller)
    {
      var recepcion = BuscarRecepcionPorId(idGuid).FirstOrDefault();
      if (recepcion != null)
      {
        recepcion.FechaHoraRecepcion = fechaHoraRecepcion;
        recepcion.ObservacionesSobreElCliente = observacionesSobreElCliente;
        recepcion.NombreContactoAdicional = nombreContactoAdicional;
        recepcion.TelefonoContactoAdicional = telefonoContactoAdicional;
        recepcion.SubTotal = subTotal;
        recepcion.ImporteIva = importeIva;
        recepcion.Kilometraje = kilometraje;
        recepcion.Prediagnostico = prediagnostico;
        recepcion.ContenidoAdicional = contenidoAdicional;
        recepcion.Combustible = combustible;
        recepcion.FechaHoraEntrega = fechaHoraEntrega;
        recepcion.Observaciones = observaciones;
        recepcion.IdCliente = idCliente;
        recepcion.IdVehiculo = idVehiculo;
        recepcion.IdEmpleado = idEmpleado;
        recepcion.CodigoTaller = codigoTaller;

        ChangeRecepcion(recepcion);
      }
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="recepcion"><see cref="IGestionRecepcionVehiculoService"/></param>
    public void ChangeRecepcion(Recepcion recepcion)
    {
      if (recepcion == null) throw new ArgumentNullException("recepcion");

      var unitOfWork = _recepcionRepository.UnitOfWork;

      _recepcionRepository.Modify(recepcion);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionRecepcionVehiculoService"/></param>
    public void RemoveRecepcion(Guid idGuid)
    {
      var recepcion = BuscarRecepcionPorId(idGuid).FirstOrDefault();
      if (recepcion != null)
      {
        RemoveRecepcion(recepcion);
      }
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="recepcion"><see cref="IGestionRecepcionVehiculoService"/></param>
    public void RemoveRecepcion(Recepcion recepcion)
    {
      if (recepcion == null) throw new ArgumentNullException("recepcion");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _recepcionRepository.UnitOfWork;

      _recepcionRepository.Remove(recepcion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionRecepcionVehiculoService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionRecepcionVehiculoService"/> </param>
    /// <returns><see cref="IGestionRecepcionVehiculoService"/></returns>
    public List<Recepcion> BuscarRecepcionPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del recepcion en el repositorio
      var spec = new IdRecepcionSpecification(id);

      return _recepcionRepository.BuscarRecepcion(spec);
    }

    public List<Recepcion> BuscarRecepcionVehiculoPorMatricula(string matricula)
    {
      //Crear especificación para la búsqueda del recepcion en el repositorio
      var spec = new MatriculaRecepcionSpecification(matricula);

      return _recepcionRepository.BuscarRecepcion(spec);
    }

    public List<Recepcion> BuscarRecepcionVehiculoPorNifCifCliente(string nifCif)
    {
      //Crear especificación para la búsqueda del recepcion en el repositorio
      var spec = new NifCifRecepcionSpecification(nifCif);

      return _recepcionRepository.BuscarRecepcion(spec);
    }

    public List<Recepcion> BuscarRecepcionVehiculoPorNombreCliente(string nombreCliente)
    {
      //Crear especificación para la búsqueda del recepcion en el repositorio
      var spec = new NombreClienteRecepcionSpecification(nombreCliente);

      return _recepcionRepository.BuscarRecepcion(spec);
    }

    #endregion
  }
}