﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionRecepciones
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los recepciones
  /// </summary>
  public interface IGestionRecepcionVehiculoService
  {
    /// <summary>
    /// Añadir una nueva recepcion
    /// </summary>
    /// <param name="recepcion">Recepcion a añadir</param>
    void AddRecepcion(Recepcion recepcion);

    /// <summary>
    /// Añadir una nueva recepcion
    /// </summary>    
    /// <param name="idGuid">Identificador único de la recepcion a añadir </param>
    /// <param name="fechaHoraRecepcion">FechaHora de Recepcion de la recepcion a añadir </param>
    /// <param name="observacionesSobreElCliente">Observaciones sobre el cliente de la recepcion a añadir </param>
    /// <param name="nombreContactoAdicional">Nombre del Contacto Adicional de la recepcion a añadir </param>
    /// <param name="telefonoContactoAdicional">Telefono del Contacto Adicional de la recepcion a añadir </param>
    /// <param name="subTotal">SubTotal de la recepcion a añadir </param>
    /// <param name="importeIva">Importe Iva de la recepcion a añadir </param>
    /// <param name="kilometraje">Kilometraje del vehículo recepcionado en la recepcion a añadir </param>
    /// <param name="prediagnostico">Prediagnostico del vehículo recepcionado en la recepcion a añadir </param>
    /// <param name="contenidoAdicional">Contenido adicional en el vehículo de la recepcion a añadir </param>
    /// <param name="combustible">Cantidad de combustible del vehículo de la recepcion a añadir </param>
    /// <param name="fechaHoraEntrega">FechaHora entrega aproximada del vehículo de la recepcion a añadir </param>
    /// <param name="observaciones">Observaciones de la recepcion a añadir </param>
    /// <param name="idCliente">IdCliente de la recepcion a añadir </param>
    /// <param name="idVehiculo">IdVehiculo de la recepcion a añadir </param>
    /// <param name="idEmpleado">IdEmpleado de la recepcion a añadir </param>
    /// <param name="codigoTaller">CodigoTaller de la recepcion a añadir </param>
    void AddRecepcion(Guid idGuid, DateTime? fechaHoraRecepcion, string observacionesSobreElCliente,
                      string nombreContactoAdicional, string telefonoContactoAdicional, decimal? subTotal,
                      decimal? importeIva, decimal? kilometraje, string prediagnostico, string contenidoAdicional,
                      int? combustible, DateTime? fechaHoraEntrega, string observaciones, Guid? idCliente,
                      Guid? idVehiculo, Guid? idEmpleado, string codigoTaller);

    /// <summary>
    /// Modificar una recepcion existente
    /// </summary>
    /// <param name="recepcion">Recepcion con cambios a modificar</param>
    void ChangeRecepcion(Recepcion recepcion);

    /// <summary>
    /// Modificar una recepcion existente
    /// </summary>
    /// <param name="idGuid">Identificador único de la recepcion a modificar </param>
    /// <param name="fechaHoraRecepcion">FechaHora de Recepcion de la recepcion a modificar </param>
    /// <param name="observacionesSobreElCliente">Observaciones sobre el cliente de la recepcion a modificar </param>
    /// <param name="nombreContactoAdicional">Nombre del Contacto Adicional de la recepcion a modificar </param>
    /// <param name="telefonoContactoAdicional">Telefono del Contacto Adicional de la recepcion a modificar </param>
    /// <param name="subTotal">SubTotal de la recepcion a modificar </param>
    /// <param name="importeIva">Importe Iva de la recepcion a modificar </param>
    /// <param name="kilometraje">Kilometraje del vehículo recepcionado en la recepcion a modificar </param>
    /// <param name="prediagnostico">Prediagnostico del vehículo recepcionado en la recepcion a modificar </param>
    /// <param name="contenidoAdicional">Contenido adicional en el vehículo de la recepcion a modificar </param>
    /// <param name="combustible">Cantidad de combustible del vehículo de la recepcion a modificar </param>
    /// <param name="fechaHoraEntrega">FechaHora entrega aproximada del vehículo de la recepcion a modificar </param>
    /// <param name="observaciones">Observaciones de la recepcion a modificar </param>
    /// <param name="idCliente">IdCliente de la recepcion a modificar </param>
    /// <param name="idVehiculo">IdVehiculo de la recepcion a modificar </param>
    /// <param name="idEmpleado">IdEmpleado de la recepcion a modificar </param>
    /// <param name="codigoTaller">CodigoTaller de la recepcion a modificar </param>    
    void ChangeRecepcion(Guid idGuid, DateTime? fechaHoraRecepcion, string observacionesSobreElCliente,
                         string nombreContactoAdicional, string telefonoContactoAdicional, decimal? subTotal,
                         decimal? importeIva, decimal? kilometraje, string prediagnostico, string contenidoAdicional,
                         int? combustible, DateTime? fechaHoraEntrega, string observaciones, Guid? idCliente,
                         Guid? idVehiculo, Guid? idEmpleado, string codigoTaller);

    /// <summary>
    /// Borrar una recepcion existente 
    /// </summary>
    /// <param name="recepcion">Recepcion a eliminar</param>
    void RemoveRecepcion(Recepcion recepcion);

    /// <summary>
    /// Borrar una recepcion existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del recepcion a eliminar</param>
    void RemoveRecepcion(Guid idGuid);

    /// <summary>
    /// Buscar recepcion por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id de la recepcion</param>
    /// <returns>Lista de recepciones si existen o null si no existen recepciones con el valor id igual a <paramref name="id"/></returns>
    List<Recepcion> BuscarRecepcionPorId(Guid? id);

    List<Recepcion> BuscarRecepcionVehiculoPorMatricula(string matricula);
    List<Recepcion> BuscarRecepcionVehiculoPorNifCifCliente(string nifCif);
    List<Recepcion> BuscarRecepcionVehiculoPorNombreCliente(string nombreCliente);
  }
}