﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionReparaciones
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los reparaciones
  /// </summary>
  public interface IGestionReparacionService
  {
    /// <summary>
    /// Añadir una nueva reparacion
    /// </summary>
    /// <param name="reparacion">Reparacion a añadir</param>
    void AddReparacion(Reparacion reparacion);

    /// <summary>
    /// Añadir una nueva reparacion
    /// </summary>    
    void AddReparacion();

    /// <summary>
    /// Modificar una reparacion existente
    /// </summary>
    /// <param name="reparacion">Reparacion con cambios a modificar</param>
    void ChangeReparacion(Reparacion reparacion);

    /// <summary>
    /// Modificar una reparacion existente
    /// </summary>
    /// <param name="idReparacion">Identificador único del reparacion a modificar </param>
    /// <param name="duracionSugerida">Duración sugerida para la reparación </param>
    /// <param name="duracionReal">Duración real de la reparación introducida por el empleado </param>
    /// <param name="estado">Estado de la reparación </param>
    /// <param name="idEmpleado">Identificador del empleado que ha realizado la reparación </param>
    /// <param name="idOrdenDeReparacion">Identificador de la orden de reparación que contiene la reparación </param>    
    void ChangeReparacion(Guid idReparacion, decimal? duracionSugerida, decimal? duracionReal, int? estado,
                          Guid? idEmpleado, Guid idOrdenDeReparacion);

    /// <summary>
    /// Borrar una reparacion existente 
    /// </summary>
    /// <param name="reparacion">Reparacion a eliminar</param>
    void RemoveReparacion(Reparacion reparacion);

    /// <summary>
    /// Borrar una reparacion existente 
    /// </summary>
    /// <param name="idGuid">Identificador único de la reparacion a eliminar</param>
    void RemoveReparacion(Guid idGuid);

    /// <summary>
    /// Buscar reparacion por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del reparacion</param>
    /// <returns>Lista de reparaciones si existen o null si no existen reparaciones con el valor id igual a <paramref name="id"/></returns>
    List<Reparacion> BuscarReparacionPorId(Guid? id);
  }
}