﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Talleres;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTalleres
{
  /// <summary>
  /// Implementación de IGestionTallerService
  /// </summary>
  public class GestionTallerService : IAplicacionService, IGestionTallerService
  {
    #region Atributos

    private readonly ITallerRepository _tallerRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="tallerRepository">Dependencia al repositorio de talleres, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionTallerService(ITallerRepository tallerRepository)
    {
      if (tallerRepository == null) throw new ArgumentNullException("tallerRepository");

      _tallerRepository = tallerRepository;
    }

    #endregion

    #region Miembros de IGestionTallerService

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="taller"><see cref="IGestionTallerService"/></param>
    public void AddTaller(Taller taller)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _tallerRepository.UnitOfWork;

      _tallerRepository.Add(taller);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionTallerService"/></param>    
    public void AddTaller(string idGuid)
    {
      var taller = new Taller
                {
                  Id = idGuid
                };

      AddTaller(taller);
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    public void AddTaller()
    {
      throw new NotImplementedException();
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionTallerService"/></param>    
    public void ChangeTaller(string idGuid)
    {
      var taller = BuscarTallerPorId(idGuid).FirstOrDefault();
      if (taller != null)
      {
        // TODO: Falta poner propiedades a actualizar	
        //cliente.Nombre = nombre;

        ChangeTaller(taller);
      }
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="taller"><see cref="IGestionTallerService"/></param>
    public void ChangeTaller(Taller taller)
    {
      if (taller == null) throw new ArgumentNullException("taller");

      var unitOfWork = _tallerRepository.UnitOfWork;

      _tallerRepository.Modify(taller);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionTallerService"/></param>
    public void RemoveTaller(string idGuid)
    {
      var taller = BuscarTallerPorId(idGuid).FirstOrDefault();
      if (taller != null)
      {
        RemoveTaller(taller);
      }
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="taller"><see cref="IGestionTallerService"/></param>
    public void RemoveTaller(Taller taller)
    {
      if (taller == null) throw new ArgumentNullException("taller");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _tallerRepository.UnitOfWork;

      _tallerRepository.Remove(taller);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionTallerService"/> </param>
    /// <returns><see cref="IGestionTallerService"/></returns>
    public List<Taller> BuscarTallerPorId(string id)
    {
      //Crear especificación para la búsqueda del taller en el repositorio
      var spec = new IdTallerSpecification(id);

      return _tallerRepository.BuscarTaller(spec);
    }

    /// <summary>
    /// <see cref="IGestionTallerService"/>
    /// </summary>
    /// <returns>
    /// <see cref="IGestionTallerService"/>
    /// </returns>
    public List<Taller> BuscarTodosLosTalleres()
    {
      return _tallerRepository.BuscarTodosLosTalleres();
    }

    #endregion
  }
}