﻿using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTalleres
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los talleres
  /// </summary>
  public interface IGestionTallerService
  {
    /// <summary>
    /// Añadir un nuevo taller
    /// </summary>
    /// <param name="taller">Taller a añadir</param>
    void AddTaller(Taller taller);

    /// <summary>
    /// Añadir un nuevo taller
    /// </summary>    
    void AddTaller();

    /// <summary>
    /// Modificar un taller existente
    /// </summary>
    /// <param name="taller">Taller con cambios a modificar</param>
    void ChangeTaller(Taller taller);

    /// <summary>
    /// Modificar un taller existente
    /// </summary>
    /// <param name="idGuid">Identificador único del taller a modificar </param>    
    void ChangeTaller(string idGuid);

    /// <summary>
    /// Borrar un taller existente 
    /// </summary>
    /// <param name="taller">Taller a eliminar</param>
    void RemoveTaller(Taller taller);

    /// <summary>
    /// Borrar un taller existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del taller a eliminar</param>
    void RemoveTaller(string idGuid);

    /// <summary>
    /// Buscar taller por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del taller</param>
    /// <returns>Lista de talleres si existen o null si no existen talleres con el valor id igual a <paramref name="id"/></returns>
    List<Taller> BuscarTallerPorId(string id);

    /// <summary>
    /// Buscar todos los talleres.
    /// </summary>
    /// <returns>Lista de todos los talleres</returns>
    List<Taller> BuscarTodosLosTalleres();
  }
}