﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.TiposDeCombustible;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTiposCombustible
{
  /// <summary>
  /// Implementación de IGestionFormaDePagoService
  /// </summary>
  public class GestionTipoCombustibleService : IAplicacionService, IGestionTipoCombustibleService
  {
    #region Atributos

    readonly ITipoCombustibleRepository _tipoCombustibleRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="tipoCombustibleRepository">Dependencia al repositorio de tipos de combustible, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionTipoCombustibleService(ITipoCombustibleRepository tipoCombustibleRepository)
    {
      if (tipoCombustibleRepository == null) throw new ArgumentNullException("tipoCombustibleRepository");

      _tipoCombustibleRepository = tipoCombustibleRepository;
    }

    #endregion

    #region Miembros de IGestionTipoCombustibleService

    /// <summary>
    /// <see cref="IGestionTipoCombustibleService"/>
    /// </summary>
    /// <returns><see cref="TipoCombustible"/></returns>
    public List<TipoCombustible> BuscarTodosLosTiposDeCombustible()
    {
      return _tipoCombustibleRepository.BuscarTipoCombustible();
    }
    #endregion
  }
}
