﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.TiposEmpleado;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTiposEmpleado
{
  /// <summary>
  /// Implementación de IGestionTipoVehiculoService
  /// </summary>
  public class GestionTipoEmpleadoService : IAplicacionService, IGestionTipoEmpleadoService
  {
    #region Atributos

    readonly ITipoEmpleadoRepository _tipoEmpleadoRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="tipoEmpleadoRepository">Dependencia al repositorio de Tipos de empleado, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionTipoEmpleadoService(ITipoEmpleadoRepository tipoEmpleadoRepository)
    {
      if (tipoEmpleadoRepository == null) throw new ArgumentNullException("tipoEmpleadoRepository");

      _tipoEmpleadoRepository = tipoEmpleadoRepository;
    }

    #endregion

    #region Miembros de IGestionTipoEmpleadoService

    /// <summary>
    /// <see cref="IGestionTipoEmpleadoService"/>
    /// </summary>
    /// <returns><see cref="TipoEmpleado"/></returns>
    public List<TipoEmpleado> BuscarTodosLosTiposDeEmpleado()
    {
      return _tipoEmpleadoRepository.BuscarTodosLosTiposDeEmpleado();
    }
    #endregion
  }
}
