﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Vehiculos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos
{
  /// <summary>
  /// Implementación de IGestionVehiculoService
  /// </summary>
  public class GestionVehiculoService : IAplicacionService, IGestionVehiculoService
  {
    #region Atributos

    readonly IVehiculoRepository _vehiculoRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="vehiculoRepository">Dependencia al repositorio de vehiculos, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionVehiculoService(IVehiculoRepository vehiculoRepository)
    {
      if (vehiculoRepository == null) throw new ArgumentNullException("vehiculoRepository");

      _vehiculoRepository = vehiculoRepository;
    }

    #endregion

    #region Miembros de IGestionVehiculoService

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="vehiculo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></param>
    public void AddVehiculo(Vehiculo vehiculo)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _vehiculoRepository.UnitOfWork;

      _vehiculoRepository.Add(vehiculo);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="id"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="matricula"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="marca"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="modelo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="color"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="fecha"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="bastidor"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    public void AddVehiculo(Guid id, string matricula, string marca, string modelo, string color, DateTime? fecha,
                            string bastidor)
    {
      var vehiculo = new Vehiculo
                       {
                         Id = id,
                         Matricula = matricula,
                         Marca = marca,
                         Modelo = modelo,
                         Color = color,
                         Fecha = fecha,
                         Bastidor = bastidor
                       };
      AddVehiculo(vehiculo);
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="matricula"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="marca"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="modelo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="color"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="fecha"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="bastidor"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    public void ChangeVehiculo(Guid idGuid, string matricula, string marca, string modelo, string color, DateTime? fecha,
                               string bastidor)
    {
      var vehiculo = BuscarVehiculoPorId(idGuid).FirstOrDefault();
      if (vehiculo != null)
      {
        vehiculo.Matricula = matricula;
        vehiculo.Marca = marca;
        vehiculo.Modelo = modelo;
        vehiculo.Color = color;
        vehiculo.Fecha = fecha;
        vehiculo.Bastidor = bastidor;

        ChangeVehiculo(vehiculo);
      }
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="vehiculo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></param>
    public void ChangeVehiculo(Vehiculo vehiculo)
    {
      if (vehiculo == null) throw new ArgumentNullException("vehiculo");

      var unitOfWork = _vehiculoRepository.UnitOfWork;

      _vehiculoRepository.Modify(vehiculo);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></param>
    public void RemoveVehiculo(Guid idGuid)
    {
      var vehiculo = BuscarVehiculoPorId(idGuid).FirstOrDefault();
      if (vehiculo != null)
      {
        RemoveVehiculo(vehiculo);
      }
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="vehiculo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></param>
    public void RemoveVehiculo(Vehiculo vehiculo)
    {
      if (vehiculo == null) throw new ArgumentNullException("vehiculo");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _vehiculoRepository.UnitOfWork;

      _vehiculoRepository.Remove(vehiculo);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="marca"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <param name="modelo"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <returns><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></returns>
    public List<Vehiculo> BuscarVehiculoPorMarcaYModelo(string marca, string modelo)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new MarcaModeloVehiculoSpecification(marca, modelo);

      return _vehiculoRepository.BuscarVehiculo(spec);
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="matricula"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <returns><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></returns>
    public List<Vehiculo> BuscarVehiculoPorMatricula(string matricula)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new MatriculaVehiculoSpecification(matricula);

      return _vehiculoRepository.BuscarVehiculo(spec);
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="id"><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/> </param>
    /// <returns><see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/></returns>
    public List<Vehiculo> BuscarVehiculoPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new IdVehiculoSpecification(id);

      return _vehiculoRepository.BuscarVehiculo(spec);
    }
    #endregion
  }
}
