﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los vehiculos
  /// </summary>
  public interface IGestionVehiculoService
  {
    /// <summary>
    /// Añadir un nuevo vehículo
    /// </summary>
    /// <param name="vehiculo">Vehículo a añadir</param>
    void AddVehiculo(Vehiculo vehiculo);

    /// <summary>
    /// Añadir un nuevo vehiculo
    /// </summary>
    /// <param name="id">Id del vehículo </param>
    /// <param name="matricula">Matricula del vehiculo.</param>
    /// <param name="marca">Marca del vehiculo.</param>
    /// <param name="modelo">Modelo del vehiculo.</param>
    /// <param name="color">Color del vehiculo.</param>
    /// <param name="fecha">Fecha del vehiculo.</param>
    /// <param name="bastidor">Bastidor del vehiculo.</param>
    void AddVehiculo(Guid id, string matricula, string marca, string modelo, string color, DateTime? fecha, string bastidor);

    /// <summary>
    /// Modificar un vehículo existente
    /// </summary>
    /// <param name="vehiculo">Vehiculo con cambios a modificar</param>
    void ChangeVehiculo(Vehiculo vehiculo);

    /// <summary>
    /// Modificar un vehículo existente
    /// </summary>
    /// <param name="idGuid">Identificador único del vehículo a modificar </param>
    /// <param name="matricula">Matricula del vehiculo.</param>
    /// <param name="marca">Marca del vehiculo.</param>
    /// <param name="modelo">Modelo del vehiculo.</param>
    /// <param name="color">Color del vehiculo.</param>
    /// <param name="fecha">Fecha del vehiculo.</param>
    /// <param name="bastidor">Bastidor del vehiculo.</param>
    void ChangeVehiculo(Guid idGuid, string matricula, string marca, string modelo, string color, DateTime? fecha, string bastidor);

    /// <summary>
    /// Borrar un vehículo existente 
    /// </summary>
    /// <param name="vehiculo">Vehículo a eliminar</param>
    void RemoveVehiculo(Vehiculo vehiculo);

    /// <summary>
    /// Borrar un vehículo existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del vehículo a eliminar</param>
    void RemoveVehiculo(Guid idGuid);

    /// <summary>
    /// Buscar vehículo por el valor de las propiedades marca y modelo
    /// </summary>
    /// <param name="marca">Especificación de marca del vehiculo</param>
    /// <param name="modelo">Especificación de modelo del vehiculo</param>
    /// <returns>Lista de vehículos si existen o null si no existen vehículos con los valores marca y modelo iguales a <paramref name="marca"/> y <paramref name="modelo"/></returns>
    List<Vehiculo> BuscarVehiculoPorMarcaYModelo(string marca, string modelo);

    /// <summary>
    /// Buscar vehículo por el valor de la propiedad matricula
    /// </summary>
    /// <param name="matricula">Especificación de matricula del vehiculo</param>
    /// <returns>Lista de vehículos si existen o null si no existen vehículos con el valor matricula igual a <paramref name="matricula"/></returns>
    List<Vehiculo> BuscarVehiculoPorMatricula(string matricula);

    /// <summary>
    /// Buscar vehículo por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del vehiculo</param>
    /// <returns>Lista de vehículos si existen o null si no existen vehículos con el valor id igual a <paramref name="id"/></returns>
    List<Vehiculo> BuscarVehiculoPorId(Guid? id);
  }
}
