﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de los vehiculos de venta
  /// </summary>
  public interface IGestionVehiculoVentaService
  {
    /// <summary>
    /// Añadir un nuevo vehículo de venta
    /// </summary>
    /// <param name="vehiculoVenta">Vehículo de venta a añadir</param>
    void AddVehiculoVenta(VehiculoVenta vehiculoVenta);

    /// <summary>
    /// Añadir un nuevo vehiculo de venta
    /// </summary>
    /// <param name="idGuid">Identificador único del vehículo de venta </param>
    /// <param name="codigo">Código del vehiculo de venta. </param>
    /// <param name="fechaDeEntrada">Fecha de entrada en el concesionario del vehiculo de venta.</param>
    /// <param name="estado">Estado (Nuevo u ocasión) del vehiculo de venta. </param>
    /// <param name="matricula">Matricula del vehiculo de venta.</param>
    /// <param name="marca">Marca del vehiculo de venta.</param>
    /// <param name="modelo">Modelo del vehiculo de venta.</param>
    /// <param name="kilometros">Kilómetros que tiene el vehiculo de venta. </param>
    /// <param name="color">Color del vehiculo de venta.</param>
    /// <param name="tipo">Tipo de vehículo del vehiculo de venta. </param>
    /// <param name="fechaMatriculacion">Fecha de matriculación del vehiculo de venta. </param>
    /// <param name="cilindrada">Cilindrada del vehiculo de venta. </param>
    /// <param name="potencia">Potencia del vehiculo de venta. </param>
    /// <param name="combustible">Tipo de combustible del vehiculo de venta. </param>
    /// <param name="bastidor">Bastidor del vehiculo de venta.</param>
    /// <param name="itv">Si el vehículo de venta ha de pasar ITV o no </param>
    /// <param name="fechaUltimaInspeccion">Fecha de la última inspección de itv </param>
    /// <param name="fechaProximaInspeccion">Fecha de la próxima inspección de itv </param>
    /// <param name="observaciones">Observaciones del vehiculo de venta. </param>
    /// <param name="precio">Precio del vehiculo de venta. </param>
    /// <param name="fechaInicioGarantia">En caso de venta, fecha de inicio de la garantía </param>
    /// <param name="mesesGarantia">En caso de venta, meses de garantía </param>
    void AddVehiculoVenta(Guid idGuid, string codigo, DateTime? fechaDeEntrada, int? estado, string matricula, string marca, string modelo, decimal? kilometros, string color, string tipo, DateTime? fechaMatriculacion, decimal? cilindrada, decimal? potencia, string combustible, string bastidor, int? itv, DateTime? fechaUltimaInspeccion, DateTime? fechaProximaInspeccion, string observaciones, decimal? precio, DateTime? fechaInicioGarantia, int? mesesGarantia);

    /// <summary>
    /// Modificar un vehículo de venta existente
    /// </summary>
    /// <param name="vehiculo">Vehiculo de venta con cambios a modificar</param>
    void ChangeVehiculoVenta(VehiculoVenta vehiculo);

    /// <summary>
    /// Modificar un vehículo de venta existente
    /// </summary>
    /// <param name="idGuid">Identificador único del vehículo de venta a modificar </param>
    /// <param name="codigo">Código del vehiculo de venta. </param>
    /// <param name="fechaDeEntrada">Fecha de entrada en el concesionario del vehiculo de venta.</param>
    /// <param name="estado">Estado (Nuevo u ocasión) del vehiculo de venta. </param>
    /// <param name="matricula">Matricula del vehiculo de venta.</param>
    /// <param name="marca">Marca del vehiculo de venta.</param>
    /// <param name="modelo">Modelo del vehiculo de venta.</param>
    /// <param name="kilometros">Kilómetros que tiene el vehiculo de venta. </param>
    /// <param name="color">Color del vehiculo de venta.</param>
    /// <param name="tipo">Tipo de vehículo del vehiculo de venta. </param>
    /// <param name="fechaMatriculacion">Fecha de matriculación del vehiculo de venta. </param>
    /// <param name="cilindrada">Cilindrada del vehiculo de venta. </param>
    /// <param name="potencia">Potencia del vehiculo de venta. </param>
    /// <param name="combustible">Tipo de combustible del vehiculo de venta. </param>
    /// <param name="bastidor">Bastidor del vehiculo de venta.</param>
    /// <param name="itv">Si el vehículo de venta ha de pasar ITV o no </param>
    /// <param name="fechaUltimaInspeccion">Fecha de la última inspección de itv </param>
    /// <param name="fechaProximaInspeccion">Fecha de la próxima inspección de itv </param>
    /// <param name="observaciones">Observaciones del vehiculo de venta. </param>
    /// <param name="precio">Precio del vehiculo de venta. </param>
    /// <param name="fechaInicioGarantia">En caso de venta, fecha de inicio de la garantía </param>
    /// <param name="mesesGarantia">En caso de venta, meses de garantía </param>
    void ChangeVehiculoVenta(Guid idGuid, string codigo, DateTime? fechaDeEntrada, int? estado, string matricula, string marca, string modelo, decimal? kilometros, string color, string tipo, DateTime? fechaMatriculacion, decimal? cilindrada, decimal? potencia, string combustible, string bastidor, int? itv, DateTime? fechaUltimaInspeccion, DateTime? fechaProximaInspeccion, string observaciones, decimal? precio, DateTime? fechaInicioGarantia, int? mesesGarantia);

    /// <summary>
    /// Borrar un vehículo de venta existente 
    /// </summary>
    /// <param name="vehiculoVenta">Vehículo de venta a eliminar</param>
    void RemoveVehiculoVenta(VehiculoVenta vehiculoVenta);

    /// <summary>
    /// Borrar un vehículo de venta existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del vehículo de venta a eliminar</param>
    void RemoveVehiculoVenta(Guid idGuid);

    /// <summary>
    /// Buscar vehículo de venta por el valor de las propiedades marca y modelo
    /// </summary>
    /// <param name="marca">Especificación de marca del vehiculo de venta</param>
    /// <param name="modelo">Especificación de modelo del vehiculo de venta</param>
    /// <returns>Lista de vehículos de venta si existen o null si no existen vehículos de venta con los valores marca y modelo iguales a <paramref name="marca"/> y <paramref name="modelo"/></returns>
    List<VehiculoVenta> BuscarVehiculoVentaPorMarcaYModelo(string marca, string modelo);

    /// <summary>
    /// Buscar vehículo de venta por el valor de la propiedad matricula
    /// </summary>
    /// <param name="matricula">Especificación de matricula del vehiculo de venta</param>
    /// <returns>Lista de vehículos de venta si existen o null si no existen vehículos de venta con el valor matricula igual a <paramref name="matricula"/></returns>
    List<VehiculoVenta> BuscarVehiculoVentaPorMatricula(string matricula);

    /// <summary>
    /// Buscar vehículo de venta por el valor de la propiedad matricula
    /// </summary>
    /// <param name="codigo">Especificación de matricula del vehiculo de venta</param>
    /// <returns>Lista de vehículos de venta si existen o null si no existen vehículos de venta con el valor codigo igual a <paramref name="codigo"/></returns>
    List<VehiculoVenta> BuscarVehiculoVentaPorCodigo(string codigo);

    /// <summary>
    /// Buscar vehículo de venta por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del vehiculo de venta</param>
    /// <returns>Lista de vehículos de venta si existen o null si no existen vehículos de venta con el valor id igual a <paramref name="id"/></returns>
    List<VehiculoVenta> BuscarVehiculoVentaPorId(Guid? id);
  }
}
