﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas
{
  /// <summary>
  /// Contrato base para las operaciones en la capa de Aplicación de las ventas
  /// </summary>
  public interface IGestionVentaService
  {
    /// <summary>
    /// Añadir una nueva venta
    /// </summary>
    /// <param name="venta">Venta a añadir</param>
    void AddVenta(Venta venta);

    /// <summary>
    /// Añadir una nueva venta
    /// </summary>    
    void AddVenta(Guid idVenta, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller, int? formaDePago, string cuentaBancaria);

    /// <summary>
    /// Modificar una venta existente
    /// </summary>
    /// <param name="venta">Venta con cambios a modificar</param>
    void ChangeVenta(Venta venta);

    /// <summary>
    /// Modificar una venta existente
    /// </summary>
    void ChangeVenta(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller, int? formaDePago, string cuentaBancaria);

    /// <summary>
    /// Borrar un venta existente 
    /// </summary>
    /// <param name="venta">Venta a eliminar</param>
    void RemoveVenta(Venta venta);

    /// <summary>
    /// Borrar una venta existente 
    /// </summary>
    /// <param name="idGuid">Identificador único del venta a eliminar</param>
    void RemoveVenta(Guid idGuid);

    /// <summary>
    /// Buscar venta por el valor de la propiedad Id
    /// </summary>
    /// <param name="id">Especificación de Id del venta</param>
    /// <returns>Lista de ventas si existen o null si no existen ventas con el valor id igual a <paramref name="id"/></returns>
    List<Venta> BuscarVentaPorId(Guid? id);

    /// <summary>
    /// Buscar venta por numero.
    /// </summary>
    /// <param name="numero">Especificación de número de venta.</param>
    /// <returns>Lista de ventas si existen o null si no existen ventas con el valor numero igual a <paramref name="numero"/></returns>
    List<Venta> BuscarVentaPorNumero(string numero);

    /// <summary>
    /// Buscar venta por cliente y fecha.
    /// </summary>
    /// <param name="cliente">Especificación de cliente del venta.</param>
    /// <param name="fecha">Especificación de fecha a partir de la cual se realiza la búsqueda.</param>
    /// <returns>Lista de ventas si existen o null si no existen ventas que coincidan con los criterios anteriores</returns>
    List<Venta> BuscarVentaPorClienteYFecha(Guid? cliente, DateTime? fecha);
  }
}