﻿/*
Script de implementación para Concesionarios.Database
*/

GO
SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;

SET NUMERIC_ROUNDABORT OFF;


GO
:setvar DatabaseName "Concesionarios.Database"
:setvar DefaultDataPath ""
:setvar DefaultLogPath ""

GO
USE [master]

GO
:on error exit
GO
IF (DB_ID(N'$(DatabaseName)') IS NOT NULL
    AND DATABASEPROPERTYEX(N'$(DatabaseName)','Status') <> N'ONLINE')
BEGIN
    RAISERROR(N'El estado de la base de datos de destino, %s, no está establecido en ONLINE. Para implementarlo en esta base de datos, debe tener el estado ONLINE.', 16, 127,N'$(DatabaseName)') WITH NOWAIT
    RETURN
END

GO
IF (DB_ID(N'$(DatabaseName)') IS NOT NULL) 
BEGIN
    ALTER DATABASE [$(DatabaseName)]
    SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
    DROP DATABASE [$(DatabaseName)];
END

GO
PRINT N'Creando $(DatabaseName)...'
GO
CREATE DATABASE [$(DatabaseName)]
    ON 
    PRIMARY(NAME = [Concesionarios], FILENAME = '$(DefaultDataPath)$(DatabaseName).mdf', FILEGROWTH = 1024 KB)
    LOG ON (NAME = [Concesionarios_log], FILENAME = '$(DefaultLogPath)$(DatabaseName)_log.ldf', MAXSIZE = 2097152 MB, FILEGROWTH = 10 %) COLLATE Traditional_Spanish_CI_AS
GO
EXECUTE sp_dbcmptlevel [$(DatabaseName)], 100;


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET ANSI_NULLS ON,
                ANSI_PADDING ON,
                ANSI_WARNINGS ON,
                ARITHABORT ON,
                CONCAT_NULL_YIELDS_NULL ON,
                NUMERIC_ROUNDABORT OFF,
                QUOTED_IDENTIFIER ON,
                ANSI_NULL_DEFAULT ON,
                CURSOR_DEFAULT LOCAL,
                RECOVERY FULL,
                CURSOR_CLOSE_ON_COMMIT OFF,
                AUTO_CREATE_STATISTICS ON,
                AUTO_SHRINK OFF,
                AUTO_UPDATE_STATISTICS ON,
                RECURSIVE_TRIGGERS OFF 
            WITH ROLLBACK IMMEDIATE;
        ALTER DATABASE [$(DatabaseName)]
            SET AUTO_CLOSE OFF 
            WITH ROLLBACK IMMEDIATE;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET ALLOW_SNAPSHOT_ISOLATION OFF;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET READ_COMMITTED_SNAPSHOT OFF;
    END


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        ALTER DATABASE [$(DatabaseName)]
            SET AUTO_UPDATE_STATISTICS_ASYNC OFF,
                PAGE_VERIFY NONE,
                DATE_CORRELATION_OPTIMIZATION OFF,
                DISABLE_BROKER,
                PARAMETERIZATION SIMPLE,
                SUPPLEMENTAL_LOGGING OFF 
            WITH ROLLBACK IMMEDIATE;
    END


GO
IF IS_SRVROLEMEMBER(N'sysadmin') = 1
    BEGIN
        IF EXISTS (SELECT 1
                   FROM   [master].[dbo].[sysdatabases]
                   WHERE  [name] = N'$(DatabaseName)')
            BEGIN
                EXECUTE sp_executesql N'ALTER DATABASE [$(DatabaseName)]
    SET TRUSTWORTHY OFF,
        DB_CHAINING OFF 
    WITH ROLLBACK IMMEDIATE';
            END
    END
ELSE
    BEGIN
        PRINT N'No se puede modificar la configuración de la base de datos. Debe ser un administrador del sistema para poder aplicar esta configuración.';
    END


GO
IF IS_SRVROLEMEMBER(N'sysadmin') = 1
    BEGIN
        IF EXISTS (SELECT 1
                   FROM   [master].[dbo].[sysdatabases]
                   WHERE  [name] = N'$(DatabaseName)')
            BEGIN
                EXECUTE sp_executesql N'ALTER DATABASE [$(DatabaseName)]
    SET HONOR_BROKER_PRIORITY OFF 
    WITH ROLLBACK IMMEDIATE';
            END
    END
ELSE
    BEGIN
        PRINT N'No se puede modificar la configuración de la base de datos. Debe ser un administrador del sistema para poder aplicar esta configuración.';
    END


GO
USE [$(DatabaseName)]

GO
IF fulltextserviceproperty(N'IsFulltextInstalled') = 1
    EXECUTE sp_fulltext_database 'enable';


GO
/*
 Plantilla de script anterior a la implementación							
--------------------------------------------------------------------------------------
 Este archivo contiene instrucciones de SQL que se ejecutarán antes del script de compilación	
 Use sintaxis de SQLCMD para incluir un archivo en el script anterior a la implementación			
 Ejemplo:      :r .\miarchivo.sql								
 Use sintaxis de SQLCMD para hacer referencia a una variable del script anterior a la implementación		
 Ejemplo:      :setvar TableName Mi tabla							
               SELECT * FROM [$(TableName)]					
--------------------------------------------------------------------------------------
*/

GO
PRINT N'Creando [dbo].[Articulos]...';


GO
CREATE TABLE [dbo].[Articulos] (
    [Id]            UNIQUEIDENTIFIER NOT NULL,
    [Nombre]        NVARCHAR (50)    NULL,
    [Descripcion]   NVARCHAR (50)    NULL,
    [IdCategoria]   NVARCHAR (50)    NULL,
    [CosteActual]   MONEY            NULL,
    [CosteAnterior] MONEY            NULL,
    [IdProveedor]   UNIQUEIDENTIFIER NULL
);


GO
PRINT N'Creando PK_ARTICULOS...';


GO
ALTER TABLE [dbo].[Articulos]
    ADD CONSTRAINT [PK_ARTICULOS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[ArticulosAlmacen]...';


GO
CREATE TABLE [dbo].[ArticulosAlmacen] (
    [Id]           UNIQUEIDENTIFIER NOT NULL,
    [IdEmpleado]   UNIQUEIDENTIFIER NULL,
    [Fecha]        DATETIME         NULL,
    [StockMinimo]  NUMERIC (10, 2)  NULL,
    [StockMaximo]  NUMERIC (10, 2)  NULL,
    [StockActual]  NUMERIC (10, 2)  NULL,
    [Ubicacion]    NVARCHAR (50)    NULL,
    [IdArticulo]   UNIQUEIDENTIFIER NULL,
    [CodigoTaller] NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_ARTICULOSALMACEN...';


GO
ALTER TABLE [dbo].[ArticulosAlmacen]
    ADD CONSTRAINT [PK_ARTICULOSALMACEN] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[CategoriasArticulo]...';


GO
CREATE TABLE [dbo].[CategoriasArticulo] (
    [Id]          NVARCHAR (50)  NOT NULL,
    [Descripcion] NVARCHAR (100) NOT NULL
);


GO
PRINT N'Creando PK_CategoriasArticulos...';


GO
ALTER TABLE [dbo].[CategoriasArticulo]
    ADD CONSTRAINT [PK_CategoriasArticulos] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Clientes]...';


GO
CREATE TABLE [dbo].[Clientes] (
    [Id]                UNIQUEIDENTIFIER NOT NULL,
    [Nombre]            NVARCHAR (50)    NULL,
    [PrimerApellido]    NVARCHAR (50)    NULL,
    [SegundoApellido]   NVARCHAR (50)    NULL,
    [NifCif]            NVARCHAR (15)    NULL,
    [Observaciones]     NVARCHAR (300)   NULL,
    [Telefonos]         NVARCHAR (50)    NULL,
    [CorreoElectronico] NVARCHAR (50)    NULL,
    [IdCodigoPostal]    INT              NULL,
    [Direccion]         NVARCHAR (50)    NULL,
    [FormaDePago]       INT              NULL,
    [CuentaBancaria]    NVARCHAR (50)    NULL,
    [NombreComercial]   NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_CLIENTES...';


GO
ALTER TABLE [dbo].[Clientes]
    ADD CONSTRAINT [PK_CLIENTES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[CodigosPostales]...';


GO
CREATE TABLE [dbo].[CodigosPostales] (
    [Id]          INT           NOT NULL,
    [Codigo]      NVARCHAR (10) NULL,
    [IdPoblacion] INT           NULL
);


GO
PRINT N'Creando PK_CODIGOSPOSTALES...';


GO
ALTER TABLE [dbo].[CodigosPostales]
    ADD CONSTRAINT [PK_CODIGOSPOSTALES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[ContenidoReparaciones]...';


GO
CREATE TABLE [dbo].[ContenidoReparaciones] (
    [Id]           UNIQUEIDENTIFIER NOT NULL,
    [IdReparacion] UNIQUEIDENTIFIER NULL,
    [IdArticulo]   UNIQUEIDENTIFIER NULL,
    [IdManoDeObra] UNIQUEIDENTIFIER NULL,
    [IdPaquete]    UNIQUEIDENTIFIER NULL,
    [Cantidad]     NUMERIC (10, 2)  NULL
);


GO
PRINT N'Creando PK_CONTENIDOREPARACIONES...';


GO
ALTER TABLE [dbo].[ContenidoReparaciones]
    ADD CONSTRAINT [PK_CONTENIDOREPARACIONES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Empleados]...';


GO
CREATE TABLE [dbo].[Empleados] (
    [Id]           UNIQUEIDENTIFIER NOT NULL,
    [Codigo]       NVARCHAR (20)    NULL,
    [Nombre]       NVARCHAR (50)    NULL,
    [Tipo]         INT              NULL,
    [CodigoTaller] NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_EMPLEADOS...';


GO
ALTER TABLE [dbo].[Empleados]
    ADD CONSTRAINT [PK_EMPLEADOS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_EMPLEADOS_CODIGO...';


GO
ALTER TABLE [dbo].[Empleados]
    ADD CONSTRAINT [UQ_EMPLEADOS_CODIGO] UNIQUE NONCLUSTERED ([Codigo] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando [dbo].[EstadosOrdenReparacion]...';


GO
CREATE TABLE [dbo].[EstadosOrdenReparacion] (
    [Id]          INT           NOT NULL,
    [Descripcion] NVARCHAR (30) NOT NULL
);


GO
PRINT N'Creando PK_EstadosOrdenReparacion...';


GO
ALTER TABLE [dbo].[EstadosOrdenReparacion]
    ADD CONSTRAINT [PK_EstadosOrdenReparacion] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[EstadosReparacion]...';


GO
CREATE TABLE [dbo].[EstadosReparacion] (
    [Id]          INT           NOT NULL,
    [Descripcion] NVARCHAR (30) NOT NULL
);


GO
PRINT N'Creando PK_EstadosReparacion...';


GO
ALTER TABLE [dbo].[EstadosReparacion]
    ADD CONSTRAINT [PK_EstadosReparacion] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[FormasDePago]...';


GO
CREATE TABLE [dbo].[FormasDePago] (
    [Id]          INT            NOT NULL,
    [Descripcion] NVARCHAR (100) NOT NULL
);


GO
PRINT N'Creando PK_FormasDePago...';


GO
ALTER TABLE [dbo].[FormasDePago]
    ADD CONSTRAINT [PK_FormasDePago] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[LookUp]...';


GO
CREATE TABLE [dbo].[LookUp] (
    [Id]          INT            NOT NULL,
    [Codigo]      NVARCHAR (50)  NOT NULL,
    [Descripcion] NVARCHAR (100) NULL
);


GO
PRINT N'Creando PK_LOOKUP...';


GO
ALTER TABLE [dbo].[LookUp]
    ADD CONSTRAINT [PK_LOOKUP] PRIMARY KEY CLUSTERED ([Id] ASC, [Codigo] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[ManosDeObra]...';


GO
CREATE TABLE [dbo].[ManosDeObra] (
    [Id]            UNIQUEIDENTIFIER NOT NULL,
    [Descripcion]   NVARCHAR (50)    NULL,
    [Duracion]      NUMERIC (10, 2)  NULL,
    [PrecioPorHora] MONEY            NULL
);


GO
PRINT N'Creando PK_MANOSDEOBRA...';


GO
ALTER TABLE [dbo].[ManosDeObra]
    ADD CONSTRAINT [PK_MANOSDEOBRA] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Marcas]...';


GO
CREATE TABLE [dbo].[Marcas] (
    [Id]          NVARCHAR (50) NOT NULL,
    [Descripcion] NVARCHAR (50) NOT NULL
);


GO
PRINT N'Creando PK_Marcas...';


GO
ALTER TABLE [dbo].[Marcas]
    ADD CONSTRAINT [PK_Marcas] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Modelos]...';


GO
CREATE TABLE [dbo].[Modelos] (
    [Id]          NVARCHAR (50) NOT NULL,
    [Descripcion] NVARCHAR (50) NOT NULL,
    [IdMarca]     NVARCHAR (50) NOT NULL
);


GO
PRINT N'Creando PK_Modelos...';


GO
ALTER TABLE [dbo].[Modelos]
    ADD CONSTRAINT [PK_Modelos] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[OrdenesReparacion]...';


GO
CREATE TABLE [dbo].[OrdenesReparacion] (
    [Id]                 UNIQUEIDENTIFIER NOT NULL,
    [FechaHoraEntrega]   DATETIME         NULL,
    [Estado]             INT              NULL,
    [IdPersonalAsignado] UNIQUEIDENTIFIER NULL,
    [SubTotal]           MONEY            NULL,
    [ImporteIva]         MONEY            NULL,
    [Diagnostico]        NVARCHAR (500)   NULL,
    [IdRecepcion]        UNIQUEIDENTIFIER NULL,
    [IdVehiculo]         UNIQUEIDENTIFIER NULL,
    [CodigoTaller]       NVARCHAR (50)    NULL,
    [NumeroOrden]        NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_ORDENESREPARACION...';


GO
ALTER TABLE [dbo].[OrdenesReparacion]
    ADD CONSTRAINT [PK_ORDENESREPARACION] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_ORDENESREPARACION_IDRECEPCION...';


GO
ALTER TABLE [dbo].[OrdenesReparacion]
    ADD CONSTRAINT [UQ_ORDENESREPARACION_IDRECEPCION] UNIQUE NONCLUSTERED ([IdRecepcion] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando [dbo].[Paquetes]...';


GO
CREATE TABLE [dbo].[Paquetes] (
    [IdPaqueteDeServicio] UNIQUEIDENTIFIER NOT NULL,
    [IdArticulo]          UNIQUEIDENTIFIER NOT NULL,
    [IdManoDeObra]        UNIQUEIDENTIFIER NOT NULL,
    [Cantidad]            NUMERIC (10, 2)  NULL
);


GO
PRINT N'Creando PK_PAQUETES...';


GO
ALTER TABLE [dbo].[Paquetes]
    ADD CONSTRAINT [PK_PAQUETES] PRIMARY KEY CLUSTERED ([IdPaqueteDeServicio] ASC, [IdArticulo] ASC, [IdManoDeObra] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[PaquetesDeServicio]...';


GO
CREATE TABLE [dbo].[PaquetesDeServicio] (
    [Id]          UNIQUEIDENTIFIER NOT NULL,
    [Descripcion] NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_PAQUETESDESERVICIO...';


GO
ALTER TABLE [dbo].[PaquetesDeServicio]
    ADD CONSTRAINT [PK_PAQUETESDESERVICIO] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Poblaciones]...';


GO
CREATE TABLE [dbo].[Poblaciones] (
    [Id]          INT            NOT NULL,
    [Nombre]      NVARCHAR (100) NULL,
    [IdProvincia] NVARCHAR (10)  NULL
);


GO
PRINT N'Creando PK_POBLACIONES...';


GO
ALTER TABLE [dbo].[Poblaciones]
    ADD CONSTRAINT [PK_POBLACIONES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Presupuestos]...';


GO
CREATE TABLE [dbo].[Presupuestos] (
    [Id]            UNIQUEIDENTIFIER NOT NULL,
    [Numero]        NVARCHAR (15)    NULL,
    [Fecha]         DATE             NULL,
    [BaseImponible] MONEY            NULL,
    [PorcentajeIva] NUMERIC (10, 2)  NULL,
    [IdEmpleado]    UNIQUEIDENTIFIER NULL,
    [IdCliente]     UNIQUEIDENTIFIER NULL,
    [CodigoTaller]  NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_PRESUPUESTOS...';


GO
ALTER TABLE [dbo].[Presupuestos]
    ADD CONSTRAINT [PK_PRESUPUESTOS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Proveedores]...';


GO
CREATE TABLE [dbo].[Proveedores] (
    [Id]                UNIQUEIDENTIFIER NOT NULL,
    [Codigo]            NVARCHAR (50)    NOT NULL,
    [RazonSocial]       NVARCHAR (50)    NULL,
    [Direccion]         NVARCHAR (50)    NULL,
    [CifDni]            NVARCHAR (15)    NULL,
    [Telefonos]         NVARCHAR (50)    NULL,
    [Observaciones]     NVARCHAR (500)   NULL,
    [CorreoElectronico] NVARCHAR (50)    NULL,
    [PaginaWeb]         NVARCHAR (50)    NULL,
    [IdCodigoPostal]    INT              NULL
);


GO
PRINT N'Creando PK_PROVEEDOR...';


GO
ALTER TABLE [dbo].[Proveedores]
    ADD CONSTRAINT [PK_PROVEEDOR] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_PROVEEDOR_CODIGO...';


GO
ALTER TABLE [dbo].[Proveedores]
    ADD CONSTRAINT [UQ_PROVEEDOR_CODIGO] UNIQUE NONCLUSTERED ([Codigo] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando [dbo].[Provincias]...';


GO
CREATE TABLE [dbo].[Provincias] (
    [Id]     NVARCHAR (10) NOT NULL,
    [Nombre] NVARCHAR (50) NULL
);


GO
PRINT N'Creando PK_PROVINCIAS...';


GO
ALTER TABLE [dbo].[Provincias]
    ADD CONSTRAINT [PK_PROVINCIAS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Recepciones]...';


GO
CREATE TABLE [dbo].[Recepciones] (
    [Id]                          UNIQUEIDENTIFIER NOT NULL,
    [FechaHoraRecepcion]          DATETIME         NULL,
    [ObservacionesSobreElCliente] NVARCHAR (500)   NULL,
    [NombreContactoAdicional]     NVARCHAR (50)    NULL,
    [TelefonoContactoAdicional]   NVARCHAR (50)    NULL,
    [SubTotal]                    MONEY            NULL,
    [ImporteIva]                  MONEY            NULL,
    [Kilometraje]                 NUMERIC (10, 2)  NULL,
    [Prediagnostico]              NVARCHAR (500)   NULL,
    [ContenidoAdicional]          NVARCHAR (300)   NULL,
    [Combustible]                 INT              NULL,
    [FechaHoraEntrega]            DATETIME         NULL,
    [Observaciones]               NVARCHAR (500)   NULL,
    [IdCliente]                   UNIQUEIDENTIFIER NULL,
    [IdVehiculo]                  UNIQUEIDENTIFIER NULL,
    [IdEmpleado]                  UNIQUEIDENTIFIER NULL,
    [CodigoTaller]                NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_RECEPCIONES...';


GO
ALTER TABLE [dbo].[Recepciones]
    ADD CONSTRAINT [PK_RECEPCIONES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Reparaciones]...';


GO
CREATE TABLE [dbo].[Reparaciones] (
    [Id]                UNIQUEIDENTIFIER NOT NULL,
    [DuracionSugerida]  NUMERIC (10, 2)  NULL,
    [DuracionReal]      NUMERIC (10, 2)  NULL,
    [Estado]            INT              NULL,
    [IdEmpleado]        UNIQUEIDENTIFIER NULL,
    [IdOrdenReparacion] UNIQUEIDENTIFIER NULL
);


GO
PRINT N'Creando PK_REPARACIONES...';


GO
ALTER TABLE [dbo].[Reparaciones]
    ADD CONSTRAINT [PK_REPARACIONES] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Talleres]...';


GO
CREATE TABLE [dbo].[Talleres] (
    [Id]          NVARCHAR (50) NOT NULL,
    [Descripcion] NVARCHAR (50) NOT NULL
);


GO
PRINT N'Creando PK_Talleres...';


GO
ALTER TABLE [dbo].[Talleres]
    ADD CONSTRAINT [PK_Talleres] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[TiposCombustible]...';


GO
CREATE TABLE [dbo].[TiposCombustible] (
    [Id]          NVARCHAR (20) NOT NULL,
    [Descripcion] NVARCHAR (50) NOT NULL
);


GO
PRINT N'Creando PK_TiposCombustible...';


GO
ALTER TABLE [dbo].[TiposCombustible]
    ADD CONSTRAINT [PK_TiposCombustible] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[TiposEmpleado]...';


GO
CREATE TABLE [dbo].[TiposEmpleado] (
    [Id]          INT            NOT NULL,
    [Descripcion] NVARCHAR (100) NOT NULL
);


GO
PRINT N'Creando PK_TiposEmpleado...';


GO
ALTER TABLE [dbo].[TiposEmpleado]
    ADD CONSTRAINT [PK_TiposEmpleado] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[TiposVehiculo]...';


GO
CREATE TABLE [dbo].[TiposVehiculo] (
    [Id]          NCHAR (50)    NOT NULL,
    [Descripcion] NVARCHAR (50) NOT NULL
);


GO
PRINT N'Creando PK_TiposVehiculo...';


GO
ALTER TABLE [dbo].[TiposVehiculo]
    ADD CONSTRAINT [PK_TiposVehiculo] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando [dbo].[Vehiculos]...';


GO
CREATE TABLE [dbo].[Vehiculos] (
    [Id]        UNIQUEIDENTIFIER NOT NULL,
    [Matricula] NVARCHAR (10)    NULL,
    [Marca]     NVARCHAR (50)    NULL,
    [Modelo]    NVARCHAR (50)    NULL,
    [Color]     NVARCHAR (50)    NULL,
    [Fecha]     DATE             NULL,
    [Bastidor]  NVARCHAR (20)    NULL,
    [IdCliente] UNIQUEIDENTIFIER NULL
);


GO
PRINT N'Creando PK_VEHICULOS...';


GO
ALTER TABLE [dbo].[Vehiculos]
    ADD CONSTRAINT [PK_VEHICULOS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_VEHICULOS_MATRICULA...';


GO
ALTER TABLE [dbo].[Vehiculos]
    ADD CONSTRAINT [UQ_VEHICULOS_MATRICULA] UNIQUE NONCLUSTERED ([Matricula] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando [dbo].[VehiculosVenta]...';


GO
CREATE TABLE [dbo].[VehiculosVenta] (
    [Id]                  UNIQUEIDENTIFIER NOT NULL,
    [Codigo]              NVARCHAR (15)    NULL,
    [FechaDeEntrada]      DATE             NULL,
    [Estado]              INT              NULL,
    [Marca]               NVARCHAR (50)    NULL,
    [Modelo]              NVARCHAR (50)    NULL,
    [Matricula]           NVARCHAR (10)    NULL,
    [Kilometros]          NUMERIC (10, 2)  NULL,
    [Color]               NVARCHAR (50)    NULL,
    [Tipo]                NVARCHAR (50)    NULL,
    [FechaMatriculacion]  DATE             NULL,
    [Cilindrada]          NUMERIC (10, 2)  NULL,
    [Potencia]            NUMERIC (10, 2)  NULL,
    [Combustible]         NVARCHAR (20)    NULL,
    [Bastidor]            NVARCHAR (20)    NULL,
    [Itv]                 INT              NULL,
    [UltimaInspeccion]    DATE             NULL,
    [ProximaInspeccion]   DATE             NULL,
    [Observaciones]       NVARCHAR (300)   NULL,
    [Precio]              MONEY            NULL,
    [FechaInicioGarantia] DATE             NULL,
    [MesesGarantia]       INT              NULL,
    [IdPresupuesto]       UNIQUEIDENTIFIER NULL,
    [IdVenta]             UNIQUEIDENTIFIER NULL,
    [CodigoTaller]        NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_VEHICULOSVENTA...';


GO
ALTER TABLE [dbo].[VehiculosVenta]
    ADD CONSTRAINT [PK_VEHICULOSVENTA] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_VEHICULOSVENTA_CODIGO...';


GO
ALTER TABLE [dbo].[VehiculosVenta]
    ADD CONSTRAINT [UQ_VEHICULOSVENTA_CODIGO] UNIQUE NONCLUSTERED ([Codigo] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando [dbo].[Ventas]...';


GO
CREATE TABLE [dbo].[Ventas] (
    [Id]             UNIQUEIDENTIFIER NOT NULL,
    [Numero]         NVARCHAR (20)    NULL,
    [Fecha]          DATE             NULL,
    [BaseImponible]  MONEY            NULL,
    [PorcentajeIva]  FLOAT            NULL,
    [FormaDePago]    INT              NULL,
    [CuentaBancaria] NVARCHAR (50)    NULL,
    [IdCliente]      UNIQUEIDENTIFIER NULL,
    [IdEmpleado]     UNIQUEIDENTIFIER NULL,
    [CodigoTaller]   NVARCHAR (50)    NULL
);


GO
PRINT N'Creando PK_VENTAS...';


GO
ALTER TABLE [dbo].[Ventas]
    ADD CONSTRAINT [PK_VENTAS] PRIMARY KEY CLUSTERED ([Id] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF);


GO
PRINT N'Creando UQ_VENTAS_NUMERO...';


GO
ALTER TABLE [dbo].[Ventas]
    ADD CONSTRAINT [UQ_VENTAS_NUMERO] UNIQUE NONCLUSTERED ([Numero] ASC) WITH (ALLOW_PAGE_LOCKS = ON, ALLOW_ROW_LOCKS = ON, PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF, STATISTICS_NORECOMPUTE = OFF) ON [PRIMARY];


GO
PRINT N'Creando FK_ARTICULOS_PROVEEDORES...';


GO
ALTER TABLE [dbo].[Articulos] WITH NOCHECK
    ADD CONSTRAINT [FK_ARTICULOS_PROVEEDORES] FOREIGN KEY ([IdProveedor]) REFERENCES [dbo].[Proveedores] ([Id]) ON DELETE SET NULL ON UPDATE CASCADE;


GO
PRINT N'Creando FK_ARTICULOSALMACEN_ARTICULOS...';


GO
ALTER TABLE [dbo].[ArticulosAlmacen] WITH NOCHECK
    ADD CONSTRAINT [FK_ARTICULOSALMACEN_ARTICULOS] FOREIGN KEY ([IdArticulo]) REFERENCES [dbo].[Articulos] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_ARTICULOSALMACEN_EMPLEADOS...';


GO
ALTER TABLE [dbo].[ArticulosAlmacen] WITH NOCHECK
    ADD CONSTRAINT [FK_ARTICULOSALMACEN_EMPLEADOS] FOREIGN KEY ([IdEmpleado]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE SET NULL ON UPDATE CASCADE;


GO
PRINT N'Creando FK_CLIENTES_CODIGOSPOSTALES...';


GO
ALTER TABLE [dbo].[Clientes] WITH NOCHECK
    ADD CONSTRAINT [FK_CLIENTES_CODIGOSPOSTALES] FOREIGN KEY ([IdCodigoPostal]) REFERENCES [dbo].[CodigosPostales] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_CODIGOSPOSTALES_POBLACIONES...';


GO
ALTER TABLE [dbo].[CodigosPostales] WITH NOCHECK
    ADD CONSTRAINT [FK_CODIGOSPOSTALES_POBLACIONES] FOREIGN KEY ([IdPoblacion]) REFERENCES [dbo].[Poblaciones] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Creando FK_ContenidoReparaciones_Reparaciones...';


GO
ALTER TABLE [dbo].[ContenidoReparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_ContenidoReparaciones_Reparaciones] FOREIGN KEY ([IdReparacion]) REFERENCES [dbo].[Reparaciones] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[ContenidoReparaciones] NOCHECK CONSTRAINT [FK_ContenidoReparaciones_Reparaciones];


GO
PRINT N'Creando FK_REPARACIONES_ARTICULOS...';


GO
ALTER TABLE [dbo].[ContenidoReparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_REPARACIONES_ARTICULOS] FOREIGN KEY ([IdArticulo]) REFERENCES [dbo].[Articulos] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[ContenidoReparaciones] NOCHECK CONSTRAINT [FK_REPARACIONES_ARTICULOS];


GO
PRINT N'Creando FK_REPARACIONES_MANOSDEOBRA...';


GO
ALTER TABLE [dbo].[ContenidoReparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_REPARACIONES_MANOSDEOBRA] FOREIGN KEY ([IdManoDeObra]) REFERENCES [dbo].[ManosDeObra] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[ContenidoReparaciones] NOCHECK CONSTRAINT [FK_REPARACIONES_MANOSDEOBRA];


GO
PRINT N'Creando FK_REPARACIONES_PAQUETES...';


GO
ALTER TABLE [dbo].[ContenidoReparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_REPARACIONES_PAQUETES] FOREIGN KEY ([IdPaquete]) REFERENCES [dbo].[PaquetesDeServicio] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[ContenidoReparaciones] NOCHECK CONSTRAINT [FK_REPARACIONES_PAQUETES];


GO
PRINT N'Creando FK_Marcas_Modelos...';


GO
ALTER TABLE [dbo].[Modelos] WITH NOCHECK
    ADD CONSTRAINT [FK_Marcas_Modelos] FOREIGN KEY ([IdMarca]) REFERENCES [dbo].[Marcas] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
ALTER TABLE [dbo].[Modelos] NOCHECK CONSTRAINT [FK_Marcas_Modelos];


GO
PRINT N'Creando FK_ORDENESREPARACION_EMPLEADOS...';


GO
ALTER TABLE [dbo].[OrdenesReparacion] WITH NOCHECK
    ADD CONSTRAINT [FK_ORDENESREPARACION_EMPLEADOS] FOREIGN KEY ([IdPersonalAsignado]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE SET NULL ON UPDATE CASCADE;


GO
PRINT N'Creando FK_OrdenesReparacion_Recepciones...';


GO
ALTER TABLE [dbo].[OrdenesReparacion] WITH NOCHECK
    ADD CONSTRAINT [FK_OrdenesReparacion_Recepciones] FOREIGN KEY ([IdRecepcion]) REFERENCES [dbo].[Recepciones] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_ORDENESREPARACION_VEHICULOS...';


GO
ALTER TABLE [dbo].[OrdenesReparacion] WITH NOCHECK
    ADD CONSTRAINT [FK_ORDENESREPARACION_VEHICULOS] FOREIGN KEY ([IdVehiculo]) REFERENCES [dbo].[Vehiculos] ([Id]) ON DELETE SET NULL ON UPDATE CASCADE;


GO
PRINT N'Creando FK_Paquetes_Articulos...';


GO
ALTER TABLE [dbo].[Paquetes] WITH NOCHECK
    ADD CONSTRAINT [FK_Paquetes_Articulos] FOREIGN KEY ([IdArticulo]) REFERENCES [dbo].[Articulos] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[Paquetes] NOCHECK CONSTRAINT [FK_Paquetes_Articulos];


GO
PRINT N'Creando FK_Paquetes_ManosDeObra...';


GO
ALTER TABLE [dbo].[Paquetes] WITH NOCHECK
    ADD CONSTRAINT [FK_Paquetes_ManosDeObra] FOREIGN KEY ([IdManoDeObra]) REFERENCES [dbo].[ManosDeObra] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[Paquetes] NOCHECK CONSTRAINT [FK_Paquetes_ManosDeObra];


GO
PRINT N'Creando FK_PAQUETES_PAQUETESDESERVICIO...';


GO
ALTER TABLE [dbo].[Paquetes] WITH NOCHECK
    ADD CONSTRAINT [FK_PAQUETES_PAQUETESDESERVICIO] FOREIGN KEY ([IdPaqueteDeServicio]) REFERENCES [dbo].[PaquetesDeServicio] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Creando FK_POBLACIONES_PROVINCIAS...';


GO
ALTER TABLE [dbo].[Poblaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_POBLACIONES_PROVINCIAS] FOREIGN KEY ([IdProvincia]) REFERENCES [dbo].[Provincias] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Creando FK_Presupuestos_Clientes...';


GO
ALTER TABLE [dbo].[Presupuestos] WITH NOCHECK
    ADD CONSTRAINT [FK_Presupuestos_Clientes] FOREIGN KEY ([IdCliente]) REFERENCES [dbo].[Clientes] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_Presupuestos_Empleados...';


GO
ALTER TABLE [dbo].[Presupuestos] WITH NOCHECK
    ADD CONSTRAINT [FK_Presupuestos_Empleados] FOREIGN KEY ([IdEmpleado]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_PROVEEDORES_CodigoPostal...';


GO
ALTER TABLE [dbo].[Proveedores] WITH NOCHECK
    ADD CONSTRAINT [FK_PROVEEDORES_CodigoPostal] FOREIGN KEY ([IdCodigoPostal]) REFERENCES [dbo].[CodigosPostales] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_RECEPCIONES_CLIENTES...';


GO
ALTER TABLE [dbo].[Recepciones] WITH NOCHECK
    ADD CONSTRAINT [FK_RECEPCIONES_CLIENTES] FOREIGN KEY ([IdCliente]) REFERENCES [dbo].[Clientes] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_RECEPCIONES_EMPLEADOS...';


GO
ALTER TABLE [dbo].[Recepciones] WITH NOCHECK
    ADD CONSTRAINT [FK_RECEPCIONES_EMPLEADOS] FOREIGN KEY ([IdEmpleado]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_RECEPCIONES_VEHICULOS...';


GO
ALTER TABLE [dbo].[Recepciones] WITH NOCHECK
    ADD CONSTRAINT [FK_RECEPCIONES_VEHICULOS] FOREIGN KEY ([IdVehiculo]) REFERENCES [dbo].[Vehiculos] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_Reparaciones_Empleados...';


GO
ALTER TABLE [dbo].[Reparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_Reparaciones_Empleados] FOREIGN KEY ([Id]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[Reparaciones] NOCHECK CONSTRAINT [FK_Reparaciones_Empleados];


GO
PRINT N'Creando FK_Reparaciones_OrdenesReparacion...';


GO
ALTER TABLE [dbo].[Reparaciones] WITH NOCHECK
    ADD CONSTRAINT [FK_Reparaciones_OrdenesReparacion] FOREIGN KEY ([IdOrdenReparacion]) REFERENCES [dbo].[OrdenesReparacion] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[Reparaciones] NOCHECK CONSTRAINT [FK_Reparaciones_OrdenesReparacion];


GO
PRINT N'Creando FK_VEHICULOS_CLIENTES...';


GO
ALTER TABLE [dbo].[Vehiculos] WITH NOCHECK
    ADD CONSTRAINT [FK_VEHICULOS_CLIENTES] FOREIGN KEY ([IdCliente]) REFERENCES [dbo].[Clientes] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION NOT FOR REPLICATION;


GO
ALTER TABLE [dbo].[Vehiculos] NOCHECK CONSTRAINT [FK_VEHICULOS_CLIENTES];


GO
PRINT N'Creando FK_VEHICULOSVENTA_PRESUPUESTOS...';


GO
ALTER TABLE [dbo].[VehiculosVenta] WITH NOCHECK
    ADD CONSTRAINT [FK_VEHICULOSVENTA_PRESUPUESTOS] FOREIGN KEY ([IdPresupuesto]) REFERENCES [dbo].[Presupuestos] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Creando FK_VEHICULOSVENTA_VENTAS...';


GO
ALTER TABLE [dbo].[VehiculosVenta] WITH NOCHECK
    ADD CONSTRAINT [FK_VEHICULOSVENTA_VENTAS] FOREIGN KEY ([IdVenta]) REFERENCES [dbo].[Ventas] ([Id]) ON DELETE CASCADE ON UPDATE CASCADE;


GO
PRINT N'Creando FK_Ventas_Clientes...';


GO
ALTER TABLE [dbo].[Ventas] WITH NOCHECK
    ADD CONSTRAINT [FK_Ventas_Clientes] FOREIGN KEY ([IdCliente]) REFERENCES [dbo].[Clientes] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
PRINT N'Creando FK_Ventas_Empleados...';


GO
ALTER TABLE [dbo].[Ventas] WITH NOCHECK
    ADD CONSTRAINT [FK_Ventas_Empleados] FOREIGN KEY ([IdEmpleado]) REFERENCES [dbo].[Empleados] ([Id]) ON DELETE NO ACTION ON UPDATE NO ACTION;


GO
-- Paso de refactorización para actualizar el servidor de destino con los registros de transacciones implementadas
CREATE TABLE  [dbo].[__RefactorLog] (OperationKey UNIQUEIDENTIFIER NOT NULL PRIMARY KEY)
GO
sp_addextendedproperty N'microsoft_database_tools_support', N'refactoring log', N'schema', N'dbo', N'table', N'__RefactorLog'
GO

GO
/*
Plantilla de script posterior a la implementación							
--------------------------------------------------------------------------------------
 Este archivo contiene instrucciones de SQL que se anexarán al script de compilación		
 Use sintaxis de SQLCMD para incluir un archivo en el script posterior a la implementación			
 Ejemplo:      :r .\miarchivo.sql								
 Use sintaxis de SQLCMD para hacer referencia a una variable del script posterior a la implementación		
 Ejemplo:      :setvar TableName Mi tabla							
               SELECT * FROM [$(TableName)]					
--------------------------------------------------------------------------------------
*/

GO
PRINT N'Comprobando los datos existentes con las restricciones recién creadas';


GO
USE [$(DatabaseName)];


GO
ALTER TABLE [dbo].[Articulos] WITH CHECK CHECK CONSTRAINT [FK_ARTICULOS_PROVEEDORES];

ALTER TABLE [dbo].[ArticulosAlmacen] WITH CHECK CHECK CONSTRAINT [FK_ARTICULOSALMACEN_ARTICULOS];

ALTER TABLE [dbo].[ArticulosAlmacen] WITH CHECK CHECK CONSTRAINT [FK_ARTICULOSALMACEN_EMPLEADOS];

ALTER TABLE [dbo].[Clientes] WITH CHECK CHECK CONSTRAINT [FK_CLIENTES_CODIGOSPOSTALES];

ALTER TABLE [dbo].[CodigosPostales] WITH CHECK CHECK CONSTRAINT [FK_CODIGOSPOSTALES_POBLACIONES];

ALTER TABLE [dbo].[OrdenesReparacion] WITH CHECK CHECK CONSTRAINT [FK_ORDENESREPARACION_EMPLEADOS];

ALTER TABLE [dbo].[OrdenesReparacion] WITH CHECK CHECK CONSTRAINT [FK_OrdenesReparacion_Recepciones];

ALTER TABLE [dbo].[OrdenesReparacion] WITH CHECK CHECK CONSTRAINT [FK_ORDENESREPARACION_VEHICULOS];

ALTER TABLE [dbo].[Paquetes] WITH CHECK CHECK CONSTRAINT [FK_PAQUETES_PAQUETESDESERVICIO];

ALTER TABLE [dbo].[Poblaciones] WITH CHECK CHECK CONSTRAINT [FK_POBLACIONES_PROVINCIAS];

ALTER TABLE [dbo].[Presupuestos] WITH CHECK CHECK CONSTRAINT [FK_Presupuestos_Clientes];

ALTER TABLE [dbo].[Presupuestos] WITH CHECK CHECK CONSTRAINT [FK_Presupuestos_Empleados];

ALTER TABLE [dbo].[Proveedores] WITH CHECK CHECK CONSTRAINT [FK_PROVEEDORES_CodigoPostal];

ALTER TABLE [dbo].[Recepciones] WITH CHECK CHECK CONSTRAINT [FK_RECEPCIONES_CLIENTES];

ALTER TABLE [dbo].[Recepciones] WITH CHECK CHECK CONSTRAINT [FK_RECEPCIONES_EMPLEADOS];

ALTER TABLE [dbo].[Recepciones] WITH CHECK CHECK CONSTRAINT [FK_RECEPCIONES_VEHICULOS];

ALTER TABLE [dbo].[VehiculosVenta] WITH CHECK CHECK CONSTRAINT [FK_VEHICULOSVENTA_PRESUPUESTOS];

ALTER TABLE [dbo].[VehiculosVenta] WITH CHECK CHECK CONSTRAINT [FK_VEHICULOSVENTA_VENTAS];

ALTER TABLE [dbo].[Ventas] WITH CHECK CHECK CONSTRAINT [FK_Ventas_Clientes];

ALTER TABLE [dbo].[Ventas] WITH CHECK CHECK CONSTRAINT [FK_Ventas_Empleados];


GO
IF EXISTS (SELECT 1
           FROM   [master].[dbo].[sysdatabases]
           WHERE  [name] = N'$(DatabaseName)')
    BEGIN
        DECLARE @VarDecimalSupported AS BIT;
        SELECT @VarDecimalSupported = 0;
        IF ((ServerProperty(N'EngineEdition') = 3)
            AND (((@@microsoftversion / power(2, 24) = 9)
                  AND (@@microsoftversion & 0xffff >= 3024))
                 OR ((@@microsoftversion / power(2, 24) = 10)
                     AND (@@microsoftversion & 0xffff >= 1600))))
            SELECT @VarDecimalSupported = 1;
        IF (@VarDecimalSupported > 0)
            BEGIN
                EXECUTE sp_db_vardecimal_storage_format N'$(DatabaseName)', 'ON';
            END
    END


GO
ALTER DATABASE [$(DatabaseName)]
    SET MULTI_USER 
    WITH ROLLBACK IMMEDIATE;


GO
