﻿using System.Linq;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo
{
  public partial class OrdenReparacion
  {
    /// <summary>
    /// Obtiene el Id del estado de la orden de reparación.
    /// </summary>
    /// <remarks>
    /// Si la orden de reparación tiene un estado inferior a EnProceso hay que consultar
    /// el estado de las reparaciones de la misma. Si todas están finalizadas la orden
    /// pasará a estado Cerrada. Sino permanece en el mismo estado.
    /// En el caso de no tener reparaciones también permanece en el mismo estado
    /// </remarks>
    public int? IdEstado
    {
      get
      {
        if (Reparacion.Count == 0) return Estado;
        return Estado <= EstadoOrdenReparacion.EnProceso
                 ? (Reparacion.Any(
                   reparacion =>
                   reparacion.ContenidoReparacion.Any(
                     contenidoReparacion => contenidoReparacion.ManoDeObra != null && reparacion.Estado == 0))
                      ? Estado
                      : EstadoOrdenReparacion.Cerrada)
                 : Estado;
      }
    }
  }
}
