﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos
{
  /// <summary>
  /// Especificación AdHoc para buscar Articulos por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdArticuloSpecification : Specification<Articulo>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Articulo que debe coincidir en la búsqueda </param>
    public IdArticuloSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Articulo>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Articulo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Articulo}"/></returns>
    public override Expression<Func<Articulo, bool>> SatisfiedBy()
    {
      Specification<Articulo> spec = new TrueSpecification<Articulo>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Articulo, bool>> expression = articulo => articulo.Id == _id;

        spec &= new DirectSpecification<Articulo>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}