﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Articulos
{
  /// <summary>
  /// Especificación AdHoc para buscar Articulos por su nombre
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class NombreArticuloSpecification : Specification<Articulo>
  {
    #region Members

    private readonly string _nombre;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="nombre">Nombre del Articulo que debe coincidir en la búsqueda </param>
    public NombreArticuloSpecification(string nombre)
    {
      _nombre = nombre;
    }

    #endregion

    #region Miembros de Specification<Articulo>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Articulo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Articulo}"/></returns>
    public override Expression<Func<Articulo, bool>> SatisfiedBy()
    {
      Specification<Articulo> spec = new TrueSpecification<Articulo>();

      if (!string.IsNullOrWhiteSpace(_nombre))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Articulo, bool>> expression = articulo => articulo.Nombre.Contains(_nombre);

        spec &= new DirectSpecification<Articulo>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
