﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.ArticulosAlmacen
{
  /// <summary>
  /// Especificación AdHoc para buscar ArticuloAlmacens por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdArticuloAlmacenSpecification : Specification<ArticuloAlmacen>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del ArticuloAlmacen que debe coincidir en la búsqueda </param>
    public IdArticuloAlmacenSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<ArticuloAlmacen>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ArticuloAlmacen}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ArticuloAlmacen}"/></returns>
    public override Expression<Func<ArticuloAlmacen, bool>> SatisfiedBy()
    {
      Specification<ArticuloAlmacen> spec = new TrueSpecification<ArticuloAlmacen>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<ArticuloAlmacen, bool>> expression = articuloAlmacen => articuloAlmacen.Id == _id;

        spec &= new DirectSpecification<ArticuloAlmacen>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}