﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.ArticulosAlmacen
{
  /// <summary>
  /// Especificación AdHoc para buscar Articulos en el Almacen por su idArticulo
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdArticuloArticuloAlmacenSpecification : Specification<ArticuloAlmacen>
  {
    #region Members

    private readonly Guid? _idArticulo;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idArticulo">IdArticulo del ArticuloAlmacen que debe coincidir en la búsqueda </param>
    public IdArticuloArticuloAlmacenSpecification(Guid? idArticulo)
    {
      _idArticulo = idArticulo;
    }

    #endregion

    #region Miembros de Specification<ArticuloAlmacen>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ArticuloAlmacen}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ArticuloAlmacen}"/></returns>
    public override Expression<Func<ArticuloAlmacen, bool>> SatisfiedBy()
    {
      Specification<ArticuloAlmacen> spec = new TrueSpecification<ArticuloAlmacen>();

      if (_idArticulo != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<ArticuloAlmacen, bool>> expression = articuloAlmacen => articuloAlmacen.IdArticulo == _idArticulo;

        spec &= new DirectSpecification<ArticuloAlmacen>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
