﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Clientes
{
  /// <summary>
  /// Especificación AdHoc para buscar clientes por su nombre, primer o segundo apellido
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class ClienteSpecification : Specification<Cliente>
  {
    #region Atributos

    readonly string _nombre = default(String);
    readonly string _primerApellido = default(String);
    readonly string _segundoApellido = default(String);

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="nombre">Nombre del cliente que debe coincidir en la búsqueda</param>
    /// <param name="primerApellido">Primer apellido del cliente que debe coincidir en la búsqueda </param>
    /// <param name="segundoApellido">Segundo apellido del cliente que debe coincidir en la búsqueda </param>
    public ClienteSpecification(string nombre, string primerApellido, string segundoApellido)
    {
      _nombre = nombre;
      _primerApellido = primerApellido;
      _segundoApellido = segundoApellido;
    }

    #endregion

    #region Miembros de Specification<Cliente>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/></returns>
    public override Expression<Func<Cliente, bool>> SatisfiedBy()
    {
      Specification<Cliente> spec = new TrueSpecification<Cliente>();

      if (!String.IsNullOrEmpty(_nombre) && !String.IsNullOrWhiteSpace(_nombre))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Cliente, bool>> expression = cliente => cliente.Nombre.ToLower() == _nombre.ToLower();

        spec &= new DirectSpecification<Cliente>(expression);
      }
      if (!String.IsNullOrEmpty(_primerApellido) && !String.IsNullOrWhiteSpace(_primerApellido))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Cliente, bool>> expression = cliente => cliente.PrimerApellido.ToLower() == _primerApellido.ToLower();

        spec &= new DirectSpecification<Cliente>(expression);
      }
      if (!String.IsNullOrEmpty(_segundoApellido) && !String.IsNullOrWhiteSpace(_segundoApellido))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Cliente, bool>> expression = cliente => cliente.SegundoApellido.ToLower() == _segundoApellido.ToLower();

        spec &= new DirectSpecification<Cliente>(expression);
      }

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
