﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Clientes
{
  /// <summary>
  /// Especificación AdHoc para buscar clientes por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdClienteSpecification : Specification<Cliente>
  {
    #region Atributos

    readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del cliente que debe coincidir en la búsqueda </param>
    public IdClienteSpecification(Guid? id)
    {
      _id = id;
    }
    #endregion

    #region Miembros de Specification<Cliente>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/></returns>
    public override Expression<Func<Cliente, bool>> SatisfiedBy()
    {
      Specification<Cliente> spec = new TrueSpecification<Cliente>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Cliente, bool>> expression = cliente => cliente.Id == _id;

        spec &= new DirectSpecification<Cliente>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
