﻿using System;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Clientes
{
  /// <summary>
  /// Especificación AdHoc para buscar clientes por su nif/cif
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class NifCifClienteSpecification : Specification<Cliente>
  {
    #region Atributos

    readonly string _nifCif;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="nifCif">Nif/Cif del cliente que debe coincidir en la búsqueda </param>
    public NifCifClienteSpecification(string nifCif)
    {
      if (string.IsNullOrEmpty(nifCif) || string.IsNullOrWhiteSpace(nifCif))
      {
        throw new ArgumentNullException("nifCif");
      }
      _nifCif = nifCif;
    }

    #endregion
    
    #region Miembros de Specification<Cliente>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Cliente}"/></returns>
    public override System.Linq.Expressions.Expression<Func<Cliente, bool>> SatisfiedBy()
    {
      Specification<Cliente> spec = new TrueSpecification<Cliente>();

      spec &= new DirectSpecification<Cliente>(c => c.NifCif != null && (c.NifCif.ToUpper() == _nifCif.ToUpper()));

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
