﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.ContenidosReparaciones
{
  /// <summary>
  /// Especificación AdHoc para buscar Contenidos Reparacion por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdContenidoReparacionSpecification : Specification<ContenidoReparacion>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del ContenidoReparacion que debe coincidir en la búsqueda </param>
    public IdContenidoReparacionSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<ContenidoReparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ContenidoReparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ContenidoReparacion}"/></returns>
    public override Expression<Func<ContenidoReparacion, bool>> SatisfiedBy()
    {
      Specification<ContenidoReparacion> spec = new TrueSpecification<ContenidoReparacion>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<ContenidoReparacion, bool>> expression = contenidoReparacion => contenidoReparacion.Id == _id;

        spec &= new DirectSpecification<ContenidoReparacion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}