﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Empleados
{
  /// <summary>
  /// Especificación AdHoc para buscar Empleados por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdEmpleadoSpecification : Specification<Empleado>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Empleado que debe coincidir en la búsqueda </param>
    public IdEmpleadoSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Empleado>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Empleado}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Empleado}"/></returns>
    public override Expression<Func<Empleado, bool>> SatisfiedBy()
    {
      Specification<Empleado> spec = new TrueSpecification<Empleado>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Empleado, bool>> expression = empleado => empleado.Id == _id;

        spec &= new DirectSpecification<Empleado>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}