﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.ManosDeObra
{
  /// <summary>
  /// Especificación AdHoc para buscar Manos de obra por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdManoDeObraSpecification : Specification<ManoDeObra>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id de la ManoDeObra que debe coincidir en la búsqueda </param>
    public IdManoDeObraSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<ManoDeObra>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ManoDeObra}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{ManoDeObra}"/></returns>
    public override Expression<Func<ManoDeObra, bool>> SatisfiedBy()
    {
      Specification<ManoDeObra> spec = new TrueSpecification<ManoDeObra>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<ManoDeObra, bool>> expression = manoDeObra => manoDeObra.Id == _id;

        spec &= new DirectSpecification<ManoDeObra>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}