﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Modelos
{
  /// <summary>
  /// Especificación AdHoc para buscar modelos por su idMarca (Marca)
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdMarcaModeloSpecification : Specification<Modelo>
  {
    #region Atributos

    readonly string _idMarca = string.Empty;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idMarca">Identificador IdMarca de los modelos que debe coincidir en la búsqueda </param>
    public IdMarcaModeloSpecification(string idMarca)
    {
      _idMarca = idMarca;
    }
    #endregion

    #region Miembros de Specification<Cliente>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Modelo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Modelo}"/></returns>
    public override Expression<Func<Modelo, bool>> SatisfiedBy()
    {
      Specification<Modelo> spec = new TrueSpecification<Modelo>();

      //construir expresion y añadir una nueva condición a la misma
      Expression<Func<Modelo, bool>> expression = modelo => modelo.IdMarca == _idMarca;

      spec &= new DirectSpecification<Modelo>(expression);

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
