﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion
{
  /// <summary>
  /// Especificación AdHoc para buscar Ordenes de Reparacion por el nombre del empleado y el estado de la Orden
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdEmpleadoYEstadoOrdenReparacionSpecification : Specification<OrdenReparacion>
  {
    #region Members

    private readonly Guid? _idEmpleado = default(Guid);
    private readonly int _estado = default(int);

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idEmpleado">Id del empleado de la OrdenReparacion que debe coincidir en la búsqueda </param>
    /// <param name="estado">Estado de la OrdenReparacion que debe coincidir en la búsqueda </param>
    public IdEmpleadoYEstadoOrdenReparacionSpecification(Guid? idEmpleado, int estado)
    {
      _idEmpleado = idEmpleado;
      _estado = estado;
    }

    #endregion

    #region Miembros de Specification<OrdenReparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/></returns>
    public override Expression<Func<OrdenReparacion, bool>> SatisfiedBy()
    {
      Specification<OrdenReparacion> spec = new TrueSpecification<OrdenReparacion>();
      
      //construir expresion y añadir una nueva condición a la misma
      Expression<Func<OrdenReparacion, bool>> expression = ordenReparacion => ordenReparacion.IdPersonalAsignado == _idEmpleado;
      spec &= new DirectSpecification<OrdenReparacion>(expression);
      
      //construir expresion y añadir una nueva condición a la misma
      expression = ordenReparacion => ordenReparacion.Estado <= _estado;
      spec &= new DirectSpecification<OrdenReparacion>(expression);
      
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
