﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion
{
  /// <summary>
  /// Especificación AdHoc para buscar Ordenes de Reparacion por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdOrdenReparacionSpecification : Specification<OrdenReparacion>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id de la OrdenReparacion que debe coincidir en la búsqueda </param>
    public IdOrdenReparacionSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<OrdenReparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/></returns>
    public override Expression<Func<OrdenReparacion, bool>> SatisfiedBy()
    {
      Specification<OrdenReparacion> spec = new TrueSpecification<OrdenReparacion>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<OrdenReparacion, bool>> expression = ordenReparacion => ordenReparacion.Id == _id;

        spec &= new DirectSpecification<OrdenReparacion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}