﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion
{
  /// <summary>
  /// Especificación AdHoc para buscar Ordenes de Reparacion por la matricula del vehículo
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class MatriculaOrdenReparacionSpecification : Specification<OrdenReparacion>
  {
    #region Members

    private readonly string _matricula;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="matricula">Matrícula del vehículo de la OrdenReparacion que debe coincidir en la búsqueda </param>
    public MatriculaOrdenReparacionSpecification(string matricula)
    {
      _matricula = matricula;
    }

    #endregion

    #region Miembros de Specification<OrdenReparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/></returns>
    public override Expression<Func<OrdenReparacion, bool>> SatisfiedBy()
    {
      Specification<OrdenReparacion> spec = new TrueSpecification<OrdenReparacion>();

      if (!string.IsNullOrWhiteSpace(_matricula))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<OrdenReparacion, bool>> expression = ordenReparacion => ordenReparacion.Vehiculo.Matricula == _matricula;

        spec &= new DirectSpecification<OrdenReparacion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
