﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion
{
  /// <summary>
  /// Especificación AdHoc para buscar Ordenes de Reparacion por el nombre del cliente
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class NombreClienteOrdenReparacionSpecification : Specification<OrdenReparacion>
  {
    #region Members

    private readonly string _nombreCliente;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="nombreCliente">Nombre del cliente de la OrdenReparacion que debe coincidir en la búsqueda </param>
    public NombreClienteOrdenReparacionSpecification(string nombreCliente)
    {
      _nombreCliente = nombreCliente;
    }

    #endregion

    #region Miembros de Specification<OrdenReparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{OrdenReparacion}"/></returns>
    public override Expression<Func<OrdenReparacion, bool>> SatisfiedBy()
    {
      Specification<OrdenReparacion> spec = new TrueSpecification<OrdenReparacion>();

      if (!string.IsNullOrWhiteSpace(_nombreCliente))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<OrdenReparacion, bool>> expression = ordenReparacion => ordenReparacion.Recepcion.Cliente.Nombre.Contains(_nombreCliente);

        spec &= new DirectSpecification<OrdenReparacion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
