﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Paquetes
{
  /// <summary>
  /// Especificación AdHoc para buscar Paquetes por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdPaqueteSpecification : Specification<Paquete>
  {
    #region Members

    private readonly Guid? _idArticulo;
    private readonly Guid? _idManoDeObra;
    private readonly Guid? _idPaqueteDeServicio;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idArticulo">IdArticulo del Paquete que debe coincidir en la búsqueda </param>
    /// <param name="idManoDeObra">IdManoDeObra del Paquete que debe coincidir en la búsqueda </param>
    /// <param name="idPaqueteDeServicio">IdPaqueteDeServicio del Paquete que debe coincidir en la búsqueda </param>
    public IdPaqueteSpecification(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio)
    {
      _idArticulo = idArticulo;
      _idManoDeObra = idManoDeObra;
      _idPaqueteDeServicio = idPaqueteDeServicio;
    }

    #endregion

    #region Miembros de Specification<Paquete>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Paquete}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Paquete}"/></returns>
    public override Expression<Func<Paquete, bool>> SatisfiedBy()
    {
      Specification<Paquete> spec = new TrueSpecification<Paquete>();

      if (_idArticulo!=null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Paquete, bool>> expression = paquete => paquete.IdArticulo == _idArticulo;

        spec &= new DirectSpecification<Paquete>(expression);
      }
      if (_idManoDeObra != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Paquete, bool>> expression = paquete => paquete.IdManoDeObra == _idManoDeObra;

        spec &= new DirectSpecification<Paquete>(expression);
      }
      if (_idPaqueteDeServicio != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Paquete, bool>> expression = paquete => paquete.IdPaqueteDeServicio == _idPaqueteDeServicio;

        spec &= new DirectSpecification<Paquete>(expression);
      }

      return spec.SatisfiedBy();
    }

    #endregion
  }
}