﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.PaquetesDeServicio
{
  /// <summary>
  /// Especificación AdHoc para buscar Paquetes De Servicio por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdPaqueteDeServicioSpecification : Specification<PaqueteDeServicio>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del PaqueteDeServicio que debe coincidir en la búsqueda </param>
    public IdPaqueteDeServicioSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<PaqueteDeServicio>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{PaqueteDeServicio}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{PaqueteDeServicio}"/></returns>
    public override Expression<Func<PaqueteDeServicio, bool>> SatisfiedBy()
    {
      Specification<PaqueteDeServicio> spec = new TrueSpecification<PaqueteDeServicio>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<PaqueteDeServicio, bool>> expression = paqueteDeServicio => paqueteDeServicio.Id == _id;

        spec &= new DirectSpecification<PaqueteDeServicio>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}