﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Poblaciones
{
  /// <summary>
  /// Especificación AdHoc para buscar poblaciones por su idProvincia
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdProvinciaPoblacionSpecification : Specification<Poblacion>
  {
    #region Members

    readonly string _idProvincia = string.Empty;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idProvincia">Identificador IdProvincia de la población que debe coincidir en la búsqueda </param>
    public IdProvinciaPoblacionSpecification(string idProvincia)
    {
      _idProvincia = idProvincia;
    }
    #endregion

    #region Miembros de Specification<Cliente>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Poblacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Poblacion}"/></returns>
    public override Expression<Func<Poblacion, bool>> SatisfiedBy()
    {
      Specification<Poblacion> spec = new TrueSpecification<Poblacion>();

      //construir expresion y añadir una nueva condición a la misma
      Expression<Func<Poblacion, bool>> expression = poblacion => poblacion.IdProvincia == _idProvincia;

      spec &= new DirectSpecification<Poblacion>(expression);

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
