﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Presupuestos
{
  /// <summary>
  /// Especificación AdHoc para buscar presupuestos por el cliente y fecha
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class ClienteYFechaDelPresupuestoSpecification : Specification<Presupuesto>
  {
    #region Atributos

    readonly Guid? _idCliente = default(Guid);
    readonly DateTime? _fecha = default(DateTime);

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idCliente">Id del cliente que debe coincidir en la búsqueda</param>
    /// <param name="fecha">Valor mínimo de la fecha del presupuesto </param>
    public ClienteYFechaDelPresupuestoSpecification(Guid? idCliente, DateTime? fecha)
    {
      _idCliente = idCliente;
      _fecha = fecha;
    }

    #endregion

    #region Miembros de Specification<Presupuesto>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Presupuesto}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Presupuesto}"/></returns>
    public override Expression<Func<Presupuesto, bool>> SatisfiedBy()
    {
      Specification<Presupuesto> spec = new TrueSpecification<Presupuesto>();

      if (_idCliente != Guid.Empty)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Presupuesto, bool>> expression = presupuesto => presupuesto.IdCliente == _idCliente;

        spec &= new DirectSpecification<Presupuesto>(expression);
      }
      if (_fecha.HasValue)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Presupuesto, bool>> expression = presupuesto => presupuesto.Fecha >= _fecha;

        spec &= new DirectSpecification<Presupuesto>(expression);
      }

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
