﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Presupuestos
{
  /// <summary>
  /// Especificación AdHoc para buscar Presupuestos por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdPresupuestoSpecification : Specification<Presupuesto>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Presupuesto que debe coincidir en la búsqueda </param>
    public IdPresupuestoSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Presupuesto>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Presupuesto}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Presupuesto}"/></returns>
    public override Expression<Func<Presupuesto, bool>> SatisfiedBy()
    {
      Specification<Presupuesto> spec = new TrueSpecification<Presupuesto>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Presupuesto, bool>> expression = presupuesto => presupuesto.Id == _id;

        spec &= new DirectSpecification<Presupuesto>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}