﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Proveedores
{
  /// <summary>
  /// Especificación AdHoc para buscar Proveedores por su codigo
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class CodigoProveedorSpecification : Specification<Proveedor>
  {
    #region Members

    private readonly string _codigo;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="codigo">Codigo del Proveedor que debe coincidir en la búsqueda </param>
    public CodigoProveedorSpecification(string codigo)
    {
      _codigo = codigo;
    }

    #endregion

    #region Miembros de Specification<Proveedor>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Proveedor}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Proveedor}"/></returns>
    public override Expression<Func<Proveedor, bool>> SatisfiedBy()
    {
      Specification<Proveedor> spec = new TrueSpecification<Proveedor>();

      if (!string.IsNullOrWhiteSpace(_codigo))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Proveedor, bool>> expression = proveedor => proveedor.Codigo == _codigo;

        spec &= new DirectSpecification<Proveedor>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
