﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Proveedores
{
  /// <summary>
  /// Especificación AdHoc para buscar Proveedores por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdProveedorSpecification : Specification<Proveedor>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Proveedor que debe coincidir en la búsqueda </param>
    public IdProveedorSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Proveedor>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Proveedor}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Proveedor}"/></returns>
    public override Expression<Func<Proveedor, bool>> SatisfiedBy()
    {
      Specification<Proveedor> spec = new TrueSpecification<Proveedor>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Proveedor, bool>> expression = proveedor => proveedor.Id == _id;

        spec &= new DirectSpecification<Proveedor>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}