﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Recepciones
{
  /// <summary>
  /// Especificación AdHoc para buscar Recepciones por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdRecepcionSpecification : Specification<Recepcion>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Recepcion que debe coincidir en la búsqueda </param>
    public IdRecepcionSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Recepcion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Recepcion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Recepcion}"/></returns>
    public override Expression<Func<Recepcion, bool>> SatisfiedBy()
    {
      Specification<Recepcion> spec = new TrueSpecification<Recepcion>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Recepcion, bool>> expression = recepcion => recepcion.Id == _id;

        spec &= new DirectSpecification<Recepcion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}