﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Recepciones
{
  /// <summary>
  /// Especificación AdHoc para buscar Recepciones por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class NombreClienteRecepcionSpecification : Specification<Recepcion>
  {
    #region Members

    private readonly string _nombreCliente;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="nombreCliente">Nombre del Cliente de la Recepción que debe coincidir en la búsqueda </param>
    public NombreClienteRecepcionSpecification(string nombreCliente)
    {
      _nombreCliente = nombreCliente;
    }

    #endregion

    #region Miembros de Specification<Recepcion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Recepcion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Recepcion}"/></returns>
    public override Expression<Func<Recepcion, bool>> SatisfiedBy()
    {
      Specification<Recepcion> spec = new TrueSpecification<Recepcion>();

      if (!string.IsNullOrWhiteSpace(_nombreCliente))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Recepcion, bool>> expression = recepcion => recepcion.Cliente.Nombre.Contains(_nombreCliente);

        spec &= new DirectSpecification<Recepcion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
