﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Reparaciones
{
  /// <summary>
  /// Especificación AdHoc para buscar Reparaciones por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdReparacionSpecification : Specification<Reparacion>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Reparacion que debe coincidir en la búsqueda </param>
    public IdReparacionSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Reparacion>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Reparacion}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Reparacion}"/></returns>
    public override Expression<Func<Reparacion, bool>> SatisfiedBy()
    {
      Specification<Reparacion> spec = new TrueSpecification<Reparacion>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Reparacion, bool>> expression = reparacion => reparacion.Id == _id;

        spec &= new DirectSpecification<Reparacion>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}