﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Talleres
{
  /// <summary>
  /// Especificación AdHoc para buscar Talleres por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdTallerSpecification : Specification<Taller>
  {
    #region Members

    private readonly string _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Taller que debe coincidir en la búsqueda </param>
    public IdTallerSpecification(string id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Taller>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Taller}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Taller}"/></returns>
    public override Expression<Func<Taller, bool>> SatisfiedBy()
    {
      Specification<Taller> spec = new TrueSpecification<Taller>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Taller, bool>> expression = taller => taller.Id == _id;

        spec &= new DirectSpecification<Taller>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}