﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Vehiculos
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdVehiculoSpecification : Specification<Vehiculo>
  {
    #region Members

    readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del vehículo que debe coincidir en la búsqueda </param>
    public IdVehiculoSpecification(Guid? id)
    {
      _id = id;
    }
    #endregion

    #region Miembros de Specification<Vehiculo>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/></returns>
    public override Expression<Func<Vehiculo, bool>> SatisfiedBy()
    {
      Specification<Vehiculo> spec = new TrueSpecification<Vehiculo>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Vehiculo, bool>> expression = vehiculo => vehiculo.Id == _id;

        spec &= new DirectSpecification<Vehiculo>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
