﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Vehiculos
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos por su marca y modelo
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class MarcaModeloVehiculoSpecification : Specification<Vehiculo>
  {
    #region Members

    readonly string _marca = default(String);
    readonly string _modelo = default(String);
    
    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="marca">Marca del vehículo que debe coincidir en la búsqueda</param>
    /// <param name="modelo">Modelo del vehículo que debe coincidir en la búsqueda </param>
    public MarcaModeloVehiculoSpecification(string marca, string modelo)
    {
      _marca = marca;
      _modelo = modelo;
    }

    #endregion

    #region Miembros de Specification<Vehiculo>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/></returns>
    public override Expression<Func<Vehiculo, bool>> SatisfiedBy()
    {
      Specification<Vehiculo> spec = new TrueSpecification<Vehiculo>();

      if (!String.IsNullOrEmpty(_marca) && !String.IsNullOrWhiteSpace(_modelo))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Vehiculo, bool>> expression = vehiculo => vehiculo.Marca.ToLower() == _marca.ToLower();

        spec &= new DirectSpecification<Vehiculo>(expression);
      }
      if (!String.IsNullOrEmpty(_modelo) && !String.IsNullOrWhiteSpace(_modelo))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Vehiculo, bool>> expression = vehiculo => vehiculo.Modelo.ToLower() == _modelo.ToLower();

        spec &= new DirectSpecification<Vehiculo>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
