﻿using System;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Vehiculos
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos por su matrícula
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class MatriculaVehiculoSpecification : Specification<Vehiculo>
  {
    #region Members

    readonly string _matricula;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="matricula">Matrícula del vehículo que debe coincidir en la búsqueda </param>
    public MatriculaVehiculoSpecification(string matricula)
    {
      if (string.IsNullOrEmpty(matricula) || string.IsNullOrWhiteSpace(matricula))
      {
        throw new ArgumentNullException("matricula");
      }
      _matricula = matricula;
    }

    #endregion

    #region Miembros de Specification<Vehiculo>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/></returns>
    public override System.Linq.Expressions.Expression<Func<Vehiculo, bool>> SatisfiedBy()
    {
      Specification<Vehiculo> spec = new TrueSpecification<Vehiculo>();
      
      spec &= new DirectSpecification<Vehiculo>(c => c.Matricula != null && (c.Matricula.ToUpper() == _matricula.ToUpper()));

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
