﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.VehiculosVenta
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class CodigoVehiculoVentaSpecification : Specification<VehiculoVenta>
  {
    #region Members

    private readonly string _codigo;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="codigo">Código del vehículo de venta que debe coincidir en la búsqueda </param>
    public CodigoVehiculoVentaSpecification(string codigo)
    {
      _codigo = codigo;
    }
    #endregion

    #region Miembros de Specification<VehiculoVenta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{VehiculoVenta}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{VehiculoVenta}"/></returns>
    public override Expression<Func<VehiculoVenta, bool>> SatisfiedBy()
    {
      Specification<VehiculoVenta> spec = new TrueSpecification<VehiculoVenta>();

      if (_codigo != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<VehiculoVenta, bool>> expression = vehiculoVenta => vehiculoVenta.Codigo == _codigo;

        spec &= new DirectSpecification<VehiculoVenta>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
