﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.VehiculosVenta
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos de venta por su marca y modelo
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class MarcaModeloVehiculoVentaSpecification : Specification<VehiculoVenta>
  {
    #region Members

    readonly string _marca = default(String);
    readonly string _modelo = default(String);

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="marca">Marca del vehículo de venta que debe coincidir en la búsqueda</param>
    /// <param name="modelo">Modelo del vehículo de venta que debe coincidir en la búsqueda </param>
    public MarcaModeloVehiculoVentaSpecification(string marca, string modelo)
    {
      _marca = marca;
      _modelo = modelo;
    }

    #endregion

    #region Miembros de Specification<VehiculoVenta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Vehiculo}"/></returns>
    public override Expression<Func<VehiculoVenta, bool>> SatisfiedBy()
    {
      Specification<VehiculoVenta> spec = new TrueSpecification<VehiculoVenta>();

      if (!String.IsNullOrEmpty(_marca) && !String.IsNullOrWhiteSpace(_modelo))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<VehiculoVenta, bool>> expression = vehiculoVenta => vehiculoVenta.Marca.ToLower() == _marca.ToLower();

        spec &= new DirectSpecification<VehiculoVenta>(expression);
      }
      if (!String.IsNullOrEmpty(_modelo) && !String.IsNullOrWhiteSpace(_modelo))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<VehiculoVenta, bool>> expression = vehiculoVenta => vehiculoVenta.Modelo.ToLower() == _modelo.ToLower();

        spec &= new DirectSpecification<VehiculoVenta>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
