﻿using System;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.VehiculosVenta
{
  /// <summary>
  /// Especificación AdHoc para buscar vehiculos por su matrícula
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class MatriculaVehiculoVentaSpecification : Specification<VehiculoVenta>
  {
    #region Members

    readonly string _matricula;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="matricula">Matrícula del vehículo de venta que debe coincidir en la búsqueda </param>
    public MatriculaVehiculoVentaSpecification(string matricula)
    {
      if (string.IsNullOrEmpty(matricula) || string.IsNullOrWhiteSpace(matricula))
      {
        throw new ArgumentNullException("matricula");
      }
      _matricula = matricula;
    }

    #endregion

    #region Miembros de Specification<VehiculoVenta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{VehiculoVenta}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{VehiculoVenta}"/></returns>
    public override System.Linq.Expressions.Expression<Func<VehiculoVenta, bool>> SatisfiedBy()
    {
      Specification<VehiculoVenta> spec = new TrueSpecification<VehiculoVenta>();

      spec &= new DirectSpecification<VehiculoVenta>(c => c.Matricula != null && (c.Matricula.ToUpper() == _matricula.ToUpper()));

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
