﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Ventas
{
  /// <summary>
  /// Especificación AdHoc para buscar ventas por el cliente y fecha
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class ClienteYFechaDeLaVentaSpecification : Specification<Venta>
  {
    #region Atributos

    readonly Guid? _idCliente = default(Guid);
    readonly DateTime? _fecha = default(DateTime);

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="idCliente">Id del cliente que debe coincidir en la búsqueda</param>
    /// <param name="fecha">Valor mínimo de la fecha de la venta </param>
    public ClienteYFechaDeLaVentaSpecification(Guid? idCliente, DateTime? fecha)
    {
      _idCliente = idCliente;
      _fecha = fecha;
    }

    #endregion

    #region Miembros de Specification<Venta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/></returns>
    public override Expression<Func<Venta, bool>> SatisfiedBy()
    {
      Specification<Venta> spec = new TrueSpecification<Venta>();

      if (_idCliente != Guid.Empty)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Venta, bool>> expression = venta => venta.IdCliente == _idCliente;

        spec &= new DirectSpecification<Venta>(expression);
      }
      if (_fecha.HasValue)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Venta, bool>> expression = venta => venta.Fecha >= _fecha;

        spec &= new DirectSpecification<Venta>(expression);
      }

      return spec.SatisfiedBy();
    }

    #endregion
  }
}
