﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Ventas
{
  /// <summary>
  /// Especificación AdHoc para buscar Ventas por su id
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class IdVentaSpecification : Specification<Venta>
  {
    #region Members

    private readonly Guid? _id;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="id">Id del Venta que debe coincidir en la búsqueda </param>
    public IdVentaSpecification(Guid? id)
    {
      _id = id;
    }

    #endregion

    #region Miembros de Specification<Venta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/></returns>
    public override Expression<Func<Venta, bool>> SatisfiedBy()
    {
      Specification<Venta> spec = new TrueSpecification<Venta>();

      if (_id != null)
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Venta, bool>> expression = venta => venta.Id == _id;

        spec &= new DirectSpecification<Venta>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}