﻿using System;
using System.Linq.Expressions;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Dominio.ModuloPrincipal.Ventas
{
  /// <summary>
  /// Especificación AdHoc para buscar Ventas por su numero
  /// Para mas información sobre la clase specification (que implementa el patrón specification) <see cref=" GestionConcesionarios.Dominio.Core.Specification.ISpecification{TEntity}"/>
  /// </summary>
  public class NumeroVentaSpecification : Specification<Venta>
  {
    #region Members

    private readonly string _numero;

    #endregion

    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="numero">Numero de la Venta que debe coincidir en la búsqueda </param>
    public NumeroVentaSpecification(string numero)
    {
      _numero = numero;
    }

    #endregion

    #region Miembros de Specification<Venta>

    /// <summary>
    /// <see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/>
    /// </summary>
    /// <returns><see cref="GestionConcesionarios.Dominio.Core.Specification.Specification{Venta}"/></returns>
    public override Expression<Func<Venta, bool>> SatisfiedBy()
    {
      Specification<Venta> spec = new TrueSpecification<Venta>();

      if (!string.IsNullOrWhiteSpace(_numero))
      {
        //construir expresion y añadir una nueva condición a la misma
        Expression<Func<Venta, bool>> expression = venta => venta.Numero == _numero;

        spec &= new DirectSpecification<Venta>(expression);
      }
      return spec.SatisfiedBy();
    }

    #endregion
  }
}
