﻿using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Data.EntityClient;
using System.Data.Metadata.Edm;
using System.Data.Objects;
using System.Reflection;
using System.Windows.Forms;
using Autofac;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Dominio.ModuloPrincipal.Clientes;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;
using GestionConcesionarios.Infraestructura.Helpers;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.CasosDeUso.FormularioPrincipal;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using Module = Autofac.Module;

namespace GestionConcesionariosEjecutable.Container
{
  class GestionConcesionariosViewModule : Module
  {
    protected override void Load(ContainerBuilder builder)
    {
      builder.RegisterType<FormPrincipal>().SingleInstance().Named<Form>("FormPrincipal");

      //Información sobre el usuario de la aplicación
      builder.RegisterType<InformacionUsuario>().SingleInstance();

      //Servicios de aplicacion
      builder.RegisterAssemblyTypes(typeof(GestionClienteService).Assembly).Where(l => l.ImplementsInterface(typeof(IAplicacionService))).AsImplementedInterfaces().AsSelf();

      //Repositorios
      builder.RegisterAssemblyTypes(typeof(IClienteRepository).Assembly).Where(l => l.ImplementsInterface(typeof(IRepository<>))).AsImplementedInterfaces().AsSelf();
      builder.RegisterAssemblyTypes(typeof(ClienteRepository).Assembly).Where(l => l.ImplementsInterface(typeof(IRepository<>))).AsImplementedInterfaces().AsSelf();
      builder.RegisterAssemblyTypes(typeof(ClienteRepository).Assembly).Where(l => l.ImplementsInterface(typeof(IMainModuleUnitOfWork))).AsImplementedInterfaces().AsSelf();
      
      //Entidades dominio
      builder.RegisterType<Articulo>();
      builder.RegisterType<ArticuloAlmacen>();
      builder.RegisterType<Cliente>();
      builder.RegisterType<CodigoPostal>();
      builder.RegisterType<ContenidoReparacion>();
      builder.RegisterType<Empleado>();
      builder.RegisterType<LookUp>();
      builder.RegisterType<ManoDeObra>();
      builder.RegisterType<OrdenReparacion>();
      builder.RegisterType<Paquete>();
      builder.RegisterType<PaqueteDeServicio>();
      builder.RegisterType<Poblacion>();
      builder.RegisterType<Presupuesto>();
      builder.RegisterType<Proveedor>();
      builder.RegisterType<Provincia>();
      builder.RegisterType<Recepcion>();
      builder.RegisterType<Reparacion>();
      builder.RegisterType<Vehiculo>();
      builder.RegisterType<VehiculoVenta>();
      builder.RegisterType<Venta>();
      builder.RegisterType<CategoriaArticulo>();
      builder.RegisterType<FormaDePago>();
      builder.RegisterType<TipoEmpleado>();
      builder.RegisterType<Marca>();
      builder.RegisterType<Modelo>();
      builder.RegisterType<Taller>();
      builder.RegisterType<EstadoOrdenReparacion>();
      builder.RegisterType<EstadoReparacion>();
      builder.RegisterType<TipoVehiculo>();
      builder.RegisterType<TipoCombustible>();

      builder.RegisterType(typeof(EntidadesGestionConcesionarios)).AsImplementedInterfaces().UsingConstructor();
      
      builder.Register(c => new ObjectContext(c.Resolve<EntityConnection>())).InstancePerLifetimeScope();
      builder.Register(c => new ObjectContext(c.Resolve<string>())).InstancePerLifetimeScope();

      builder.RegisterAssemblyTypes(typeof(ClienteRepository).Assembly);
      builder.RegisterType<EntityConnection>();
      builder.Register(c => new EntityConnection(c.Resolve<string>()));
      builder.Register(c => new MetadataWorkspace(c.Resolve<IEnumerable<string>>(), c.Resolve<IEnumerable<Assembly>>()));
      builder.RegisterType<DbConnection>();
      builder.RegisterType<MetadataWorkspace>();
      builder.RegisterType<Assembly>();
      builder.Register(c => Assembly.GetAssembly(c.Resolve<Type>())).As<Assembly>();
      builder.Register(c => Assembly.GetCallingAssembly()).As<Assembly>();
      builder.Register(c => Assembly.GetEntryAssembly()).As<Assembly>();
      builder.Register(c => Assembly.GetExecutingAssembly()).As<Assembly>();
      builder.Register(c => Assembly.Load(c.Resolve<string>())).As<Assembly>();
      builder.Register(c => Assembly.LoadFile(c.Resolve<string>())).As<Assembly>();
      builder.Register(c => Assembly.LoadFrom(c.Resolve<string>())).As<Assembly>();
      builder.RegisterType<String>();
    }
  }
}
