﻿using System;
using System.Windows.Forms;
using Autofac;
using DevExpress.LookAndFeel;
using GestionConcesionarios.Infraestructura.Presentacion;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionariosEjecutable.Container;

namespace GestionConcesionariosEjecutable
{
  static class Program
  {
    /// <summary>
    /// The main entry point for the application.
    /// </summary>
    [STAThread]
    static void Main()
    {
      Application.EnableVisualStyles();
      Application.SetCompatibleTextRenderingDefault(false);

      DevExpress.Skins.SkinManager.EnableFormSkins();
      DevExpress.UserSkins.BonusSkins.Register();
      UserLookAndFeel.Default.SetSkinStyle("DevExpress Style");
      
      BootStrapper bootstrapper =
        (PresentationBootStrapperBuilder)new PresentationBootStrapperBuilder()
                                            .AddPresentationAssembly(typeof(ClienteViewModel).Assembly)
                                            .WithContainerRegistrationAction(b => b.RegisterModule(new GestionConcesionariosViewModule()))
                                            .AddNamespace("Almacen")
                                            .AddNamespace("Articulos")
                                            .AddNamespace("BuscarArticulo")
                                            .AddNamespace("BuscarCliente")
                                            .AddNamespace("BuscarManoDeObra")
                                            .AddNamespace("BuscarOrdenDeReparacion")
                                            .AddNamespace("BuscarPaqueteDeServicio")
                                            .AddNamespace("BuscarPresupuesto")
                                            .AddNamespace("BuscarProveedor")
                                            .AddNamespace("BuscarRecepcionVehiculo")
                                            .AddNamespace("BuscarReparaciones")
                                            .AddNamespace("BuscarVehiculo")
                                            .AddNamespace("BuscarVehiculoVenta")
                                            .AddNamespace("BuscarVenta")
                                            .AddNamespace("Clientes")
                                            .AddNamespace("FinalizarReparaciones")
                                            .AddNamespace("FormularioPrincipal")
                                            .AddNamespace("HistorialOrdenesDeReparacion")
                                            .AddNamespace("ManosDeObra")
                                            .AddNamespace("Login")
                                            .AddNamespace("OrdenesDeReparacion")
                                            .AddNamespace("PaquetesDeServicio")
                                            .AddNamespace("Presupuestos")
                                            .AddNamespace("Proveedores")
                                            .AddNamespace("RecepcionVehiculo")
                                            .AddNamespace("Ventas")
                                            .AddNamespace("Vehiculos")
                                            .AddNamespace("VehiculosDeVenta");

      using (var presentationContainer = bootstrapper.Container)
      {
        var formPrincipal = presentationContainer.ResolveNamed<Form>("FormPrincipal");
        Application.Run(formPrincipal);
      }
      
      /*
      //Perform application configuration
      IBootStrapper bootStrapper = new DefaultBootStrapper(IoCFactory.Instance.CurrentContainer);
      bootStrapper.Boot();
      */
      Environment.Exit(0);

      /*
      var pantallaBienvenida = new PantallaBienvenida();
      pantallaBienvenida.ShowDialog();
      
      var formLogin = new FormLogin();
      formLogin.ShowDialog();
      
      Application.Run(new FormPrincipal());
      Environment.Exit(0);
      */

      /*
       var unitOfWork = new UnitOfWork<ProjectCodeEntities>();
            var blogRepository = unitOfWork.GetRepository<BlogPost>();
            var catRepository = unitOfWork.GetRepository<Category>();
            using (unitOfWork)
            {
                Guid postId = Guid.NewGuid();
                Guid catId = Guid.NewGuid();
                System.Console.WriteLine("Enter Category of your blog post:");
                var cat = new Category { Id = catId, Name = System.Console.ReadLine() };
                System.Console.WriteLine("Enter Title of your blog post:");
                var post = new BlogPost { Id = postId, Title = System.Console.ReadLine(), Content = "Hello Universe!", PublishDate = new DateTime(2011, 1, 1), Category = cat };

                blogRepository.Add(post);

                int i = unitOfWork.Save();
                if (i > 0)
                    System.Console.WriteLine("Successfully Saved!!");
                else
                    System.Console.WriteLine("Failed to save!");

                System.Console.WriteLine("List of all post:");
                var resultPost = blogRepository.SelectAll();

                foreach (var item in resultPost)
                {
                    System.Console.WriteLine(item.Title);
                }

                System.Console.WriteLine("List of all category:");
                var resultCat = catRepository.SelectAll();

                foreach (var item in resultCat)
                {
                    System.Console.WriteLine(item.Name);
                }

                System.Console.WriteLine("Press any key to quit!");
                System.Console.Read();
            }
       */
    }
  }
}