﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.Articulos;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IArticuloRepository
  /// <see cref="IArticuloRepository"/>
  /// </summary>
  public class ArticuloRepository : Repository<Articulo>, IArticuloRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public ArticuloRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IArticuloRepository

    /// <summary>
    /// <see cref="IArticuloRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IArticuloRepository"/></param>
    /// <returns>Articulos que coinciden con la especificación <paramref name="specification"/></returns>
    public List<Articulo> BuscarArticulo(ISpecification<Articulo> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.Articulos.
        Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}