﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.CategoriasArticulos;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio ICategoriaArticuloRepository
  /// <see cref="ICategoriaArticuloRepository"/>
  /// </summary>
  public class CategoriaArticuloRepository : Repository<CategoriaArticulo>, ICategoriaArticuloRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public CategoriaArticuloRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de ICategoriaArticuloRepository

    /// <summary>
    /// <see cref="ICategoriaArticuloRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="ICategoriaArticuloRepository"/></param>
    /// <returns>Categorias Articulo que coinciden con la especificación <paramref name="specification"/></returns>
    public List<CategoriaArticulo> BuscarCategoriaArticulo(ISpecification<CategoriaArticulo> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.CategoriasArticulo.Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    /// <summary>
    /// <see cref="ICategoriaArticuloRepository"/>
    /// </summary>
    /// <returns>Todas las Categorias Articulo </returns>
    public List<CategoriaArticulo> BuscarTodasLasCategoriasArticulo()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.CategoriasArticulo.ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}