﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.ModuloPrincipal.CodigosPostales;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio ICodigoPostalRepository
  /// <see cref="ICodigoPostalRepository"/>
  /// </summary>
  public class CodigoPostalRepository : Repository<CodigoPostal>, ICodigoPostalRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public CodigoPostalRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork) { }

    #endregion

    #region Implementación de ICodigoPostalRepository

    /// <summary>
    /// <see cref="ICodigoPostalRepository"/>
    /// </summary>
    /// <returns>Todos los códigos postales </returns>
    public List<CodigoPostal> BuscarCodigoPostal()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.CodigosPostales.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Resources.Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion


  }
}