﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.EstadosOrdenReparacion;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IEstadoOrdenReparacionRepository
  /// <see cref="IEstadoOrdenReparacionRepository"/>
  /// </summary>
  public class EstadoOrdenReparacionRepository : Repository<EstadoOrdenReparacion>, IEstadoOrdenReparacionRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public EstadoOrdenReparacionRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IEstadoOrdenReparacionRepository

    /// <summary>
    /// <see cref="IEstadoOrdenReparacionRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IEstadoOrdenReparacionRepository"/></param>
    /// <returns>Estados OrdenReparacion que coinciden con la especificación <paramref name="specification"/></returns>
    public List<EstadoOrdenReparacion> BuscarEstadoOrdenReparacion(ISpecification<EstadoOrdenReparacion> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.EstadosOrdenReparacion.Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    /// <summary>
    /// <see cref="IEstadoOrdenReparacionRepository"/>
    /// </summary>
    /// <returns>Todas los estados orden reparación </returns>
    public List<EstadoOrdenReparacion> BuscarTodosLosEstadosOrdenReparacion()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.EstadosOrdenReparacion.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }
    #endregion
  }
}