﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.EstadosReparacion;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IEstadoReparacionRepository
  /// <see cref="IEstadoReparacionRepository"/>
  /// </summary>
  public class EstadoReparacionRepository : Repository<EstadoReparacion>, IEstadoReparacionRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public EstadoReparacionRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IEstadoReparacionRepository

    /// <summary>
    /// <see cref="IEstadoReparacionRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IEstadoReparacionRepository"/></param>
    /// <returns>Estados Reparacion que coinciden con la especificación <paramref name="specification"/></returns>
    public List<EstadoReparacion> BuscarEstadoReparacion(ISpecification<EstadoReparacion> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.EstadoReparacion.Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    /// <summary>
    /// <see cref="IEstadoReparacionRepository"/>
    /// </summary>
    /// <returns>Todas los estados reparación </returns>
    public List<EstadoReparacion> BuscarTodosLosEstadosReparacion()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.EstadoReparacion.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }
    #endregion
  }
}