﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.ModuloPrincipal.Marcas;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IMarcaRepository
  /// <see cref="IMarcaRepository"/>
  /// </summary>
  public class MarcaRepository : Repository<Marca>, IMarcaRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public MarcaRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork) { }

    #endregion

    #region Implementación de IMarcaRepository

    /// <summary>
    /// <see cref="IMarcaRepository"/>
    /// </summary>
    /// <returns>Todas las marcas </returns>
    public List<Marca> BuscarMarcas()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.Marcas.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Resources.Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion


  }
}