﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.Core.Extensiones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IOrdenReparacionRepository
  /// <see cref="IOrdenReparacionRepository"/>
  /// </summary>
  public class OrdenReparacionRepository : Repository<OrdenReparacion>, IOrdenReparacionRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public OrdenReparacionRepository(IMainModuleUnitOfWork unitOfWork)
      : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IOrdenReparacionRepository

    /// <summary>
    /// <see cref="IOrdenReparacionRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IOrdenReparacionRepository"/></param>
    /// <returns>Ordenes Reparacion que coinciden con la especificación <paramref name="specification"/></returns>
    public List<OrdenReparacion> BuscarOrdenReparacion(ISpecification<OrdenReparacion> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return
          activeContext.OrdenesReparacions
          .Include("Recepcion.Cliente")
          .Include("Reparacion.ContenidoReparacion.Articulo")
          .Include("Reparacion.ContenidoReparacion.ManoDeObra")
          .Include("Reparacion.ContenidoReparacion.PaqueteDeServicio")
          .Include(c => c.Empleado)
          .Include(c => c.Vehiculo).Where(
            specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}