﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.PaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.Core.Extensiones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IPaqueteDeServicioRepository
  /// <see cref="IPaqueteDeServicioRepository"/>
  /// </summary>
  public class PaqueteDeServicioRepository : Repository<PaqueteDeServicio>, IPaqueteDeServicioRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public PaqueteDeServicioRepository(IMainModuleUnitOfWork unitOfWork)
      : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IPaqueteDeServicioRepository

    /// <summary>
    /// <see cref="IPaqueteDeServicioRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IPaqueteDeServicioRepository"/></param>
    /// <returns>Paquetes De Servicios que coinciden con la especificación <paramref name="specification"/></returns>
    public List<PaqueteDeServicio> BuscarPaqueteDeServicio(ISpecification<PaqueteDeServicio> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        var listaArticulos = activeContext.PaquetesDeServicios
          .Include("Paquete.Articulo")
          .Where(specification.SatisfiedBy()).ToList();

        var listaManosDeObra = activeContext.PaquetesDeServicios
          .Include("Paquete.ManoDeObra")
          .Where(specification.SatisfiedBy()).ToList();

        var lista = listaArticulos.ToList();
        lista.AddRange(listaManosDeObra);

        return lista;
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}