﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.Presupuestos;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.Core.Extensiones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IPresupuestoRepository
  /// <see cref="IPresupuestoRepository"/>
  /// </summary>
  public class PresupuestoRepository : Repository<Presupuesto>, IPresupuestoRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public PresupuestoRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IPresupuestoRepository

    /// <summary>
    /// <see cref="IPresupuestoRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IPresupuestoRepository"/></param>
    /// <returns>Presupuestos que coinciden con la especificación <paramref name="specification"/></returns>
    public List<Presupuesto> BuscarPresupuesto(ISpecification<Presupuesto> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.Presupuestos.Include(c=>c.VehiculoVenta).Include(c => c.Cliente).Include(c => c.Empleado).Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}