﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.ModuloPrincipal.Provincias;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IProvinciaRepository
  /// <see cref="IProvinciaRepository"/>
  /// </summary>
  public class ProvinciaRepository : Repository<Provincia>, IProvinciaRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public ProvinciaRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork) { }

    #endregion

    #region IProvinciaRepository implementation

    /// <summary>
    /// <see cref="IProvinciaRepository"/>
    /// </summary>
    /// <returns>Todas las provincias </returns>
    public List<Provincia> BuscarProvincia()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.Provincias.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Resources.Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion


  }
}