﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.Reparaciones;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IReparacionRepository
  /// <see cref="IReparacionRepository"/>
  /// </summary>
  public class ReparacionRepository : Repository<Reparacion>, IReparacionRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public ReparacionRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de IReparacionRepository

    /// <summary>
    /// <see cref="IReparacionRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IReparacionRepository"/></param>
    /// <returns>Reparaciones que coinciden con la especificación <paramref name="specification"/></returns>
    public List<Reparacion> BuscarReparacion(ISpecification<Reparacion> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.Reparaciones.Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}