﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.TiposEmpleado;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Resources;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio ITipoEmpleadoRepository
  /// <see cref="ITipoEmpleadoRepository"/>
  /// </summary>
  public class TipoEmpleadoRepository : Repository<TipoEmpleado>, ITipoEmpleadoRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public TipoEmpleadoRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork)
    {
    }

    #endregion

    #region Implementación de ITipoEmpleadoRepository

    /// <summary>
    /// <see cref="ITipoEmpleadoRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="ITipoEmpleadoRepository"/></param>
    /// <returns>Tipos Empleado que coinciden con la especificación <paramref name="specification"/></returns>
    public List<TipoEmpleado> BuscarTipoEmpleado(ISpecification<TipoEmpleado> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.TiposEmpleado.Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    /// <summary>
    /// <see cref="ITipoEmpleadoRepository"/>
    /// </summary>
    /// <returns>Todas los tipos de empleado </returns>
    public List<TipoEmpleado> BuscarTodosLosTiposDeEmpleado()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.TiposEmpleado.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Messages.exception_InvalidStoreContext,
        GetType().Name));
    }
    #endregion
  }
}