﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.ModuloPrincipal.TiposDeVehiculo;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio ITipoVehiculoRepository
  /// <see cref="ITipoVehiculoRepository"/>
  /// </summary>
  public class TipoVehiculoRepository : Repository<TipoVehiculo>, ITipoVehiculoRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public TipoVehiculoRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork) { }

    #endregion

    #region Implementación de ITipoVehiculoRepository

    /// <summary>
    /// <see cref="ITipoVehiculoRepository"/>
    /// </summary>
    /// <returns>Todos los tipos de vehiculo </returns>
    public List<TipoVehiculo> BuscarTipoVehiculo()
    {
      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.TiposVehiculo.ToList();

      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Resources.Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion
  }
}
