﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Dominio.Core.Specification;
using GestionConcesionarios.Dominio.ModuloPrincipal.Vehiculos;
using GestionConcesionarios.Dominio.ModuloPrincipal.VehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.Core;
using GestionConcesionarios.Infraestructura.Data.Core.Extensiones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.UnitOfWork;

namespace GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Repositorios
{
  /// <summary>
  /// Implementación del repositorio IVehiculoRepository
  /// <see cref="IVehiculoRepository"/>
  /// </summary>
  public class VehiculoVentaRepository : Repository<VehiculoVenta>, IVehiculoVentaRepository
  {
    #region Constructor

    /// <summary>
    /// Constructor por defecto
    /// </summary>
    /// <param name="unitOfWork">unitOfWork específico para este repositorio</param>
    public VehiculoVentaRepository(IMainModuleUnitOfWork unitOfWork) : base(unitOfWork) { }

    #endregion

    #region Implementación de IVehiculoRepository

    /// <summary>
    /// <see cref="IVehiculoRepository"/>
    /// </summary>
    /// <param name="specification">Especificación de los criterios de búsqueda<see cref="IVehiculoRepository"/></param>
    /// <returns>Vehiculos de venta que coinciden con la especificación <paramref name="specification"/></returns>
    public List<VehiculoVenta> BuscarVehiculoVenta(ISpecification<VehiculoVenta> specification)
    {
      //Validar la especificacion
      if (specification == null) throw new ArgumentNullException("specification");

      var activeContext = UnitOfWork as IMainModuleUnitOfWork;
      if (activeContext != null)
      {
        //Ejecutar la operación de búsqueda en este repositorio
        return activeContext.VehiculosVentas.Include("Venta.Empleado").Where(specification.SatisfiedBy()).ToList();
      }
      throw new InvalidOperationException(string.Format(
        CultureInfo.InvariantCulture,
        Resources.Messages.exception_InvalidStoreContext,
        GetType().Name));
    }

    #endregion


  }
}