﻿using System;
using System.Linq.Expressions;
using FluentValidation;

namespace GestionConcesionarios.Infraestructura.Helpers
{
  public static class RuleBuilderValidatorExtensions
  {
    public static IRuleBuilderOptions<T, string> DefaultLength<T>(this IRuleBuilderOptions<T, string> ruleBuilder)
    {
      return ruleBuilder.Length(0, 50);
    }
    public static IRuleBuilderOptions<T, string> DefaultLength<T>(this IRuleBuilderInitial<T, string> ruleBuilder)
    {
      return ruleBuilder.Length(0, 50);
    }

    public static IRuleBuilderOptions<T, decimal> Percentage<T>(this IRuleBuilderOptions<T, decimal> ruleBuilder)
    {
      return ruleBuilder.GreaterThanOrEqualTo(0).LessThanOrEqualTo(100);
    }

    public static IRuleBuilderOptions<T, decimal> Percentage<T>(this IRuleBuilderInitial<T, decimal> ruleBuilder)
    {
      return ruleBuilder.GreaterThanOrEqualTo(0).LessThanOrEqualTo(100);
    }

    public static IRuleBuilderOptions<T, DateTime> DateRange<T>(this IRuleBuilderOptions<T, DateTime> ruleBuilder,Expression<Func<T,DateTime>> endDateExpression)
    {
      return ruleBuilder.LessThan(endDateExpression);
    }

    public static IRuleBuilderOptions<T, DateTime> DateRange<T>(this IRuleBuilderInitial<T, DateTime> ruleBuilder,Expression<Func<T,DateTime>> endDateExpression)
    {
      return ruleBuilder.LessThan(endDateExpression);
    }

    public static IRuleBuilderOptions<T, DateTime> DateRangeIncluding<T>(this IRuleBuilderOptions<T, DateTime> ruleBuilder, Expression<Func<T, DateTime>> endDateExpression)
    {
      return ruleBuilder.LessThanOrEqualTo(endDateExpression);
    }

    public static IRuleBuilderOptions<T, DateTime> DateRangeIncluding<T>(this IRuleBuilderInitial<T, DateTime> ruleBuilder, Expression<Func<T, DateTime>> endDateExpression)
    {
      return ruleBuilder.LessThanOrEqualTo(endDateExpression);
    }

    public static IRuleBuilderOptions<T, decimal> Positive<T>(this IRuleBuilderOptions<T, decimal> ruleBuilder)
    {
      return ruleBuilder.GreaterThan(0);
    }

    public static IRuleBuilderOptions<T, decimal> Positive<T>(this IRuleBuilderInitial<T, decimal> ruleBuilder)
    {
      return ruleBuilder.GreaterThan(0);
    }

    public static IRuleBuilderOptions<T, decimal> NotNegative<T>(this IRuleBuilderOptions<T, decimal> ruleBuilder)
    {
        return ruleBuilder.GreaterThanOrEqualTo(0);
    }

    public static IRuleBuilderOptions<T, decimal> NotNegative<T>(this IRuleBuilderInitial<T, decimal> ruleBuilder)
    {
        return ruleBuilder.GreaterThanOrEqualTo(0);
    }

    public static IRuleBuilderOptions<T, int> NotNegative<T>(this IRuleBuilderOptions<T, int> ruleBuilder)
    {
      return ruleBuilder.GreaterThanOrEqualTo(0);
    }

    public static IRuleBuilderOptions<T, int> NotNegative<T>(this IRuleBuilderInitial<T, int> ruleBuilder)
    {
      return ruleBuilder.GreaterThanOrEqualTo(0);
    }

    public static IRuleBuilderOptions<T, int> Positive<T>(this IRuleBuilderOptions<T, int> ruleBuilder)
    {
      return ruleBuilder.GreaterThan(0);
    }

    public static IRuleBuilderOptions<T, int> Positive<T>(this IRuleBuilderInitial<T, int> ruleBuilder)
    {
      return ruleBuilder.GreaterThan(0);
    }
    /*
    public static IRuleBuilderOptions<T, int> InEnum<T>(this IRuleBuilderOptions<T, int> ruleBuilder,Type enumType)
    {
        return ruleBuilder.SetValidator(new InEnumValidator(enumType, ""));
    }

    public static IRuleBuilderOptions<T, int> InEnum<T>(this IRuleBuilderInitial<T, int> ruleBuilder, Type enumType)
    {
        return ruleBuilder.SetValidator(new InEnumValidator(enumType,""));
    }
    */
    public static IRuleBuilderOptions<T, long> MustBeSelected<T>(this IRuleBuilderOptions<T, long> ruleBuilder)
    {
        return ruleBuilder.GreaterThan(0);
    }
    public static IRuleBuilderOptions<T, long> MustBeSelected<T>(this IRuleBuilderInitial<T, long> ruleBuilder)
    {
      return ruleBuilder.GreaterThan(0);
    }

    public static IRuleBuilderOptions<T, int> MustBeSelected<T>(this IRuleBuilderOptions<T, int> ruleBuilder)
    {
        return ruleBuilder.GreaterThan(0);
    }
    public static IRuleBuilderOptions<T, int> MustBeSelected<T>(this IRuleBuilderInitial<T, int> ruleBuilder)
    {
        return ruleBuilder.GreaterThan(0);
    }

    public static IRuleBuilderOptions<T, TProperty> OnFailure<T,TProperty>(this IRuleBuilderOptions<T, TProperty> ruleBuilder, Action action)
    {
        return ruleBuilder.Configure(pr => pr.OnFailure = o=>action());
    }

    public static IRuleBuilderInitial<T, TProperty> OnFailure<T,TProperty>(this IRuleBuilderInitial<T, TProperty> ruleBuilder, Action action)
    {
        return ruleBuilder.Configure(pr => pr.OnFailure = o=>action());
    }
    
    public static IRuleBuilderOptions<T, string> IsHour<T>(this IRuleBuilderOptions<T, string> ruleBuilder)
    {
      return ruleBuilder.Matches(@"([0-1][0-9]|2[0-3]):[0-5][0-9]");
    }
    public static IRuleBuilderOptions<T, string> IsHour<T>(this IRuleBuilderInitial<T, string> ruleBuilder)
    {
      return ruleBuilder.Matches(@"([0-1][0-9]|2[0-3]):[0-5][0-9]");
    }
  }
}
