using System;
using System.Collections.Generic;
using System.Reflection;
using GestionConcesionarios.Aplicacion.Core.Container;

namespace GestionConcesionarios.Infraestructura.Presentacion
{
  public class ApplicationBootStrapperBuilder : BootStrapperBuilder
  {
    #region Atributos
    private readonly List<Assembly> _applicationAssemblies = new List<Assembly>();
    private readonly List<Assembly> _domainModelAssemblies = new List<Assembly>();
    private Assembly _mappingsAssembly;
    private readonly List<Assembly> _mappingOverridesAssemblies = new List<Assembly>();
    #endregion

    #region Metodos Privados

    private static void Validate(ApplicationBootStrapperBuilder bootStrapperBuilder)
    {
      if (bootStrapperBuilder == null) throw new ArgumentNullException("bootStrapperBuilder");
      if (bootStrapperBuilder._mappingsAssembly == null && bootStrapperBuilder._domainModelAssemblies.Count == 0)
        throw new ArgumentException("A mappings assembly or domain model assemblies must be specified");
      if (bootStrapperBuilder._applicationNamespaces.Count == 0) throw new ArgumentException("No namespaces specified");
      if (bootStrapperBuilder._applicationAssemblies.Count == 0) throw new ArgumentException("No application assemblies specified");
    }

    #endregion

    #region Metodos Publicos
    
    public ApplicationBootStrapperBuilder AddDomainModelAssembly(Assembly assembly)
    {
      return (ApplicationBootStrapperBuilder)AddTolist(assembly, _domainModelAssemblies);
    }

    public ApplicationBootStrapperBuilder AddApplicationAssembly(Assembly assembly)
    {
      return (ApplicationBootStrapperBuilder)AddTolist(assembly, _applicationAssemblies);
    }

    public BootStrapperBuilder UseMappingsAssembly(Assembly mappingsAssembly)
    {
      if (mappingsAssembly == null) throw new ArgumentNullException("mappingsAssembly");

      _mappingsAssembly = mappingsAssembly;
      return this;
    }

    public BootStrapperBuilder AddMappingsOverridesAssembly(Assembly assembly)
    {
      return AddTolist(assembly, _mappingOverridesAssemblies);
    }

    #endregion
    
    #region Operadores

    public static implicit operator BootStrapper(ApplicationBootStrapperBuilder bootStrapperBuilder)
    {
      Validate(bootStrapperBuilder);

      var applicationContainer =
        ApplicationContainerBuilderHelper.BuildApplicationContainer(bootStrapperBuilder._mappingsAssembly,
                                                                    bootStrapperBuilder._applicationNamespaces.ToArray(),
                                                                    bootStrapperBuilder._applicationAssemblies.ToArray(),
                                                                    bootStrapperBuilder._domainModelAssemblies.ToArray());
      return new BootStrapper(applicationContainer);
    }

    #endregion
  }
}