﻿using System;
using System.Collections.Generic;
using System.Linq;
using Autofac;

namespace GestionConcesionarios.Infraestructura.Presentacion
{
  public abstract class BootStrapperBuilder
  {
    #region Atributos
    
    protected readonly List<string> _applicationNamespaces = new List<string>();
    protected Action<ContainerBuilder> _containerRegistrationAction = a => { };
    protected IEnumerable<Type> _defaultCommandTypesToLoad;

    #endregion

    #region Metodos Publicos
    
    public BootStrapperBuilder WithDomainPrefix(string prefix)
    {
      if (prefix == null) throw new ArgumentNullException("prefix");
      return this;
    }

    public BootStrapperBuilder WithContainerRegistrationAction(Action<ContainerBuilder> containerRegistrationAction)
    {
      if (containerRegistrationAction == null) throw new ArgumentNullException("containerRegistrationAction");

      _containerRegistrationAction = containerRegistrationAction;
      return this;
    }

    public BootStrapperBuilder AddNamespace(string @namespace)
    {
      return AddTolist(@namespace, _applicationNamespaces);
    }
    #endregion

    #region Metodos Protegidos
    protected BootStrapperBuilder AddTolist<T>(T element, ICollection<T> list) where T : class
    {
      if (element == null) throw new ArgumentNullException("element");
      if (list == null) throw new ArgumentNullException("list");

      if (!list.Contains(element)) list.Add(element);
      return this;
    }

    #endregion

  }
}