using System;
using System.Collections.Generic;
using System.Reflection;
using GestionConcesionarios.Presentacion.Infraestructura.Container;

namespace GestionConcesionarios.Infraestructura.Presentacion
{
  public class PresentationBootStrapperBuilder : BootStrapperBuilder
  {
    #region Atributos

    private readonly List<Assembly> _presentationAssemblies = new List<Assembly>();

    #endregion

    #region Private 

    private static void Validate(PresentationBootStrapperBuilder bootStrapperBuilder)
    {
      if (bootStrapperBuilder == null) throw new ArgumentNullException("bootStrapperBuilder");
      if (bootStrapperBuilder._applicationNamespaces.Count == 0) throw new ArgumentException("No namespaces specified");
      if (bootStrapperBuilder._presentationAssemblies.Count == 0) throw new ArgumentException("No presentation assemblies specified");
    }

    #endregion

    #region Metodos Publicos
    public PresentationBootStrapperBuilder AddPresentationAssembly(Assembly assembly)
    {
      return (PresentationBootStrapperBuilder)AddTolist(assembly, _presentationAssemblies);
    }
    #endregion

    #region Operators
    
    public static implicit operator BootStrapper(PresentationBootStrapperBuilder bootStrapperBuilder)
    {
      Validate(bootStrapperBuilder);
      var presentationContainerParams = new BuildPresentationContainerParams(bootStrapperBuilder._containerRegistrationAction,
                                                                             bootStrapperBuilder._presentationAssemblies.ToArray(),
                                                                             bootStrapperBuilder._applicationNamespaces.ToArray());

      var presentationContainer = PresentationContainerBuilderHelper.BuildPresentationContainer(presentationContainerParams);
      return new BootStrapper(presentationContainer);
    }

    #endregion
  }
}