﻿using System;
using System.Configuration;
using System.Linq.Expressions;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;
using Kent.Boogaart.Truss;

namespace GestionConcesionarios.Presentacion.Base
{
  /// <summary>
  /// Clase base para los formularios.
  /// </summary>
  public partial class BindableFormBase : DevExpress.XtraEditors.XtraForm, IView
  {
    #region Constructores

    public BindableFormBase()
    {
      BindingManager = new BindingManager();
      InitializeComponent();
      BoolToLayoutVisibilityConverter = new BoolToLayoutVisibilityValueConverter();
    }

    #endregion

    #region Propiedades

    protected BindingManager BindingManager { get; set; }

    public IValueConverter BoolToLayoutVisibilityConverter { get; set; }

    #endregion

    #region Metodos Protegidos

    protected void Bind<TTarget, TSource>(TTarget target, Expression<Func<TTarget, object>> targetExpression, TSource source, Expression<Func<TSource, object>> sourceExpression, IValueConverter converter = null)
    {
      var typedBinding = new TypedBinding<TTarget, TSource>(target, targetExpression, source, sourceExpression);
      if (converter != null) typedBinding.Converter = converter;
      BindingManager.Bindings.Add(typedBinding);
    }

    protected void ConfigurarAyuda(HelpProvider helpProvider)
    {
      if (ConfigurationSettings.AppSettings != null)
        helpProvider.HelpNamespace = Application.StartupPath + ConfigurationSettings.AppSettings["RutaDesdeStartupPathDelSistemaDeAyuda"];
    }
    #endregion
  }
}