﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using GestionConcesionarios.Presentacion.Infraestructura;

namespace GestionConcesionarios.Presentacion.Base
{
  #region Interfaces

  public interface IPersistentModel
  {
    bool HasBeenPersisted();
  }

  public interface IModel : IPropertyChangedTracker
  {
    object GetTarget();
  }

  #endregion

  public abstract class ModelBase : IModel, INotifyPropertyChanged, IDataErrorInfo, IDisposable
  {
    #region Atributos

    private readonly IPropertyChangedTracker _propertyChangedTracker = new PropertyChangedTracker();

    #endregion

    #region Events
    public virtual event PropertyChangedEventHandler PropertyChanged;
    #endregion

    #region Properties

    public virtual IEnumerable<string> ChangedProperties
    {
      get { return _propertyChangedTracker.ChangedProperties; }
    }

    public virtual bool IsDirty
    {
      get { return _propertyChangedTracker.IsDirty; }
    }

    public virtual string this[string columnName]
    {
      get { return string.Empty; }
    }

    public virtual string Error
    {
      get { return string.Empty; }
    }

    public bool IsValid { get; set; }

    #endregion

    #region Metodos Publicos

    public virtual void ClearChangedProperties()
    {
      _propertyChangedTracker.ClearChangedProperties();
    }

    public virtual void TrackPropertyChange(string propertyName)
    {
      if (propertyName == null) throw new ArgumentNullException("propertyName");

      _propertyChangedTracker.TrackPropertyChange(propertyName);
    }

    public void OnPropertyChanged(string propertyName)
    {
      if (PropertyChanged != null)
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    public virtual object GetTarget()
    {
      return this;
    }

    protected virtual void DoDispose()
    {

    }

    public void Dispose()
    {
      PropertyChanged = null;
    }

    #endregion
  }
}
