﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;

namespace GestionConcesionarios.Presentacion.Base
{
  public abstract class PresenterBase : INotifyPropertyChanged, IPresenter, IDisposable
  {
    #region Eventos

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    #region Metodos Publicos

    public virtual void Dispose()
    {
      PropertyChanged = null;
    }

    public void NotifyPropertyChanged<T>(Expression<Func<T, object>> exp) where T : PresenterBase
    {
      if (exp.Body is MemberExpression)
      {
        OnPropertyChanged(((MemberExpression)exp.Body).Member.Name);
      }
      else if (exp.Body is UnaryExpression)
      {
        OnPropertyChanged(((MemberExpression)((UnaryExpression)exp.Body).Operand).Member.Name);
      }
      else
        throw new ArgumentException(string.Format("Expressions of {0} type are not supported.",
                                                  exp.Body.GetType().Name));
    }

    #endregion

    #region Metodos Privados

    private void OnPropertyChanged(string propertyName)
    {
      if (PropertyChanged != null)
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    #endregion
  }
}
