﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulosAlmacen;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Almacen
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de las entradas al almacén
  /// </summary>
  public class EntradaAlAlmacenViewModel : ModelBase
  {
    #region Modelos Auxiliares
    public class LineaEntradaAlAlmacenViewModel : ModelBase, IPersistentModel
    {
      public Guid IdArticuloAlmacen { get; set; }
      public Guid IdArticulo { get; set; }
      public string Descripcion { get; set; }
      public decimal CantidadEntrada { get; set; }
      public string Ubicacion { get; set; }
      public decimal? Precio { get; set; }
      public decimal? StockMinimo { get; set; }
      public decimal? StockMaximo { get; set; }
      public decimal? StockActual { get; set; }
      public bool Borrar { get; set; }

      public bool HasBeenPersisted()
      {
        return !IdArticuloAlmacen.Equals(Guid.Empty);
      }
    }

    #endregion

    #region Atributos

    private readonly IGestionArticuloAlmacenService _gestionArticuloAlmacenService;
    private BindingList<LineaEntradaAlAlmacenViewModel> _lineasDeLaEntradaAlAlmacen = new BindingList<LineaEntradaAlAlmacenViewModel>();

    #endregion

    #region Constructores
    public EntradaAlAlmacenViewModel(IGestionArticuloAlmacenService gestionArticuloAlmacenService)
    {
      _gestionArticuloAlmacenService = gestionArticuloAlmacenService;
    }

    #endregion

    #region Propiedades

    public virtual Guid? IdEmpleado { get; set; }
    public virtual DateTime? FechaEntrada { get; set; }

    public BindingList<LineaEntradaAlAlmacenViewModel> LineasDeLaEntradaAlAlmacen { get { return _lineasDeLaEntradaAlAlmacen; } }

    #endregion

    #region Métodos Públicos

    public List<ArticuloAlmacen> ObtenerLineasDeLaEntradaAlAlmacen(IEnumerable<LineaEntradaAlAlmacenViewModel> lineasDeLaEntradaAlAlmacenViewModel)
    {
      return
        lineasDeLaEntradaAlAlmacenViewModel.Select(
          lineaEntradaAlAlmacenViewModel => _gestionArticuloAlmacenService.BuscarArticuloAlmacenPorId(lineaEntradaAlAlmacenViewModel.IdArticuloAlmacen).First()).ToList();
    }

    public void SetLineasDeLaEntradaAlAlmacen(BindingList<LineaEntradaAlAlmacenViewModel> lineasDeLaEntradaAlAlmacen)
    {
      _lineasDeLaEntradaAlAlmacen = lineasDeLaEntradaAlAlmacen;
    }

    public void EliminarLineaEntradaAlAlmacen(LineaEntradaAlAlmacenViewModel lineaEntradaAlAlmacenViewModel)
    {
      lineaEntradaAlAlmacenViewModel.Borrar = true;
    }

    public void AñadirLineaEntradaAlAlmacen(BuscarArticuloListItemModel buscarArticuloListItemModel)
    {
      var articuloAlmacen =
        _gestionArticuloAlmacenService.BuscarArticuloAlmacenPorIdArticulo(buscarArticuloListItemModel.ArticuloId).FirstOrDefault() ??
        new ArticuloAlmacen();

      var nuevaLineaEntradaAlAlmacen = new LineaEntradaAlAlmacenViewModel
                            {
                              IdArticuloAlmacen = articuloAlmacen.Id,
                              IdArticulo = buscarArticuloListItemModel.ArticuloId,
                              Descripcion = buscarArticuloListItemModel.Descripcion,
                              CantidadEntrada = 0,
                              Ubicacion = articuloAlmacen.Ubicacion,
                              Precio = buscarArticuloListItemModel.CosteActual,
                              StockMinimo = articuloAlmacen.StockMinimo,
                              StockMaximo = articuloAlmacen.StockMaximo,
                              StockActual = articuloAlmacen.StockActual
                            };

      _lineasDeLaEntradaAlAlmacen.Add(nuevaLineaEntradaAlAlmacen);
    }
    
    #endregion
  }
}
