﻿using System;
using System.ComponentModel;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Almacen
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de entradas al almacén,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class EntradaAlAlmacenViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<EntradaAlAlmacenViewModel> _crearViewModel;

    #endregion

    #region Constructors

    public EntradaAlAlmacenViewModelFactory(Func<EntradaAlAlmacenViewModel> crearViewModel)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo EntradaAlAlmacenViewModel vacío</returns>
    public EntradaAlAlmacenViewModel CrearEntradaAlAlmacenViewModel()
    {
      var entradaAlmacenViewModel = _crearViewModel();

      entradaAlmacenViewModel.IdEmpleado = Guid.Empty;
      entradaAlmacenViewModel.FechaEntrada = null;

      entradaAlmacenViewModel.SetLineasDeLaEntradaAlAlmacen(new BindingList<EntradaAlAlmacenViewModel.LineaEntradaAlAlmacenViewModel>());

      return entradaAlmacenViewModel;
    }

    #endregion
  }
}
