﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Almacen
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista de las entradas al almacén,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class EntradaAlAlmacenViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<Guid>> _empleados = new BindingList<LookUpItem<Guid>>();

    #endregion

    #region Constructores

    #endregion

    #region Propiedades
    
    public IEnumerable<LookUpItem<Guid>> Empleados { get { return _empleados; } }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece los empleados en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="empleados">Códigos postales a poner en correspondiente lista LookUp.</param>
    public void SetEmpleados(IList<Empleado> empleados)
    {
      _empleados =
        empleados.Select(empleado => new LookUpItem<Guid> { Codigo = empleado.Id, Descripcion = empleado.Nombre }).ToList();
    }

    #endregion
  }
}
