﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Almacen
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista de las entradas al almacén,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class EntradaAlAlmacenViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<EntradaAlAlmacenViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionEmpleadoService _gestionEmpleadoService;

    #endregion

    #region Constructores

    public EntradaAlAlmacenViewModelReadOnlyFactory(Func<EntradaAlAlmacenViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionEmpleadoService gestionEmpleadoService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionEmpleadoService = gestionEmpleadoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo PresupuestoViewModelReadOnly vacío</returns>
    public EntradaAlAlmacenViewModelReadOnly CreateEntradaAlAlmacenViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetEmpleados(_gestionEmpleadoService.BuscarTodosLosEmpleados());
      return model;
    }

    #endregion
  }
}
