﻿using System;
using FluentValidation;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Articulos
{
  /// <summary>
  /// Para definir un conjunto de reglas de validación para un objeto en particular, hay que 
  /// crear una clase que herede de AbstractValidator <T/>, donde T es el tipo de
  /// Clase que se desea validar.
  /// Para mas información ir a http://fluentvalidation.codeplex.com/documentation
  /// </summary>
  public class ArticuloViewModelValidator : AbstractValidator<ArticuloViewModel>
  {
    /// <summary>
    /// Inicializa una nueva instancia de la clase<see cref="ArticuloViewModelValidator" />.
    /// Configurando las reglas de validación a cumplir.
    /// </summary>
    public ArticuloViewModelValidator()
    {
      RuleFor(t => t.Nombre).NotEmpty();
      RuleFor(t => t.Descripcion).NotEmpty();
      RuleFor(t => t.IdProveedor).NotNull().NotEmpty().NotEqual(Guid.Empty).WithMessage("Es necesario rellenar el campo");
      RuleFor(t => t.IdCategoriaArticulo).NotNull().NotEmpty();
      RuleFor(t => t.CosteActualPorUnidad).GreaterThan(0);
    }
  }
}
