﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.Proveedores;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Articulos
{
  public partial class FormArticulo : BindableFormBase
  {
    #region Atributos

    private readonly ArticuloPresenter _articuloPresenter;

    #endregion

    #region Constructores

    public FormArticulo(ArticuloPresenter articuloPresenter)
    {
      InitializeComponent();
      _articuloPresenter = articuloPresenter;
      ConfigurarAyuda(helpProviderFormArticulo);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaArticulosBuscados()
    {
      dockPanelBuscarArticulo.Show();
    }

    private void OcultarListaArticulosBuscados()
    {
      dockPanelBuscarArticulo.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void FormArticulo_Load(object sender, EventArgs e)
    {
      if (_articuloPresenter != null)
      {
        _articuloPresenter.SetModelReadOnly();
        _articuloPresenter.NuevoArticulo();
        _articuloPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _articuloPresenter;
      }
      helpProviderFormArticulo.SetShowHelp(this, true);
    }
    
    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormArticulo.HelpNamespace);
    }
    
    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _articuloPresenter.NuevoArticulo();
      textEditNombre.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _articuloPresenter.GuardarArticulo();
      
      if (_articuloPresenter.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion)
        Close();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _articuloPresenter.BuscarArticulo();
      MostrarListaArticulosBuscados();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _articuloPresenter.NuevaBusquedaArticulo();
      textEditNombre.Focus();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _articuloPresenter.EliminarArticulo();
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {
      gridViewListaArticulos_DoubleClick(sender, e);
    }

    private void gridViewListaArticulos_DoubleClick(object sender, System.EventArgs e)
    {
      var articuloSeleccionado = gridViewListaArticulos.GetFocusedRow() as BuscarArticuloListItemModel;
      if (articuloSeleccionado != null) _articuloPresenter.SetArticulo(articuloSeleccionado.ArticuloId);
      OcultarListaArticulosBuscados();
      
      if (_articuloPresenter.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion)
        Close();
    }

    private void simpleButtonBuscarProveedor_Click(object sender, EventArgs e)
    {
      _articuloPresenter.MostrarFormProveedorConNuevoProveedor();
    }
    #endregion
  }
}