﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista de búsquedas de artículo,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class BuscarArticuloViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<Guid>> _proveedores = new BindingList<LookUpItem<Guid>>();
    private IList<LookUpItem<string>> _categoriasArticulo = new BindingList<LookUpItem<string>>();

    #endregion
    
    #region Propiedades

    public IEnumerable<LookUpItem<Guid>> Proveedores { get { return _proveedores; } }
    public IEnumerable<LookUpItem<string>> CategoriasArticulo { get { return _categoriasArticulo; } }
    
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece los proveedores en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="proveedores">Proveedores a poner en correspondiente lista LookUp.</param>
    public void SetProveedores(IList<Proveedor> proveedores)
    {
      _proveedores =
        proveedores.Select(proveedor => new LookUpItem<Guid> { Codigo = proveedor.Id, Descripcion = proveedor.RazonSocial }).ToList();
    }

    /// <summary>
    /// Establece las categorias de artículo en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="categoriasArticulo">Categorías de articulo a poner en correspondiente lista LookUp.</param>
    public void SetCategoriasArticulo(IList<CategoriaArticulo> categoriasArticulo)
    {
      _categoriasArticulo =
        categoriasArticulo.Select(categoriaArticulo => new LookUpItem<string> { Codigo = categoriaArticulo.Id, Descripcion = categoriaArticulo.Descripcion }).ToList();
    }
    
    #endregion
  }
}
