﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionCategoriasArticulos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista de búsquedas de artículo,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarArticuloViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<BuscarArticuloViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionCategoriasArticuloService _gestionCategoriasArticuloService;
    private readonly IGestionProveedorService _gestionProveedorService;

    #endregion

    #region Constructores

    public BuscarArticuloViewModelReadOnlyFactory(Func<BuscarArticuloViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionCategoriasArticuloService gestionCategoriasArticuloService, 
                                           IGestionProveedorService gestionProveedorService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionCategoriasArticuloService = gestionCategoriasArticuloService;
      _gestionProveedorService = gestionProveedorService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarArticuloViewModelReadOnly vacío</returns>
    public BuscarArticuloViewModelReadOnly CreateArticuloViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetCategoriasArticulo(_gestionCategoriasArticuloService.BuscarTodasLasCategoriasArticulo());
      model.SetProveedores(_gestionProveedorService.BuscarTodosLosProveedores());
      return model;
    }

    #endregion
  }
}
