﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarCliente)]
  public class BuscarClientePresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarClienteViewModelFactory _buscarClienteViewModelFactory;

    #endregion

    #region Constructor

    public BuscarClientePresenter(IMediatorManager mediatorManager, BuscarClienteViewModelFactory buscarClienteViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarClienteViewModelFactory = buscarClienteViewModelFactory;
      Model = _buscarClienteViewModelFactory.CrearBuscarClienteViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarCliente];
    }

    #endregion

    #region Propiedades

    public BuscarClienteViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.Nombre).Do(x =>
      {
        Buscar(Model.Nombre, string.Empty, string.Empty);
        NotifyPropertyChanged<BuscarClientePresenter>(t => t.Model);
      });
    }

    /// <summary>
    /// Notifica que se ha seleccinado el cliente con Id=clienteId
    /// </summary>
    /// <param name="buscarClienteListItemModel">Entidad modelo de vista seleccionada</param>
    public void ClienteSeleccionado(BuscarClienteListItemModel buscarClienteListItemModel)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarClienteClienteSeleccionado, buscarClienteListItemModel);
    }

    /// <summary>
    /// Muestra la vista de clientes para añadir un nuevo cliente
    /// </summary>
    public void MostrarFormCliente()
    {
      //AlGuardarElClienteOSeleccionarloDesdeFormClientesAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormClientes));
    }

    /// <summary>
    /// Ejecuta la búsqueda de clientes en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="nombre">Nombre a buscar.</param>
    /// <param name="primerApellido">Primer Apellido a buscar.</param>
    /// <param name="segundoApellido">Segundo Apellido a buscar.</param>
    public void Buscar(string nombre, string primerApellido, string segundoApellido)
    {
      Model = _buscarClienteViewModelFactory.CrearBuscarClienteViewModelFactory(nombre, primerApellido, segundoApellido);
      NotifyPropertyChanged<BuscarClientePresenter>(t => t.Model);
    }

    #endregion
    /*
    #region Métodos Privados

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos de la orden de reparación
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticuloAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion);
    }

    #endregion
    */
  }
}
