﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.ManosDeObra;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarManoDeObra)]
  public class BuscarManoDeObraPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarManoDeObraViewModelFactory _buscarManoDeObraViewModelFactory;

    #endregion

    #region Constructor

    public BuscarManoDeObraPresenter(IMediatorManager mediatorManager, BuscarManoDeObraViewModelFactory buscarManoDeObraViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarManoDeObraViewModelFactory = buscarManoDeObraViewModelFactory;
      Model = _buscarManoDeObraViewModelFactory.CrearBuscarManoDeObraViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarManoDeObra];
    }

    #endregion

    #region Propiedades

    public BuscarManoDeObraViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos
    /*
    /// <summary>
    /// Notifica que se han seleccionado manos de obra
    /// </summary>
    public void AsignarManosDeObra()
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarManoDeObraManoDeObraSeleccionada, Model);
    }
    */
    /// <summary>
    /// Notifica que se ha seleccionado una mano de obra
    /// </summary>
    /// <param name="manoDeObraSeleccionada">La mano de obra seleccionada.</param>
    public void AsignarManoDeObra(BuscarManoDeObraListItemModel manoDeObraSeleccionada)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarManoDeObraManoDeObraSeleccionada, manoDeObraSeleccionada);
    }

    /// <summary>
    /// Ejecuta la búsqueda de manos de obra en función de los valores del modelo de vista
    /// </summary>
    public void Buscar()
    {
      Buscar(Model.Descripcion);
    }

    /// <summary>
    /// Ejecuta la búsqueda de manos de obra en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="descripcion">Descripcion de la mano de obra a buscar.</param>
    public void Buscar(string descripcion)
    {
      Model = _buscarManoDeObraViewModelFactory.CrearBuscarManoDeObraViewModelFactory(descripcion);
      NotifyPropertyChanged<BuscarManoDeObraPresenter>(t => t.Model);
    }

    /// <summary>
    /// Muestra la vista de manos de obra para añadir una nueva mano de obra
    /// </summary>
    public void MostrarFormManoDeObra()
    {
      //AlGuardarElClienteOSeleccionarloDesdeFormClientesAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormManoDeObra));
    }
    #endregion
    /*
    #region Métodos Privados

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos de la orden de reparación
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticuloAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion);
    }

    #endregion
    */
  }
}
