﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarOrdenDeReparacion
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarOrdenDeReparacion)]
  public class BuscarOrdenDeReparacionPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarOrdenDeReparacionViewModelFactory _buscarOrdenDeReparacionViewModelFactory;

    #endregion

    #region Constructor

    public BuscarOrdenDeReparacionPresenter(IMediatorManager mediatorManager, BuscarOrdenDeReparacionViewModelFactory buscarOrdenDeReparacionViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarOrdenDeReparacionViewModelFactory = buscarOrdenDeReparacionViewModelFactory;
      Model = _buscarOrdenDeReparacionViewModelFactory.CrearBuscarOrdenDeReparacionViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarOrdenDeReparacion];
    }

    #endregion

    #region Propiedades

    public BuscarOrdenDeReparacionViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado la orden de reparación con Id=ordenReparacionId
    /// </summary>
    /// <param name="ordenReparacionId">Id de la entidad seleccionada</param>
    public void OrdenDeReparacionSeleccionada(Guid ordenReparacionId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarOrdenDeReparacionOrdenDeReparacionSeleccionada, ordenReparacionId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de órdenes de reparación en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula del vehículo en la orden de reparación a buscar.</param>
    /// <param name="numeroOrden">Número de orden de la orden a buscar.</param>
    /// <param name="nombreCliente">Nombre del cliente en la orden a buscar.</param>
    public void Buscar(string matricula, string numeroOrden, string nombreCliente)
    {
      var criterioBusqueda = new CriteriosBusquedaOrdenDeReparacion
      {
        Matricula = matricula,
        NumeroOrden = numeroOrden,
        NombreCliente = nombreCliente
      };

      Model = _buscarOrdenDeReparacionViewModelFactory.CrearBuscarOrdenDeReparacionViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarOrdenDeReparacionPresenter>(t => t.Model);
    }
    #endregion
  }
}
