﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarPaqueteDeServicio)]
  public class BuscarPaqueteDeServicioPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarPaqueteDeServicioViewModelFactory _buscarPaqueteDeServicioViewModelFactory;

    #endregion

    #region Constructor

    public BuscarPaqueteDeServicioPresenter(IMediatorManager mediatorManager, BuscarPaqueteDeServicioViewModelFactory buscarPaqueteDeServicioViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarPaqueteDeServicioViewModelFactory = buscarPaqueteDeServicioViewModelFactory;
      Model = _buscarPaqueteDeServicioViewModelFactory.CrearBuscarPaqueteDeServicioViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarPaqueteDeServicio];
    }

    #endregion

    #region Propiedades

    public BuscarPaqueteDeServicioViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se han seleccionado paquetes de servicio
    /// </summary>
    public void AsignarPaquetesDeServicio()
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarPaqueteDeServicioPaqueteDeServicioSeleccionado, Model);
    }

    /// <summary>
    /// Ejecuta la búsqueda de paquetes de servicio en función de los valores del modelo de vista
    /// </summary>
    public void Buscar()
    {
      Buscar(Model.Descripcion);
    }

    /// <summary>
    /// Ejecuta la búsqueda de paquetes de servicio en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="descripcion">Descripcion del paquete de servicio a buscar.</param>
    public void Buscar(string descripcion)
    {
      Model = _buscarPaqueteDeServicioViewModelFactory.CrearBuscarPaqueteDeServicioViewModelFactory(descripcion);
      NotifyPropertyChanged<BuscarPaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Muestra la vista de paquetes de servicio para añadir un nuevo paquete de servicio
    /// </summary>
    public void MostrarFormPaqueteDeServicio()
    {
      //AlGuardarElClienteOSeleccionarloDesdeFormClientesAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormPaqueteDeServicio));
    }
    #endregion
    /*
    #region Métodos Privados

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos de la orden de reparación
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticuloAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion);
    }

    #endregion
    */
  }
}
