﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarPresupuesto
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarPresupuesto)]
  public class BuscarPresupuestoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarPresupuestoViewModelFactory _buscarPresupuestoViewModelFactory;

    #endregion

    #region Constructor

    public BuscarPresupuestoPresenter(IMediatorManager mediatorManager, BuscarPresupuestoViewModelFactory buscarPresupuestoViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarPresupuestoViewModelFactory = buscarPresupuestoViewModelFactory;
      Model = _buscarPresupuestoViewModelFactory.CrearBuscarPresupuestoViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarPresupuesto];
    }

    #endregion

    #region Propiedades

    public BuscarPresupuestoViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado el vehículo de venta con Id=vehiculoVentaId
    /// </summary>
    /// <param name="vehiculoVentaId">Id de la entidad seleccionada</param>
    public void PresupuestoSeleccionado(Guid vehiculoVentaId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarPresupuestoPresupuestoSeleccionado, vehiculoVentaId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de presupuestos en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="numeroPresupuesto">Número de presupuesto a buscar.</param>
    /// <param name="fecha">Fecha del presupuesto a partir de la cual a buscar.</param>
    /// <param name="cliente">Cliente del presupuesto a buscar.</param>
    public void Buscar(string numeroPresupuesto, DateTime? fecha, Guid? cliente)
    {
      var criterioBusqueda = new CriteriosBusquedaPresupuesto
      {
        Numero = numeroPresupuesto,
        Fecha = fecha,
        Cliente = cliente
      };

      Model = _buscarPresupuestoViewModelFactory.CrearBuscarPresupuestoViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarPresupuestoPresenter>(t => t.Model);
    }
    #endregion
  }
}
