﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarProveedor
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarProveedor)]
  public class BuscarProveedorPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarProveedorViewModelFactory _buscarProveedorViewModelFactory;

    #endregion

    #region Constructor

    public BuscarProveedorPresenter(IMediatorManager mediatorManager, BuscarProveedorViewModelFactory buscarProveedorViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarProveedorViewModelFactory = buscarProveedorViewModelFactory;
      Model = _buscarProveedorViewModelFactory.CrearBuscarProveedorViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarProveedor];
    }

    #endregion

    #region Propiedades

    public BuscarProveedorViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado el proveedor con Id=proveedorId
    /// </summary>
    /// <param name="proveedorId">Id de la entidad seleccionada</param>
    public void ProveedorSeleccionado(Guid proveedorId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarProveedorProveedorSeleccionado, proveedorId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de proveedores en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="codigo">Codigo a buscar.</param>
    /// <param name="razonSocial">Razon social a buscar.</param>
    /// <param name="cifDni">Cif Dni a buscar.</param>
    public void Buscar(string codigo, string razonSocial, string cifDni)
    {
      var criterioBusqueda = new CriteriosBusquedaProveedor
      {
        CifDni = cifDni,
        Codigo = codigo,
        RazonSocial = razonSocial
      };

      Model = _buscarProveedorViewModelFactory.CrearBuscarProveedorViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarProveedorPresenter>(t => t.Model);
    }
    #endregion
  }
}
