﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarRecepcionVehiculo
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarRecepcionVehiculo)]
  public class BuscarRecepcionVehiculoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarRecepcionVehiculoViewModelFactory _buscarRecepcionVehiculoViewModelFactory;

    #endregion

    #region Constructor

    public BuscarRecepcionVehiculoPresenter(IMediatorManager mediatorManager, BuscarRecepcionVehiculoViewModelFactory buscarRecepcionVehiculoViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarRecepcionVehiculoViewModelFactory = buscarRecepcionVehiculoViewModelFactory;
      Model = _buscarRecepcionVehiculoViewModelFactory.CrearBuscarRecepcionVehiculoViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarRecepcionVehiculo];
    }

    #endregion

    #region Propiedades

    public BuscarRecepcionVehiculoViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado la recepción vehículo con Id=recepcionVehiculoId
    /// </summary>
    /// <param name="recepcionVehiculoId">Id de la entidad seleccionada</param>
    public void RecepcionVehiculoSeleccionada(Guid recepcionVehiculoId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarRecepcionVehiculoRecepcionVehiculoSeleccionada, recepcionVehiculoId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de recepciones de vehículos en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula del vehículo en la recepción a buscar.</param>
    /// <param name="nifCif">Nif/Cif del cliente a buscar.</param>
    /// <param name="nombreCliente">Nombre del cliente a buscar.</param>
    public void Buscar(string matricula, string nifCif, string nombreCliente)
    {
      var criterioBusqueda = new CriteriosBusquedaRecepcionVehiculo
      {
        Matricula = matricula,
        NifCif = nifCif,
        NombreCliente = nombreCliente
      };

      Model = _buscarRecepcionVehiculoViewModelFactory.CrearBuscarRecepcionVehiculoViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarRecepcionVehiculoPresenter>(t => t.Model);
    }
    #endregion
  }
}
