﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionRecepciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarRecepcionVehiculo
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de vehículos,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarRecepcionVehiculoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionRecepcionVehiculoService _gestionRecepcionVehiculoService;

    #endregion

    #region Constructor

    public BuscarRecepcionVehiculoViewModelFactory(IGestionRecepcionVehiculoService gestionRecepcionVehiculoService)
    {
      _gestionRecepcionVehiculoService = gestionRecepcionVehiculoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarRecepcionVehiculoViewModel vacío</returns>
    public BuscarRecepcionVehiculoViewModel CrearBuscarRecepcionVehiculoViewModelFactory()
    {
      return new BuscarRecepcionVehiculoViewModel { ListaBuscarRecepcionVehiculos = new BindingList<BuscarRecepcionVehiculoListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaRecepcionVehiculo">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarRecepcionVehiculoViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarRecepcionVehiculoViewModel CrearBuscarRecepcionVehiculoViewModelFactory(CriteriosBusquedaRecepcionVehiculo criteriosBusquedaRecepcionVehiculo)
    {
      List<Recepcion> lista;
      if (criteriosBusquedaRecepcionVehiculo.HayMatricula)
        lista = _gestionRecepcionVehiculoService.BuscarRecepcionVehiculoPorMatricula(criteriosBusquedaRecepcionVehiculo.Matricula);
      else if (criteriosBusquedaRecepcionVehiculo.HayNifCif)
        lista = _gestionRecepcionVehiculoService.BuscarRecepcionVehiculoPorNifCifCliente(criteriosBusquedaRecepcionVehiculo.NifCif);
      else
      {
        lista = _gestionRecepcionVehiculoService.BuscarRecepcionVehiculoPorNombreCliente(criteriosBusquedaRecepcionVehiculo.NombreCliente);
      }
      var buscarRecepcionVehiculoViewModel = new BuscarRecepcionVehiculoViewModel { ListaBuscarRecepcionVehiculos = CrearListaBuscarRecepcionVehiculos(lista) };

      return buscarRecepcionVehiculoViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarRecepcionVehiculoListItemModel> CrearListaBuscarRecepcionVehiculos(IEnumerable<Recepcion> listaRecepcionVehiculos)
    {
      var listaBuscarRecepcionVehiculos = new BindingList<BuscarRecepcionVehiculoListItemModel>();

      foreach (var recepcionVehiculoListItemModel in listaRecepcionVehiculos.Select(recepcionVehiculo => new BuscarRecepcionVehiculoListItemModel
      {
        RecepcionVehiculoId = recepcionVehiculo.Id,
        IdCliente = recepcionVehiculo.IdCliente,
        Cliente = recepcionVehiculo.Cliente.Nombre,
        Matricula = recepcionVehiculo.Vehiculo.Matricula,
        Marca = recepcionVehiculo.Vehiculo.Marca,
        Modelo = recepcionVehiculo.Vehiculo.Modelo,
        Color = recepcionVehiculo.Vehiculo.Color,
        Fecha = recepcionVehiculo.FechaHoraRecepcion
      }))
      {
        listaBuscarRecepcionVehiculos.Add(recepcionVehiculoListItemModel);
      }
      return listaBuscarRecepcionVehiculos;
    }

    #endregion
  }
}
