﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarReparaciones
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarReparaciones)]
  public class BuscarReparacionPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarReparacionViewModelFactory _buscarReparacionViewModelFactory;

    #endregion

    #region Constructor

    public BuscarReparacionPresenter(IMediatorManager mediatorManager, BuscarReparacionViewModelFactory buscarReparacionViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarReparacionViewModelFactory = buscarReparacionViewModelFactory;
      Model = _buscarReparacionViewModelFactory.CrearBuscarReparacionViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarReparaciones];
    }

    #endregion

    #region Propiedades

    public BuscarReparacionViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado la orden de reparación con Id=ordenReparacionId
    /// </summary>
    /// <param name="ordenReparacionId">Id de la entidad seleccionada</param>
    public void OrdenDeReparacionSeleccionada(Guid ordenReparacionId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarReparacionesOrdenDeReparacionSeleccionada, ordenReparacionId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de reparaciones en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula del vehículo en la orden de reparación a buscar.</param>
    /// <param name="idOrdenDeReparacion">Id de la orden a buscar.</param>
    /// <param name="idEmpleado">Empleado asignado en la orden a buscar.</param>
    public void Buscar(string matricula, Guid idOrdenDeReparacion, Guid? idEmpleado)
    {
      var criterioBusqueda = new CriteriosBusquedaReparacion
      {
        Matricula = matricula,
        IdOrdenDeReparacion = idOrdenDeReparacion,
        IdEmpleado = idEmpleado
      };

      Model = _buscarReparacionViewModelFactory.CrearBuscarReparacionViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarReparacionPresenter>(t => t.Model);
    }
    #endregion
  }
}
