﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculo
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarVehiculo)]
  public class BuscarVehiculoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarVehiculoViewModelFactory _buscarVehiculoViewModelFactory;

    #endregion

    #region Constructor

    public BuscarVehiculoPresenter(IMediatorManager mediatorManager, BuscarVehiculoViewModelFactory buscarVehiculoViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarVehiculoViewModelFactory = buscarVehiculoViewModelFactory;
      Model = _buscarVehiculoViewModelFactory.CrearBuscarVehiculoViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarVehiculo];
    }

    #endregion

    #region Propiedades

    public BuscarVehiculoViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado el vehículo con Id=vehiculoId
    /// </summary>
    /// <param name="vehiculoId">Id de la entidad seleccionada</param>
    public void VehiculoSeleccionado(Guid vehiculoId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarVehiculoVehiculoSeleccionado, vehiculoId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de vehículos en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula a buscar.</param>
    /// <param name="marca">Marca a buscar.</param>
    /// <param name="modelo">Modelo a buscar.</param>
    public void Buscar(string matricula, string marca, string modelo)
    {
      var criterioBusqueda = new CriteriosBusquedaVehiculo
      {
        Matricula = matricula,
        Marca = marca,
        Modelo = modelo
      };

      Model = _buscarVehiculoViewModelFactory.CrearBuscarVehiculoViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarVehiculoPresenter>(t => t.Model);
    }
    #endregion
  }
}
