﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculoVenta
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarVehiculoVenta)]
  public class BuscarVehiculoVentaPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarVehiculoVentaViewModelFactory _buscarVehiculoVentaViewModelFactory;

    #endregion

    #region Constructor

    public BuscarVehiculoVentaPresenter(IMediatorManager mediatorManager, BuscarVehiculoVentaViewModelFactory buscarVehiculoVentaViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarVehiculoVentaViewModelFactory = buscarVehiculoVentaViewModelFactory;
      Model = _buscarVehiculoVentaViewModelFactory.CrearBuscarVehiculoVentaViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarVehiculoVenta];
    }

    #endregion

    #region Propiedades

    public BuscarVehiculoVentaViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado el vehículo de venta con Id=vehiculoVentaId
    /// </summary>
    /// <param name="vehiculoVentaId">Id de la entidad seleccionada</param>
    public void VehiculoVentaSeleccionado(Guid vehiculoVentaId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarVehiculoVentaVehiculoVentaSeleccionado, vehiculoVentaId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de vehículos de venta en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula a buscar.</param>
    /// <param name="marca">Marca a buscar.</param>
    /// <param name="modelo">Modelo a buscar.</param>
    public void Buscar(string matricula, string marca, string modelo)
    {
      var criterioBusqueda = new CriteriosBusquedaVehiculoVenta
      {
        Matricula = matricula,
        Marca = marca,
        Modelo = modelo
      };

      Model = _buscarVehiculoVentaViewModelFactory.CrearBuscarVehiculoVentaViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarVehiculoVentaPresenter>(t => t.Model);
    }
    #endregion
  }
}
