﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVenta
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarFactura)]
  public class BuscarFacturaPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarFacturaViewModelFactory _buscarFacturaViewModelFactory;

    #endregion

    #region Constructor

    public BuscarFacturaPresenter(IMediatorManager mediatorManager, BuscarFacturaViewModelFactory buscarFacturaViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarFacturaViewModelFactory = buscarFacturaViewModelFactory;
      Model = _buscarFacturaViewModelFactory.CrearBuscarFacturaViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarFactura];
    }

    #endregion

    #region Propiedades

    public BuscarFacturaViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado la factura con Id=facturaId
    /// </summary>
    /// <param name="facturaId">Id de la entidad seleccionada</param>
    public void FacturaSeleccionado(Guid facturaId)
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarFacturaFacturaSeleccionada, facturaId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de facturas en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="numeroFactura">Número de factura a buscar.</param>
    /// <param name="fecha">Fecha de la factura a partir de la cual a buscar.</param>
    /// <param name="cliente">Cliente de la factura a buscar.</param>
    public void Buscar(string numeroFactura, DateTime? fecha, Guid? cliente)
    {
      var criterioBusqueda = new CriteriosBusquedaFactura
      {
        Numero = numeroFactura,
        Fecha = fecha,
        Cliente = cliente
      };

      Model = _buscarFacturaViewModelFactory.CrearBuscarFacturaViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<BuscarFacturaPresenter>(t => t.Model);
    }
    #endregion
  }
}
