﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista del cliente
  /// </summary>
  public class ClienteViewModel : ModelBase, IPersistentModel
  {
    #region Modelos Auxiliares
    public class VehiculoViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Matricula { get; set; }
      public string Marca { get; set; }
      public string Modelo { get; set; }
      public string Bastidor { get; set; }
      public DateTime? Año { get; set; }
      public string Color { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionVehiculoService _gestionVehiculoService;
    private BindingList<VehiculoViewModel> _vehiculosDelCliente = new BindingList<VehiculoViewModel>();
    private readonly List<VehiculoViewModel> _vehiculosDelClienteEliminados = new List<VehiculoViewModel>();

    #endregion

    #region Constructores
    public ClienteViewModel(IGestionVehiculoService gestionVehiculoService)
    {
      _gestionVehiculoService = gestionVehiculoService;
    }

    #endregion

    #region Propiedades

    #region Información personal

    public virtual Guid Id { get; set; }
    public virtual string Nombre { get; set; }
    public virtual string PrimerApellido { get; set; }
    public virtual string SegundoApellido { get; set; }
    public virtual string NifCif { get; set; }
    public virtual string Observaciones { get; set; }
    public virtual string ListaTelefonos { get; set; }
    public virtual string Direccion { get; set; }
    public virtual string CorreoElectronico { get; set; }
    public virtual int? FormaDePago { get; set; }
    public virtual int? IdCodigoPostal { get; set; }
    public virtual string CodigoPostal { get; set; }
    public virtual string IdProvincia { get; set; }
    public virtual int? IdPoblacion { get; set; }

    #endregion

    #region Información fiscal

    public virtual string CuentaBancaria { get; set; }
    public virtual string NombreComercial { get; set; }

    #endregion

    #region Vehículos del cliente

    public BindingList<VehiculoViewModel> VehiculosDelCliente { get { return _vehiculosDelCliente; } }
    public List<VehiculoViewModel> VehiculosDelClienteEliminados { get { return _vehiculosDelClienteEliminados; } }

    #endregion

    #endregion

    #region Métodos Públicos

    public List<Vehiculo> ObtenerListaVehiculos(IEnumerable<VehiculoViewModel> listaVehiculosViewModel)
    {
      return
        listaVehiculosViewModel.Select(
          vehiculoViewModel => _gestionVehiculoService.BuscarVehiculoPorId(vehiculoViewModel.Id).First()).ToList();
    }

    public void SetVehiculosDelCliente(BindingList<VehiculoViewModel> vehiculosDelCliente)
    {
      _vehiculosDelCliente = vehiculosDelCliente;
    }

    public void EliminarVehiculo(VehiculoViewModel vehiculoViewModel)
    {
      AñadirVehiculoAListaVehiculosDelClienteEliminados(vehiculoViewModel);
      VehiculosDelCliente.Remove(vehiculoViewModel);
    }
    
    public void AñadirVehiculo(Vehiculos.VehiculoViewModel vehiculoViewModel)
    {
      var nuevoVehiculo = new VehiculoViewModel
                            {
                              Id = vehiculoViewModel.Id,
                              Matricula = vehiculoViewModel.Matricula,
                              Marca = vehiculoViewModel.Marca,
                              Modelo = vehiculoViewModel.Modelo,
                              Año = vehiculoViewModel.Año,
                              Bastidor = vehiculoViewModel.Bastidor,
                              Color = vehiculoViewModel.Color
                            };
      _vehiculosDelCliente.Add(nuevoVehiculo);

      QuitarVehiculoDeListaVehiculosDelClienteEliminados(nuevoVehiculo);
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion

    #region Métodos Privados

    private void QuitarVehiculoDeListaVehiculosDelClienteEliminados(VehiculoViewModel nuevoVehiculo)
    {
      var existe = false;
      foreach (
        var vehiculoDelClienteEliminado in
          _vehiculosDelClienteEliminados.Where(
            vehiculoDelClienteEliminado => vehiculoDelClienteEliminado.Matricula.Equals(nuevoVehiculo.Matricula)))
        existe = true;
      if (existe)
        _vehiculosDelClienteEliminados.Remove(nuevoVehiculo);
    }

    private void AñadirVehiculoAListaVehiculosDelClienteEliminados(VehiculoViewModel vehiculoViewModel)
    {
      var existe = false;
      foreach (
        var vehiculoDelClienteEliminado in
          _vehiculosDelClienteEliminados.Where(
            vehiculoDelClienteEliminado => vehiculoDelClienteEliminado.Matricula.Equals(vehiculoViewModel.Matricula)))
        existe = true;
      if (!existe)
        _vehiculosDelClienteEliminados.Add(vehiculoViewModel);
    }

    #endregion
  }
}
