﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista del cliente,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class ClienteViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<int>> _formasDePago = new BindingList<LookUpItem<int>>();
    private IList<LookUpItem<int>> _codigosPostales = new BindingList<LookUpItem<int>>();
    private IList<LookUpItem<int>> _poblaciones = new BindingList<LookUpItem<int>>();
    private IList<LookUpItem<string>> _provincias = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _marcas = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _modelos = new BindingList<LookUpItem<string>>();

    #endregion

    #region Constructores

    #endregion

    #region Propiedades

    public IEnumerable<LookUpItem<int>> FormasDePago { get { return _formasDePago; } }
    public IEnumerable<LookUpItem<int>> CodigosPostales { get { return _codigosPostales; } }
    public IEnumerable<LookUpItem<int>> Poblaciones { get { return _poblaciones; } }
    public IEnumerable<LookUpItem<string>> Provincias { get { return _provincias; } }
    public IEnumerable<LookUpItem<string>> Marcas { get { return _marcas; } }
    public IEnumerable<LookUpItem<string>> Modelos { get { return _modelos; } }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece las formas de pago en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="formasDePago">Formas de pago a poner en correspondiente lista LookUp.</param>
    public void SetFormasDePago(IList<FormaDePago> formasDePago)
    {
      _formasDePago =
        formasDePago.Select(formaDePago => new LookUpItem<int> { Codigo = formaDePago.Id, Descripcion = formaDePago.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los códigos postales en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="codigosPostales">Códigos postales a poner en correspondiente lista LookUp.</param>
    public void SetCodigosPostales(IList<CodigoPostal> codigosPostales)
    {
      _codigosPostales =
        codigosPostales.Select(codigoPostal => new LookUpItem<int> { Codigo = codigoPostal.Id, Descripcion = codigoPostal.Codigo }).ToList();
    }

    /// <summary>
    /// Establece las poblaciones en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="poblaciones">Poblaciones a poner en correspondiente lista LookUp.</param>
    public void SetPoblaciones(IList<Poblacion> poblaciones)
    {
      _poblaciones =
        poblaciones.Select(poblacion => new LookUpItem<int> { Codigo = poblacion.Id, Descripcion = poblacion.Nombre }).ToList();
    }

    /// <summary>
    /// Establece las provincias en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="provincias">Provincias a poner en correspondiente lista LookUp.</param>
    public void SetProvincias(IList<Provincia> provincias)
    {
      _provincias =
        provincias.Select(provincia => new LookUpItem<string> { Codigo = provincia.Id, Descripcion = provincia.Nombre }).ToList();
    }

    /// <summary>
    /// Establece las marcas en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="marcas">Marcas a poner en correspondiente lista LookUp.</param>
    public void SetMarcas(IList<Marca> marcas)
    {
      _marcas =
        marcas.Select(marca => new LookUpItem<string> { Codigo = marca.Id, Descripcion = marca.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los modelos en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="modelos">Modelos a poner en correspondiente lista LookUp.</param>
    public void SetModelos(IList<Modelo> modelos)
    {
      _modelos =
        modelos.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    #endregion
  }
}
