﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.CodigosPostales;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionFormasDePago;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProvincias;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del cliente,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ClienteViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ClienteViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionFormaDePagoService _gestionFormaDePagoService;
    private readonly IGestionCodigoPostalService _gestionCodigoPostalService;
    private readonly IGestionProvinciaService _gestionProvinciaService;
    private readonly IGestionMarcaService _gestionMarcaService;
    private readonly IGestionModeloService _gestionModeloService;

    #endregion

    #region Constructores

    public ClienteViewModelReadOnlyFactory(Func<ClienteViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionFormaDePagoService gestionFormaDePagoService, 
                                           IGestionProvinciaService gestionProvinciaService, 
                                           IGestionMarcaService gestionMarcaService, 
                                           IGestionModeloService gestionModeloService, 
                                           IGestionCodigoPostalService gestionCodigoPostalService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionFormaDePagoService = gestionFormaDePagoService;
      _gestionProvinciaService = gestionProvinciaService;
      _gestionMarcaService = gestionMarcaService;
      _gestionModeloService = gestionModeloService;
      _gestionCodigoPostalService = gestionCodigoPostalService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ClienteViewModelReadOnly vacío</returns>
    public ClienteViewModelReadOnly CreateClienteViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetFormasDePago(_gestionFormaDePagoService.BuscarTodasLasFormasDePago());
      model.SetProvincias(_gestionProvinciaService.BuscarTodasLasProvincias());
      model.SetMarcas(_gestionMarcaService.BuscarTodasLasMarcas());
      model.SetModelos(_gestionModeloService.BuscarTodosLosModelos());
      model.SetCodigosPostales(_gestionCodigoPostalService.BuscarTodosLosCodigosPostales());
      return model;
    }

    #endregion
  }
}
