﻿using FluentValidation;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Para definir un conjunto de reglas de validación para un objeto en particular, hay que 
  /// crear una clase que herede de AbstractValidator <T/>, donde T es el tipo de
  /// Clase que se desea validar.
  /// Para mas información ir a http://fluentvalidation.codeplex.com/documentation
  /// </summary>
  public class ClienteViewModelValidator : AbstractValidator<ClienteViewModel>
  {
    /// <summary>
    /// Inicializa una nueva instancia de la clase<see cref="ClienteViewModelValidator" />.
    /// Configurando las reglas de validación a cumplir.
    /// </summary>
    public ClienteViewModelValidator()
    {
      RuleFor(t => t.Nombre).NotEmpty();
      RuleFor(t => t.PrimerApellido).NotEmpty();
      RuleFor(t => t.NifCif).NotEmpty();
      RuleFor(t => t.IdCodigoPostal).NotNull().NotEqual(0).WithMessage("Se debe introducir un código postal");
      RuleFor(t => t.FormaDePago).NotNull().NotEqual(0).WithMessage("Se debe introducir una forma de pago");
    }
  }
}
