﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  public partial class FormClientes : BindableFormBase
  {
    #region Atributos

    private readonly ClientePresenter _clientePresenter;
    #endregion

    #region Constructores
    public FormClientes(ClientePresenter clientePresenter)
    {
      InitializeComponent();
      _clientePresenter = clientePresenter;
      ConfigurarAyuda(helpProviderFormCliente);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaClientesBuscados()
    {
      dockPanelBuscarCliente.Show();
    }

    private void OcultarListaClientesBuscados()
    {
      dockPanelBuscarCliente.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void FormClientes_Load(object sender, EventArgs e)
    {
      if (_clientePresenter != null)
      {
        _clientePresenter.SetModelReadOnly();
        _clientePresenter.NuevoCliente();
        _clientePresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _clientePresenter;
      }
      helpProviderFormCliente.SetShowHelp(this, true);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _clientePresenter.NuevoCliente();
      textEditNombre.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _clientePresenter.GuardarCliente();
      if (_clientePresenter.AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura ||
          _clientePresenter.AlGuardarElClienteOSeleccionarloAñadirloALaRecepcion)
        Close();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _clientePresenter.BuscarCliente();
      MostrarListaClientesBuscados();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _clientePresenter.NuevaBusquedaCliente();
      textEditNombre.Focus();
    }

    private void gridViewBuscarCliente_DoubleClick(object sender, EventArgs e)
    {
      var clienteSeleccionado = gridViewBuscarCliente.GetFocusedRow() as BuscarClienteListItemModel;
      if (clienteSeleccionado != null) _clientePresenter.SetCliente(clienteSeleccionado.ClienteId);
      OcultarListaClientesBuscados();
      if (_clientePresenter.AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura ||
          _clientePresenter.AlGuardarElClienteOSeleccionarloAñadirloALaRecepcion)
        Close();
    }

    private void simpleButtonTransferir_Click(object sender, EventArgs e)
    {
      gridViewBuscarCliente_DoubleClick(sender, e);
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _clientePresenter.EliminarCliente();
    }

    private void simpleButtonNuevoVehiculo_Click(object sender, EventArgs e)
    {
      _clientePresenter.MostrarFormVehiculoConNuevoVehiculo();
    }

    private void simpleButtonAddVehiculo_Click(object sender, EventArgs e)
    {
      _clientePresenter.MostrarFormVehiculoConNuevoVehiculo();
    }

    private void simpleButtonEliminarVehiculo_Click(object sender, EventArgs e)
    {
      var vehiculoSeleccionado = gridViewVehiculosCliente.GetFocusedRow() as ClienteViewModel.VehiculoViewModel;
      if (vehiculoSeleccionado != null) _clientePresenter.EliminarVehiculoAlCliente(vehiculoSeleccionado);
    }

    private void simpleButtonModificarVehiculo_Click(object sender, EventArgs e)
    {
      var vehiculoSeleccionado = gridViewVehiculosCliente.GetFocusedRow() as ClienteViewModel.VehiculoViewModel;
      if (vehiculoSeleccionado != null) _clientePresenter.MostrarFormVehiculoConVehiculoSeleccionado(vehiculoSeleccionado.Id);
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormCliente.HelpNamespace);
    }

    #endregion
  }
}