﻿using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionReparaciones;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarReparaciones;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormFinalizarReparaciones)]
  public class FinalizarReparacionesPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private FinalizarReparacionesViewModel _model;
    private readonly FinalizarReparacionesViewModelFactory _modelFactory;

    private FinalizarReparacionesViewModelReadOnly _modelReadOnly;
    private readonly FinalizarReparacionesViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly IGestionReparacionService _gestionReparacionService;
    private readonly InformacionUsuario _informacionUsuario;

    private readonly BuscarReparacionPresenter _buscarReparacionPresenter;

    #endregion

    #region Constructores

    public FinalizarReparacionesPresenter(FinalizarReparacionesViewModelFactory modelFactory,
                                FinalizarReparacionesViewModelReadOnlyFactory modelReadOnlyFactory,
                                IMediatorManager mediatorManager,
                                IGestionReparacionService gestionReparacionService,
                                InformacionUsuario informacionUsuario,
                                BuscarReparacionPresenter buscarReparacionPresenter)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _gestionReparacionService = gestionReparacionService;
      _informacionUsuario = informacionUsuario;
      _buscarReparacionPresenter = buscarReparacionPresenter;
    }

    #endregion

    #region Propiedades

    public FinalizarReparacionesViewModel Model
    {
      get { return _model; }
    }

    public FinalizarReparacionesViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarReparacionListItemModel> ListaBuscarReparaciones
    {
      get { return _buscarReparacionPresenter.Model.ListaBuscarReparaciones; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {

    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateFinalizarReparacionesViewModelReadOnly(_informacionUsuario.IdEmpleado);
      NotifyPropertyChanged<FinalizarReparacionesPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaFinalizarReparaciones()
    {
      _model = _modelFactory.CrearFinalizarReparacionesViewModel();
      NotifyPropertyChanged<FinalizarReparacionesPresenter>(t => t.Model);
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarFinalizarReparaciones()
    {
      foreach (var lineaReparacion in _model.LineasDeFinalizarReparaciones)
      {
        _gestionReparacionService.ChangeReparacion(lineaReparacion.IdReparacion, lineaReparacion.DuracionSugerida,
                                                   lineaReparacion.DuracionReal, lineaReparacion.Estado,
                                                   _informacionUsuario.IdEmpleado, _model.IdOrdenDeReparacion);
      }
      MessageBox.Show(Resources.FinalizarReparacionesPresenter_GuardarFinalizarReparaciones_Reparaciones_guardadas_con_éxito,
                      Resources.FinalizarReparacionesPresenter_GuardarFinalizarReparaciones_Guardar_reparaciones, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Añade las reparaciones de la orden contenida en buscarReparacionListItemModel
    /// </summary>
    /// <param name="buscarReparacionListItemModel">Orden con las reparaciones a añadir</param>
    [MediatorMessageSink(MediatorMessages.AñadirReparacionesAFinalizarReparaciones, ParameterType = typeof(BuscarReparacionListItemModel))]
    public void AñadirReparacionesAFinalizarReparaciones(BuscarReparacionListItemModel buscarReparacionListItemModel)
    {
      _model = _modelFactory.CrearFinalizarReparacionesViewModel(buscarReparacionListItemModel.OrdenDeReparacionId);
      NotifyPropertyChanged<FinalizarReparacionesPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de órdenes de reparación
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void Buscar()
    {
      _buscarReparacionPresenter.Buscar(_model.Matricula, _model.IdOrdenDeReparacion, _informacionUsuario.IdEmpleado);
      NotifyPropertyChanged<FinalizarReparacionesPresenter>(t => t.ListaBuscarReparaciones);
    }

    #endregion
  }
}
