﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionReparaciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de finalizar reparaciones
  /// </summary>
  public class FinalizarReparacionesViewModel : ModelBase
  {
    #region Modelos Auxiliares
    public class LineaFinalizarReparacionesViewModel : ModelBase
    {
      public Guid IdReparacion { get; set; }
      public string NumeroOrden { get; set; }
      public string Matricula { get; set; }
      public string Descripcion { get; set; }
      public decimal? DuracionReal { get; set; }
      public decimal? DuracionSugerida { get; set; }
      public int? Estado { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionReparacionService _gestionReparacionService;
    private BindingList<LineaFinalizarReparacionesViewModel> _lineasDeFinalizarReparaciones = new BindingList<LineaFinalizarReparacionesViewModel>();

    #endregion

    #region Constructores
    public FinalizarReparacionesViewModel(IGestionReparacionService gestionReparacionService)
    {
      _gestionReparacionService = gestionReparacionService;
    }

    #endregion

    #region Propiedades

    public virtual string Empleado { get; set; }
    public virtual Guid IdOrdenDeReparacion { get; set; }
    public virtual string Matricula { get; set; }

    public BindingList<LineaFinalizarReparacionesViewModel> LineasDeFinalizarReparaciones { get { return _lineasDeFinalizarReparaciones; } }

    #endregion

    #region Métodos Públicos

    public List<Reparacion> ObtenerListaReparaciones(IEnumerable<LineaFinalizarReparacionesViewModel> lineasDeFinalizarReparacionesViewModel)
    {
      return
        lineasDeFinalizarReparacionesViewModel.Select(
          lineaFinalizarReparacionesViewModel => _gestionReparacionService.BuscarReparacionPorId(lineaFinalizarReparacionesViewModel.IdReparacion).First()).ToList();
    }

    public void SetLineasDeFinalizarReparaciones(BindingList<LineaFinalizarReparacionesViewModel> lineasDeFinalizarReparaciones)
    {
      _lineasDeFinalizarReparaciones = lineasDeFinalizarReparaciones;
    }
    
    #endregion
  }
}
