﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista de finalizar reparaciones,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class FinalizarReparacionesViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<Guid>> _ordenesDeReparacion = new BindingList<LookUpItem<Guid>>();
    private IList<LookUpItem<int>> _estadosReparacion = new BindingList<LookUpItem<int>>();
    private IList<LookUpItem<int>> _estadosOrdenReparacion = new BindingList<LookUpItem<int>>();

    #endregion

    #region Constructores

    #endregion

    #region Propiedades
    
    public IEnumerable<LookUpItem<Guid>> OrdenesDeReparacion { get { return _ordenesDeReparacion; } }
    public IEnumerable<LookUpItem<int>> EstadosReparacion { get { return _estadosReparacion; } }
    public IEnumerable<LookUpItem<int>> EstadosOrdenReparacion { get { return _estadosOrdenReparacion; } }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece los estados de la orden en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="estadosOrdenReparacion">Estados de la orden a poner en correspondiente lista LookUp.</param>
    public void SetEstadosOrdenReparacion(IList<EstadoOrdenReparacion> estadosOrdenReparacion)
    {
      _estadosOrdenReparacion =
        estadosOrdenReparacion.Select(estado => new LookUpItem<int> { Codigo = estado.Id, Descripcion = estado.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece las órdenes de reparación en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="ordenesDeReparacion">Órdenes de reparación a poner en correspondiente lista LookUp.</param>
    public void SetOrdenesDeReparacion(IList<OrdenReparacion> ordenesDeReparacion)
    {
      _ordenesDeReparacion =
        ordenesDeReparacion.Select(ordenReparacion => new LookUpItem<Guid> { Codigo = ordenReparacion.Id, Descripcion = ordenReparacion.NumeroOrden }).ToList();
    }

    /// <summary>
    /// Establece los estados de la reparación en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="estadosReparacion">Estados reparación a poner en correspondiente lista LookUp.</param>
    public void SetEstadosReparacion(IList<EstadoReparacion> estadosReparacion)
    {
      _estadosReparacion =
        estadosReparacion.Select(estado => new LookUpItem<int> { Codigo = estado.Id, Descripcion = estado.Descripcion }).ToList();
    }

    #endregion
  }
}
