﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEstadosOrdenReparacion;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEstadosReparacion;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista de las entradas al almacén,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class FinalizarReparacionesViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<FinalizarReparacionesViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;
    private readonly IGestionEstadoReparacionService _gestionEstadoReparacionService;
    private readonly IGestionEstadoOrdenReparacionService _gestionEstadoOrdenReparacionService;

    #endregion

    #region Constructores

    public FinalizarReparacionesViewModelReadOnlyFactory(Func<FinalizarReparacionesViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionOrdenDeReparacionService gestionOrdenDeReparacionService, IGestionEstadoReparacionService gestionEstadoReparacionService, IGestionEstadoOrdenReparacionService gestionEstadoOrdenReparacionService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
      _gestionEstadoReparacionService = gestionEstadoReparacionService;
      _gestionEstadoOrdenReparacionService = gestionEstadoOrdenReparacionService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo FinalizarReparacionesViewModelReadOnly vacío</returns>
    public FinalizarReparacionesViewModelReadOnly CreateFinalizarReparacionesViewModelReadOnly(Guid? idEmpleado)
    {
      var model = _createModelWithProxy();
      const int estadoEnProceso = 1;
      model.SetOrdenesDeReparacion(_gestionOrdenDeReparacionService.BuscarPorEmpleadoYEstado(idEmpleado, estadoEnProceso));
      model.SetEstadosReparacion(_gestionEstadoReparacionService.BuscarTodosLosEstadosReparacion());
      model.SetEstadosOrdenReparacion(_gestionEstadoOrdenReparacionService.BuscarTodosLosEstadosOrdenReparacion());
      return model;
    }

    #endregion
  }
}
