﻿using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarReparaciones;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  public partial class FormFinalizarReparaciones : BindableFormBase
  {
    #region Atributos

    private readonly FinalizarReparacionesPresenter _finalizarReparacionesPresenter;

    #endregion

    #region Constructores

    public FormFinalizarReparaciones(FinalizarReparacionesPresenter finalizarReparacionesPresenter)
    {
      InitializeComponent();
      _finalizarReparacionesPresenter = finalizarReparacionesPresenter;
      ConfigurarAyuda(helpProviderFinalizarReparaciones);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaReparacionesBuscadas()
    {
      dockPanelListaBuscarReparaciones.Show();
    }

    private void OcultarListaReparacionesBuscadas()
    {
      dockPanelListaBuscarReparaciones.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFinalizarReparaciones.HelpNamespace);
    }

    private void FormFinalizarReparaciones_Load(object sender, System.EventArgs e)
    {
      if (_finalizarReparacionesPresenter != null)
      {
        _finalizarReparacionesPresenter.SetModelReadOnly();
        _finalizarReparacionesPresenter.NuevaFinalizarReparaciones();
        _finalizarReparacionesPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _finalizarReparacionesPresenter;
      }
      helpProviderFinalizarReparaciones.SetShowHelp(this, true);
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {
      gridViewListaOrdenes_DoubleClick(sender, e);
    }

    private void gridViewListaOrdenes_DoubleClick(object sender, System.EventArgs e)
    {
      var reparacionSeleccionada = gridViewListaOrdenes.GetFocusedRow() as BuscarReparacionListItemModel;
      if (reparacionSeleccionada != null) _finalizarReparacionesPresenter.AñadirReparacionesAFinalizarReparaciones(reparacionSeleccionada);
      OcultarListaReparacionesBuscadas();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      gridViewListaReparaciones.PostEditor();
      _finalizarReparacionesPresenter.GuardarFinalizarReparaciones();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _finalizarReparacionesPresenter.NuevaFinalizarReparaciones();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _finalizarReparacionesPresenter.Buscar();
      MostrarListaReparacionesBuscadas();
    }

    private void repositoryItemCheckEditFinalizada_EditValueChanged(object sender, System.EventArgs e)
    {
      gridViewListaReparaciones.PostEditor();
    }
    #endregion
  }
}