﻿using System;
using System.Windows.Forms;
using Autofac;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Extensiones;
using GestionConcesionarios.Presentacion.Infraestructura.MessageService;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FormularioPrincipal
{
  public partial class FormPrincipal : FormPrincipalBase
  {
    #region Atributos

    private readonly FormPrincipalPresenter _presenter;

    #endregion

    #region Constructores

    public FormPrincipal(FormPrincipalPresenter presenter, ILifetimeScope lifetimeScope,
                         WindowsFormsMessageService formsMessage)
      : base(lifetimeScope)
    {
      _presenter = presenter;
      InitializeComponent();
      ConfigurarAyuda(helpProviderFormPrincipal);
    }

    #endregion

    #region Manejadores de eventos

    private void barButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormPrincipal.HelpNamespace);
    }

    private void barButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void FormPrincipal_Load(object sender, EventArgs e)
    {
      if (!_presenter.ShowLogin())
        this.Close();

      ConfigurarBindingsConPresenter();
      _presenter.ConfigurarLaVistaSegunElTipoDeUsuario();
    }

    private void ConfigurarBindingsConPresenter()
    {
      Bind(layoutControlGroupMecanicos, t => t.Enabled, _presenter, v => v.HabilitadoMecanicos);
      Bind(layoutControlGroupTaller, t => t.Enabled, _presenter, v => v.HabilitadoTaller);
      Bind(layoutControlGroupVentas, t => t.Enabled, _presenter, v => v.HabilitadoVentas);
    }

    private void simpleButtonClientes_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarCliente());
    }

    private void simpleButtonPresupuestos_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarPresupuesto());
    }

    private void simpleButtonVentas_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarVenta());
    }

    private void simpleButtonVehiculosVenta_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarVehiculoVenta());
    }

    private void simpleButtonRecepcionVehiculo_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarRecepcionVehiculo());
    }

    private void simpleButtonOrdenesReparacion_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarOrdenReparacion());
    }

    private void simpleButtonVehiculos_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarVehiculos());
    }

    private void simpleButtonManosDeObra_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarManoDeObra());
    }

    private void simpleButtonArticulos_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarArticulo());
    }

    private void simpleButtonPaquetesDeServicio_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarPaqueteDeServicio());
    }

    private void simpleButtonAlmacen_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarEntradaAlmacen());
    }

    private void simpleButtonReparaciones_Click(object sender, EventArgs e)
    {
      this.BeginInvoke(() => _presenter.MostrarFinalizarReparacion());
    }

    #endregion
  }
}