﻿using System;
using System.Windows.Forms;
using Autofac;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Almacen;
using GestionConcesionarios.Presentacion.CasosDeUso.Articulos;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.ManosDeObra;
using GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion;
using GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio;
using GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos;
using GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo;
using GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos;
using GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta;
using GestionConcesionarios.Presentacion.CasosDeUso.Ventas;
using GestionConcesionarios.Presentacion.Infraestructura;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FormularioPrincipal
{
  public class FormPrincipalPresenter : PresenterBase
  {
    #region Atributos

    private readonly Func<Type, IFormContext> _formContextFactory;
    private readonly FormManager _formManager;
    private readonly ILifetimeScope _lifetimeScope;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructor

    public FormPrincipalPresenter(Func<Type, IFormContext> formContextFactory, FormManager formManager, ILifetimeScope lifetimeScope, InformacionUsuario informacionUsuario)
    {
      if (formContextFactory == null) throw new ArgumentNullException("formContextFactory");
      if (formManager == null) throw new ArgumentNullException("formManager");

      _formContextFactory = formContextFactory;
      _formManager = formManager;
      _lifetimeScope = lifetimeScope;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Propiedades

    public bool HabilitadoVentas { get; set; }
    public bool HabilitadoTaller { get; set; }
    public bool HabilitadoMecanicos { get; set; }

    #endregion

    #region Metodos Publicos

    public void ConfigurarLaVistaSegunElTipoDeUsuario()
    {
      if (_informacionUsuario != null)
        switch (_informacionUsuario.TipoUsuario)
        {
          case TipoUsuario.Comercial:
            HabilitadoVentas = true;
            HabilitadoMecanicos = false;
            HabilitadoTaller = false;
            break;
          case TipoUsuario.JefeTaller:
            HabilitadoVentas = false;
            HabilitadoMecanicos = false;
            HabilitadoTaller = true;
            break;
          case TipoUsuario.Mecanico:
            HabilitadoVentas = false;
            HabilitadoMecanicos = true;
            HabilitadoTaller = false;
            break;
          case TipoUsuario.Administrador:
            HabilitadoVentas = true;
            HabilitadoMecanicos = true;
            HabilitadoTaller = true;
            break;
        }
      NotifyPropertyChanged<FormPrincipalPresenter>(t => t.HabilitadoMecanicos);
      NotifyPropertyChanged<FormPrincipalPresenter>(t => t.HabilitadoTaller);
      NotifyPropertyChanged<FormPrincipalPresenter>(t => t.HabilitadoVentas);
    }

    public void MostrarCliente()
    {
      using (var presenterContext = _formContextFactory(typeof(FormClientes)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarPresupuesto()
    {
      using (var presenterContext = _formContextFactory(typeof(FormPresupuesto)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarVenta()
    {
      using (var presenterContext = _formContextFactory(typeof(FormFactura)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarVehiculoVenta()
    {
      using (var presenterContext = _formContextFactory(typeof(FormVehículoVenta)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarRecepcionVehiculo()
    {
      using (var presenterContext = _formContextFactory(typeof(FormRecepcionVehiculo)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarOrdenReparacion()
    {
      using (var presenterContext = _formContextFactory(typeof(FormOrdenDeReparacion)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarVehiculos()
    {
      using (var presenterContext = _formContextFactory(typeof(FormVehiculos)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarManoDeObra()
    {
      using (var presenterContext = _formContextFactory(typeof(FormManoDeObra)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarArticulo()
    {
      using (var presenterContext = _formContextFactory(typeof(FormArticulo)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarPaqueteDeServicio()
    {
      using (var presenterContext = _formContextFactory(typeof(FormPaqueteDeServicio)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarEntradaAlmacen()
    {
      using (var presenterContext = _formContextFactory(typeof(FormEntradaAlAlmacen)))
      {
        presenterContext.Init();
      }
    }

    public void MostrarFinalizarReparacion()
    {
      using (var presenterContext = _formContextFactory(typeof(FormFinalizarReparaciones)))
      {
        presenterContext.Init();
      }
    }

    public bool ShowLogin()
    {
      using (var presenterContext = _formContextFactory(typeof(FormLogin)))
      {
        if (presenterContext.Init() == DialogResult.OK)
        {
          return true;
        }
        return false;
      }
    }

    #endregion
  }
}
