﻿using System;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.HistorialOrdenesDeReparacion
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.HistorialOrdenesDeReparacion)]
  public class HistorialOrdenesDeReparacionPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly HistorialOrdenesDeReparacionViewModelFactory _historialOrdenesDeReparacionViewModelFactory;

    #endregion

    #region Constructor

    public HistorialOrdenesDeReparacionPresenter(IMediatorManager mediatorManager, HistorialOrdenesDeReparacionViewModelFactory historialOrdenesDeReparacionViewModelFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _historialOrdenesDeReparacionViewModelFactory = historialOrdenesDeReparacionViewModelFactory;
      Model = _historialOrdenesDeReparacionViewModelFactory.CrearHistorialOrdenesDeReparacionViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.HistorialOrdenesDeReparacion];
    }

    #endregion

    #region Propiedades

    public HistorialOrdenesDeReparacionViewModel Model { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Notifica que se ha seleccinado la orden de reparación con Id=ordenReparacionId
    /// </summary>
    /// <param name="ordenReparacionId">Id de la entidad seleccionada</param>
    public void OrdenDeReparacionSeleccionada(Guid ordenReparacionId)
    {
      _mediator.NotifyColleagues(MediatorMessages.HistorialOrdenesDeReparacionOrdenDeReparacionSeleccionada, ordenReparacionId);
    }

    /// <summary>
    /// Ejecuta la búsqueda de órdenes de reparación en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="matricula">Matricula del vehículo en la orden de reparación a buscar.</param>
    [MediatorMessageSink(MediatorMessages.BuscarOrdenesDeReparacionDelVehiculo, ParameterType = typeof(string))]
    public void Buscar(string matricula)
    {
      var criterioBusqueda = new CriteriosBusquedaOrdenDeReparacion
      {
        Matricula = matricula
      };

      Model = _historialOrdenesDeReparacionViewModelFactory.CrearHistorialOrdenesDeReparacionViewModelFactory(criterioBusqueda);
      NotifyPropertyChanged<HistorialOrdenesDeReparacionPresenter>(t => t.Model);
    }
    #endregion
  }
}
