﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.HistorialOrdenesDeReparacion
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de historial de ordenes de reparación,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class HistorialOrdenesDeReparacionViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;

    #endregion

    #region Constructor

    public HistorialOrdenesDeReparacionViewModelFactory(IGestionOrdenDeReparacionService gestionOrdenDeReparacionService)
    {
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo HistorialOrdenesDeReparacionViewModel vacío</returns>
    public HistorialOrdenesDeReparacionViewModel CrearHistorialOrdenesDeReparacionViewModelFactory()
    {
      return new HistorialOrdenesDeReparacionViewModel { ListaHistorialOrdenesDeReparacion = new BindingList<HistorialOrdenesDeReparacionListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaOrdenDeReparacion">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo HistorialOrdenesDeReparacionViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public HistorialOrdenesDeReparacionViewModel CrearHistorialOrdenesDeReparacionViewModelFactory(CriteriosBusquedaOrdenDeReparacion criteriosBusquedaOrdenDeReparacion)
    {
      var lista = _gestionOrdenDeReparacionService.BuscarOrdenDeReparacionPorMatricula(criteriosBusquedaOrdenDeReparacion.Matricula);
      
      var buscarOrdenDeReparacionViewModel = new HistorialOrdenesDeReparacionViewModel { ListaHistorialOrdenesDeReparacion = CrearListaHistorialOrdenesDeReparacion(lista) };

      return buscarOrdenDeReparacionViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<HistorialOrdenesDeReparacionListItemModel> CrearListaHistorialOrdenesDeReparacion(IEnumerable<OrdenReparacion> listaOrdenesDeReparacion)
    {
      var listaHistorialOrdenesDeReparacion = new BindingList<HistorialOrdenesDeReparacionListItemModel>();

      foreach (var ordenReparacionListItemModel in listaOrdenesDeReparacion.Select(ordenReparacion => new HistorialOrdenesDeReparacionListItemModel
      {
        OrdenDeReparacionId = ordenReparacion.Id,
        Taller = ordenReparacion.CodigoTaller,
        Prediagnostico = ordenReparacion.Recepcion.Prediagnostico,
        Fecha = ordenReparacion.Recepcion.FechaHoraRecepcion,
        Estado = ordenReparacion.IdEstado.ToString(),
        ObservacionesDelCliente = ordenReparacion.Recepcion.ObservacionesSobreElCliente,
        Responsable = ordenReparacion.Recepcion.Empleado.Nombre
      }))
      {
        listaHistorialOrdenesDeReparacion.Add(ordenReparacionListItemModel);
      }
      return listaHistorialOrdenesDeReparacion;
    }

    #endregion
  }
}
