﻿using System;
using System.Reflection;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Login
{
  public partial class FormLogin : BindableFormBase
  {
    #region Atributos

    private readonly LoginPresenter _loginPresenter;
    private decimal _numeroIntentos;
    private const decimal MaxIntentos = 3;

    #endregion

    #region Constructores

    public FormLogin(LoginPresenter loginPresenter)
    {
      InitializeComponent();
      _loginPresenter = loginPresenter;
      //this.Text = String.Format("Acerca de {0}", AssemblyTitle);
    }

    #endregion

    #region Descriptores de acceso de atributos de ensamblado

    public string AssemblyTitle
    {
      get
      {
        object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
        if (attributes.Length > 0)
        {
          var titleAttribute = (AssemblyTitleAttribute)attributes[0];
          if (titleAttribute.Title != "")
          {
            return titleAttribute.Title;
          }
        }
        return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
      }
    }

    public string AssemblyVersion
    {
      get
      {
        return Assembly.GetExecutingAssembly().GetName().Version.ToString();
      }
    }

    public string AssemblyDescription
    {
      get
      {
        object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyDescriptionAttribute), false);
        if (attributes.Length == 0)
        {
          return "";
        }
        return ((AssemblyDescriptionAttribute)attributes[0]).Description;
      }
    }

    public string AssemblyProduct
    {
      get
      {
        object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyProductAttribute), false);
        if (attributes.Length == 0)
        {
          return "";
        }
        return ((AssemblyProductAttribute)attributes[0]).Product;
      }
    }

    public string AssemblyCopyright
    {
      get
      {
        object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
        if (attributes.Length == 0)
        {
          return "";
        }
        return ((AssemblyCopyrightAttribute)attributes[0]).Copyright;
      }
    }

    public string AssemblyCompany
    {
      get
      {
        object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyCompanyAttribute), false);
        if (attributes.Length == 0)
        {
          return "";
        }
        return ((AssemblyCompanyAttribute)attributes[0]).Company;
      }
    }
    #endregion

    #region Manejadores de eventos

    private void FormLogin_Load(object sender, EventArgs e)
    {
      if (_loginPresenter != null)
      {
        _loginPresenter.SetModelReadOnly();
        _loginPresenter.NuevoModeloVista();
        _loginPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _loginPresenter;
      }
    }

    private void simpleButtonCancelar_Click(object sender, EventArgs e)
    {
      DialogResult = DialogResult.None;
      Close();
    }

    private void simpleButtonAceptar_Click(object sender, EventArgs e)
    {
      if (_loginPresenter.AutentificarUsuario())
      {
        DialogResult = DialogResult.OK;
        this.Close();
      }
      else
      {
        _numeroIntentos++;
        if (_numeroIntentos == MaxIntentos)
        {
          DialogResult = DialogResult.None;
          MessageBox.Show("Login Incorrecto", "Login",
                          MessageBoxButtons.OK, MessageBoxIcon.Error);
          Close();
        }
        else
          MessageBox.Show("Login Incorrecto, te quedan " + (MaxIntentos - _numeroIntentos) + " intentos!", "Login",
                          MessageBoxButtons.OK, MessageBoxIcon.Warning);
      }
    }

    #endregion
  }
}
