﻿using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Proveedores;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Login
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormLogin)]
  public class LoginPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private LoginViewModel _model;
    private readonly LoginViewModelFactory _modelFactory;

    private LoginViewModelReadOnly _modelReadOnly;
    private readonly LoginViewModelReadOnlyFactory _modelReadOnlyFactory;
    private readonly InformacionUsuario _informacionUsuario;

    private readonly IGestionEmpleadoService _gestionEmpleadoService;

    #endregion

    #region Constructores

    public LoginPresenter(LoginViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             LoginViewModelReadOnlyFactory modelReadOnlyFactory,
                             InformacionUsuario informacionUsuario,
                             IGestionEmpleadoService gestionEmpleadoService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _informacionUsuario = informacionUsuario;
      _gestionEmpleadoService = gestionEmpleadoService;
    }

    #endregion

    #region Propiedades

    public LoginViewModel Model
    {
      get { return _model; }
    }

    public LoginViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.IdEmpleado).Do(x =>
                                                  {
                                                    var empleado =
                                                      _gestionEmpleadoService.BuscarEmpleadoPorId(Model.IdEmpleado)[0];
                                                    if (empleado != null)
                                                      Model.IdTaller = empleado.CodigoTaller;
                                                    NotifyPropertyChanged<LoginPresenter>(t => t.Model);
                                                  });
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevoModeloVista()
    {
      _model = _modelFactory.CrearLoginViewModel();
      NotifyPropertyChanged<LoginPresenter>(t => t.Model);
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateLoginViewModelReadOnly();
      NotifyPropertyChanged<ProveedorPresenter>(t => t.ModelReadOnly);
    }

    public bool AutentificarUsuario()
    {
      var empleado = _gestionEmpleadoService.BuscarEmpleadoPorId(Model.IdEmpleado)[0];
      if (empleado != null)
      {
        if (empleado.Codigo.Equals(_model.Codigo))
        {
          _informacionUsuario.IdEmpleado = Model.IdEmpleado;
          _informacionUsuario.CodigoTaller = _model.IdTaller;
          _informacionUsuario.Usuario = empleado.Nombre;
          if (empleado.Tipo != null) _informacionUsuario.TipoUsuario = (TipoUsuario) empleado.Tipo;
          return true;
        }
      }
      return false;
    }

    #endregion
  }
}
