﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Login
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del logion,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class LoginViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<LoginViewModel> _crearViewModel;
    private readonly IGestionEmpleadoService _gestionEmpleadoService;

    #endregion

    #region Constructores

    public LoginViewModelFactory(Func<LoginViewModel> crearViewModel, IGestionEmpleadoService gestionEmpleadoService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionEmpleadoService = gestionEmpleadoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo LoginViewModel vacío</returns>
    public LoginViewModel CrearLoginViewModel()
    {
      var loginViewModel = _crearViewModel();

      loginViewModel.NombreEmpleado = string.Empty;
      loginViewModel.IdEmpleado = Guid.Empty;
      loginViewModel.Codigo = string.Empty;
      loginViewModel.IdTaller = string.Empty;

      return loginViewModel;
    }

    #endregion
  }
}
