﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Login
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista del login,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class LoginViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<string>> _talleres = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<Guid>> _empleados = new BindingList<LookUpItem<Guid>>();

    #endregion
    
    #region Propiedades

    public IEnumerable<LookUpItem<string>> Talleres { get { return _talleres; } }
    public IEnumerable<LookUpItem<Guid>> Empleados { get { return _empleados; } }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece los talleres en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="talleres">Talleres a poner en correspondiente lista LookUp.</param>
    public void SetTalleres(IList<Taller> talleres)
    {
      _talleres =
        talleres.Select(taller => new LookUpItem<string> { Codigo = taller.Id, Descripcion = taller.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los empleados en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="empleados">Empleados a poner en correspondiente lista LookUp.</param>
    public void SetEmpleados(IList<Empleado> empleados)
    {
      _empleados =
        empleados.Select(empleado => new LookUpItem<Guid> { Codigo = empleado.Id, Descripcion = empleado.Nombre }).ToList();
    }

    #endregion
  }
}
