﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTalleres;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Login
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del login,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class LoginViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<LoginViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionTallerService _gestionTallerService;
    private readonly IGestionEmpleadoService _gestionEmpleadoService;

    #endregion

    #region Constructores

    public LoginViewModelReadOnlyFactory(Func<LoginViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory, IGestionTallerService gestionTallerService, IGestionEmpleadoService gestionEmpleadoService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionTallerService = gestionTallerService;
      _gestionEmpleadoService = gestionEmpleadoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo LoginViewModelReadOnly vacío</returns>
    public LoginViewModelReadOnly CreateLoginViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetTalleres(_gestionTallerService.BuscarTodosLosTalleres());
      model.SetEmpleados(_gestionEmpleadoService.BuscarTodosLosEmpleados());
      return model;
    }

    #endregion
  }
}
