﻿using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;

namespace GestionConcesionarios.Presentacion.CasosDeUso.ManosDeObra
{
  public partial class FormManoDeObra : BindableFormBase
  {
    #region Atributos

    private readonly ManoDeObraPresenter _manoDeObraPresenter;

    #endregion

    #region Constructores

    public FormManoDeObra(ManoDeObraPresenter manoDeObraPresenter)
    {
      InitializeComponent();
      _manoDeObraPresenter = manoDeObraPresenter;
      ConfigurarAyuda(helpProviderFormManoDeObra);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaManosDeObraBuscadas()
    {
      dockPanelBuscarManosDeObra.Show();
    }

    private void OcultarListaManosDeObraBuscadas()
    {
      dockPanelBuscarManosDeObra.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormManoDeObra.HelpNamespace);
    }

    private void FormManoDeObra_Load(object sender, System.EventArgs e)
    {
      if (_manoDeObraPresenter != null)
      {
        _manoDeObraPresenter.NuevaManoDeObra();
        bindingSourcePresenter.DataSource = _manoDeObraPresenter;
      }
      helpProviderFormManoDeObra.SetShowHelp(this, true);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _manoDeObraPresenter.NuevaManoDeObra();
      textEditDescripcion.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _manoDeObraPresenter.GuardarManoDeObra();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _manoDeObraPresenter.NuevaBusquedaManoDeObra();
      textEditDescripcion.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _manoDeObraPresenter.BuscarManoDeObra();
      MostrarListaManosDeObraBuscadas();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _manoDeObraPresenter.EliminarManoDeObra();
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {

    }

    private void gridViewListaManosDeObra_DoubleClick(object sender, System.EventArgs e)
    {
      var manoDeObraSeleccionada = gridViewListaManosDeObra.GetFocusedRow() as BuscarManoDeObraListItemModel;
      if (manoDeObraSeleccionada != null) _manoDeObraPresenter.SetManoDeObra(manoDeObraSeleccionada.ManoDeObraId);
      OcultarListaManosDeObraBuscadas();
    }

    #endregion
  }
}