﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionManosDeObra;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.ManosDeObra
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormManoDeObra)]
  public class ManoDeObraPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private ManoDeObraViewModel _model;
    private readonly ManoDeObraViewModelFactory _modelFactory;

    private readonly BuscarManoDeObraPresenter _buscarManoDeObraPresenter;
    private readonly IGestionManoDeObraService _gestionManoDeObraService;
    //private bool _alGuardarElVehiculoOSeleccionarloAñadirloAlCliente;

    #endregion

    #region Constructores

    public ManoDeObraPresenter(ManoDeObraViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             BuscarManoDeObraPresenter buscarManoDeObraPresenter,
                             IGestionManoDeObraService gestionManoDeObraService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarManoDeObraPresenter = buscarManoDeObraPresenter;
      _gestionManoDeObraService = gestionManoDeObraService;
    }

    #endregion

    #region Propiedades

    public ManoDeObraViewModel Model
    {
      get { return _model; }
    }

    public BindingList<BuscarManoDeObraListItemModel> ListaBuscarManoDeObraViewModel
    {
      get { return _buscarManoDeObraPresenter.Model.ListaBuscarManosDeObra; }
    }
    /*
    public bool AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista
    {
      get { return _alGuardarElVehiculoOSeleccionarloAñadirloAlCliente; }
    }
    */
    #endregion

    #region Metodos Publicos
    /*
    /// <summary>
    /// Pone el flag _alGuardarElVehiculoOSeleccionarloAñadirloAlCliente = true, de forma
    /// que cuando se guarde un nuevo vehículo o se seleccione uno existente, se envíe un 
    /// mensaje al presenter del cliente para añadir el vehículo al cliente.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)]
    public void SetAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente()
    {
      _alGuardarElVehiculoOSeleccionarloAñadirloAlCliente = true;
    }
    */
    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    //[MediatorMessageSink(MediatorMessages.NuevaManoDeObra)]
    public void NuevaManoDeObra()
    {
      _model = _modelFactory.CrearManoDeObraViewModel();
      NotifyPropertyChanged<ManoDeObraPresenter>(t => t.Model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id pasado por parámetro.
    /// </summary>
    /// <param name="vehiculoId">Id del vehículo con el se se creará un nuevo modelo de vista.</param>
    //[MediatorMessageSink(MediatorMessages.EditarManoDeObra, ParameterType = typeof(Guid))]
    public void EditarManoDeObra(Guid vehiculoId)
    {
      SetManoDeObra(vehiculoId);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaManoDeObra()
    {
      _model = _modelFactory.CrearManoDeObraViewModel();
      NotifyPropertyChanged<ManoDeObraPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de manos de obra
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarManoDeObra()
    {
      _buscarManoDeObraPresenter.Buscar(_model.Descripcion);
      NotifyPropertyChanged<ManoDeObraPresenter>(t => t.ListaBuscarManoDeObraViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarManoDeObra()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.ManoDeObraPresenter_EliminarManoDeObra_, Resources.ManoDeObraPresenter_EliminarManoDeObra_Eliminar_mano_de_obra, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionManoDeObraService.RemoveManoDeObra(_model.Id);
          NuevaManoDeObra();
          MessageBox.Show(Resources.ManoDeObraPresenter_EliminarManoDeObra_Mano_de_obra_eliminada_con_éxito,
                          Resources.ManoDeObraPresenter_EliminarManoDeObra_Eliminar_mano_de_obra, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.ManoDeObraPresenter_EliminarManoDeObra_No_se_puede_eliminar_la_mano_de_obra_porque_todavía_no_se_ha_guardado_nada,
                          Resources.ManoDeObraPresenter_EliminarManoDeObra_Eliminar_mano_de_obra, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    /// <remarks>
    /// Si el flag _alGuardarElVehiculoOSeleccionarloAñadirloAlCliente = True, se envía un mensaje
    /// al presenter del cliente para añadir el vehículo al cliente
    /// </remarks>
    public void GuardarManoDeObra()
    {
      if (!_model.HasBeenPersisted())
        _gestionManoDeObraService.AddManoDeObra(Guid.NewGuid(), _model.Descripcion, _model.Duracion, _model.PrecioPorHora);
      else
      {
        _gestionManoDeObraService.ChangeManoDeObra(_model.Id, _model.Descripcion, _model.Duracion, _model.PrecioPorHora);
      }
      MessageBox.Show(Resources.ManoDeObraPresenter_GuardarManoDeObra_Mano_de_obra_guardada_con_éxito,
                      Resources.ManoDeObraPresenter_GuardarManoDeObra_Guardar_mano_de_obra, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
      /*
      if (_alGuardarElManoDeObraOSeleccionarloAñadirloAlCliente)
        _mediatorManager[MediatorEnumerator.FormCliente].NotifyColleagues(MediatorMessages.AñadirManoDeObraAlCliente, _model);
      */
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="manoDeObraId">Id de la entidad a cargar</param>
    public void SetManoDeObra(Guid manoDeObraId)
    {
      _model = _modelFactory.CrearManoDeObraViewModel(manoDeObraId);
      NotifyPropertyChanged<ManoDeObraPresenter>(t => t.Model);
      /*
      if (_alGuardarElVehiculoOSeleccionarloAñadirloAlCliente)
        _mediatorManager[MediatorEnumerator.FormCliente].NotifyColleagues(MediatorMessages.AñadirVehiculo, _model);
      */
    }
    #endregion
  }
}
