﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarOrdenDeReparacion;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  public partial class FormOrdenDeReparacion : BindableFormBase
  {
    #region Atributos

    private readonly OrdenDeReparacionPresenter _ordenDeReparacionPresenter;

    #endregion

    #region Constructores

    public FormOrdenDeReparacion(OrdenDeReparacionPresenter ordenDeReparacionPresenter)
    {
      InitializeComponent();
      _ordenDeReparacionPresenter = ordenDeReparacionPresenter;
      ConfigurarAyuda(helpProviderFormOrdenReparacion);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaOrdenesDeReparacionBuscadas()
    {
      dockPanelBuscarOrdenDeReparacion.Show();
    }

    private void OcultarListaOrdenesDeReparacionBuscadas()
    {
      dockPanelBuscarOrdenDeReparacion.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void FormOrdenDeReparacion_Load(object sender, EventArgs e)
    {
      if (_ordenDeReparacionPresenter != null)
      {
        _ordenDeReparacionPresenter.SetModelReadOnly();
        _ordenDeReparacionPresenter.NuevaOrdenDeReparacion();
        _ordenDeReparacionPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _ordenDeReparacionPresenter;
      }
      helpProviderFormOrdenReparacion.SetShowHelp(this, true);
    }

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormOrdenReparacion.HelpNamespace);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _ordenDeReparacionPresenter.NuevaOrdenDeReparacion();
      textEditNumeroOrden.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _ordenDeReparacionPresenter.GuardarOrdenDeReparacion();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _ordenDeReparacionPresenter.NuevaBusquedaOrdenDeReparacion();
      textEditNumeroOrden.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _ordenDeReparacionPresenter.BuscarOrdenDeReparacion();
      MostrarListaOrdenesDeReparacionBuscadas();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _ordenDeReparacionPresenter.EliminarOrdenDeReparacion();
    }

    private void simpleButtonAñadirReparacion_Click(object sender, EventArgs e)
    {
      _ordenDeReparacionPresenter.MostrarFormArticulos();
    }

    private void simpleButtonEliminarReparacion_Click(object sender, EventArgs e)
    {
      var reparacionSeleccionada = gridViewListaReparaciones.GetFocusedRow() as OrdenDeReparacionViewModel.LineaArticuloViewModel;
      if (reparacionSeleccionada != null)
        _ordenDeReparacionPresenter.EliminarLineaArticuloDeLaOrdenDeReparacion(reparacionSeleccionada);
    }

    private void simpleButtonAñadirManoDeObra_Click(object sender, EventArgs e)
    {
      _ordenDeReparacionPresenter.MostrarFormManosDeObra();
    }

    private void simpleButtonEliminarManoDeObra_Click(object sender, EventArgs e)
    {
      var manoDeObraSeleccionada =
        gridViewListaManosDeObra.GetFocusedRow() as OrdenDeReparacionViewModel.LineaManoDeObraViewModel;
      if (manoDeObraSeleccionada != null)
        _ordenDeReparacionPresenter.EliminarLineaManoDeObraDeLaOrdenDeReparacion(manoDeObraSeleccionada);
    }

    private void simpleButtonAñadirPaquete_Click(object sender, EventArgs e)
    {
      _ordenDeReparacionPresenter.MostrarFormPaquetesDeServicio();
    }

    private void simpleButtonEliminarPaquete_Click(object sender, EventArgs e)
    {
      var paqueteSeleccionado =
        gridViewListaPaquetes.GetFocusedRow() as OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel;
      if (paqueteSeleccionado != null)
        _ordenDeReparacionPresenter.EliminarPaqueteDeServicioDeLaOrdenDeReparacion(paqueteSeleccionado);
    }

    private void simpleButtonVerDetallesPaquete_Click(object sender, EventArgs e)
    {
      var paqueteSeleccionado =
        gridViewListaPaquetes.GetFocusedRow() as OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel;
      if (paqueteSeleccionado != null)
        _ordenDeReparacionPresenter.MostrarFormPaqueteDeServicio(paqueteSeleccionado);
    }

    private void simpleButtonTransferir_Click(object sender, EventArgs e)
    {
      gridViewListaOrdenesReparacion_DoubleClick(sender, e);
    }

    private void gridViewListaOrdenesReparacion_DoubleClick(object sender, EventArgs e)
    {
      var ordenDeReparacionSeleccionada = gridViewListaOrdenesReparacion.GetFocusedRow() as BuscarOrdenDeReparacionListItemModel;
      if (ordenDeReparacionSeleccionada != null) _ordenDeReparacionPresenter.SetOrdenDeReparacion(ordenDeReparacionSeleccionada.OrdenDeReparacionId);
      OcultarListaOrdenesDeReparacionBuscadas();
    }

    #endregion
  }
}