﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionContenidosReparaciones;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionReparaciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarOrdenDeReparacion;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormOrdenDeReparacion)]
  [RegisterInMediator(MediatorEnumerator.BuscarArticulo)]
  [RegisterInMediator(MediatorEnumerator.FormArticulo)]
  [RegisterInMediator(MediatorEnumerator.BuscarManoDeObra)]
  [RegisterInMediator(MediatorEnumerator.BuscarPaqueteDeServicio)]
  public class OrdenDeReparacionPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private OrdenDeReparacionViewModel _model;
    private readonly OrdenDeReparacionViewModelFactory _modelFactory;

    private OrdenDeReparacionViewModelReadOnly _modelReadOnly;
    private readonly OrdenDeReparacionViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarOrdenDeReparacionPresenter _buscarOrdenDeReparacionPresenter;

    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;
    
    private readonly IGestionReparacionService _gestionReparacionService;
    private readonly IGestionContenidoReparacionService _gestionContenidoReparacionService;

    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructores

    public OrdenDeReparacionPresenter(OrdenDeReparacionViewModelFactory modelFactory,
                            OrdenDeReparacionViewModelReadOnlyFactory modelReadOnlyFactory,
                            IMediatorManager mediatorManager,
                            BuscarOrdenDeReparacionPresenter buscarOrdenDeReparacionPresenter,
                            IGestionOrdenDeReparacionService gestionOrdenDeReparacionService, 
                            IGestionReparacionService gestionReparacionService, 
                            IGestionContenidoReparacionService gestionContenidoReparacionService, 
                            InformacionUsuario informacionUsuario)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _buscarOrdenDeReparacionPresenter = buscarOrdenDeReparacionPresenter;
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
      _gestionReparacionService = gestionReparacionService;
      _gestionContenidoReparacionService = gestionContenidoReparacionService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Propiedades

    public OrdenDeReparacionViewModel Model
    {
      get { return _model; }
    }

    public OrdenDeReparacionViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarOrdenDeReparacionListItemModel> ListaBuscarOrdenDeReparacionViewModel
    {
      get { return _buscarOrdenDeReparacionPresenter.Model.ListaBuscarOrdenesDeReparacion; }
    }
    
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      /*
      Model.OnChanged(c => Model.IdProvincia).Do(x =>
                                                   {
                                                     ModelReadOnly.SetPoblaciones(_gestionPoblacionService.BuscarListaPoblacionPorIdProvincia(Model.IdProvincia));
                                                     NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.ModelReadOnly);
                                                   });
      */
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateOrdenDeReparacionViewModelReadOnly();
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaOrdenDeReparacion()
    {
      _model = _modelFactory.CrearOrdenDeReparacionViewModel();
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaOrdenDeReparacion()
    {
      _model = _modelFactory.CrearOrdenDeReparacionViewModel();
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de clientes
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarOrdenDeReparacion()
    {
      _buscarOrdenDeReparacionPresenter.Buscar(_model.Matricula, _model.NumeroOrden, _model.Nombre);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.ListaBuscarOrdenDeReparacionViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarOrdenDeReparacion()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_, Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_Eliminar_orden_de_reparación, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionOrdenDeReparacionService.RemoveOrdenReparacion(_model.Id);
          NuevaOrdenDeReparacion();
          MessageBox.Show(Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_Orden_de_Reparación_eliminada_con_éxito,
                          Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_Eliminar_orden_de_reparación, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_No_se_puede_eliminar_la_orden_de_reparación_porque_todavía_no_se_ha_guardado_nada,
                          Resources.OrdenDeReparacionPresenter_EliminarOrdenDeReparacion_Eliminar_orden_de_reparación, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarOrdenDeReparacion()
    {
      Guid idOrdenDeReparacion;
      if (!_model.HasBeenPersisted())
      {
        idOrdenDeReparacion = Guid.NewGuid();
        _gestionOrdenDeReparacionService.AddOrdenReparacion(idOrdenDeReparacion, _model.NumeroOrden, _model.FechaEntrega,
                                                            _model.IdEstado, _model.IdPersonalAsignado, _model.SubTotal,
                                                            _model.ImporteIva, _model.Diagnostico, _model.IdRecepcion,
                                                            _model.IdVehiculo, _informacionUsuario.CodigoTaller);
      }
      else
      {
        idOrdenDeReparacion = _model.Id;
        _gestionOrdenDeReparacionService.ChangeOrdenReparacion(idOrdenDeReparacion, _model.NumeroOrden, _model.FechaEntrega,
                                                            _model.IdEstado, _model.IdPersonalAsignado, _model.SubTotal,
                                                            _model.ImporteIva, _model.Diagnostico, _model.IdRecepcion,
                                                            _model.IdVehiculo, _informacionUsuario.CodigoTaller);
      }
      
      GuardarLineasArticulo(idOrdenDeReparacion, _model.LineasArticuloDeLaOrdenDeReparacion);
      GuardarLineasManoDeObra(idOrdenDeReparacion, _model.LineasManoDeObraDeLaOrdenDeReparacion);
      GuardarLineasPaqueteDeServicio(idOrdenDeReparacion, _model.LineasPaqueteDeServicioDeLaOrdenDeReparacion);
      
      MessageBox.Show(Resources.OrdenDeReparacionPresenter_GuardarOrdenDeReparacion_Orden_de_Reparación_guardada_con_éxito,
                      Resources.OrdenDeReparacionPresenter_GuardarOrdenDeReparacion_Guardar_orden_de_reparación, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="ordenDeReparacionId">Id de la entidad a cargar</param>
    public void SetOrdenDeReparacion(Guid ordenDeReparacionId)
    {
      _model = _modelFactory.CrearOrdenDeReparacionViewModel(ordenDeReparacionId);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormArticulos()
    {
      AlGuardarElArticuloOSeleccionarloDesdeFormArticulosAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormArticulos));
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormManosDeObra()
    {
      AlGuardarLaManoDeObraOSeleccionarlaDesdeFormManosDeObraAñadirlaALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormManosDeObra));
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormPaquetesDeServicio()
    {
      AlGuardarElPaqueteDeServicioOSeleccionarloDesdeFormPaquetesDeServicioAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormPaquetesDeServicio));
    }

    public void MostrarFormPaqueteDeServicio(OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel lineaPaqueteDeServicioViewModel)
    {
      // TODO: Ver detalles del paquete de servicio desde las órdenes de reparación
      MessageBox.Show("Esta funcionalidad no está implementada");
    }

    /// <summary>
    /// Añade líneas de artículos a la orden de reparación
    /// </summary>
    /// <param name="buscarArticuloViewModel">Líneas de artículos a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarArticuloArticuloSeleccionado, ParameterType = typeof(BuscarArticuloViewModel))]
    public void AñadirLineasArticuloALaOrdenDeReparacion(BuscarArticuloViewModel buscarArticuloViewModel)
    {
      foreach (var articulo in buscarArticuloViewModel.ListaBuscarArticulos)
      {
        if (articulo.Seleccionar)
          _model.AñadirLineaArticulo(articulo);
      }
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea artículo a la orden de reparación
    /// </summary>
    /// <param name="lineaArticuloViewModel">Línea de artículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, ParameterType = typeof(OrdenDeReparacionViewModel.LineaArticuloViewModel))]
    public void AñadirLineaArticuloALaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.AñadirLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de artículo de la orden de reparación
    /// </summary>
    /// <param name="lineaArticuloViewModel">Linea de artículo a eliminar.</param>
    public void EliminarLineaArticuloDeLaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.EliminarLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de mano de obra a la orden de reparación
    /// </summary>
    /// <param name="manoDeObraSeleccionada">Mano de obra a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarManoDeObraManoDeObraSeleccionada, ParameterType = typeof(BuscarManoDeObraListItemModel))]
    public void AñadirLineaManoDeObraAlPaqueteDeServicio(BuscarManoDeObraListItemModel manoDeObraSeleccionada)
    {
      _model.AñadirLineaManoDeObra(manoDeObraSeleccionada);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de mano de obra a la orden de reparación
    /// </summary>
    /// <param name="lineaManoDeObraViewModel">Línea de mano de obra a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirManoDeObraALaOrdenDeReparacion, ParameterType = typeof(OrdenDeReparacionViewModel.LineaManoDeObraViewModel))]
    public void AñadirLineaManoDeObraALaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      _model.AñadirLineaManoDeObra(lineaManoDeObraViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de mano de obra de la orden de reparación
    /// </summary>
    /// <param name="lineaManoDeObraViewModel">Linea de mano de obra a eliminar.</param>
    public void EliminarLineaManoDeObraDeLaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      _model.EliminarLineaManoDeObra(lineaManoDeObraViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade paquetes de servicio a la orden de reparación
    /// </summary>
    /// <param name="buscarPaqueteDeServicioViewModel">Paquetes de servicio a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarPaqueteDeServicioPaqueteDeServicioSeleccionado, ParameterType = typeof(BuscarPaqueteDeServicioViewModel))]
    public void AñadirPaqueteDeServicioALaOrdenDeReparacion(BuscarPaqueteDeServicioViewModel buscarPaqueteDeServicioViewModel)
    {
      foreach (var paqueteDeServicio in buscarPaqueteDeServicioViewModel.ListaBuscarPaquetesDeServicio)
      {
        if (paqueteDeServicio.Seleccionar)
          _model.AñadirLineaPaqueteDeServicio(paqueteDeServicio);
      }
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de paquete de servicio a la orden de reparación
    /// </summary>
    /// <param name="lineaPaqueteDeServicioViewModel">Línea de paquete de servicio a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, ParameterType = typeof(OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel))]
    public void AñadirPaqueteDeServicioALaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel lineaPaqueteDeServicioViewModel)
    {
      _model.AñadirLineaPaqueteDeServicio(lineaPaqueteDeServicioViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de paquete de servicio de la orden de reparación
    /// </summary>
    /// <param name="lineaPaqueteDeServicioViewModel">Linea de paquete de servicio a eliminar.</param>
    public void EliminarPaqueteDeServicioDeLaOrdenDeReparacion(OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel lineaPaqueteDeServicioViewModel)
    {
      _model.EliminarLineaPaqueteDeServicio(lineaPaqueteDeServicioViewModel);
      NotifyPropertyChanged<OrdenDeReparacionPresenter>(t => t.Model);
    }
    
    #endregion

    #region Metodos Privados
    
    /// <summary>
    /// Guarda las lineas de reparación (artículos) en la base de datos
    /// </summary>
    /// <param name="idOrdenDeReparacion">Id de la orden de reparación cargada en la vista.</param>
    /// <param name="lineasArticuloViewModel">Lista de artículos visualizados en la vista.</param>
    private void GuardarLineasArticulo(Guid idOrdenDeReparacion, IEnumerable<OrdenDeReparacionViewModel.LineaArticuloViewModel> lineasArticuloViewModel)
    {
      foreach (var lineaArticuloViewModel in lineasArticuloViewModel)
      {
        var reparacion = new Reparacion
                           {
                             Id = Guid.NewGuid(),
                             IdOrdenReparacion = idOrdenDeReparacion,
                             IdEmpleado = _model.IdPersonalAsignado
                           };

        var contenidoReparacion = new ContenidoReparacion
                                    {
                                      Id = Guid.NewGuid(),
                                      IdReparacion = reparacion.Id,
                                      IdArticulo = lineaArticuloViewModel.Id,
                                      Cantidad = lineaArticuloViewModel.Cantidad
                                    };

        _gestionReparacionService.AddReparacion(reparacion);
        _gestionContenidoReparacionService.AddContenidoReparacion(contenidoReparacion);
      }
    }

    /// <summary>
    /// Guarda las lineas de reparación (manos de obra) en la base de datos
    /// </summary>
    /// <param name="idOrdenDeReparacion">Id de la orden de reparación cargada en la vista.</param>
    /// <param name="lineasManoDeObraViewModel">Lista de manos de obra visualizados en la vista.</param>
    private void GuardarLineasManoDeObra(Guid idOrdenDeReparacion, IEnumerable<OrdenDeReparacionViewModel.LineaManoDeObraViewModel> lineasManoDeObraViewModel)
    {
      foreach (var lineaManoDeObraViewModel in lineasManoDeObraViewModel)
      {
        var reparacion = new Reparacion
        {
          Id = Guid.NewGuid(),
          DuracionSugerida = lineaManoDeObraViewModel.DuracionSugerida,
          IdOrdenReparacion = idOrdenDeReparacion,
          IdEmpleado = _model.IdPersonalAsignado
        };

        var contenidoReparacion = new ContenidoReparacion
        {
          Id = Guid.NewGuid(),
          IdReparacion = reparacion.Id,
          IdManoDeObra = lineaManoDeObraViewModel.Id,
          Cantidad = lineaManoDeObraViewModel.Cantidad
        };

        _gestionReparacionService.AddReparacion(reparacion);
        _gestionContenidoReparacionService.AddContenidoReparacion(contenidoReparacion);
      }
    }

    /// <summary>
    /// Guarda las lineas de reparación (paquetes de servicio) en la base de datos
    /// </summary>
    /// <param name="idOrdenDeReparacion">Id de la orden de reparación cargada en la vista.</param>
    /// <param name="lineasPaqueteDeServicioViewModel">Lista de paquetes visualizados en la vista.</param>
    private void GuardarLineasPaqueteDeServicio(Guid idOrdenDeReparacion, IEnumerable<OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel> lineasPaqueteDeServicioViewModel)
    {
      foreach (var lineaPaqueteDeServicioViewModel in lineasPaqueteDeServicioViewModel)
      {
        var reparacion = new Reparacion
        {
          Id = Guid.NewGuid(),
          DuracionSugerida = lineaPaqueteDeServicioViewModel.DuracionSugerida,
          IdOrdenReparacion = idOrdenDeReparacion,
          IdEmpleado = _model.IdPersonalAsignado
        };

        var contenidoReparacion = new ContenidoReparacion
        {
          Id = Guid.NewGuid(),
          IdReparacion = reparacion.Id,
          IdPaquete = lineaPaqueteDeServicioViewModel.Id,
          Cantidad = lineaPaqueteDeServicioViewModel.Cantidad
        };

        _gestionReparacionService.AddReparacion(reparacion);
        _gestionContenidoReparacionService.AddContenidoReparacion(contenidoReparacion);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos de la orden de reparación
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticulosAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulos].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion);
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de las manos de obra
    /// que indique que la nueva mano de obra creada o seleccionada en dicho presenter se añada
    /// a la lista de manos de obra de la orden de reparación
    /// </summary>
    private void AlGuardarLaManoDeObraOSeleccionarlaDesdeFormManosDeObraAñadirlaALaOrdenDeReparacion()
    {
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion = new Timer();
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion.Tick += TimerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion_Tick;
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormManoDeObra].NotifyColleagues(MediatorMessages.AlGuardarLaManoDeObraOSeleccionarlaAñadirlaALaOrdenDeReparacion);
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los paquetes de servicio
    /// que indique que el nuevo paquete creado o seleccionado en dicho presenter se añada
    /// a la lista de paquetes de servicio de la orden de reparación
    /// </summary>
    private void AlGuardarElPaqueteDeServicioOSeleccionarloDesdeFormPaquetesDeServicioAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormManoDeObra].NotifyColleagues(MediatorMessages.AlGuardarElPaqueteDeServicioOSeleccionarloAñadirloALaOrdenDeReparacion);
    }
    
    #endregion
  }
}
