﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de la orden de reparación
  /// </summary>
  public class OrdenDeReparacionViewModel : ModelBase, IPersistentModel
  {
    #region Modelos Auxiliares
    public class LineaArticuloViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Descripcion { get; set; }
      public decimal? Cantidad { get; set; }
      public decimal? PrecioUnidad { get; set; }
      public decimal? ImporteLinea { get; set; }
    }

    public class LineaManoDeObraViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Descripcion { get; set; }
      public decimal? Cantidad { get; set; }
      public decimal? DuracionSugerida { get; set; }
      public decimal? PrecioUnidad { get; set; }
      public decimal? ImporteLinea { get; set; }
    }

    public class LineaPaqueteDeServicioViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Descripcion { get; set; }
      public decimal? Cantidad { get; set; }
      public decimal? DuracionSugerida { get; set; }
      public decimal? PrecioUnidad { get; set; }
      public decimal? ImporteLinea { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionArticuloService _gestionArticuloService;
    private BindingList<LineaArticuloViewModel> _lineasArticuloDeLaOrdenDeReparacion = new BindingList<LineaArticuloViewModel>();
    private BindingList<LineaManoDeObraViewModel> _lineasManoDeObraDeLaOrdenDeReparacion = new BindingList<LineaManoDeObraViewModel>();
    private BindingList<LineaPaqueteDeServicioViewModel> _lineasPaqueteDeServicioDeLaOrdenDeReparacion = new BindingList<LineaPaqueteDeServicioViewModel>();

    #endregion

    #region Constructores
    public OrdenDeReparacionViewModel(IGestionArticuloService gestionArticuloService)
    {
      _gestionArticuloService = gestionArticuloService;
    }

    #endregion

    #region Propiedades

    #region Orden reparación

    public virtual Guid Id { get; set; }
    public virtual string NumeroOrden { get; set; }
    public virtual DateTime? FechaRecepcion { get; set; }
    public virtual DateTime? HoraRecepcion { get; set; }
    public virtual DateTime? FechaEntrega { get; set; }
    public virtual DateTime? HoraEntrega { get; set; }
    public virtual int? IdEstado { get; set; }
    public virtual Guid? IdPersonalAsignado { get; set; }
    public virtual decimal? SubTotal { get; set; }
    public virtual decimal? ImporteIva { get; set; }
    public virtual decimal? ImporteTotal { get; set; }
    public virtual Guid? IdRecepcion { get; set; }

    #endregion

    #region Cliente

    public virtual Guid? IdCliente { get; set; }
    public virtual string NifCif { get; set; }
    public virtual string Nombre { get; set; }
    public virtual string ListaTelefonos { get; set; }
    public virtual string Observaciones { get; set; }

    #endregion

    #region Vehículo

    public virtual Guid? IdVehiculo { get; set; }
    public virtual string Matricula { get; set; }
    public virtual string Marca { get; set; }
    public virtual string Modelo { get; set; }
    public virtual string Color { get; set; }
    public virtual DateTime? Año { get; set; }
    public virtual string Bastidor { get; set; }

    #endregion

    #region Observaciones

    public virtual string Prediagnostico { get; set; }
    public virtual string Diagnostico { get; set; }

    #endregion

    #region Líneas

    public BindingList<LineaArticuloViewModel> LineasArticuloDeLaOrdenDeReparacion { get { return _lineasArticuloDeLaOrdenDeReparacion; } }
    public BindingList<LineaManoDeObraViewModel> LineasManoDeObraDeLaOrdenDeReparacion { get { return _lineasManoDeObraDeLaOrdenDeReparacion; } }
    public BindingList<LineaPaqueteDeServicioViewModel> LineasPaqueteDeServicioDeLaOrdenDeReparacion { get { return _lineasPaqueteDeServicioDeLaOrdenDeReparacion; } }

    #endregion

    #endregion

    #region Métodos Públicos

    public List<Articulo> ObtenerListaArticulos(IEnumerable<LineaArticuloViewModel> lineasArticuloViewModel)
    {
      return
        lineasArticuloViewModel.Select(
          lineaArticuloViewModel => _gestionArticuloService.BuscarArticuloPorId(lineaArticuloViewModel.Id).First()).ToList();
    }

    public void SetLineasArticuloDeLaOrdenDeReparacion(BindingList<LineaArticuloViewModel> lineasArticuloDeLaOrdenDeReparacion)
    {
      _lineasArticuloDeLaOrdenDeReparacion = lineasArticuloDeLaOrdenDeReparacion;
    }

    public void EliminarLineaArticulo(LineaArticuloViewModel lineaArticuloViewModel)
    {
      LineasArticuloDeLaOrdenDeReparacion.Remove(lineaArticuloViewModel);
      ActualizarImportes();
    }

    public void AñadirLineaArticulo(BuscarArticuloListItemModel buscarArticuloListItemModel)
    {
      var nuevaLineaArticulo = new LineaArticuloViewModel
      {
        Id = buscarArticuloListItemModel.ArticuloId,
        Descripcion = buscarArticuloListItemModel.Descripcion,
        Cantidad = 1,
        PrecioUnidad = buscarArticuloListItemModel.CosteActual,
        ImporteLinea = buscarArticuloListItemModel.CosteActual
      };
      _lineasArticuloDeLaOrdenDeReparacion.Add(nuevaLineaArticulo);
      ActualizarImportes();
    }

    public void AñadirLineaArticulo(LineaArticuloViewModel lineaArticuloViewModel)
    {
      var nuevaLineaArticulo = new LineaArticuloViewModel
                                 {
                                   Id = lineaArticuloViewModel.Id,
                                   Descripcion = lineaArticuloViewModel.Descripcion,
                                   Cantidad = lineaArticuloViewModel.Cantidad,
                                   PrecioUnidad = lineaArticuloViewModel.PrecioUnidad,
                                   ImporteLinea = lineaArticuloViewModel.ImporteLinea
                                 };
      _lineasArticuloDeLaOrdenDeReparacion.Add(nuevaLineaArticulo);
      ActualizarImportes();
    }

    public void SetLineasManoDeObraDeLaOrdenDeReparacion(BindingList<LineaManoDeObraViewModel> lineasManoDeObraDeLaOrdenDeReparacion)
    {
      _lineasManoDeObraDeLaOrdenDeReparacion = lineasManoDeObraDeLaOrdenDeReparacion;
    }

    public void EliminarLineaManoDeObra(LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      LineasManoDeObraDeLaOrdenDeReparacion.Remove(lineaManoDeObraViewModel);
      ActualizarImportes();
    }

    public void AñadirLineaManoDeObra(LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      var nuevaLineaManoDeObra = new LineaManoDeObraViewModel
                                   {
                                     Id = lineaManoDeObraViewModel.Id,
                                     Descripcion = lineaManoDeObraViewModel.Descripcion,
                                     DuracionSugerida = lineaManoDeObraViewModel.DuracionSugerida,
                                     Cantidad = lineaManoDeObraViewModel.Cantidad,
                                     PrecioUnidad = lineaManoDeObraViewModel.PrecioUnidad,
                                     ImporteLinea = lineaManoDeObraViewModel.ImporteLinea
                                   };
      _lineasManoDeObraDeLaOrdenDeReparacion.Add(nuevaLineaManoDeObra);
      ActualizarImportes();
    }

    public void AñadirLineaManoDeObra(BuscarManoDeObraListItemModel buscarManoDeObraListItemModel)
    {
      var nuevaLineaManoDeObra = new LineaManoDeObraViewModel
      {
        Id = buscarManoDeObraListItemModel.ManoDeObraId,
        Descripcion = buscarManoDeObraListItemModel.Descripcion,
        DuracionSugerida = buscarManoDeObraListItemModel.Duracion,
        Cantidad = 1,
        PrecioUnidad = buscarManoDeObraListItemModel.PrecioPorHora,
        ImporteLinea = (buscarManoDeObraListItemModel.PrecioPorHora * buscarManoDeObraListItemModel.Duracion) / 60
      };
      _lineasManoDeObraDeLaOrdenDeReparacion.Add(nuevaLineaManoDeObra);
      ActualizarImportes();
    }

    public void SetLineasPaqueteDeServicioDeLaOrdenDeReparacion(BindingList<LineaPaqueteDeServicioViewModel> lineasPaqueteDeServicioDeLaOrdenDeReparacion)
    {
      _lineasPaqueteDeServicioDeLaOrdenDeReparacion = lineasPaqueteDeServicioDeLaOrdenDeReparacion;
    }

    public void EliminarLineaPaqueteDeServicio(LineaPaqueteDeServicioViewModel lineaPaqueteDeServicioViewModel)
    {
      LineasPaqueteDeServicioDeLaOrdenDeReparacion.Remove(lineaPaqueteDeServicioViewModel);
      ActualizarImportes();
    }

    public void AñadirLineaPaqueteDeServicio(LineaPaqueteDeServicioViewModel lineaPaqueteDeServicioViewModel)
    {
      var nuevaLineaPaqueteDeServicio = new LineaPaqueteDeServicioViewModel
                                          {
                                            Id = lineaPaqueteDeServicioViewModel.Id,
                                            Descripcion = lineaPaqueteDeServicioViewModel.Descripcion,
                                            DuracionSugerida = lineaPaqueteDeServicioViewModel.DuracionSugerida,
                                            Cantidad = lineaPaqueteDeServicioViewModel.Cantidad,
                                            PrecioUnidad = lineaPaqueteDeServicioViewModel.PrecioUnidad,
                                            ImporteLinea = lineaPaqueteDeServicioViewModel.ImporteLinea
                                          };
      _lineasPaqueteDeServicioDeLaOrdenDeReparacion.Add(nuevaLineaPaqueteDeServicio);
      ActualizarImportes();
    }

    public void AñadirLineaPaqueteDeServicio(BuscarPaqueteDeServicioListItemModel buscarPaqueteDeServicioListItemModel)
    {
      var nuevaLineaPaqueteDeServicio = new LineaPaqueteDeServicioViewModel
      {
        Id = buscarPaqueteDeServicioListItemModel.PaqueteDeServicioId,
        Descripcion = buscarPaqueteDeServicioListItemModel.Descripcion,
        DuracionSugerida = buscarPaqueteDeServicioListItemModel.DuracionSugerida,
        Cantidad = 1,
        PrecioUnidad = buscarPaqueteDeServicioListItemModel.Precio,
        ImporteLinea = buscarPaqueteDeServicioListItemModel.Precio
      };
      _lineasPaqueteDeServicioDeLaOrdenDeReparacion.Add(nuevaLineaPaqueteDeServicio);
      ActualizarImportes();
    }

    public void ActualizarImportes()
    {
      var subTotal =
        _lineasArticuloDeLaOrdenDeReparacion.Sum(
          lineaArticuloViewModel => lineaArticuloViewModel.ImporteLinea ?? 0) +
        _lineasManoDeObraDeLaOrdenDeReparacion.Sum(
          lineaManoDeObraViewModel => lineaManoDeObraViewModel.ImporteLinea ?? 0) +
        _lineasPaqueteDeServicioDeLaOrdenDeReparacion.Sum(
          lineaPaqueteDeServicioViewModel => lineaPaqueteDeServicioViewModel.ImporteLinea ?? 0);

      SubTotal = subTotal;
      ImporteIva = (subTotal * InformacionUsuario.PorcentajeIva) / 100;
      ImporteTotal = SubTotal + ImporteIva;
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion
  }
}
