﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista de la orden de reparacion,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class OrdenDeReparacionViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<int>> _estados = new BindingList<LookUpItem<int>>();
    private IList<LookUpItem<Guid>> _empleados = new BindingList<LookUpItem<Guid>>();
    private IList<LookUpItem<string>> _marcas = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _modelos = new BindingList<LookUpItem<string>>();

    #endregion

    #region Constructores

    #endregion

    #region Propiedades

    public IEnumerable<LookUpItem<int>> Estados { get { return _estados; } }
    public IEnumerable<LookUpItem<Guid>> Empleados { get { return _empleados; } }
    public IEnumerable<LookUpItem<string>> Marcas { get { return _marcas; } }
    public IEnumerable<LookUpItem<string>> Modelos { get { return _modelos; } }
    
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece las marcas en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="marcas">Marcas a poner en correspondiente lista LookUp.</param>
    public void SetMarcas(IList<Marca> marcas)
    {
      _marcas =
        marcas.Select(marca => new LookUpItem<string> { Codigo = marca.Id, Descripcion = marca.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los modelos en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="modelos">Modelos a poner en correspondiente lista LookUp.</param>
    public void SetModelos(IList<Modelo> modelos)
    {
      _modelos =
        modelos.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los estados de la orden en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="estadosOrdenReparacion">Estados de la orden a poner en correspondiente lista LookUp.</param>
    public void SetEstados(IList<EstadoOrdenReparacion> estadosOrdenReparacion)
    {
      _estados =
        estadosOrdenReparacion.Select(estado => new LookUpItem<int> { Codigo = estado.Id, Descripcion = estado.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los empleados en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="empleados">Empleados a poner en correspondiente lista LookUp.</param>
    public void SetEmpleados(IList<Empleado> empleados)
    {
      _empleados =
        empleados.Select(empleado => new LookUpItem<Guid> { Codigo = empleado.Id, Descripcion = empleado.Nombre }).ToList();
    }
    
    #endregion
  }
}
