﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEstadosOrdenReparacion;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista de la orden de reparación,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class OrdenDeReparacionViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<OrdenDeReparacionViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionEstadoOrdenReparacionService _gestionEstadoOrdenReparacionService;
    private readonly IGestionEmpleadoService _gestionEmpleadoService;
    private readonly IGestionMarcaService _gestionMarcaService;
    private readonly IGestionModeloService _gestionModeloService;
    #endregion

    #region Constructores

    public OrdenDeReparacionViewModelReadOnlyFactory(Func<OrdenDeReparacionViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionEstadoOrdenReparacionService gestionEstadoOrdenReparacionService,
                                           IGestionEmpleadoService gestionEmpleadoService, IGestionMarcaService gestionMarcaService, IGestionModeloService gestionModeloService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionEstadoOrdenReparacionService = gestionEstadoOrdenReparacionService;
      _gestionEmpleadoService = gestionEmpleadoService;
      _gestionMarcaService = gestionMarcaService;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo OrdenDeReparacionViewModelReadOnly vacío</returns>
    public OrdenDeReparacionViewModelReadOnly CreateOrdenDeReparacionViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetEstados(_gestionEstadoOrdenReparacionService.BuscarTodosLosEstadosOrdenReparacion());
      model.SetEmpleados(_gestionEmpleadoService.BuscarTodosLosEmpleados());
      model.SetMarcas(_gestionMarcaService.BuscarTodasLasMarcas());
      model.SetModelos(_gestionModeloService.BuscarTodosLosModelos());
      return model;
    }

    #endregion
  }
}
