﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio;

namespace GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio
{
  public partial class FormPaqueteDeServicio : BindableFormBase
  {
    #region Atributos

    private readonly PaqueteDeServicioPresenter _paqueteDeServicioPresenter;

    #endregion

    #region Constructores

    public FormPaqueteDeServicio(PaqueteDeServicioPresenter paqueteDeServicioPresenter)
    {
      InitializeComponent();
      _paqueteDeServicioPresenter = paqueteDeServicioPresenter;
      ConfigurarAyuda(helpProviderFormPaqueteDeServicio);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaPaquetesDeServicioBuscados()
    {
      dockPanelBuscarPaqueteDeServicio.Show();
    }

    private void OcultarListaPaquetesDeServicioBuscados()
    {
      dockPanelBuscarPaqueteDeServicio.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void FormPaqueteDeServicio_Load(object sender, EventArgs e)
    {
      if (_paqueteDeServicioPresenter != null)
      {
        _paqueteDeServicioPresenter.NuevoPaqueteDeServicio();
        _paqueteDeServicioPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _paqueteDeServicioPresenter;
      }
      helpProviderFormPaqueteDeServicio.SetShowHelp(this, true);
    }

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormPaqueteDeServicio.HelpNamespace);
    }

    private void simpleButtonTransferir_Click(object sender, EventArgs e)
    {
      gridViewListaPaquetes_DoubleClick(sender, e);
    }

    private void gridViewListaPaquetes_DoubleClick(object sender, EventArgs e)
    {
      var paqueteSeleccionado = gridViewListaPaquetes.GetFocusedRow() as BuscarPaqueteDeServicioListItemModel;
      if (paqueteSeleccionado != null) _paqueteDeServicioPresenter.SetPaqueteDeServicio(paqueteSeleccionado.PaqueteDeServicioId);
      OcultarListaPaquetesDeServicioBuscados();
    }

    private void simpleButtonAgregarArticulo_Click(object sender, EventArgs e)
    {
      _paqueteDeServicioPresenter.MostrarFormArticulo();
    }

    private void simpleButtonEliminarArticulo_Click(object sender, EventArgs e)
    {
      var articuloSeleccionado = gridViewLineasArticulo.GetFocusedRow() as PaqueteDeServicioViewModel.LineaArticuloViewModel;
      if (articuloSeleccionado != null)
        _paqueteDeServicioPresenter.EliminarLineaArticuloDelPaqueteDeServicio(articuloSeleccionado);
    }

    private void simpleButtonAgregarManoDeObra_Click(object sender, EventArgs e)
    {
      _paqueteDeServicioPresenter.MostrarFormManoDeObra();
    }

    private void simpleButtonEliminarManoDeObra_Click(object sender, EventArgs e)
    {
      var manoDeObraSeleccionada = gridViewLineasManoDeObra.GetFocusedRow() as PaqueteDeServicioViewModel.LineaManoDeObraViewModel;
      if (manoDeObraSeleccionada != null)
        _paqueteDeServicioPresenter.EliminarLineaManoDeObraDelPaqueteDeServicio(manoDeObraSeleccionada);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _paqueteDeServicioPresenter.NuevoPaqueteDeServicio();
      textEditDescripcionPaquete.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _paqueteDeServicioPresenter.GuardarPaqueteDeServicio();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _paqueteDeServicioPresenter.NuevoPaqueteDeServicio();
      textEditDescripcionPaquete.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _paqueteDeServicioPresenter.BuscarPaqueteDeServicio();
      MostrarListaPaquetesDeServicioBuscados();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _paqueteDeServicioPresenter.EliminarPaqueteDeServicio();
    }

    #endregion
  }
}
