﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetes;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormPaqueteDeServicio)]
  [RegisterInMediator(MediatorEnumerator.BuscarArticulo)]
  [RegisterInMediator(MediatorEnumerator.BuscarManoDeObra)]
  public class PaqueteDeServicioPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private PaqueteDeServicioViewModel _model;
    private readonly PaqueteDeServicioViewModelFactory _modelFactory;

    private readonly BuscarPaqueteDeServicioPresenter _buscarPaqueteDeServicioPresenter;

    private readonly IGestionPaqueteDeServicioService _gestionPaqueteDeServicioService;
    private readonly IGestionPaqueteService _gestionPaqueteService;

    #endregion

    #region Constructores

    public PaqueteDeServicioPresenter(PaqueteDeServicioViewModelFactory modelFactory,
                            IMediatorManager mediatorManager,
                            BuscarPaqueteDeServicioPresenter buscarPaqueteDeServicioPresenter,
                            IGestionPaqueteDeServicioService gestionPaqueteDeServicioService,
                            IGestionPaqueteService gestionPaqueteService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarPaqueteDeServicioPresenter = buscarPaqueteDeServicioPresenter;
      _gestionPaqueteDeServicioService = gestionPaqueteDeServicioService;
      _gestionPaqueteService = gestionPaqueteService;
    }

    #endregion

    #region Propiedades

    public PaqueteDeServicioViewModel Model
    {
      get { return _model; }
    }

    public BindingList<BuscarPaqueteDeServicioListItemModel> ListaBuscarPaqueteDeServicioViewModel
    {
      get { return _buscarPaqueteDeServicioPresenter.Model.ListaBuscarPaquetesDeServicio; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      /*
      Model.OnChanged(c => Model.IdProvincia).Do(x =>
                                                   {
                                                     ModelReadOnly.SetPoblaciones(_gestionPoblacionService.BuscarListaPoblacionPorIdProvincia(Model.IdProvincia));
                                                     NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.ModelReadOnly);
                                                   });
      */
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevoPaqueteDeServicio()
    {
      _model = _modelFactory.CrearPaqueteDeServicioViewModel();
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaPaqueteDeServicio()
    {
      _model = _modelFactory.CrearPaqueteDeServicioViewModel();
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de clientes
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarPaqueteDeServicio()
    {
      _buscarPaqueteDeServicioPresenter.Buscar(_model.Descripcion);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.ListaBuscarPaqueteDeServicioViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarPaqueteDeServicio()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_, Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_Eliminar_paquete, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionPaqueteDeServicioService.RemovePaqueteDeServicio(_model.Id);
          NuevoPaqueteDeServicio();
          MessageBox.Show(Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_Paquete_de_servicio_eliminado_con_éxito,
                          Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_Eliminar_paquete, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_No_se_puede_eliminar_el_paquete_de_servicio_porque_todavía_no_se_ha_guardado_nada,
                          Resources.PaqueteDeServicioPresenter_EliminarPaqueteDeServicio_Eliminar_paquete, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarPaqueteDeServicio()
    {
      Guid idPaqueteDeServicio;
      if (!_model.HasBeenPersisted())
      {
        idPaqueteDeServicio = Guid.NewGuid();
        _gestionPaqueteDeServicioService.AddPaqueteDeServicio(idPaqueteDeServicio, _model.Descripcion);}
      else
      {
        idPaqueteDeServicio = _model.Id;
        _gestionPaqueteDeServicioService.ChangePaqueteDeServicio(idPaqueteDeServicio, _model.Descripcion);
      }

      GuardarLineasArticulo(idPaqueteDeServicio, _model.LineasArticuloDelPaqueteDeServicio);
      GuardarLineasManoDeObra(idPaqueteDeServicio, _model.LineasManoDeObraDelPaqueteDeServicio);

      MessageBox.Show(Resources.PaqueteDeServicioPresenter_GuardarPaqueteDeServicio_Paquete_de_servicio_guardado_con_éxito,
                      Resources.PaqueteDeServicioPresenter_GuardarPaqueteDeServicio_Guardar_paquete_de_servicio, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="paqueteDeServicioId">Id de la entidad a cargar</param>
    public void SetPaqueteDeServicio(Guid paqueteDeServicioId)
    {
      _model = _modelFactory.CrearPaqueteDeServicioViewModel(paqueteDeServicioId);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormArticulo()
    {
      AlGuardarElArticuloOSeleccionarloDesdeFormArticulosAñadirloAlPaqueteDeServicio();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormArticulos));
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormManoDeObra()
    {
      AlGuardarLaManoDeObraOSeleccionarlaDesdeFormManosDeObraAñadirlaAlPaqueteDeServicio();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormManosDeObra));
    }

    /// <summary>
    /// Añade líneas de artículos al paquete de servicio
    /// </summary>
    /// <param name="buscarArticuloViewModel">Líneas de artículos a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarArticuloArticuloSeleccionado, ParameterType = typeof(BuscarArticuloViewModel))]
    public void AñadirLineasArticuloALaOrdenDeReparacion(BuscarArticuloViewModel buscarArticuloViewModel)
    {
      foreach (var articulo in buscarArticuloViewModel.ListaBuscarArticulos)
      {
        if (articulo.Seleccionar)
          _model.AñadirLineaArticulo(articulo);
      }
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea artículo al paquete de servicio
    /// </summary>
    /// <param name="lineaArticuloViewModel">Línea de artículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirArticuloAlPaqueteDeServicio, ParameterType = typeof(PaqueteDeServicioViewModel.LineaArticuloViewModel))]
    public void AñadirLineaArticuloAlPaqueteDeServicio(PaqueteDeServicioViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.AñadirLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de artículo de la orden de reparación
    /// </summary>
    /// <param name="lineaArticuloViewModel">Linea de artículo a eliminar.</param>
    public void EliminarLineaArticuloDelPaqueteDeServicio(PaqueteDeServicioViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.EliminarLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de mano de obra al paquete de servicio
    /// </summary>
    /// <param name="manoDeObraSeleccionada">Mano de obra a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarManoDeObraManoDeObraSeleccionada, ParameterType = typeof(BuscarManoDeObraListItemModel))]
    public void AñadirLineaManoDeObraAlPaqueteDeServicio(BuscarManoDeObraListItemModel manoDeObraSeleccionada)
    {
      _model.AñadirLineaManoDeObra(manoDeObraSeleccionada);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de mano de obra al paquete de servicio
    /// </summary>
    /// <param name="lineaManoDeObraViewModel">Línea de mano de obra a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirManoDeObraAlPaqueteDeServicio, ParameterType = typeof(PaqueteDeServicioViewModel.LineaManoDeObraViewModel))]
    public void AñadirLineaManoDeObraAlPaqueteDeServicio(PaqueteDeServicioViewModel.LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      _model.AñadirLineaManoDeObra(lineaManoDeObraViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de mano de obra de la orden de reparación
    /// </summary>
    /// <param name="lineaManoDeObraViewModel">Linea de mano de obra a eliminar.</param>
    public void EliminarLineaManoDeObraDelPaqueteDeServicio(PaqueteDeServicioViewModel.LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      _model.EliminarLineaManoDeObra(lineaManoDeObraViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade una línea de artículo al paquete de servicio
    /// </summary>
    /// <param name="lineaArticuloViewModel">Línea de artículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirArticuloAlPaqueteDeServicio, ParameterType = typeof(PaqueteDeServicioViewModel.LineaArticuloViewModel))]
    public void AñadirPaqueteDeServicioAlPaqueteDeServicio(PaqueteDeServicioViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.AñadirLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    /// <summary>
    /// Eliminar una línea de artículo al paquete de servicio
    /// </summary>
    /// <param name="lineaArticuloViewModel">Linea de artículo a eliminar.</param>
    public void EliminarPaqueteDeServicioDeLaPaqueteDeServicio(PaqueteDeServicioViewModel.LineaArticuloViewModel lineaArticuloViewModel)
    {
      _model.EliminarLineaArticulo(lineaArticuloViewModel);
      NotifyPropertyChanged<PaqueteDeServicioPresenter>(t => t.Model);
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Guarda las lineas de artículos en la base de datos
    /// </summary>
    /// <param name="idPaqueteDeServicio">Id del paquete de servicio cargado en la vista.</param>
    /// <param name="lineasArticuloViewModel">Lista de artículos visualizados en la vista.</param>
    private void GuardarLineasArticulo(Guid idPaqueteDeServicio, IEnumerable<PaqueteDeServicioViewModel.LineaArticuloViewModel> lineasArticuloViewModel)
    {
      foreach (var paquete in lineasArticuloViewModel.Select(lineaArticuloViewModel => new Paquete
                                                                                         {
                                                                                           IdPaqueteDeServicio = idPaqueteDeServicio,
                                                                                           IdArticulo = lineaArticuloViewModel.Id,
                                                                                           Cantidad = lineaArticuloViewModel.Cantidad
                                                                                         }))
      {
        _gestionPaqueteService.AddPaquete(paquete);
      }
    }

    /// <summary>
    /// Guarda las lineas de manos de obra en la base de datos
    /// </summary>
    /// <param name="idPaqueteDeServicio">Id de la orden de reparación cargada en la vista.</param>
    /// <param name="lineasManoDeObraViewModel">Lista de manos de obra visualizadas en la vista.</param>
    private void GuardarLineasManoDeObra(Guid idPaqueteDeServicio, IEnumerable<PaqueteDeServicioViewModel.LineaManoDeObraViewModel> lineasManoDeObraViewModel)
    {
      foreach (var paquete in lineasManoDeObraViewModel.Select(lineaManoDeObraViewModel => new Paquete
                                                                                             {
                                                                                               IdPaqueteDeServicio = idPaqueteDeServicio,
                                                                                               IdManoDeObra = lineaManoDeObraViewModel.Id,
                                                                                               Cantidad = lineaManoDeObraViewModel.Cantidad
                                                                                             }))
      {
        _gestionPaqueteService.AddPaquete(paquete);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos del paquete de servicio
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticulosAñadirloAlPaqueteDeServicio()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloAlPaqueteDeServicio);
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de las manos de obra
    /// que indique que la nueva mano de obra creada o seleccionada en dicho presenter se añada
    /// a la lista de manos de obra del paquete de servicio
    /// </summary>
    private void AlGuardarLaManoDeObraOSeleccionarlaDesdeFormManosDeObraAñadirlaAlPaqueteDeServicio()
    {
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio = new Timer();
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio.Tick += TimerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio_Tick;
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio.Interval = 1000;
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio.Enabled = true;
    }
    Timer _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio = new Timer();
    private void TimerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormManoDeObra].NotifyColleagues(MediatorMessages.AlGuardarLaManoDeObraOSeleccionarlaAñadirlaAlPaqueteDeServicio);
    }

    #endregion
  }
}
