﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra;

namespace GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de la orden de reparación
  /// </summary>
  public class PaqueteDeServicioViewModel : ModelBase, IPersistentModel
  {
    #region Modelos Auxiliares
    public class LineaArticuloViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Descripcion { get; set; }
      public decimal Cantidad { get; set; }
      public decimal? PrecioUnidad { get; set; }
      public decimal? ImporteLinea { get; set; }
    }

    public class LineaManoDeObraViewModel : ModelBase
    {
      public Guid Id { get; set; }
      public string Descripcion { get; set; }
      public decimal Cantidad { get; set; }
      public decimal? PrecioUnidad { get; set; }
      public decimal? ImporteLinea { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionArticuloService _gestionArticuloService;
    private BindingList<LineaArticuloViewModel> _lineasArticuloDelPaqueteDeServicio = new BindingList<LineaArticuloViewModel>();
    private BindingList<LineaManoDeObraViewModel> _lineasManoDeObraDelPaqueteDeServicio = new BindingList<LineaManoDeObraViewModel>();

    #endregion

    #region Constructores
    public PaqueteDeServicioViewModel(IGestionArticuloService gestionArticuloService)
    {
      _gestionArticuloService = gestionArticuloService;
    }

    #endregion

    #region Propiedades

    public virtual Guid Id { get; set; }
    public virtual string Descripcion { get; set; }
    public virtual decimal? Precio { get; set; }

    #region Líneas

    public BindingList<LineaArticuloViewModel> LineasArticuloDelPaqueteDeServicio { get { return _lineasArticuloDelPaqueteDeServicio; } }
    public BindingList<LineaManoDeObraViewModel> LineasManoDeObraDelPaqueteDeServicio { get { return _lineasManoDeObraDelPaqueteDeServicio; } }

    #endregion

    #endregion

    #region Métodos Públicos

    public List<Articulo> ObtenerListaArticulos(IEnumerable<LineaArticuloViewModel> lineasArticuloViewModel)
    {
      return
        lineasArticuloViewModel.Select(
          lineaArticuloViewModel => _gestionArticuloService.BuscarArticuloPorId(lineaArticuloViewModel.Id).First()).ToList();
    }

    public void SetLineasArticuloDelPaqueteDeServicio(BindingList<LineaArticuloViewModel> lineasArticuloDelPaqueteDeServicio)
    {
      _lineasArticuloDelPaqueteDeServicio = lineasArticuloDelPaqueteDeServicio;
      var importeLineasArticulo = lineasArticuloDelPaqueteDeServicio.Aggregate<LineaArticuloViewModel, decimal?>(0, (current, lineaArticuloViewModel) => current + lineaArticuloViewModel.ImporteLinea);
      ActualizarPrecio(importeLineasArticulo);
    }

    public void EliminarLineaArticulo(LineaArticuloViewModel lineaArticuloViewModel)
    {
      LineasArticuloDelPaqueteDeServicio.Remove(lineaArticuloViewModel);
      ActualizarPrecio(lineaArticuloViewModel.ImporteLinea * -1);
    }

    public void AñadirLineaArticulo(LineaArticuloViewModel lineaArticuloViewModel)
    {
      var nuevaLineaArticulo = new LineaArticuloViewModel
                                 {
                                   Id = lineaArticuloViewModel.Id,
                                   Descripcion = lineaArticuloViewModel.Descripcion,
                                   Cantidad = lineaArticuloViewModel.Cantidad,
                                   PrecioUnidad = lineaArticuloViewModel.PrecioUnidad,
                                   ImporteLinea = lineaArticuloViewModel.ImporteLinea
                                 };
      _lineasArticuloDelPaqueteDeServicio.Add(nuevaLineaArticulo);
      ActualizarPrecio(nuevaLineaArticulo.ImporteLinea);
    }

    public void AñadirLineaArticulo(BuscarArticuloListItemModel buscarArticuloListItemModel)
    {
      var nuevaLineaArticulo = new LineaArticuloViewModel
      {
        Id = buscarArticuloListItemModel.ArticuloId,
        Descripcion = buscarArticuloListItemModel.Descripcion,
        Cantidad = 1,
        PrecioUnidad = buscarArticuloListItemModel.CosteActual,
        ImporteLinea = buscarArticuloListItemModel.CosteActual
      };
      _lineasArticuloDelPaqueteDeServicio.Add(nuevaLineaArticulo);
      ActualizarPrecio(nuevaLineaArticulo.ImporteLinea);
    }

    public void SetLineasManoDeObraDelPaqueteDeServicio(BindingList<LineaManoDeObraViewModel> lineasManoDeObraDelPaqueteDeServicio)
    {
      _lineasManoDeObraDelPaqueteDeServicio = lineasManoDeObraDelPaqueteDeServicio;
      var importeLineasManoDeObra= lineasManoDeObraDelPaqueteDeServicio.Aggregate<LineaManoDeObraViewModel, decimal?>(0, (current, lineaManoDeObraViewModel) => current + lineaManoDeObraViewModel.ImporteLinea);
      ActualizarPrecio(importeLineasManoDeObra);
    }

    public void EliminarLineaManoDeObra(LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      LineasManoDeObraDelPaqueteDeServicio.Remove(lineaManoDeObraViewModel);
      ActualizarPrecio(lineaManoDeObraViewModel.ImporteLinea * -1);
    }

    public void AñadirLineaManoDeObra(LineaManoDeObraViewModel lineaManoDeObraViewModel)
    {
      var nuevaLineaManoDeObra = new LineaManoDeObraViewModel
                                   {
                                     Id = lineaManoDeObraViewModel.Id,
                                     Descripcion = lineaManoDeObraViewModel.Descripcion,
                                     Cantidad = lineaManoDeObraViewModel.Cantidad,
                                     PrecioUnidad = lineaManoDeObraViewModel.PrecioUnidad,
                                     ImporteLinea = lineaManoDeObraViewModel.ImporteLinea
                                   };
      _lineasManoDeObraDelPaqueteDeServicio.Add(nuevaLineaManoDeObra);
      ActualizarPrecio(nuevaLineaManoDeObra.ImporteLinea);
    }

    public void AñadirLineaManoDeObra(BuscarManoDeObraListItemModel buscarManoDeObraListItemModel)
    {
      var nuevaLineaManoDeObra = new LineaManoDeObraViewModel
      {
        Id = buscarManoDeObraListItemModel.ManoDeObraId,
        Descripcion = buscarManoDeObraListItemModel.Descripcion,
        Cantidad = 1,
        PrecioUnidad = buscarManoDeObraListItemModel.PrecioPorHora,
        ImporteLinea = buscarManoDeObraListItemModel.PrecioPorHora
      };
      _lineasManoDeObraDelPaqueteDeServicio.Add(nuevaLineaManoDeObra);
      ActualizarPrecio(nuevaLineaManoDeObra.ImporteLinea);
    }

    public void ActualizarPrecio(decimal? importeLinea)
    {
      Precio += importeLinea;
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion
  }
}
