﻿using System.Windows.Forms;
using DevExpress.XtraLayout.Utils;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPresupuesto;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos
{
  public partial class FormPresupuesto : BindableFormBase
  {
    #region Atributos

    private readonly PresupuestoPresenter _presupuestoPresenter;
    #endregion

    #region Constructores

    public FormPresupuesto(PresupuestoPresenter presupuestoPresenter)
    {
      InitializeComponent();
      _presupuestoPresenter = presupuestoPresenter;
      ConfigurarAyuda(helpProviderFormPresupuesto);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaPresupuestosBuscados()
    {
      dockPanelBuscarPresupuesto.Show();
    }

    private void OcultarListaPresupuestosBuscados()
    {
      dockPanelBuscarPresupuesto.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void gridViewListaPresupuestos_DoubleClick(object sender, System.EventArgs e)
    {
      var presupuestoSeleccionado = gridViewListaPresupuestos.GetFocusedRow() as BuscarPresupuestoListItemModel;
      if (presupuestoSeleccionado != null) _presupuestoPresenter.SetPresupuesto(presupuestoSeleccionado.PresupuestoId);
      OcultarListaPresupuestosBuscados();
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {
      gridViewListaPresupuestos_DoubleClick(sender, e);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.NuevoPresupuesto();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.GuardarPresupuesto();
    }

    private void barLargeButtonItemImprimir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.ImprimirPresupuesto();
    }

    private void barLargeButtonItemConvertirPresupuestoEnFactura_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.ConvertirPresupuestoEnFactura();
      Close();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.NuevaBusquedaPresupuesto();
      textEditNumero.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.BuscarPresupuesto();
      MostrarListaPresupuestosBuscados();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _presupuestoPresenter.EliminarPresupuesto();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormPresupuesto.HelpNamespace);
    }

    private void simpleButtonBuscarCliente_Click(object sender, System.EventArgs e)
    {
      _presupuestoPresenter.MostrarFormClienteConNuevoCliente();
    }

    private void simpleButtonAgregarLinea_Click(object sender, System.EventArgs e)
    {
      _presupuestoPresenter.MostrarFormVehiculoVentaConNuevoVehiculo();
    }

    private void simpleButtonEliminarLinea_Click(object sender, System.EventArgs e)
    {
      var lineaSeleccionada = gridViewLineas.GetFocusedRow() as PresupuestoViewModel.LineaPresupuestoViewModel;
      if (lineaSeleccionada != null) _presupuestoPresenter.EliminarVehiculoVentaAlPresupuesto(lineaSeleccionada);
    }

    private void FormPresupuesto_Load(object sender, System.EventArgs e)
    {
      if (_presupuestoPresenter != null)
      {
        _presupuestoPresenter.SetModelReadOnly();
        //_presupuestoPresenter.NuevoPresupuesto();
        _presupuestoPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _presupuestoPresenter;
      }
      helpProviderFormPresupuesto.SetShowHelp(this, true);
    }

    #endregion
  }
}