﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionFormasDePago;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del presupuesto,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class PresupuestoViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<PresupuestoViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionFormaDePagoService _gestionFormaDePagoService;
    private readonly IGestionEmpleadoService _gestionEmpleadoService;
    private readonly IGestionMarcaService _gestionMarcaService;
    private readonly IGestionModeloService _gestionModeloService;

    #endregion

    #region Constructores

    public PresupuestoViewModelReadOnlyFactory(Func<PresupuestoViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionFormaDePagoService gestionFormaDePagoService,
                                           IGestionEmpleadoService gestionEmpleadoService, IGestionMarcaService gestionMarcaService, IGestionModeloService gestionModeloService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionFormaDePagoService = gestionFormaDePagoService;
      _gestionEmpleadoService = gestionEmpleadoService;
      _gestionMarcaService = gestionMarcaService;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo PresupuestoViewModelReadOnly vacío</returns>
    public PresupuestoViewModelReadOnly CreatePresupuestoViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetFormasDePago(_gestionFormaDePagoService.BuscarTodasLasFormasDePago());
      model.SetEmpleados(_gestionEmpleadoService.BuscarTodosLosEmpleados());
      model.SetMarcas(_gestionMarcaService.BuscarTodasLasMarcas());
      model.SetModelos(_gestionModeloService.BuscarTodosLosModelos());
      return model;
    }

    #endregion
  }
}
