﻿using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarProveedor;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Proveedores
{
  public partial class FormProveedor : BindableFormBase
  {
    #region Atributos

    private readonly ProveedorPresenter _proveedorPresenter;

    #endregion

    #region Constructores

    public FormProveedor(ProveedorPresenter proveedorPresenter)
    {
      InitializeComponent();
      _proveedorPresenter = proveedorPresenter;
      ConfigurarAyuda(helpProviderFormProveedor);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaProveedoresBuscados()
    {
      dockPanelBuscarProveedor.Show();
    }

    private void OcultarListaProveedoresBuscados()
    {
      dockPanelBuscarProveedor.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormProveedor.HelpNamespace);
    }

    private void FormProveedor_Load(object sender, System.EventArgs e)
    {
      if (_proveedorPresenter != null)
      {
        _proveedorPresenter.SetModelReadOnly();
        _proveedorPresenter.NuevoProveedor();
        _proveedorPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _proveedorPresenter;
      }
      helpProviderFormProveedor.SetShowHelp(this,true);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _proveedorPresenter.NuevoProveedor();
      textEditCodigoProveedor.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _proveedorPresenter.GuardarProveedor();
      
      if (_proveedorPresenter.AlGuardarElProveedorOSeleccionarloAñadirloAlArticulo)
        Close();
      
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _proveedorPresenter.BuscarProveedor();
      MostrarListaProveedoresBuscados();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _proveedorPresenter.NuevaBusquedaProveedor();
      textEditCodigoProveedor.Focus();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _proveedorPresenter.EliminarProveedor();
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {
      gridViewListaProveedores_DoubleClick(sender, e);
    }

    private void gridViewListaProveedores_DoubleClick(object sender, System.EventArgs e)
    {
      var proveedorSeleccionado = gridViewListaProveedores.GetFocusedRow() as BuscarProveedorListItemModel;
      if (proveedorSeleccionado != null) _proveedorPresenter.SetProveedor(proveedorSeleccionado.ProveedorId);
      OcultarListaProveedoresBuscados();
      
      if (_proveedorPresenter.AlGuardarElProveedorOSeleccionarloAñadirloAlArticulo)
        Close();
    }
    
    #endregion
  }
}