﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.CodigosPostales;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProvincias;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Proveedores
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del proveedor,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ProveedorViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ProveedorViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionCodigoPostalService _gestionCodigoPostalService;
    private readonly IGestionProvinciaService _gestionProvinciaService;
    
    #endregion

    #region Constructores

    public ProveedorViewModelReadOnlyFactory(Func<ProveedorViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory, 
                                           IGestionCodigoPostalService gestionCodigoPostalService, 
                                           IGestionProvinciaService gestionProvinciaService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionCodigoPostalService = gestionCodigoPostalService;
      _gestionProvinciaService = gestionProvinciaService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ProveedorViewModelReadOnly vacío</returns>
    public ProveedorViewModelReadOnly CreateProveedorViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetProvincias(_gestionProvinciaService.BuscarTodasLasProvincias());
      model.SetCodigosPostales(_gestionCodigoPostalService.BuscarTodosLosCodigosPostales());
      return model;
    }

    #endregion
  }
}
