﻿using FluentValidation;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Proveedores
{
  /// <summary>
  /// Para definir un conjunto de reglas de validación para un objeto en particular, hay que 
  /// crear una clase que herede de AbstractValidator <T/>, donde T es el tipo de
  /// Clase que se desea validar.
  /// Para mas información ir a http://fluentvalidation.codeplex.com/documentation
  /// </summary>
  public class ProveedorViewModelValidator : AbstractValidator<ProveedorViewModel>
  {
    /// <summary>
    /// Inicializa una nueva instancia de la clase<see cref="ProveedorViewModelValidator" />.
    /// Configurando las reglas de validación a cumplir.
    /// </summary>
    public ProveedorViewModelValidator()
    {
      RuleFor(t => t.Codigo).NotEmpty();
      RuleFor(t => t.RazonSocial).NotEmpty();
      RuleFor(t => t.CifDni).NotEmpty();
      RuleFor(t => t.Telefonos).NotEmpty();
      RuleFor(t => t.IdCodigoPostal).NotNull().NotEqual(0).WithMessage("Se debe introducir un código postal");
    }
  }
}
