﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarRecepcionVehiculo;
using GestionConcesionarios.Presentacion.CasosDeUso.HistorialOrdenesDeReparacion;

namespace GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo
{
  public partial class FormRecepcionVehiculo : BindableFormBase
  {
    #region Atributos

    private readonly RecepcionVehiculoPresenter _recepcionVehiculoPresenter;

    #endregion

    #region Constructores

    public FormRecepcionVehiculo(RecepcionVehiculoPresenter recepcionVehiculoPresenter)
    {
      _recepcionVehiculoPresenter = recepcionVehiculoPresenter;
      InitializeComponent();
      ConfigurarAyuda(helpProviderFormRecepcionVehiculo);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaRecepcionesBuscadas()
    {
      dockPanelListaBuscarRecepcionVehiculos.Show();
    }

    private void OcultarListaRecepcionesBuscadas()
    {
      dockPanelListaBuscarRecepcionVehiculos.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos
    
    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }
    
    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormRecepcionVehiculo.HelpNamespace);
    }

    private void FormRecepcionVehiculo_Load(object sender, EventArgs e)
    {
      if (_recepcionVehiculoPresenter != null)
      {
        backstageViewControl1.SelectedTabIndex = 0;
        _recepcionVehiculoPresenter.SetModelReadOnly();
        _recepcionVehiculoPresenter.NuevaRecepcionVehiculo();
        _recepcionVehiculoPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _recepcionVehiculoPresenter;
      }
      helpProviderFormRecepcionVehiculo.SetShowHelp(this, true);
    }

    private void gridViewListaRecepciones_DoubleClick(object sender, EventArgs e)
    {
      var recepcionSeleccionada = gridViewListaRecepciones.GetFocusedRow() as BuscarRecepcionVehiculoListItemModel;
      if (recepcionSeleccionada != null) _recepcionVehiculoPresenter.SetRecepcionVehiculo(recepcionSeleccionada.RecepcionVehiculoId);
      OcultarListaRecepcionesBuscadas();
    }

    private void simpleButtonTransferir_Click(object sender, EventArgs e)
    {
      gridViewListaRecepciones_DoubleClick(sender, e);
    }

    private void barLargeButtonItemNueva_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.NuevaRecepcionVehiculo();
      lookUpEditRecibe.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.GuardarRecepcionVehiculo();
    }

    private void barLargeButtonItemImprimir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.ImprimirRecepcion();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.NuevaRecepcionVehiculo();
      lookUpEditRecibe.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.BuscarRecepcionVehiculo();
      MostrarListaRecepcionesBuscadas();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _recepcionVehiculoPresenter.EliminarRecepcionVehiculo();
    }

    private void simpleButtonBuscarCliente_Click(object sender, EventArgs e)
    {
      _recepcionVehiculoPresenter.BuscarCliente();
    }

    private void simpleButtonBuscarVehiculo_Click(object sender, EventArgs e)
    {
      _recepcionVehiculoPresenter.BuscarVehiculo();
    }

    private void simpleButtonVerOrdenesReparacion_Click(object sender, EventArgs e)
    {
      _recepcionVehiculoPresenter.VerOrdenesDeReparacion();
    }

    #endregion
  }
}