﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionRecepciones;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarRecepcionVehiculo;
using GestionConcesionarios.Presentacion.CasosDeUso.HistorialOrdenesDeReparacion;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion;
using GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormRecepcionVehiculo)]
  [RegisterInMediator(MediatorEnumerator.FormVehiculos)]
  [RegisterInMediator(MediatorEnumerator.BuscarCliente)]
  public class RecepcionVehiculoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private RecepcionVehiculoViewModel _model;
    private readonly RecepcionVehiculoViewModelFactory _modelFactory;

    private RecepcionVehiculoViewModelReadOnly _modelReadOnly;
    private readonly RecepcionVehiculoViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarRecepcionVehiculoPresenter _buscarRecepcionVehiculoPresenter;

    private readonly IGestionRecepcionVehiculoService _gestionRecepcionVehiculoService;
    private readonly IGestionVehiculoService _gestionVehiculoService;
    private readonly InformacionUsuario _informacionUsuario;
    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;
    private readonly OrdenDeReparacionViewModelFactory _ordenDeReparacionViewModelFactory;

    #endregion

    #region Constructores

    public RecepcionVehiculoPresenter(RecepcionVehiculoViewModelFactory modelFactory,
                            RecepcionVehiculoViewModelReadOnlyFactory modelReadOnlyFactory,
                            IMediatorManager mediatorManager,
                            BuscarRecepcionVehiculoPresenter buscarRecepcionVehiculoPresenter,
                            IGestionRecepcionVehiculoService gestionRecepcionVehiculoService,
                            IGestionVehiculoService gestionVehiculoService,
                            InformacionUsuario informacionUsuario,
                            IGestionOrdenDeReparacionService gestionOrdenDeReparacionService,
                            OrdenDeReparacionViewModelFactory ordenDeReparacionViewModelFactory)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _buscarRecepcionVehiculoPresenter = buscarRecepcionVehiculoPresenter;
      _gestionRecepcionVehiculoService = gestionRecepcionVehiculoService;
      _gestionVehiculoService = gestionVehiculoService;
      _informacionUsuario = informacionUsuario;
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
      _ordenDeReparacionViewModelFactory = ordenDeReparacionViewModelFactory;
    }

    #endregion

    #region Propiedades

    public RecepcionVehiculoViewModel Model
    {
      get { return _model; }
    }

    public RecepcionVehiculoViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarRecepcionVehiculoListItemModel> ListaBuscarRecepcionVehiculoViewModel
    {
      get { return _buscarRecepcionVehiculoPresenter.Model.ListaBuscarRecepcionVehiculos; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.IdVehiculo).Do(x =>
      {
        Model.SetVehiculo(Model.IdVehiculo);
        NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
      });
      Model.OnChanged(c => Model.MedidorCombustible).Do(x =>
      {
        Model.SetInformacionMedidorCombustible();
        NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
      });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateRecepcionVehiculoViewModelReadOnly();
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaRecepcionVehiculo()
    {
      _model = _modelFactory.CrearRecepcionVehiculoViewModel();
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaRecepcionVehiculo()
    {
      _model = _modelFactory.CrearRecepcionVehiculoViewModel();
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de recepciones
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarRecepcionVehiculo()
    {
      _buscarRecepcionVehiculoPresenter.Buscar(_model.Matricula, string.Empty, _model.NombreCliente);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.ListaBuscarRecepcionVehiculoViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarRecepcionVehiculo()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_, Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_Eliminar_recepción, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionRecepcionVehiculoService.RemoveRecepcion(_model.Id);
          NuevaRecepcionVehiculo();
          MessageBox.Show(Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_Recepción_Vehiculo_eliminada_con_éxito,
                          Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_Eliminar_recepción, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_No_se_puede_eliminar_la_recepción_del_vehículo_porque_todavía_no_se_ha_guardado_nada,
                          Resources.RecepcionVehiculoPresenter_EliminarRecepcionVehiculo_Eliminar_recepción, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarRecepcionVehiculo()
    {
      Guid idRecepcionVehiculo;
      if (!_model.HasBeenPersisted())
      {
        _model.Id = idRecepcionVehiculo = Guid.NewGuid();
        _gestionRecepcionVehiculoService.AddRecepcion(idRecepcionVehiculo, _model.Fecha,
                                                      _model.ObservacionesSobreElCliente, _model.NombreContactoAdicional,
                                                      _model.TelefonoContactoAdicional, _model.SubTotal,
                                                      _model.ImporteIva, _model.Kilometraje, _model.PreDiagnostico,
                                                      _model.ContenidoAdicional, _model.MedidorCombustible,
                                                      _model.FechaHoraEstimadaEntrega, _model.ObservacionesRecepcion,
                                                      _model.IdCliente, _model.IdVehiculo, _model.IdEmpleadoRecibe,
                                                      _informacionUsuario.CodigoTaller);
        var modelOrdenDeReparacion = CrearModeloOrdenDeReparacion(_model);
        AddOrdenDeReparacion(modelOrdenDeReparacion);
      }
      else
      {
        idRecepcionVehiculo = _model.Id;
        _gestionRecepcionVehiculoService.ChangeRecepcion(idRecepcionVehiculo, _model.Fecha,
                                                      _model.ObservacionesSobreElCliente, _model.NombreContactoAdicional,
                                                      _model.TelefonoContactoAdicional, _model.SubTotal,
                                                      _model.ImporteIva, _model.Kilometraje, _model.PreDiagnostico,
                                                      _model.ContenidoAdicional, _model.MedidorCombustible,
                                                      _model.FechaHoraEstimadaEntrega, _model.ObservacionesRecepcion,
                                                      _model.IdCliente, _model.IdVehiculo, _model.IdEmpleadoRecibe,
                                                      _informacionUsuario.CodigoTaller);
        var modelOrdenDeReparacion = CrearModeloOrdenDeReparacion(idRecepcionVehiculo);
        ChangeOrdenDeReparacion(modelOrdenDeReparacion);
      }
      //GuardarVehiculos(idRecepcionVehiculo, _model.ObtenerListaVehiculos(_model.VehiculosDelRecepcionVehiculo));
      MessageBox.Show(Resources.RecepcionVehiculoPresenter_GuardarRecepcionVehiculo_Recepción_Vehiculo_guardada_con_éxito,
                      Resources.RecepcionVehiculoPresenter_GuardarRecepcionVehiculo_Guardar_recepción, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="recepcionVehiculoId">Id de la entidad a cargar</param>
    public void SetRecepcionVehiculo(Guid recepcionVehiculoId)
    {
      _model = _modelFactory.CrearRecepcionVehiculoViewModel(recepcionVehiculoId);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Abre la pantalla para buscar un cliente
    /// </summary>
    public void BuscarCliente()
    {
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormBuscarCliente));
    }

    /// <summary>
    /// Abre la pantalla para añadir un vehículo a la recepción
    /// </summary>
    public void BuscarVehiculo()
    {
      var formWithParameter = new FormWithParameterAndMediator
      {
        FormType = typeof(FormVehiculos),
        Mediator = _mediatorManager[MediatorEnumerator.FormVehiculos],
        MediatorMessage = MediatorMessages.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista,
        Parameter = null
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }

    /// <summary>
    /// Añade un vehículo a la recepción
    /// </summary>
    /// <param name="vehiculoViewModel">Vehículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirVehiculo, ParameterType = typeof(VehiculoViewModel))]
    public void AñadirVehiculo(VehiculoViewModel vehiculoViewModel)
    {
      _model.AñadirVehiculo(vehiculoViewModel);
      _model.SetVehiculo(vehiculoViewModel);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade un cliente a la recepción
    /// </summary>
    /// <param name="buscarClienteListItemModel">Cliente buscado a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarClienteClienteSeleccionado, ParameterType = typeof(BuscarClienteListItemModel))]
    public void AñadirCliente(BuscarClienteListItemModel buscarClienteListItemModel)
    {
      _model.SetCliente(buscarClienteListItemModel);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Abre la pantalla de historial de las órdenes de reparación del vehículo seleccionado
    /// </summary>
    public void VerOrdenesDeReparacion()
    {
      if (_model.IdVehiculo != null || _model.IdVehiculo != Guid.Empty)
      {
        var formWithParameter = new FormWithParameterAndMediator
        {
          FormType = typeof(FormHistorialOrdenesDeReparacion),
          Mediator = _mediatorManager[MediatorEnumerator.FormHistorialOrdenesDeReparacion],
          MediatorMessage = MediatorMessages.BuscarOrdenesDeReparacionDelVehiculo,
          Parameter = _model.Matricula
        };
        _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
      }
      else
      {
        MessageBox.Show(
          Resources.RecepcionVehiculoPresenter_VerOrdenesDeReparacion_Se_debe_seleccionar_un_vehículo_para_poder_ver_su_historial,
          Resources.RecepcionVehiculoPresenter_VerOrdenesDeReparacion_Ver_órdenes_de_reparación,
          MessageBoxButtons.OK, MessageBoxIcon.Warning);
      }
    }

    /// <summary>
    /// Imprime la recepción del vehículo
    /// </summary>
    public void ImprimirRecepcion()
    {
      MessageBox.Show("Esta funcionalidad no está implementada", "Imprimir");
    }

    /*
    /// <summary>
    /// Añade un vehículo al cliente
    /// </summary>
    /// <param name="vehiculoViewModel">Vehículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirVehiculo, ParameterType = typeof(VehiculoViewModel))]
    public void AñadirVehiculo(VehiculoViewModel vehiculoViewModel)
    {
      _model.AñadirVehiculo(vehiculoViewModel);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Elimina un vehículo al cliente
    /// </summary>
    /// <param name="vehiculoViewModel">Vehículo a eliminar.</param>
    public void EliminarVehiculoAlCliente(ClienteViewModel.VehiculoViewModel vehiculoViewModel)
    {
      _model.EliminarVehiculo(vehiculoViewModel);
      NotifyPropertyChanged<RecepcionVehiculoPresenter>(t => t.Model);
    }
    */
    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea un modelo de orden de reparacion a partir del id de la recepción del vehículo.
    /// </summary>
    /// <param name="idRecepcionVehiculo">Id de la recepción del vehículo.</param>
    /// <returns>Un modelo de vista de la orden de reparación</returns>
    private OrdenDeReparacionViewModel CrearModeloOrdenDeReparacion(Guid idRecepcionVehiculo)
    {
      var ordenDeReparacionViewModel =
        _ordenDeReparacionViewModelFactory.CrearOrdenDeReparacionViewModelPorRecepcionId(idRecepcionVehiculo);

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Crea un modelo de orden de reparacion a partir del modelo de vista de la recepción del vehículo.
    /// </summary>
    /// <param name="recepcionVehiculoViewModel">Modelo de vista de la recepción del vehículo.</param>
    /// <returns>Un modelo de vista de la orden de reparación</returns>
    private OrdenDeReparacionViewModel CrearModeloOrdenDeReparacion(RecepcionVehiculoViewModel recepcionVehiculoViewModel)
    {
      var ordenDeReparacionViewModel =
        _ordenDeReparacionViewModelFactory.CrearOrdenDeReparacionViewModel(recepcionVehiculoViewModel);

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Añade una nueva orden de reparacion.
    /// </summary>
    /// <param name="ordenDeReparacionViewModel">El modelo de vista de la orden de reparación.</param>
    private void AddOrdenDeReparacion(OrdenDeReparacionViewModel ordenDeReparacionViewModel)
    {
      _gestionOrdenDeReparacionService.AddOrdenReparacion(Guid.NewGuid(), ordenDeReparacionViewModel.NumeroOrden,
                                                          ordenDeReparacionViewModel.FechaEntrega,
                                                          ordenDeReparacionViewModel.IdEstado,
                                                          ordenDeReparacionViewModel.IdPersonalAsignado,
                                                          ordenDeReparacionViewModel.SubTotal,
                                                          ordenDeReparacionViewModel.ImporteIva,
                                                          ordenDeReparacionViewModel.Diagnostico,
                                                          ordenDeReparacionViewModel.IdRecepcion,
                                                          ordenDeReparacionViewModel.IdVehiculo,
                                                          _informacionUsuario.CodigoTaller);
    }

    /// <summary>
    /// Modifica una orden de reparacion.
    /// </summary>
    /// <param name="ordenDeReparacionViewModel">El modelo de vista de la orden de reparación.</param>
    private void ChangeOrdenDeReparacion(OrdenDeReparacionViewModel ordenDeReparacionViewModel)
    {
      _gestionOrdenDeReparacionService.ChangeOrdenReparacion(ordenDeReparacionViewModel.Id, 
                                                             ordenDeReparacionViewModel.NumeroOrden,
                                                             ordenDeReparacionViewModel.FechaEntrega,
                                                             ordenDeReparacionViewModel.IdEstado,
                                                             ordenDeReparacionViewModel.IdPersonalAsignado,
                                                             ordenDeReparacionViewModel.SubTotal,
                                                             ordenDeReparacionViewModel.ImporteIva,
                                                             ordenDeReparacionViewModel.Diagnostico,
                                                             ordenDeReparacionViewModel.IdRecepcion,
                                                             ordenDeReparacionViewModel.IdVehiculo,
                                                             _informacionUsuario.CodigoTaller);
    }

    /*
    /// <summary>
    /// Modifica los vehículos visualizados en la vista de forma que asocia a los mismos
    /// el cliente actualmente cargado
    /// </summary>
    /// <param name="idRecepcionVehiculo">Id del cliente cargado en la vista.</param>
    /// <param name="listaVehiculos">Lista de vehículos visualizados en la vista.</param>
    private void GuardarVehiculos(Guid idRecepcionVehiculo, IEnumerable<Vehiculo> listaVehiculos)
    {
      foreach (var vehiculo in listaVehiculos)
      {
        vehiculo.IdRecepcionVehiculo = idRecepcionVehiculo;
        _gestionVehiculoService.ChangeVehiculo(vehiculo);
      }
    }
    */

    #endregion
  }
}
