﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente;
using GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos;

namespace GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista del cliente
  /// </summary>
  public class RecepcionVehiculoViewModel : ModelBase, IPersistentModel
  {
    #region Constantes

    private static readonly List<string> TextosInformacionMedidorCombustible = new List<string> { "Vacío", "Casi un cuarto", "Un cuarto", "Casi medio", "Medio", "Casi tres cuartos", "Tres cuartos", "Casi lleno", "Lleno" };

    #endregion

    #region Modelos Auxiliares
    public class LineaRecepcionVehiculoViewModel : ModelBase
    {
      public Guid IdPaqueteDeServicio { get; set; }
      public decimal Cantidad { get; set; }
      public string Descripcion { get; set; }
      public decimal Precio { get; set; }
      public decimal Importe { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionPaqueteDeServicioService _gestionPaqueteDeServicioService;
    private readonly IGestionClienteService _gestionClienteService;
    private BindingList<LineaRecepcionVehiculoViewModel> _lineasRecepcionVehiculo = new BindingList<LineaRecepcionVehiculoViewModel>();
    private readonly BindingList<VehiculoViewModel> _vehiculosDelCliente = new BindingList<VehiculoViewModel>();

    #endregion

    #region Constructores
    public RecepcionVehiculoViewModel(IGestionPaqueteDeServicioService gestionPaqueteDeServicioService, IGestionClienteService gestionClienteService)
    {
      _gestionPaqueteDeServicioService = gestionPaqueteDeServicioService;
      _gestionClienteService = gestionClienteService;
    }

    #endregion

    #region Propiedades

    public virtual Guid Id { get; set; }
    public virtual Guid? IdEmpleadoRecibe { get; set; }
    public virtual DateTime? Fecha { get; set; }

    #region Datos Cliente

    public virtual Guid? IdCliente { get; set; }
    public virtual string NombreCliente { get; set; }
    public virtual string DireccionCliente { get; set; }
    public virtual string TelefonosCliente { get; set; }
    public virtual string CorreoElectronicoCliente { get; set; }
    public virtual string ObservacionesSobreElCliente { get; set; }
    public virtual string NombreContactoAdicional { get; set; }
    public virtual string TelefonoContactoAdicional { get; set; }

    public BindingList<VehiculoViewModel> VehiculosDelCliente { get { return _vehiculosDelCliente; } }

    #endregion

    #region Vehículo cliente

    public virtual Guid? IdVehiculo { get; set; }
    public virtual string InformacionSobreLosVehiculosDelCliente { get; set; }
    public virtual string Matricula { get; set; }
    public virtual string Marca { get; set; }
    public virtual string Modelo { get; set; }
    public virtual DateTime? Año { get; set; }
    public virtual string Color { get; set; }

    #endregion

    #region Diagnóstico

    public virtual decimal? Kilometraje { get; set; }
    public virtual string PreDiagnostico { get; set; }
    public virtual string ContenidoAdicional { get; set; }
    public virtual int? MedidorCombustible { get; set; }
    public virtual string InformacionMedidorCombustible { get; set; }

    #endregion

    #region Observaciones

    public virtual DateTime? FechaHoraEstimadaEntrega { get; set; }
    public virtual string ObservacionesRecepcion { get; set; }
    public virtual Guid? IdPersonalAsignado { get; set; }

    #endregion

    public virtual decimal? SubTotal { get; set; }
    public virtual decimal? ImporteIva { get; set; }
    public virtual decimal? ImporteTotal { get; set; }

    #region Líneas de recepción

    public BindingList<LineaRecepcionVehiculoViewModel> LineasRecepcionVehiculo { get { return _lineasRecepcionVehiculo; } }

    #endregion

    #endregion

    #region Métodos Públicos

    public List<PaqueteDeServicio> ObtenerLineasRecepcionVehiculo(IEnumerable<LineaRecepcionVehiculoViewModel> lineasRecepcionVehiculoViewModel)
    {
      return
        lineasRecepcionVehiculoViewModel.Select(
          recepcionVehiculoViewModel => _gestionPaqueteDeServicioService.BuscarPaqueteDeServicioPorId(recepcionVehiculoViewModel.IdPaqueteDeServicio).First()).ToList();
    }

    public void SetLineasRecepcionVehiculo(BindingList<LineaRecepcionVehiculoViewModel> lineasRecepcionVehiculoViewModel)
    {
      _lineasRecepcionVehiculo = lineasRecepcionVehiculoViewModel;
    }

    public void EliminarLineaRecepcion(LineaRecepcionVehiculoViewModel lineaRecepcionViewModel)
    {
      LineasRecepcionVehiculo.Remove(lineaRecepcionViewModel);
    }

    public void AñadirLineaRecepcion(LineaRecepcionVehiculoViewModel lineaRecepcionViewModel)
    {
      var nuevaLineaRecepcion = new LineaRecepcionVehiculoViewModel
                            {
                              IdPaqueteDeServicio = lineaRecepcionViewModel.IdPaqueteDeServicio,
                              Cantidad = 1,
                              Descripcion = lineaRecepcionViewModel.Descripcion,
                              Precio = lineaRecepcionViewModel.Precio,
                              Importe = lineaRecepcionViewModel.Precio
                            };
      _lineasRecepcionVehiculo.Add(nuevaLineaRecepcion);
    }

    public void SetCliente(BuscarClienteListItemModel buscarClienteListItemModel)
    {
      IdCliente = buscarClienteListItemModel.ClienteId;
      NombreCliente = buscarClienteListItemModel.Nombre;
      DireccionCliente = buscarClienteListItemModel.Direccion;
      TelefonosCliente = buscarClienteListItemModel.Telefonos;
      CorreoElectronicoCliente = buscarClienteListItemModel.CorreoElectronico;
      ObservacionesSobreElCliente = buscarClienteListItemModel.Observaciones;

      SetVehiculosDelCliente(IdCliente);
    }

    public void SetVehiculo(VehiculoViewModel vehiculoViewModel)
    {
      IdVehiculo = vehiculoViewModel.Id;
      Matricula = vehiculoViewModel.Matricula;
      Marca = vehiculoViewModel.Marca;
      Modelo = vehiculoViewModel.Modelo;
      Año = vehiculoViewModel.Año;
      Color = vehiculoViewModel.Color;
    }

    public void SetVehiculo(Guid? idVehiculo)
    {
      foreach (var vehiculoViewModel in VehiculosDelCliente.Where(vehiculoViewModel => vehiculoViewModel.Id.Equals(idVehiculo)))
      {
        Matricula = vehiculoViewModel.Matricula;
        Marca = vehiculoViewModel.Marca;
        Modelo = vehiculoViewModel.Modelo;
        Año = vehiculoViewModel.Año;
        Color = vehiculoViewModel.Color;
      }
    }

    public void AñadirVehiculo(VehiculoViewModel vehiculoViewModel)
    {
      _vehiculosDelCliente.Add(new VehiculoViewModel
      {
        Id = vehiculoViewModel.Id,
        Matricula = vehiculoViewModel.Matricula,
        Marca = vehiculoViewModel.Marca,
        Modelo = vehiculoViewModel.Modelo,
        Bastidor = vehiculoViewModel.Bastidor,
        Año = vehiculoViewModel.Año,
        Color = vehiculoViewModel.Color
      });
      SetInformacionSobreLosVehiculosDelCliente();
    }

    public void SetVehiculosDelCliente(Guid? idCliente)
    {
      var cliente = _gestionClienteService.BuscarClientePorId(idCliente).FirstOrDefault();
      if (cliente != null)
        foreach (var vehiculo in cliente.Vehiculos)
        {
          _vehiculosDelCliente.Add(new VehiculoViewModel
          {
            Id = vehiculo.Id,
            Matricula = vehiculo.Matricula,
            Marca = vehiculo.Marca,
            Modelo = vehiculo.Modelo,
            Bastidor = vehiculo.Bastidor,
            Año = vehiculo.Fecha,
            Color = vehiculo.Color
          });
        }

      SetInformacionSobreLosVehiculosDelCliente();
      if (_vehiculosDelCliente.Count > 0) SetVehiculo(_vehiculosDelCliente[0]);
    }

    /// <summary>
    /// Pone el texto de informacion del medidor de combustible.
    /// </summary>
    public void SetInformacionMedidorCombustible()
    {
      if (TextosInformacionMedidorCombustible.Count > MedidorCombustible)
        InformacionMedidorCombustible = TextosInformacionMedidorCombustible[(int)MedidorCombustible];
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion

    #region Métodos Privados

    private void SetInformacionSobreLosVehiculosDelCliente()
    {
      InformacionSobreLosVehiculosDelCliente = string.Empty;

      if (_vehiculosDelCliente != null && _vehiculosDelCliente.Count > 0)
      {
        InformacionSobreLosVehiculosDelCliente = string.Format("El cliente tiene {0} vehículos. Por favor, seleccione un vehículo de la lista",
                                  _vehiculosDelCliente.Count);
      }
    }

    #endregion
  }
}
