﻿using System;
using FluentValidation;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;

namespace GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo
{
  /// <summary>
  /// Para definir un conjunto de reglas de validación para un objeto en particular, hay que 
  /// crear una clase que herede de AbstractValidator <T/>, donde T es el tipo de
  /// Clase que se desea validar.
  /// Para mas información ir a http://fluentvalidation.codeplex.com/documentation
  /// </summary>
  public class RecepcionVehiculoViewModelValidator : AbstractValidator<RecepcionVehiculoViewModel>
  {
    /// <summary>
    /// Inicializa una nueva instancia de la clase<see cref="RecepcionVehiculoViewModelValidator" />.
    /// Configurando las reglas de validación a cumplir.
    /// </summary>
    public RecepcionVehiculoViewModelValidator()
    {
      RuleFor(t => t.IdEmpleadoRecibe).NotNull().NotEmpty().NotEqual(Guid.Empty).WithMessage("Es necesario rellenar el campo");
      RuleFor(t => t.Fecha).NotEmpty();
      RuleFor(t => t.NombreCliente).NotEmpty();
      //RuleFor(t => t.IdVehiculo).NotNull().NotEmpty().NotEqual(Guid.Empty).WithMessage("Es necesario rellenar el campo");
      RuleFor(t => t.Marca).NotEmpty();
      RuleFor(t => t.Modelo).NotEmpty();
      RuleFor(t => t.PreDiagnostico).NotEmpty();
      RuleFor(t => t.Kilometraje).GreaterThan(0);
      RuleFor(t => t.IdPersonalAsignado).NotNull().NotEmpty().NotEqual(Guid.Empty).WithMessage("Es necesario rellenar el campo");
    }
  }
}
