﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculo;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos
{
  public partial class FormVehiculos : BindableFormBase
  {
    #region Atributos

    private readonly VehiculoPresenter _vehiculoPresenter;
    #endregion

    #region Constructores

    public FormVehiculos(VehiculoPresenter vehiculoPresenter)
    {
      InitializeComponent();
      _vehiculoPresenter = vehiculoPresenter;
      ConfigurarAyuda(helpProviderFormVehiculos);
    }

    #endregion

    #region Metodos Privados
    private void MostrarListaVehiculosBuscados()
    {
      dockPanelBuscarVehiculos.Show();
    }

    private void OcultarListaVehiculosBuscados()
    {
      dockPanelBuscarVehiculos.HideImmediately();
    }
    #endregion

    #region Manejadores de eventos

    private void FormVehiculos_Load(object sender, EventArgs e)
    {
      if (_vehiculoPresenter != null)
      {
        _vehiculoPresenter.NuevoVehiculo();
        _vehiculoPresenter.SetModelReadOnly();
        _vehiculoPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _vehiculoPresenter;
      }
      helpProviderFormVehiculos.SetShowHelp(this, true);
    }

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _vehiculoPresenter.NuevoVehiculo();
      textEditMatricula.Focus();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _vehiculoPresenter.GuardarVehiculo();
      if (_vehiculoPresenter.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)
        Close();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _vehiculoPresenter.NuevaBusquedaVehiculo();
      textEditMatricula.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _vehiculoPresenter.BuscarVehiculo();
      MostrarListaVehiculosBuscados();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _vehiculoPresenter.EliminarVehiculo();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormVehiculos.HelpNamespace);
    }

    private void simpleButtonAsignar_Click(object sender, EventArgs e)
    {
      gridViewListaVehiculos_DoubleClick(sender, e);
    }

    private void gridViewListaVehiculos_DoubleClick(object sender, EventArgs e)
    {
      var vehiculoSeleccionado = gridViewListaVehiculos.GetFocusedRow() as BuscarVehiculoListItemModel;
      if (vehiculoSeleccionado != null) _vehiculoPresenter.SetVehiculo(vehiculoSeleccionado.VehiculoId);
      OcultarListaVehiculosBuscados();
      if (_vehiculoPresenter.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)
        Close();
    }
    #endregion
  }
}