﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del vehículo,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class VehiculoViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<VehiculoViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionMarcaService _gestionMarcaService;
    private readonly IGestionModeloService _gestionModeloService;

    #endregion

    #region Constructores

    public VehiculoViewModelReadOnlyFactory(Func<VehiculoViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionMarcaService gestionMarcaService, 
                                           IGestionModeloService gestionModeloService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionMarcaService = gestionMarcaService;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo VehiculoViewModelReadOnly vacío</returns>
    public VehiculoViewModelReadOnly CreateVehiculoViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetMarcas(_gestionMarcaService.BuscarTodasLasMarcas());
      model.SetTodosLosModelos(_gestionModeloService.BuscarTodosLosModelos());
      return model;
    }

    #endregion
  }
}
