﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta
{
  /// <summary>
  /// Clase que representa la información de solo lectura a mostrar en la vista del vehículo de venta,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// Dicha información siempre se pondrá en la clase LookUp, la cual posteriormente se
  /// enlazará con la vista. De esta forma se factoriza la visualización de este tipo de información.
  /// </summary>
  public class VehiculoVentaViewModelReadOnly : ModelBase
  {
    #region Atributos

    private IList<LookUpItem<string>> _marcas = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _modelos = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _todosLosModelos = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _tiposDeVehiculo = new BindingList<LookUpItem<string>>();
    private IList<LookUpItem<string>> _tiposDeCombustible = new BindingList<LookUpItem<string>>();

    #endregion
    
    #region Propiedades

    public IEnumerable<LookUpItem<string>> Marcas { get { return _marcas; } }
    public IEnumerable<LookUpItem<string>> Modelos { get { return _modelos; } }
    public IEnumerable<LookUpItem<string>> TodosLosModelos { get { return _todosLosModelos; } }
    public IEnumerable<LookUpItem<string>> TiposDeVehiculo { get { return _tiposDeVehiculo; } }
    public IEnumerable<LookUpItem<string>> TiposDeCombustible { get { return _tiposDeCombustible; } }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Establece las marcas en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="marcas">Marcas a poner en correspondiente lista LookUp.</param>
    public void SetMarcas(IList<Marca> marcas)
    {
      _marcas =
        marcas.Select(marca => new LookUpItem<string> { Codigo = marca.Id, Descripcion = marca.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los modelos en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="modelos">Modelos a poner en correspondiente lista LookUp.</param>
    public void SetModelos(IList<Modelo> modelos)
    {
      _modelos =
        modelos.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece todos los modelos en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="modelos">Todos los modelos a poner en correspondiente lista LookUp.</param>
    public void SetTodosLosModelos(IList<Modelo> modelos)
    {
      _todosLosModelos =
        modelos.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los tipos de vehículo en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="tiposVehiculo">Tipos de vehículo a poner en correspondiente lista LookUp.</param>
    public void SetTiposDeVehiculo(IList<TipoVehiculo> tiposVehiculo)
    {
      _tiposDeVehiculo =
        tiposVehiculo.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    /// <summary>
    /// Establece los tipos de combustible en la clase LookUp para poder ser enlazadas con la vista.
    /// </summary>
    /// <param name="tiposCombustible">Tipos de combustible a poner en correspondiente lista LookUp.</param>
    public void SetTiposDeCombustible(IList<TipoCombustible> tiposCombustible)
    {
      _tiposDeCombustible =
        tiposCombustible.Select(modelo => new LookUpItem<string> { Codigo = modelo.Id, Descripcion = modelo.Descripcion }).ToList();
    }

    #endregion
  }
}
