﻿using System;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionMarcas;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTiposCombustible;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionTiposVehiculo;
using GestionConcesionarios.Dominio.Core;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta
{
  /// <summary>
  /// Clase factoria para construir la información de solo lectura a mostrar en 
  /// la vista del vehículo de venta,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class VehiculoVentaViewModelReadOnlyFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<VehiculoVentaViewModelReadOnly> _createModelWithProxy;
    private readonly IGestionTipoCombustibleService _gestionTipoCombustibleService;
    private readonly IGestionTipoVehiculoService _gestionTipoVehiculoService;
    private readonly IGestionMarcaService _gestionMarcaService;
    private readonly IGestionModeloService _gestionModeloService;

    #endregion

    #region Constructores

    public VehiculoVentaViewModelReadOnlyFactory(Func<VehiculoVentaViewModelReadOnly> createModelWithProxy,
                                           Func<IUnitOfWork> unitofWorkFactory,
                                           IGestionTipoCombustibleService gestionTipoCombustibleService,
                                           IGestionTipoVehiculoService gestionTipoVehiculoService,
                                           IGestionMarcaService gestionMarcaService,
                                           IGestionModeloService gestionModeloService)
    {
      if (unitofWorkFactory == null) throw new ArgumentNullException("unitofWorkFactory");
      if (createModelWithProxy == null) throw new ArgumentNullException("createModelWithProxy");

      _createModelWithProxy = createModelWithProxy;
      _gestionTipoCombustibleService = gestionTipoCombustibleService;
      _gestionTipoVehiculoService = gestionTipoVehiculoService;
      _gestionMarcaService = gestionMarcaService;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo de solo lectura para la vista vacío.
    /// </summary>
    /// <returns>Un modelo VehiculoVentaViewModelReadOnly vacío</returns>
    public VehiculoVentaViewModelReadOnly CreateVehiculoVentaViewModelReadOnly()
    {
      var model = _createModelWithProxy();
      model.SetTiposDeCombustible(_gestionTipoCombustibleService.BuscarTodosLosTiposDeCombustible());
      model.SetTiposDeVehiculo(_gestionTipoVehiculoService.BuscarTodosLosTiposDeVehiculo());
      model.SetMarcas(_gestionMarcaService.BuscarTodasLasMarcas());
      //model.SetModelos(_gestionModeloService.BuscarTodosLosModelos());
      model.SetTodosLosModelos(_gestionModeloService.BuscarTodosLosModelos());
      return model;
    }

    #endregion
  }
}
