﻿using FluentValidation;

namespace GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta
{
  /// <summary>
  /// Para definir un conjunto de reglas de validación para un objeto en particular, hay que 
  /// crear una clase que herede de AbstractValidator <T/>, donde T es el tipo de
  /// Clase que se desea validar.
  /// Para mas información ir a http://fluentvalidation.codeplex.com/documentation
  /// </summary>
  public class VehiculoVentaViewModelValidator : AbstractValidator<VehiculoVentaViewModel>
  {
    /// <summary>
    /// Inicializa una nueva instancia de la clase<see cref="VehiculoVentaViewModelValidator" />.
    /// Configurando las reglas de validación a cumplir.
    /// </summary>
    public VehiculoVentaViewModelValidator()
    {
      RuleFor(t => t.Codigo).NotEmpty();
      RuleFor(t => t.Marca).NotEmpty();
      RuleFor(t => t.Modelo).NotEmpty();
      RuleFor(t => t.Matricula).NotEmpty();
      RuleFor(t => t.FechaDeEntrada).NotEmpty();
      RuleFor(t => t.Precio).GreaterThan(0).WithMessage("El precio del vehículo ha de ser mayor que 0");
    }
  }
}
