﻿using System.Windows.Forms;
using DevExpress.XtraLayout.Utils;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarVenta;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Ventas
{
  public partial class FormFactura : BindableFormBase
  {
    #region Atributos

    private readonly FacturaPresenter _facturaPresenter;
    #endregion

    #region Constructores

    public FormFactura(FacturaPresenter facturaPresenter)
    {
      InitializeComponent();
      _facturaPresenter = facturaPresenter;
      ConfigurarAyuda(helpProviderFormFactura);
    }

    #endregion

    #region Metodos Privados

    private void MostrarListaFacturasBuscadas()
    {
      dockPanelBuscarFactura.Show();
    }

    private void OcultarListaFacturasBuscadas()
    {
      dockPanelBuscarFactura.HideImmediately();
    }

    #endregion

    #region Manejadores de eventos

    private void barLargeButtonItemSalir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Close();
    }

    private void gridViewListaFacturas_DoubleClick(object sender, System.EventArgs e)
    {
      var facturaSeleccionada = gridViewListaFacturas.GetFocusedRow() as BuscarFacturaListItemModel;
      if (facturaSeleccionada != null) _facturaPresenter.SetFactura(facturaSeleccionada.FacturaId);
      OcultarListaFacturasBuscadas();
    }

    private void simpleButtonTransferir_Click(object sender, System.EventArgs e)
    {
      gridViewListaFacturas_DoubleClick(sender, e);
    }

    private void barLargeButtonItemNuevo_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.NuevaFactura();
    }

    private void barLargeButtonItemGuardar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.GuardarFactura();
    }

    private void barLargeButtonItemImprimir_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.ImprimirFactura();
    }

    private void barLargeButtonItemNuevaBusqueda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.NuevaBusquedaFactura();
      textEditNumero.Focus();
    }

    private void barLargeButtonItemBuscar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.BuscarFactura();
      MostrarListaFacturasBuscadas();
    }

    private void barLargeButtonItemEliminar_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      _facturaPresenter.EliminarFactura();
    }

    private void barLargeButtonItemAyuda_ItemClick(object sender, DevExpress.XtraBars.ItemClickEventArgs e)
    {
      Help.ShowHelp(this, helpProviderFormFactura.HelpNamespace);
    }

    private void simpleButtonBuscarCliente_Click(object sender, System.EventArgs e)
    {
      _facturaPresenter.MostrarFormClienteConNuevoCliente();
    }

    private void simpleButtonAgregarLinea_Click(object sender, System.EventArgs e)
    {
      _facturaPresenter.MostrarFormVehiculoVentaConNuevoVehiculo();
    }

    private void simpleButtonEliminarLinea_Click(object sender, System.EventArgs e)
    {
      var lineaSeleccionada = gridViewLineas.GetFocusedRow() as FacturaViewModel.LineaFacturaViewModel;
      if (lineaSeleccionada != null) _facturaPresenter.EliminarVehiculoVentaALaFactura(lineaSeleccionada);
    }

    private void FormFactura_Load(object sender, System.EventArgs e)
    {
      if (_facturaPresenter != null)
      {
        _facturaPresenter.SetModelReadOnly();
        //_facturaPresenter.NuevaFactura();
        _facturaPresenter.SubscribirAccionesACambiosEnElModelo();
        bindingSourcePresenter.DataSource = _facturaPresenter;
      }
      helpProviderFormFactura.SetShowHelp(this, true);
    }

    #endregion
  }
}