﻿using System;
using System.Linq;
using System.Reflection;
using Autofac;
using Autofac.Core;
using Module = Autofac.Module;

namespace GestionConcesionarios.Presentacion.Infraestructura.Container
{
  public class BuildPresentationContainerParams
  {
    private readonly Action<ContainerBuilder> _registerLocalModules;
    private readonly Assembly[] _presenterAssembliesToScan;
    private readonly string[] _namespaces;
    
    public BuildPresentationContainerParams(Action<ContainerBuilder> registerLocalModules,
                                            Assembly[] presenterPresenterAssembliesToScan, 
                                            string[] namespaces)
    {
      if (registerLocalModules == null) throw new ArgumentNullException("registerLocalModules");
      if (presenterPresenterAssembliesToScan == null) throw new ArgumentNullException("presenterPresenterAssembliesToScan");
      if (namespaces == null) throw new ArgumentNullException("namespaces");

      _registerLocalModules = registerLocalModules;
      _presenterAssembliesToScan = presenterPresenterAssembliesToScan;
      _namespaces = namespaces;
    }

    public string ConnectionString
    {
      get { return string.Empty; }
    }

    public Action<ContainerBuilder> RegisterLocalModules
    {
      get { return _registerLocalModules; }
    }

    public Assembly[] PresenterAssembliesToScan
    {
      get { return _presenterAssembliesToScan; }
    }

    public string[] Namespaces
    {
      get { return _namespaces; }
    }
  }

  public static class PresentationContainerBuilderHelper
  {
    public static IContainer BuildPresentationContainer(BuildPresentationContainerParams
                                                          buildPresentationContainerParams)

  {
    if (buildPresentationContainerParams == null) throw new ArgumentNullException("buildPresentationContainerParams");

    var builder = new ContainerBuilder();
    
    buildPresentationContainerParams.RegisterLocalModules(builder);
    
    BuildPresentationModulesHelper.Build(builder, 
                                         buildPresentationContainerParams.PresenterAssembliesToScan,
                                         buildPresentationContainerParams.Namespaces);

    var container = builder.Build();
    return container;
  }
  }
}