﻿using System;
using System.Windows.Forms;
using Autofac;

namespace GestionConcesionarios.Presentacion.Infraestructura
{
  public interface IFormContext : IDisposable
  {
    DialogResult Init();
    void Init(ILifetimeScope context, FormWithParameterAndMediator formType);
  }

  public class FormContext<TForm> : IFormContext where TForm : Form
  {
    #region Atributos

    private ILifetimeScope _contextLifetimeScope;
    private bool _isContextOfFather;

    #endregion

    #region Constructor

    public FormContext(ILifetimeScope lifetimeScope)
    {
      if (lifetimeScope == null) throw new ArgumentNullException("lifetimeScope");

      _contextLifetimeScope = lifetimeScope.BeginLifetimeScope();
    }

    #endregion

    #region Metodos Publicos

    public DialogResult Init()
    {
      var form = _contextLifetimeScope.Resolve<TForm>();
      return form.ShowDialog();
    }

    public void Init(ILifetimeScope context, FormWithParameterAndMediator formType)
    {
      if (context != null)
      {
        _contextLifetimeScope = context;
        _isContextOfFather = true;
      }
      var form = _contextLifetimeScope.Resolve<TForm>();

      if (formType.Parameter != null)
        formType.Mediator.NotifyColleagues(formType.MediatorMessage, formType.Parameter);
      else
        formType.Mediator.NotifyColleagues(formType.MediatorMessage);

      form.ShowDialog();
    }

    #endregion

    #region Private Methods

    #endregion

    #region IDisposable Methods

    public void Dispose()
    {
      if (!_isContextOfFather && _contextLifetimeScope != null) _contextLifetimeScope.Dispose();
    }

    #endregion
  }
}
