﻿using System;
using Autofac;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.Infraestructura
{
  [RegisterInMediator(MediatorEnumerator.FormManager)]
  public class FormManager
  {
    #region Atributos
    private readonly Func<Type, IFormContext> _formContextFactory;
    #endregion

    #region Constructors
    public FormManager(Func<Type, IFormContext> formContextFactory)
    {
      _formContextFactory = formContextFactory;
    }
    #endregion

    #region Metodos Publicos
    [MediatorMessageSink(MediatorMessages.OpenForm, ParameterType = typeof(Type))]
    public void OpenForm(Type formType)
    {
      if (formType == null) throw new ArgumentNullException("formType");

      if (formType == typeof(FormWithParameterAndMediator))
        return;

      using (var formContext = _formContextFactory(formType))
      {
        formContext.Init();
      }
    }

    [MediatorMessageSink(MediatorMessages.OpenFormWithParameter, ParameterType = typeof(FormWithParameterAndMediator))]
    public void OpenFormWithParameter(FormWithParameterAndMediator formType)
    {
      if (formType == null) throw new ArgumentNullException("formType");

      using (var formContext = _formContextFactory(formType.FormType))
      {
        formContext.Init(formType.Context, formType);
      }
    }

    #endregion
  }

  public class FormWithParameterAndMediator
  {
    public Type FormType { get; set; }
    public IMediator Mediator { get; set; }
    public object Parameter { get; set; }
    public string MediatorMessage { get; set; }
    public ILifetimeScope Context { get; set; }
  }
}
