﻿using System;
using System.Linq;
using Autofac.Features.Indexed;
using Castle.DynamicProxy;
using FluentValidation;

namespace GestionConcesionarios.Presentacion.Infraestructura.Interceptors
{
  public class DataErrorInfoInterceptor : IInterceptor
  {
    private readonly IIndex<string, IValidator> _validatorIndex;

    public DataErrorInfoInterceptor(IIndex<string, IValidator> validatorIndex)
    {
      if (validatorIndex == null) throw new ArgumentNullException("validatorIndex");

      _validatorIndex = validatorIndex;
    }

    private static string GetValidatorServiceKey(IInvocation invocation)
    {
      if (invocation == null) throw new ArgumentNullException("invocation");

      return invocation.TargetType.Name + "Validator";
    }

    private string[] GetPropertyErrors(string columnName, IInvocation invocation)
    {
      if (columnName == null) throw new ArgumentNullException("columnName");
      if (invocation == null) throw new ArgumentNullException("invocation");

      var validator = GetValidator(invocation);
      //objectIsValid = true;
      if (validator != null)
      {
        var validationResult = validator.Validate(invocation.InvocationTarget);
        //objectIsValid = validationResult.IsValid;
        var propertyErrors = validationResult.Errors.Where(l => l.PropertyName == columnName);
        return propertyErrors.Count() > 0 ? propertyErrors.Select(l => l.ErrorMessage).ToArray() : new string[0];
      }
      return new string[0];
    }

    private IValidator GetValidator(IInvocation invocation)
    {
      if (invocation == null) throw new ArgumentNullException("invocation");

      IValidator validator;
      _validatorIndex.TryGetValue(GetValidatorServiceKey(invocation), out validator);
      return validator;
    }

    public void Intercept(IInvocation invocation)
    {
      if (invocation.Method.Name == "get_Item") //Indexer is called, get the error for the property
      {
        var propertyName = (string)invocation.Arguments[0]; //property name to validate

        var errors = GetPropertyErrors(propertyName, invocation);
        invocation.ReturnValue = string.Join(Environment.NewLine, errors);
      }
      else if (invocation.Method.Name == "get_Error") //Error property is called

        invocation.ReturnValue = string.Empty;
      else
        invocation.Proceed();
    }
  }
}