﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace GestionConcesionarios.Presentacion.Infraestructura.Mediator
{
  public interface IMediatorManager
  {
    IMediator this[MediatorEnumerator index] { get; }
    void Unregister(object target);
    void Register(object target);
  }

  public class MediatorManager : IMediatorManager
  {
    private readonly Dictionary<MediatorEnumerator, IMediator> _mediators = new Dictionary<MediatorEnumerator, IMediator>();

    public MediatorManager(Func<IMediator> mediatorFactory)
    {
      foreach (MediatorEnumerator enumerator in Enum.GetValues(typeof(MediatorEnumerator)))
      {
        _mediators.Add(enumerator, mediatorFactory());
      }

    }

    public IMediator this[MediatorEnumerator index]
    {
      get
      {
        return _mediators[index];
      }
    }

    public void Unregister(object target)
    {
      var type = target.GetType();

      var attributes = (RegisterInMediatorAttribute[])type.GetCustomAttributes(typeof(RegisterInMediatorAttribute), false);
      foreach (var mediator in attributes.Select(registerInMediatorAttribute => _mediators[registerInMediatorAttribute.Mediator]))
      {
        mediator.UnRegister(target);
      }
    }

    public void Register(object target)
    {
      var type = target.GetType();

      var attributes = (RegisterInMediatorAttribute[])type.GetCustomAttributes(typeof(RegisterInMediatorAttribute), false);
      foreach (var mediator in attributes.Select(registerInMediatorAttribute => _mediators[registerInMediatorAttribute.Mediator]))
      {
        mediator.Register(target);
      }

    }
  }
}
