﻿using System;

namespace GestionConcesionarios.Presentacion.Infraestructura.Mediator
{
  /// <summary>
  /// Cada clase que se registre en un mediator, debe tener este atributo.
  /// Para más información sobre el patrón Mediador ir a http://es.wikipedia.org/wiki/Mediator_(patrón_de_diseño)
  /// </summary>
  /// <remarks>
  /// Este atributo será utilizado durante la instanciación de una clase a través del contenedor IoC
  /// para registrar la misma en el Mediator indicado por la propiedad Mediator.
  /// </remarks>
  [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = true)]
  public class RegisterInMediatorAttribute : Attribute
  {
    public MediatorEnumerator Mediator { get; private set; }

    public RegisterInMediatorAttribute(MediatorEnumerator mediator)
    {
      Mediator = mediator;
    }
  }
}
