﻿using System;
using System.Windows.Forms;

namespace GestionConcesionarios.Presentacion.Infraestructura.MessageService
{
  public class WindowsFormsMessageService : IShowMessageService,IAskService
  {

    static private class TopMostMessageBox
    {
      static public DialogResult Show(string message)
      {
        if (message == null) throw new ArgumentNullException("message");

        return Show(message, string.Empty, MessageBoxButtons.OK, MessageBoxIcon.None);
      }

      static public DialogResult Show(string message, string title)
      {
        if (message == null) throw new ArgumentNullException("message");
        if (title == null) throw new ArgumentNullException("title");

        return Show(message, title, MessageBoxButtons.OK, MessageBoxIcon.None);
      }

      static public DialogResult Show(string message, string title,MessageBoxButtons buttons, MessageBoxIcon icon)
      {
        if (message == null) throw new ArgumentNullException("message");
        if (title == null) throw new ArgumentNullException("title");

        // Create a host form that is a TopMost window which will be the 
        // parent of the MessageBox.

        // We do not want anyone to see this window so position it off the 
        // visible screen and make it as small as possible

        var topmostForm = new Form {Size = new System.Drawing.Size(1, 1), StartPosition = FormStartPosition.Manual};

        var rect = SystemInformation.VirtualScreen;
        topmostForm.Location = new System.Drawing.Point(rect.Bottom + 10,rect.Right + 10);
        topmostForm.Show();
        // Make this form the active form and make it TopMost
        topmostForm.Focus();
        topmostForm.BringToFront();
        topmostForm.TopMost = true;
        // Finally show the MessageBox with the form just created as its owner

        var result = MessageBox.Show(topmostForm, message, title, buttons, icon);
        topmostForm.Dispose(); // clean it up all the way


        return result;
      }
    }


    public void ShowMessage(string message)
    {
      if (message == null) throw new ArgumentNullException("message");
      TopMostMessageBox.Show(message);
    }

    public void ShowWarning(string message)
    {
      if (message == null) throw new ArgumentNullException("message");

      TopMostMessageBox.Show(message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Warning);
    }

    public void ShowError(string message)
    {
      if (message == null) throw new ArgumentNullException("message");

      TopMostMessageBox.Show(message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
    }

    public bool AskYesNoQuestion(string question, string caption)
    {
      if (question == null) throw new ArgumentNullException("question");
      if (caption == null) throw new ArgumentNullException("caption");

      var dialogResult = MessageBox.Show(question, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
      return (dialogResult == DialogResult.Yes);
    }
  }
}
