﻿using System;
using System.Collections.Generic;

namespace GestionConcesionarios.Presentacion.Infraestructura
{
  public interface IPropertyChangedTracker
  {
    IEnumerable<string> ChangedProperties { get; }
    void ClearChangedProperties();
    void TrackPropertyChange(string propertyName);
    bool IsDirty { get; }
  }

  public class PropertyChangedTracker : IPropertyChangedTracker
  {
    #region Atributos

    private readonly List<string> _changedProperties = new List<string>();

    #endregion

    #region Metodos Publicos

    public IEnumerable<string> ChangedProperties
    {
      get { return _changedProperties; }
    }

    public void ClearChangedProperties()
    {
      _changedProperties.Clear();
    }

    public void TrackPropertyChange(string propertyName)
    {
      if (!_changedProperties.Contains(propertyName))
        _changedProperties.Add(propertyName);
    }

    public bool IsDirty
    {
      get
      {
        return _changedProperties.Count > 0;
      }
    }

    #endregion
  }
}