//----------------------------------------------------------------------------------------
// AddFlow for WinForms: CustomShape sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Lassalle.Flow;

namespace CustomShape
{
    public partial class CustomShape : Form
    {
        public CustomShape()
        {
            InitializeComponent();
        }

        private void CustomShape_Load(object sender, EventArgs e)
        {
            // Create a graphics path which will be used for the custom shape
            GraphicsPath path = new GraphicsPath();
            path.AddArc(0, 0, 15, 15, 180, 90);
            path.AddLine(10, 0, 40, 10);
            path.AddLine(40, 10, 50, 0);
            path.AddLine(50, 0, 80, 40);
            path.AddLine(80, 40, 40, 30);
            path.AddLine(40, 30, 0, 40);
            path.AddLine(0, 40, 10, 20);
            path.CloseFigure();
            path.AddEllipse(10, 10, 10, 10);
            path.FillMode = FillMode.Alternate;

            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.AutoScroll = true;
            AddFlow1.BackColor = Color.White;
            AddFlow1.PageUnit = GraphicsUnit.Point;
            AddFlow1.DefNodeProp.FillColor = Color.LightCoral;
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Custom;
            AddFlow1.DefNodeProp.Shape.GraphicsPath = path;
            AddFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom;
            AddFlow1.DefNodeProp.Shadow.Color = Color.Silver;
            AddFlow1.DefNodeProp.LabelEdit = false;

            AddFlow1.DefLinkProp.EndCap = LineCap.ArrowAnchor;
            AddFlow1.DefLinkProp.DrawWidth = 4;

            Node node1 = AddFlow1.Nodes.Add(50, 50, 100, 50);
            Node node2 = AddFlow1.Nodes.Add(350, 50, 50, 50);
            node2.OutLinks.Add(node1); 
        }

        private void MenuExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuOrientation0_Click(object sender, EventArgs e)
        {
            AddFlow1.DefNodeProp.Shape.Orientation = ShapeOrientation.so_0;
            foreach (Node node in AddFlow1.Nodes)
                node.Shape.Orientation = ShapeOrientation.so_0;
        }

        private void MenuOrientation90_Click(object sender, EventArgs e)
        {
            AddFlow1.DefNodeProp.Shape.Orientation = ShapeOrientation.so_90;
            foreach (Node node in AddFlow1.Nodes)
                node.Shape.Orientation = ShapeOrientation.so_90;
        }

        private void MenuOrientation180_Click(object sender, EventArgs e)
        {
            AddFlow1.DefNodeProp.Shape.Orientation = ShapeOrientation.so_180;
            foreach (Node node in AddFlow1.Nodes)
                node.Shape.Orientation = ShapeOrientation.so_180;
        }

        private void MenuOrientation270_Click(object sender, EventArgs e)
        {
            AddFlow1.DefNodeProp.Shape.Orientation = ShapeOrientation.so_270;
            foreach (Node node in AddFlow1.Nodes)
                node.Shape.Orientation = ShapeOrientation.so_270;
        }

        private void MenuAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: CustomShape sample\r\nCopyright (c) 2003-2011 Lassalle Technologies");
        }
    }
}