//----------------------------------------------------------------------------------------
// AddFlow for WinForms: DemoLayout sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using Lassalle.Flow;
using Lassalle.Flow.Layout.Hierarchic;
using Lassalle.Flow.Layout.Symmetric;
using Lassalle.Flow.Layout.Tree;
using Lassalle.Flow.Layout.Orthogonal;
using Lassalle.Flow.Layout.SP;

namespace DemoLayout
{
    public partial class DemoLayout : Form
    {
        Lassalle.Flow.Layout.Symmetric.SFlow sflow = null;
        Lassalle.Flow.Layout.Orthogonal.OFlow oflow = null;
        Lassalle.Flow.Layout.Hierarchic.HFlow hflow = null;
        Lassalle.Flow.Layout.Tree.TFlow tflow = null;
        Lassalle.Flow.Layout.SP.SPFlow spflow = null;
        string strProgName = null;
        string strFileName = null;

        public DemoLayout()
        {
            InitializeComponent();
        }

        private void DemoLayout_Load(object sender, EventArgs e)
        {
            StatusBar1.Dock = DockStyle.Bottom;

            SplitContainer1.Dock = DockStyle.Fill;
            SplitContainer1.FixedPanel = FixedPanel.Panel2;
            SplitContainer1.IsSplitterFixed = true;

            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.AutoScroll = true;
            AddFlow1.BackColor = SystemColors.Window;
            AddFlow1.Grid.Snap = true;
            AddFlow1.Grid.Style = GridStyle.DottedLines;
            AddFlow1.Grid.Color = Color.Silver;
            AddFlow1.Grid.Size = new Size(20, 20);
            AddFlow1.CanFireError = true;
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            AddFlow1.DefNodeProp.Gradient = true;
            AddFlow1.DefNodeProp.GradientMode = LinearGradientMode.ForwardDiagonal;
            AddFlow1.DefNodeProp.GradientColor = Color.Yellow;
            AddFlow1.DefLinkProp.Jump = Jump.Arc;

            hflow = new HFlow();
            oflow = new OFlow();
            sflow = new SFlow();
            tflow = new TFlow();
            spflow = new SPFlow();

            this.ToolBarButtonFileNew.Tag = "New";
            this.ToolBarButtonFileOpen.Tag = "Open";
            this.ToolBarButtonFileSave.Tag = "Save";
            this.ToolBarButtonEditCut.Tag = "Cut";
            this.ToolBarButtonEditCopy.Tag = "Copy";
            this.ToolBarButtonEditPaste.Tag = "Paste";
            this.ToolBarButtonEditUndo.Tag = "Undo";
            this.ToolBarButtonEditRedo.Tag = "Redo";
            this.ToolBarButtonLayoutHierarchic.Tag = "Hierarchic";
            this.ToolBarButtonLayoutSymmetric.Tag = "Symmetric";
            this.ToolBarButtonLayoutTree.Tag = "Tree";
            this.ToolBarButtonLayoutOrthogonal.Tag = "Orthogonal";
            this.ToolBarButtonLayoutSeriesParallel.Tag = "Series Parallel";

            CheckBoxDisplayJumps.Checked = (AddFlow1.DefLinkProp.Jump != Jump.None);
            CheckBoxDirectedGraph.Checked = (AddFlow1.DefLinkProp.ArrowDst.Style == ArrowStyle.Arrow);

            // HFlow parameters
            ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.North.ToString());
            ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.East.ToString());
            ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.South.ToString());
            ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.West.ToString());
            TextBoxHFlowVertexDistance.Text = hflow.VertexDistance.ToString();
            TextBoxHFlowLayerDistance.Text = hflow.LayerDistance.ToString();
            TextBoxHFlowLayerWidth.Text = hflow.LayerWidth.ToString();
            TextBoxHFlowXMargin.Text = hflow.MarginSize.Width.ToString();
            TextBoxHFlowYMargin.Text = hflow.MarginSize.Height.ToString();
            ComboBoxHFlowOrientation.SelectedItem = hflow.Orientation.ToString();

            // OFlow parameters
            ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.North.ToString());
            ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.East.ToString());
            ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.South.ToString());
            ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.West.ToString());
            TextBoxOFlowXMargin.Text = oflow.MarginSize.Width.ToString();
            TextBoxOFlowYMargin.Text = oflow.MarginSize.Height.ToString();
            TextBoxOFlowNodeSizeRatio.Text = oflow.NodeSizeRatio.ToString();
            ComboBoxOFlowOrientation.SelectedItem = oflow.Orientation.ToString();
            TextBoxOFlowXGrid.Text = oflow.GridSize.Width.ToString();
            TextBoxOFlowYGrid.Text = oflow.GridSize.Height.ToString();

            // SFlow parameters
            TextBoxSFlowXMargin.Text = sflow.MarginSize.Width.ToString();
            TextBoxSFlowYMargin.Text = sflow.MarginSize.Height.ToString();
            TextBoxSFlowVertexDistance.Text = sflow.VertexDistance.ToString();
            CheckBoxSFlowSendStepEvent.Checked = sflow.SendStepEvent;
            CheckBoxSFlowUnmoveableNodesAccepted.Checked = sflow.UnmoveableNodesAccepted;
            CheckBoxSFlowRandomStart.Checked = sflow.RandomStart;
            CheckBoxSFlowAnimation.Checked = sflow.Animation;

            // TFlow parameters
            ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.North.ToString());
            ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.East.ToString());
            ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.South.ToString());
            ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.West.ToString());
            ComboBoxTFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.Tree.DrawingStyle.Layered.ToString());
            ComboBoxTFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.Tree.DrawingStyle.Radial.ToString());
            TextBoxTFlowVertexDistance.Text = tflow.VertexDistance.ToString();
            TextBoxTFlowLayerDistance.Text = tflow.LayerDistance.ToString();
            TextBoxTFlowXMargin.Text = tflow.MarginSize.Width.ToString();
            TextBoxTFlowYMargin.Text = tflow.MarginSize.Height.ToString();
            ComboBoxTFlowOrientation.SelectedItem = tflow.Orientation.ToString();
            ComboBoxTFlowDrawingStyle.SelectedItem = tflow.DrawingStyle.ToString();
            
            // SPFlow parameters
            ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.North.ToString());
            ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.East.ToString());
            ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.South.ToString());
            ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.West.ToString());
            ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.BusOrthogonalDrawing.ToString());
            ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.StraightLine.ToString());
            ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.VisibilityDrawing.ToString());
            TextBoxSPFlowVertexDistance.Text = spflow.VertexDistance.ToString();
            TextBoxSPFlowLayerDistance.Text = spflow.LayerDistance.ToString();
            TextBoxSPFlowXMargin.Text = spflow.MarginSize.Width.ToString();
            TextBoxSPFlowYMargin.Text = spflow.MarginSize.Height.ToString();        
            TextBoxSPFlowVertexWidth.Text = spflow.VertexSize.Width.ToString();
            TextBoxSPFlowVertexHeight.Text = spflow.VertexSize.Height.ToString();
            ComboBoxSPFlowOrientation.SelectedItem = spflow.Orientation.ToString();
            ComboBoxSPFlowDrawingStyle.SelectedItem = spflow.DrawingStyle.ToString();

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
        }

        private void DemoLayout_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void ToolBar1_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            string key = (string)e.Button.Tag;
            switch (key)
            {
                case "New":
                    MenuItemFileNew_Click(sender, e);
                    break;
                case "Open":
                    MenuItemFileOpen_Click(sender, e);
                    break;
                case "Save":
                    MenuItemFileSave_Click(sender, e);
                    break;
                case "Cut":
                    MenuItemEditCut_Click(sender, e);
                    break;
                case "Copy":
                    MenuItemEditCopy_Click(sender, e);
                    break;
                case "Paste":
                    MenuItemEditPaste_Click(sender, e);
                    break;
                case "Undo":
                    MenuItemEditUndo_Click(sender, e);
                    break;
                case "Redo":
                    MenuItemEditRedo_Click(sender, e);
                    break;
                case "Hierarchic":
                    MenuItemLayoutHierarchic_Click(sender, e);
                    break;
                case "Symmetric":
                    MenuItemLayoutSymmetric_Click(sender, e);
                    break;
                case "Tree":
                    MenuItemLayoutTree_Click(sender, e);
                    break;
                case "Orthogonal":
                    MenuItemLayoutOrthogonal_Click(sender, e);
                    break;
                case "Series Parallel":
                    MenuItemLayoutSeriesParallel_Click(sender, e);
                    break;
            }
        }

        private void AddFlow1_BeforeAddNode(object sender, BeforeAddNodeEventArgs e)
        {
            e.Size = new SizeF(AddFlow1.Grid.Size.Width, AddFlow1.Grid.Size.Height);
        }

        private void AddFlow1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Item item = AddFlow1.PointedItem;
                if (item != null)
                {
                    Lassalle.DlgFlow.DlgFlow dlgflow = new Lassalle.DlgFlow.DlgFlow();
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        dlgflow.NodePropertyPage(AddFlow1, node);
                    }
                    else if (item is Link)
                    {
                        Link link = (Link)item;
                        dlgflow.LinkPropertyPage(AddFlow1, link);
                    }
                }
            }
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {
            MenuItemEditUndo.Enabled = AddFlow1.CanUndo;
            MenuItemEditRedo.Enabled = AddFlow1.CanRedo;
            if (MenuItemEditRedo.Enabled)
                MenuItemEditRedo.Text = "Redo " + ActionName(AddFlow1.RedoCode);
            if (MenuItemEditUndo.Enabled)
                MenuItemEditUndo.Text = "Undo " + ActionName(AddFlow1.UndoCode);
            MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count != 0);
            MenuItemEditCut.Enabled = (AddFlow1.SelectedItems.Count > 0);
            MenuItemEditCopy.Enabled = (AddFlow1.SelectedItems.Count > 0);
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            MenuItemEditDelete.Enabled = (AddFlow1.SelectedItems.Count > 0);
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemViewZoomNormal_Click(object sender, EventArgs e)
        {
            MenuItemViewZoomNormal.Checked = true;
            MenuItemViewZoomFit.Checked = false;
            AddFlow1.Zoom = new Zoom(1, 1);
        }

        private void MenuItemViewZoomFit_Click(object sender, EventArgs e)
        {
            MenuItemViewZoomNormal.Checked = false;
            MenuItemViewZoomFit.Checked = true;
            RectangleF rc = new RectangleF(new PointF(0, 0), AddFlow1.Extent);
            AddFlow1.ZoomRectangle(rc, ZoomType.Isotropic);
        }

        private void MenuItemViewGrid_Click(object sender, EventArgs e)
        {
            Lassalle.DlgFlow.DlgFlow dlgflow = new Lassalle.DlgFlow.DlgFlow();
            dlgflow.GridPropertyPage(AddFlow1);
        }

        private void MenuItemViewUserInterfaceOptions_Click(object sender, EventArgs e)
        {
            Lassalle.DlgFlow.DlgFlow dlgflow = new Lassalle.DlgFlow.DlgFlow();
            dlgflow.AddFlowPropertyPage(AddFlow1);
        }

        private void MenuItemDefaultNodeProperties_Click(object sender, EventArgs e)
        {
            Lassalle.DlgFlow.DlgFlow dlgflow = new Lassalle.DlgFlow.DlgFlow();
            dlgflow.NodePropertyPage(AddFlow1, null);
        }

        private void MenuItemDefaultLinkProperties_Click(object sender, EventArgs e)
        {
            Lassalle.DlgFlow.DlgFlow dlgflow = new Lassalle.DlgFlow.DlgFlow();
            dlgflow.LinkPropertyPage(AddFlow1, null);
        }

        private void MenuItemLayoutHierarchic_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1004);

            // Updates the properties
            hflow.MarginSize = new SizeF(Convert.ToSingle(TextBoxHFlowXMargin.Text), Convert.ToSingle(TextBoxHFlowYMargin.Text));
            hflow.Orientation = (Lassalle.Flow.Layout.Hierarchic.Orientation)ComboBoxHFlowOrientation.SelectedIndex;
            hflow.VertexDistance = Convert.ToInt32(TextBoxHFlowVertexDistance.Text);
            hflow.LayerDistance = Convert.ToInt32(TextBoxHFlowLayerDistance.Text);
            hflow.LayerWidth = Convert.ToInt32(TextBoxHFlowLayerWidth.Text);

            // Execute the layout
            hflow.Layout(AddFlow1);

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        private void MenuItemLayoutSymmetric_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1006);

            // Updates the properties
            sflow.MarginSize = new SizeF(Convert.ToSingle(TextBoxSFlowXMargin.Text), Convert.ToSingle(TextBoxSFlowYMargin.Text));
            sflow.VertexDistance = Convert.ToInt32(TextBoxSFlowVertexDistance.Text);
            sflow.SendStepEvent = CheckBoxSFlowSendStepEvent.Checked;
            sflow.UnmoveableNodesAccepted = CheckBoxSFlowUnmoveableNodesAccepted.Checked;
            sflow.RandomStart = CheckBoxSFlowRandomStart.Checked;
            sflow.Animation = CheckBoxSFlowAnimation.Checked;

            // Execute the layout
            sflow.Layout(AddFlow1);

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        private void MenuItemLayoutTree_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1005);

            // Updates the properties
            tflow.MarginSize = new SizeF(Convert.ToSingle(TextBoxTFlowXMargin.Text), Convert.ToSingle(TextBoxTFlowYMargin.Text));
            tflow.Orientation = (Lassalle.Flow.Layout.Tree.Orientation)ComboBoxTFlowOrientation.SelectedIndex;
            tflow.VertexDistance = Convert.ToInt32(TextBoxTFlowVertexDistance.Text);
            tflow.LayerDistance = Convert.ToInt32(TextBoxTFlowLayerDistance.Text);
            tflow.DrawingStyle = (Lassalle.Flow.Layout.Tree.DrawingStyle)ComboBoxTFlowDrawingStyle.SelectedIndex;

            // Execute the layout
            try
            {
                tflow.Layout(AddFlow1);
            }
            catch (TFlowException err)
            {
                MessageBox.Show(err.Message, this.Text);
            }

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        private void MenuItemLayoutOrthogonal_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1007);

            // Updates the properties
            oflow.MarginSize = new SizeF(Convert.ToSingle(TextBoxOFlowXMargin.Text), Convert.ToSingle(TextBoxOFlowYMargin.Text));
            oflow.Orientation = (Lassalle.Flow.Layout.Orthogonal.Orientation)ComboBoxOFlowOrientation.SelectedIndex;
            oflow.GridSize = new SizeF(Convert.ToInt32(TextBoxOFlowXGrid.Text), Convert.ToInt32(TextBoxOFlowYGrid.Text));
            oflow.NodeSizeRatio = Convert.ToInt32(TextBoxOFlowNodeSizeRatio.Text);

            // Execute the layout
            oflow.Layout(AddFlow1);

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }
        
        private void MenuItemLayoutSeriesParallel_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1011);

            // Updates the properties
            spflow.MarginSize = new SizeF(Convert.ToSingle(TextBoxSPFlowXMargin.Text), Convert.ToSingle(TextBoxSPFlowYMargin.Text));
            spflow.VertexSize = new SizeF(Convert.ToSingle(TextBoxSPFlowVertexWidth.Text), Convert.ToSingle(TextBoxSPFlowVertexHeight.Text));
            spflow.Orientation = (Lassalle.Flow.Layout.SP.Orientation)ComboBoxSPFlowOrientation.SelectedIndex;
            spflow.VertexDistance = Convert.ToInt32(TextBoxSPFlowVertexDistance.Text);
            spflow.LayerDistance = Convert.ToInt32(TextBoxSPFlowLayerDistance.Text);
            spflow.DrawingStyle = (Lassalle.Flow.Layout.SP.DrawingStyle)ComboBoxSPFlowDrawingStyle.SelectedIndex;

            // Execute the layout
            try
            {
                spflow.Layout(AddFlow1);
            }
            catch (SPFlowException err)
            {
                MessageBox.Show(err.Message, this.Text);
            }

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }
    
        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms DemoLayout Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void ButtonNormalizeNodes_Click(object sender, EventArgs e)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1010);

            foreach (Node node in AddFlow1.Nodes)
            {
                node.Size = new SizeF(AddFlow1.Grid.Size.Width, AddFlow1.Grid.Size.Height);
                node.Shape.Style = ShapeStyle.Rectangle;
                node.Gradient = true;
                node.GradientMode = LinearGradientMode.ForwardDiagonal;
                node.GradientColor = Color.Yellow;
            }

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        private void CheckBoxDisplayJumps_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBoxDisplayJumps.Checked)
                AddFlow1.DefLinkProp.Jump = Jump.Arc;
            else
                AddFlow1.DefLinkProp.Jump = Jump.None;

            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1009);

            // Change the jump style of each existing link
            foreach (Node node in AddFlow1.Nodes)
                foreach (Link link in node.OutLinks)
                    link.Jump = AddFlow1.DefLinkProp.Jump;

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        private void CheckBoxDirectedGraph_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBoxDirectedGraph.Checked)
                AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.Arrow;
            else
                AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None;

            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1008);

            // Change the arrow style of each existing link
            foreach (Node node in AddFlow1.Nodes)
                foreach (Link link in node.OutLinks)
                    link.ArrowDst.Style = AddFlow1.DefLinkProp.ArrowDst.Style;

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();

            CheckBoxDisplayJumps.Checked = (AddFlow1.DefLinkProp.Jump != Jump.None);
            CheckBoxDirectedGraph.Checked = (AddFlow1.DefLinkProp.ArrowDst.Style == ArrowStyle.Arrow);

            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            AddFlow1.DefNodeProp.Gradient = true;
            AddFlow1.DefNodeProp.GradientMode = LinearGradientMode.ForwardDiagonal;
            AddFlow1.DefNodeProp.GradientColor = Color.Yellow;
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }

        string ActionName(Lassalle.Flow.Action code)
        {
            switch (code)
            {
                case Lassalle.Flow.Action.NodeAdd:
                    return "node creation";
                case Lassalle.Flow.Action.NodeResize:
                    return "node resize";
                case Lassalle.Flow.Action.DeleteSel:
                    return "delete selection";
                case Lassalle.Flow.Action.MoveItems:
                    return "move items";
                case Lassalle.Flow.Action.LinkAdd:
                    return "link creation";
                case Lassalle.Flow.Action.LinkStretch:
                    return "link stretch";
                case Lassalle.Flow.Action.ZOrder:
                    return "ZOrder";
                case Lassalle.Flow.Action.Org:
                    return "link origin change";
                case Lassalle.Flow.Action.Dst:
                    return "link destination change";
                case Lassalle.Flow.Action.Text:
                    return "node text change";
                case (Lassalle.Flow.Action)1001:
                    return "node properties change";
                case (Lassalle.Flow.Action)1002:
                    return "link properties change";
                case (Lassalle.Flow.Action)1003:
                    return "paste";
                case (Lassalle.Flow.Action)1004:
                    return "hierarchical layout";
                case (Lassalle.Flow.Action)1005:
                    return "tree layout";
                case (Lassalle.Flow.Action)1006:
                    return "symmetric layout";
                case (Lassalle.Flow.Action)1007:
                    return "orthogonal layout";
                case (Lassalle.Flow.Action)1008:
                    return "change of the directed status of links";
                case (Lassalle.Flow.Action)1009:
                    return "change of the jump style of links";
                case (Lassalle.Flow.Action)1010:
                    return "normalize nodes";
                case (Lassalle.Flow.Action)1011:
                    return "series parallel layout";
                default:
                    // here you can add your own action code
                    // (see the AddFlow method: BeginLassalle.Flow.Action)
                    return "";
            }
        }
    }
}