//-------------------------------------------------------------------------
// AddFlow for WinForms: GraphTest sample
// Copyright (c) 2003-2010 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//---------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using Lassalle.Flow;
using Lassalle.GraphAlgo;

namespace GraphTest
{
    public partial class GraphTest : Form
    {
        string strProgName = null;
        string strFileName = null;
        private Color[] aColor = {Color.Yellow, Color.LightSkyBlue, Color.Coral, 
									 Color.GreenYellow, Color.LightPink, Color.Wheat};

        public GraphTest()
        {
            InitializeComponent();
        }

        private void GraphTest_Load(object sender, EventArgs e)
        {
            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.BackColor = SystemColors.Window;
            AddFlow1.Grid.Draw = false;
            AddFlow1.Grid.Snap = true;
            AddFlow1.Grid.Size = new Size(12, 12);
            AddFlow1.CanFireError = true;
            AddFlow1.CanLabelEdit = false;
            AddFlow1.CanSizeNode = false;
            AddFlow1.CanReflexLink = false;
            AddFlow1.CanMultiLink = false;
            AddFlow1.AutoScroll = true;
            AddFlow1.PageUnit = GraphicsUnit.Pixel;
            AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None;
            AddFlow1.DefLinkProp.BackMode = BackMode.Opaque;
            AddFlow1.DefNodeProp.Font = new Font("verdana", 8);

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
            
            // Create a diagram programmaticaly
            AddFlow1.Nodes.Add(new Node(24, 84, 24, 24));
            AddFlow1.Nodes.Add(new Node(96, 12, 24, 24));
            AddFlow1.Nodes.Add(new Node(96, 84, 24, 24));
            AddFlow1.Nodes.Add(new Node(96, 156, 24, 24));
            AddFlow1.Nodes.Add(new Node(168, 84, 24, 24));
            AddFlow1.Nodes.Add(new Node(240, 36, 24, 24));
            AddFlow1.Nodes.Add(new Node(240, 132, 24, 24));
            AddFlow1.Nodes.Add(new Node(312, 84, 24, 24));
            AddFlow1.Nodes.Add(new Node(384, 36, 24, 24));
            AddFlow1.Nodes.Add(new Node(384, 132, 24, 24));
            AddFlow1.Nodes.Add(new Node(456, 84, 24, 24));
            AddFlow1.Nodes.Add(new Node(96, 228, 24, 24));
            AddFlow1.Nodes.Add(new Node(192, 228, 24, 24));
            AddFlow1.Nodes.Add(new Node(192, 300, 24, 24));
            AddFlow1.Nodes.Add(new Node(264, 300, 24, 24));
            AddFlow1.Nodes[1].OutLinks.Add(AddFlow1.Nodes[0]);
            AddFlow1.Nodes[2].OutLinks.Add(AddFlow1.Nodes[0]);
            AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[3]);
            AddFlow1.Nodes[4].OutLinks.Add(AddFlow1.Nodes[1]);
            AddFlow1.Nodes[4].OutLinks.Add(AddFlow1.Nodes[2]);
            AddFlow1.Nodes[3].OutLinks.Add(AddFlow1.Nodes[4]);
            AddFlow1.Nodes[4].OutLinks.Add(AddFlow1.Nodes[5]);
            AddFlow1.Nodes[4].OutLinks.Add(AddFlow1.Nodes[6]);
            AddFlow1.Nodes[5].OutLinks.Add(AddFlow1.Nodes[7]);
            AddFlow1.Nodes[6].OutLinks.Add(AddFlow1.Nodes[7]);
            AddFlow1.Nodes[7].OutLinks.Add(AddFlow1.Nodes[8]);
            AddFlow1.Nodes[7].OutLinks.Add(AddFlow1.Nodes[9]);
            AddFlow1.Nodes[10].OutLinks.Add(AddFlow1.Nodes[8]);
            AddFlow1.Nodes[9].OutLinks.Add(AddFlow1.Nodes[10]);
            AddFlow1.Nodes[11].OutLinks.Add(AddFlow1.Nodes[12]);
            AddFlow1.Nodes[11].OutLinks.Add(AddFlow1.Nodes[13]);
            AddFlow1.Nodes[13].OutLinks.Add(AddFlow1.Nodes[12]);
            AddFlow1.Nodes[14].OutLinks.Add(AddFlow1.Nodes[13]);
        }

        private void GraphTest_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void AddFlow1_BeforeAddNode(object sender, BeforeAddNodeEventArgs e)
        {
            e.Size = new SizeF(2 * AddFlow1.Grid.Size.Width, 2 * AddFlow1.Grid.Size.Height);
        }

        private void AddFlow1_DoubleClick(object sender, EventArgs e)
        {
            Item item = AddFlow1.PointedItem;
            if (item is Link)
            {
                Cost cost = new Cost();
                cost.Link = (Link)item;
                cost.ShowDialog();
            }
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {
            bool isItems = (AddFlow1.Items.Count > 0);
            bool isSelectedItems = (AddFlow1.SelectedItems.Count > 0);
            MenuItem miEdit = this.Menu.MenuItems[1];
            miEdit.MenuItems[0].Enabled = AddFlow1.CanUndo;
            miEdit.MenuItems[1].Enabled = AddFlow1.CanRedo;
            miEdit.MenuItems[3].Enabled = isSelectedItems;
            miEdit.MenuItems[4].Enabled = isSelectedItems;
            miEdit.MenuItems[5].Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            miEdit.MenuItems[6].Enabled = isSelectedItems;
            miEdit.MenuItems[8].Enabled = isItems;
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void menuItemConnectedParts_Click(object sender, EventArgs e)
        {
            ResetItems(AddFlow1);

            Graph graph = new Graph();

            OpenGraph(AddFlow1, graph);

            // Find the connected parts of the graph
            graph.SeparateConnectedParts();

            // Display each connected part
            foreach (Node node in AddFlow1.Nodes)
            {
                Vertex v = (Vertex)node.Tag;
                node.Text = v.ConnectedPart.ToString();
                node.FillColor = v.ConnectedPart < aColor.Length ? aColor[v.ConnectedPart] : Color.Silver;
            }

            CloseGraph(AddFlow1);
        }

        private void menuItemArticulationPoints_Click(object sender, EventArgs e)
        {
            ResetItems(AddFlow1);
            Graph graph = new Graph();
            OpenGraph(AddFlow1, graph);

            // Find the articulation points of the graph
            graph.ArticulationPoints();

            // Show the articulation points
            foreach (Node node in AddFlow1.Nodes)
            {
                Vertex v = (Vertex)node.Tag;
                if (v.IsArticulation)
                {
                    node.Text = "A";
                    node.FillColor = Color.Yellow;
                }
            }

            CloseGraph(AddFlow1);
        }

        private void menuItemBiconnectedParts_Click(object sender, EventArgs e)
        {
            ResetItems(AddFlow1);
            Graph graph = new Graph();
            OpenGraph(AddFlow1, graph);

            // Find the biconnected parts of the graph
            graph.ArticulationPoints();
            graph.SeparateBiconnectedParts();

            // Display the biconnected parts
            foreach (Node node in AddFlow1.Nodes)
            {
                Vertex v = (Vertex)node.Tag;
                node.Text = v.BiconnectedPart.ToString();
                node.FillColor = v.BiconnectedPart < aColor.Length ? aColor[v.BiconnectedPart] : Color.Silver;
            }

            CloseGraph(AddFlow1);
        }

        private void menuItemSpanningTree_Click(object sender, EventArgs e)
        {
            if (AddFlow1.SelectedItem == null || AddFlow1.SelectedItem is Link)
            {
                MessageBox.Show("You have first to select a node with the mouse", strProgName,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            ResetItems(AddFlow1);
            Graph graph = new Graph();
            OpenGraph(AddFlow1, graph);
            Node selnode = (Node)AddFlow1.SelectedItem;
            graph.FindSpanningTree((Vertex)selnode.Tag);
            foreach (Node node in AddFlow1.Nodes)
            {
                foreach (Link link in node.OutLinks)
                {
                    Edge edge = (Edge)link.Tag;
                    if (edge.InSpanningTree)
                        link.DrawColor = Color.Red;
                }
            }
            CloseGraph(AddFlow1);
        }

        private void menuItemShortedPath_Click(object sender, EventArgs e)
        {
            if (AddFlow1.SelectedItem == null || AddFlow1.SelectedItem is Link)
            {
                MessageBox.Show("You have first to select a node with the mouse", strProgName,
                    MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                return;
            }

            ResetItems(AddFlow1);

            Graph graph = new Graph();

            OpenGraph(AddFlow1, graph);

            Node selnode = (Node)AddFlow1.SelectedItem;
            graph.FindShortestPath((Vertex)selnode.Tag);

            foreach (Node node in AddFlow1.Nodes)
            {
                foreach (Link link in node.OutLinks)
                {
                    Edge edge = (Edge)link.Tag;
                    if (edge.InPathTree)
                        link.DrawColor = Color.Red;
                }
            }

            CloseGraph(AddFlow1);
        }

        private void menuItemReset_Click(object sender, EventArgs e)
        {
            ResetItems(AddFlow1);
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow WinForms: GraphTest Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }

        void ResetItems(AddFlow addflow)
        {
            foreach (Node node in addflow.Nodes)
            {
                // Restore colors and text
                node.FillColor = addflow.DefNodeProp.FillColor;
                node.Text = null;
            }
            foreach (Node node in addflow.Nodes)
                foreach (Link link in node.OutLinks)
                    link.DrawColor = addflow.DefLinkProp.DrawColor;
        }

        void OpenGraph(AddFlow addflow, Graph graph)
        {
            foreach (Node node in addflow.Nodes)
            {
                // Create a vertex corresponding to this node
                Vertex v = graph.AddVertex(node);
                // Save the node Tag (in case we use it) because we will use it for another purpose
                v.Tag = node.Tag;
                // Use the node Tag to save the corresponding vertex
                node.Tag = v;
            }
            foreach (Node node in addflow.Nodes)
            {
                foreach (Link link in node.OutLinks)
                {
                    // Create an edge corresponding to this link
                    Vertex org = (Vertex)link.Org.Tag;
                    Vertex dst = (Vertex)link.Dst.Tag;
                    int cost = (link.Text != null && link.Text.Length > 0) ? Int32.Parse(link.Text) : 0;
                    Edge e = graph.AddEdge(org, dst, link, cost);
                    e.Tag = link.Tag;
                    link.Tag = e;
                }
            }
        }

        void CloseGraph(AddFlow addflow)
        {
            // Restore the initial node Tags
            foreach (Node node in addflow.Nodes)
            {
                Vertex v = (Vertex)node.Tag;
                node.Tag = v.Tag;
            }
        }
    }
}