//----------------------------------------------------------------------------------------
// AddFlow for WinForms: Lists sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Lassalle.Flow;

namespace Lists
{
    public partial class Lists : Form
    {
        bool hideControls = false;
        ListView currentList = null; 

        public Lists()
        {
            InitializeComponent();
        }

        private void Lists_Load(object sender, EventArgs e)
        {
            hideControls = false;
            currentList = null;

            addFlow1.Dock = DockStyle.Fill;
            addFlow1.BackColor = SystemColors.Window;
            addFlow1.AutoScroll = true;
            addFlow1.PageUnit = GraphicsUnit.Point;
            addFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            addFlow1.DefNodeProp.LabelEdit = false;
            addFlow1.DefNodeProp.FillColor = Color.LightYellow;
            addFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom;
            addFlow1.DefNodeProp.Shadow.Color = Color.Silver;
            addFlow1.DefNodeProp.Alignment = Alignment.CenterTOP;
            addFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None;
            addFlow1.DefLinkProp.EndCap = System.Drawing.Drawing2D.LineCap.Custom;
            addFlow1.DefLinkProp.CustomEndCap = new System.Drawing.Drawing2D.AdjustableArrowCap(3, 6, true);
            addFlow1.DefLinkProp.DrawColor = Color.Blue;
            addFlow1.DefLinkProp.Jump = Jump.None;
            addFlow1.BeforeAddNode += new AddFlow.BeforeAddNodeEventHandler(addFlow1_BeforeAddNode);
            addFlow1.AfterMove += new AddFlow.AfterMoveEventHandler(addFlow1_AfterMove);
            addFlow1.AfterResize += new AddFlow.AfterResizeEventHandler(addFlow1_AfterResize);
            addFlow1.MouseDown += new MouseEventHandler(addFlow1_MouseDown);
            addFlow1.MouseUp += new MouseEventHandler(addFlow1_MouseUp);

            MyNode mynode1 = new MyNode(addFlow1, new PointF(33, 20), new SizeF(120, 100));
            MyNode mynode2 = new MyNode(addFlow1, new PointF(367, 33), new SizeF(120, 100));

            ListView lv1 = mynode1.List;
            if (lv1 != null) 
            {
                ListViewItem lvi11 = new ListViewItem("field11");
                lvi11.SubItems.Add("subfield 1");
                ListViewItem lvi12 = new ListViewItem("field12");
                lvi12.SubItems.Add("subfield 2");
                ListViewItem lvi13 = new ListViewItem("field13");
                lvi13.SubItems.Add("subfield 3");
                ListViewItem lvi14 = new ListViewItem("field14");
                lvi14.SubItems.Add("subfield 4");
                ListViewItem lvi15 = new ListViewItem("field15");
                lvi15.SubItems.Add("subfield 5");
                ListViewItem lvi16 = new ListViewItem("field16");
                lvi16.SubItems.Add("subfield 6");
                lv1.Items.AddRange(new ListViewItem[] {lvi11, lvi12, lvi13, lvi14, lvi15, lvi16});
            }

            ListView lv2= mynode2.List;
            if (lv2 != null)
            {
                ListViewItem lvi21 = new ListViewItem("field11");
                lvi21.SubItems.Add("subfield 1");
                ListViewItem lvi22 = new ListViewItem("field12");
                lvi22.SubItems.Add("subfield 2");
                ListViewItem lvi23 = new ListViewItem("field13");
                lvi23.SubItems.Add("subfield 3");
                ListViewItem lvi24 = new ListViewItem("field14");
                lvi24.SubItems.Add("subfield 4");
                lv2.Items.AddRange(new ListViewItem[] { lvi21, lvi22, lvi23, lvi24 });
            }

            // Create a link
            addFlow1.Nodes[0].OutLinks.Add(addFlow1.Nodes[1]);
        }

        private void MenuExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Lists sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void addFlow1_BeforeAddNode(object sender, BeforeAddNodeEventArgs e)
        {        
            // Cancel the node creation. Instead we create a MyNode object at the same place.
            e.Cancel.Cancel = true;
            MyNode mynode = new MyNode(addFlow1, e.Location, e.Size);
        }

        private void addFlow1_AfterResize(object sender, System.EventArgs e)
        {
            foreach (Item item in addFlow1.SelectedItems)
            {
                if (item is Node)
                {
                    MyNode mynode = (MyNode)item;
                    mynode.UpdateListPositionAndSize();
                }
            }
        }

        private void addFlow1_AfterMove(object sender, System.EventArgs e)
        {
            if (addFlow1.SelectedItem == null)
                return;
            MyNode mynode = (MyNode)addFlow1.SelectedItem;
            mynode.UpdateListPositionAndSize();
        }

        private void addFlow1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {      
            if (currentList != null) 
                currentList.Enabled = false;
            if (addFlow1.InteractiveAction == InteractiveAction.Size || addFlow1.InteractiveAction == InteractiveAction.Drag)
            {
                hideControls = true;
                foreach (Item item in addFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        MyNode mynode = (MyNode)item;
                        if (mynode.List != null)
                            mynode.List.Visible = false;
                    }
                }
            }
            else
            {
                Item item = addFlow1.SelectedItem;
                if (item is Node)
                {
                    MyNode mynode = (MyNode)item;
                    currentList = mynode.List;
                    currentList.Enabled = true;
                }
            }
        }

        private void addFlow1_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            if (hideControls)
            {
                hideControls = false;
                foreach (Item item in addFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        MyNode mynode = (MyNode)item;
                        if (mynode.List != null) 
                            mynode.List.Visible = true;
                    }
                }
            }
        }
    }

    // Create a new node class, derived from the Node class of AddFlow.
    public class MyNode : Node
    {
        ListView m_list = null;
        int xMargin = 10;
        int yMargin = 10;

        public MyNode(AddFlow addFlow1, PointF location, SizeF size) : base(addFlow1.DefNodeProp)
        {
            this.Location = location;
            this.Size = size;
            addFlow1.Nodes.Add((Node)this);

            // Create the associated control 
            m_list = new ListView();
            m_list.Parent = addFlow1;
            m_list.BorderStyle = BorderStyle.FixedSingle;
            m_list.LabelEdit = true;
            m_list.GridLines = true;
            m_list.View = View.Details;
            m_list.FullRowSelect = true;
            m_list.Enabled = false;

            m_list.Columns.Add("Col 1", 60, HorizontalAlignment.Left);
            m_list.Columns.Add("Col 2", 60, HorizontalAlignment.Left);

            UpdateListPositionAndSize();
        }

        public ListView List
        {
            get { return m_list; }
        }
        
        public void UpdateListPositionAndSize()
        {
            if (m_list == null)
                return;
            PointF pt1 = this.AddFlow.PointFToDevice(Location);
            PointF pt2 = this.AddFlow.PointFToDevice(new PointF(Location.X + Size.Width, Location.Y + Size.Height));
            m_list.SetBounds((int)pt1.X + xMargin, (int)pt1.Y + yMargin, (int)pt2.X - (int)pt1.X - 2 * xMargin, (int)pt2.Y - (int)pt1.Y - 2 * yMargin);
        }
    }
}