//-------------------------------------------------------------------------
// AddFlow for WinForms: Navig C# sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//---------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using Lassalle.Flow;

namespace Navig
{
    public partial class Navig : Form
    {
        public Navig()
        {
            InitializeComponent();
        }

        private void Navig_Load(object sender, EventArgs e)
        {
            Node node;
            Link link;

            AddFlow1.LinkCreationMode = LinkCreationMode.MiddleHandle;
            AddFlow1.CanDragScroll = false;
            AddFlow1.PageUnit = GraphicsUnit.Point;
            AddFlow1.DefNodeProp.Gradient = true;
            AddFlow1.DefNodeProp.GradientMode = System.Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal;
            AddFlow1.DefNodeProp.GradientColor = Color.LightBlue;

            node = AddFlow1.Nodes.Add(67, 77, 26, 30, "1");
            node.Shape.Style = ShapeStyle.Rectangle;
            node.FillColor = Color.Yellow;
            node = AddFlow1.Nodes.Add(7, 52, 30, 25, "2");
            node = AddFlow1.Nodes.Add(68, 24, 30, 30, "3");
            node.Selected = true;
            node = AddFlow1.Nodes.Add(150, 68, 30, 30, "4");
            node.Selected = true;
            node = AddFlow1.Nodes.Add(7, 126, 41, 38, "5: owned by 1");
            node = AddFlow1.Nodes.Add(21, 10, 25, 25, "6");
            node = AddFlow1.Nodes.Add(28, 215, 30, 30, "7");
            node = AddFlow1.Nodes.Add(100, 150, 41, 38, "8: owned by 1");
            node = AddFlow1.Nodes.Add(150, 200, 41, 38, "9: owned by 8");
            node = AddFlow1.Nodes.Add(235, 75, 30, 30, "10");
            node = AddFlow1.Nodes.Add(185, 10, 30, 30, "11");

            link = AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[1]);
            link = AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[2]);
            link = AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[5]);
            link = AddFlow1.Nodes[3].OutLinks.Add(AddFlow1.Nodes[0]);
            link = AddFlow1.Nodes[6].OutLinks.Add(AddFlow1.Nodes[0]);
            link = AddFlow1.Nodes[4].OutLinks.Add(AddFlow1.Nodes[6]);
            link.Selected = true;
            link = AddFlow1.Nodes[7].OutLinks.Add(AddFlow1.Nodes[0]);
            link.Selected = true;
            link = AddFlow1.Nodes[8].OutLinks.Add(AddFlow1.Nodes[7]);
            link = AddFlow1.Nodes[9].OutLinks.Add(AddFlow1.Nodes[10]);

            // Links that allow ownership between nodes
            link = AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[4]);
            link.Hidden = true;
            link.Rigid = true;
            link.Selectable = false;
            link.Logical = false;
            link = AddFlow1.Nodes[0].OutLinks.Add(AddFlow1.Nodes[7]);
            link.Hidden = true;
            link.Rigid = true;
            link.Selectable = false;
            link.Logical = false;
            link = AddFlow1.Nodes[7].OutLinks.Add(AddFlow1.Nodes[8]);
            link.Hidden = true;
            link.Rigid = true;
            link.Selectable = false;
            link.Logical = false;
        }

        private void ExitMenu_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AboutMenu_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Navig sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void Button0_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].Links)
                if (!link.Rigid)
                {
                    Node node = AddFlow1.Nodes[0].GetLinkedNode(link);
                    node.DrawColor = Color.Red;
                }
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].OutLinks)
                if (!link.Rigid)
                    link.Dst.DrawColor = Color.Red;
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].InLinks)
                if (!link.Rigid)
                    link.Org.DrawColor = Color.Red;  
        }

        private void Button3_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].Links)
                if (!link.Rigid)
                    link.DrawColor = Color.Red;
        }

        private void Button4_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].InLinks)
                if (!link.Rigid)
                    link.DrawColor = Color.Red;
        }

        private void Button5_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].OutLinks)
                if (!link.Rigid)
                    link.DrawColor = Color.Red;
        }

        private void Button10_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Link link in AddFlow1.Nodes[0].OutLinks)
                if (link.Rigid)
                    link.Dst.DrawColor = Color.Red;
        }

        private void Button12_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Item item in AddFlow1.Nodes[0].ConnectedItems)
                if (item is Node)
                    item.DrawColor = Color.Red;
        }

        private void Button6_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Node node in AddFlow1.Nodes)
                node.DrawColor = Color.Red;
        }

        private void Button7_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Node node in AddFlow1.Nodes)
            {
                foreach (Link link in node.OutLinks)
                    link.DrawColor = Color.Red;
            }
        }

        private void Button8_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Node node in AddFlow1.Nodes)
            {
                node.DrawColor = Color.Red;
                foreach (Link link in node.OutLinks)
                    link.DrawColor = Color.Red;
            }
        }

        private void Button9_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Item item in AddFlow1.SelectedItems)
                if (item is Node)
                    item.DrawColor = Color.Red;
        }

        private void Button11_Click(object sender, EventArgs e)
        {
            ResetColors();
            foreach (Item item in AddFlow1.SelectedItems)
                if (item is Link)
                    item.DrawColor = Color.Red;
        }

        // Reset draw color of every items (nodes and links)
        private void ResetColors()
        {
            foreach (Node node in AddFlow1.Nodes)
            {
                node.DrawColor = Color.Black;
                foreach (Link link in node.OutLinks)
                    link.DrawColor = Color.Black;
            }
        }
    }
}