//----------------------------------------------------------------------------------------
// AddFlow for WinForms: OwnerDraw sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Lassalle.Flow;

namespace OwnerDraw
{
    public partial class OwnerDraw : Form
    {
        public OwnerDraw()
        {
            InitializeComponent();
        }

        private void ExitMenu_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AboutMenu_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: OwnerDraw sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void OwnerDraw_Load(object sender, EventArgs e)
        {
            addFlow1.Dock = DockStyle.Fill;
            addFlow1.AutoScroll = true;
            addFlow1.OwnerDraw = true;
            addFlow1.PageUnit = GraphicsUnit.Point;
            addFlow1.DefNodeProp.OwnerDraw = true;
            addFlow1.DefLinkProp.OwnerDraw = true;
            addFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            addFlow1.NodeOwnerDraw += new Lassalle.Flow.AddFlow.NodeOwnerDrawEventHandler(this.addFlow1_NodeOwnerDraw);
            addFlow1.LinkOwnerDraw += new Lassalle.Flow.AddFlow.LinkOwnerDrawEventHandler(this.addFlow1_LinkOwnerDraw);
            addFlow1.DiagramOwnerDraw += new Lassalle.Flow.AddFlow.DiagramOwnerDrawEventHandler(this.addFlow1_DiagramOwnerDraw);

            // Create 2 nodes and a link
            Node node1  = new Node(30, 30, 50, 50, "node1", addFlow1.DefNodeProp);
            Node node2 = new Node(330, 30, 50, 50, "node2", addFlow1.DefNodeProp);
            Link link = new Link("link 12", addFlow1.DefLinkProp);
            link.DrawColor = Color.Red;

            // Insert them in the graph
            addFlow1.Nodes.Add(node1);
            addFlow1.Nodes.Add(node2);
            node1.OutLinks.Add(link, node2);

            // Add link points
            link.Points.Add(new PointF(55, 160));
            link.Points.Add(new PointF(355, 160));
        }

        private void addFlow1_DiagramOwnerDraw(object sender, Lassalle.Flow.DiagramOwnerDrawEventArgs e)
        {
            Graphics grfx = e.Graphics;

            // Save the graphics state because it does not belong to us
            // (although in this case, it not necessary because we do not alter the Graphics state)
            GraphicsState gs = grfx.Save();

            int swimLinesHeight = 50;
            int w = (int)(addFlow1.ClientRectangle.Size.Width + addFlow1.Extent.Width);
            int h = (int)(addFlow1.ClientRectangle.Size.Height + addFlow1.Extent.Height);
            int swimlines = h / swimLinesHeight;
            Rectangle rect;
            for (int i = 0; i <= swimlines; i++)
            {
                rect = new Rectangle(0, i * swimLinesHeight, w, swimLinesHeight);
                if (i % 2 == 0)
                    grfx.FillRectangle(Brushes.LightYellow, rect);
                else
                    grfx.FillRectangle(Brushes.LightBlue, rect);
            }

            // Restore the graphics state
            grfx.Restore(gs);
        }

        private void addFlow1_NodeOwnerDraw(object sender, Lassalle.Flow.NodeOwnerDrawEventArgs e)
        {
            Graphics grfx = e.Graphics;
            Node node = e.Node;

            // Save the graphics state because it does not belong to us
            // (although in this case, it not necessary because we do not alter the Graphics state)
            GraphicsState gs = grfx.Save();

            HatchBrush hBrush1 = new HatchBrush(HatchStyle.BackwardDiagonal, Color.White, Color.Silver);
            grfx.FillRectangle(hBrush1, node.Rect);

            // Restore the graphics state
            grfx.Restore(gs);

            // Tell AddFlow not to fill the node (because this is already done)
            e.Flags = ~NodeDrawFlags.Fill;
        }

        private void addFlow1_LinkOwnerDraw(object sender, Lassalle.Flow.LinkOwnerDrawEventArgs e)
        {
            Graphics grfx = e.Graphics;
            Link link = e.Link;
            
            if (link.Points.Count > 2)
            {
                // Save the graphics state because it does not belong to us
                // (although in this case, it not necessary because we do not alter the Graphics state)
                GraphicsState gs = grfx.Save();

                int degrees = 0;
                Brush brush = new SolidBrush(link.TextColor);
                for (int i = 0; i < link.Points.Count - 1; i++)
                {
                    float x1  = link.Points[i].X;
                    float y1 = link.Points[i].Y;
                    float x2 = link.Points[i + 1].X;
                    float y2 = link.Points[i + 1].Y;
                    float x = x2 - x1;
                    float y = -(y2 - y1);  // ... since the y axis is bottom directed 
                    float d = (float)Math.Sqrt(x * x + y * y);
                    float cosA = x / d;
                    float sinA = y / d;
                    if (d == 0)
                        degrees = 0;
                    else
                    {
                        degrees = (int)(180.0 / 3.1415899999999999 * Math.Acos(cosA));
                        if (sinA < 0)
                            degrees = -degrees;
                        if (cosA < 0) 
                            degrees += 180;
                    }
                    PointF pt = new PointF((link.Points[i].X + link.Points[i + 1].X) / 2, (link.Points[i].Y + link.Points[i + 1].Y) / 2);
                    grfx.TranslateTransform(pt.X, pt.Y);
                    if (degrees != 0)
                        grfx.RotateTransform(-degrees);
                    grfx.DrawString(link.Text, link.Font, brush, new PointF(0, 0));
                    if (degrees != 0) 
                        grfx.RotateTransform(degrees);
                    grfx.TranslateTransform(-pt.X, -pt.Y);
                }

                // Restore the graphics state
                grfx.Restore(gs);

                // Tell AddFlow not to display the text (because this is already done)
                // However, other parts of the link (line, arrow) will be displayed by AddFlow.
                e.Flags = ~LinkDrawFlags.Text;
            }
        }
    }
}