//---------------------------------------------------------------------------------------
// AddFlow for WinForms: Pins sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//---------------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using Lassalle.Flow;

namespace Pins
{
    public partial class Pins3 : Form
    {
        public static int pinDistance = 5;
        public static int pinWidth = 10;
        public static int pinHeight = 10;

        public Pins3()
        {
            InitializeComponent();
        }

        private void ExitMenu_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AboutMenu_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Pins sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void MenuDelete_Click(object sender, EventArgs e)
        {
            addFlow1.DeleteSel();
        }

        private void Pins_Load(object sender, EventArgs e)
        {
            addFlow1.Dock = DockStyle.Bottom;
            addFlow1.PageUnit = GraphicsUnit.Point;
            addFlow1.DefNodeProp.Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle;
            addFlow1.AutoScroll = true;
            addFlow1.CanReflexLink = false;
            addFlow1.CanLabelEdit = false;
            addFlow1.BackColor = SystemColors.Window; 
            addFlow1.MouseMove += new System.Windows.Forms.MouseEventHandler(addFlow1_MouseMove);
            addFlow1.AfterResize += new Lassalle.Flow.AddFlow.AfterResizeEventHandler(addFlow1_AfterResize);
            addFlow1.AfterAddNode += new Lassalle.Flow.AddFlow.AfterAddNodeEventHandler(addFlow1_AfterAddNode);
            addFlow1.BeforeAddLink += new Lassalle.Flow.AddFlow.BeforeAddLinkEventHandler(addFlow1_BeforeAddLink);

            // Some explanations
            string s = null;
            s = s + "This sample shows how to implement pins. Each created node has 4 pins.";
            s = s + "To create a link, bring the mouse cursor into one pin of the origin node, ";
            s = s + "press the left button, move the mouse towards the other node. When the ";
            s = s + "mouse cursor is into a pin of the other node, release the left button.";
            Label1.Text = s;
        }

        private void Pins_Resize(object sender, EventArgs e)
        {
            addFlow1.Location = new Point(0, Label1.Size.Height);
            addFlow1.Size = new Size(this.ClientSize.Width, this.ClientSize.Height - Label1.Size.Height);
        }

        private void addFlow1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {        
            bool canDrawLink = false;
            Item item = addFlow1.SelectedItem;
            if (item != null && item is Pin)
                canDrawLink = true;
            addFlow1.Cursor = canDrawLink ? Cursors.UpArrow : Cursors.Default;
        }

        private void addFlow1_AfterResize(object sender, System.EventArgs e)
        {
            Item item = addFlow1.SelectedItem;
            if (item != null && item is Node)
            {
                Node node = (Node)item;
                UpdateNodePosition(node);
            }
        }

        private void addFlow1_AfterAddNode(object sender, Lassalle.Flow.AfterAddNodeEventArgs e)
        {
            CreateMyComplexNode(e.Node);
        }

        private void addFlow1_BeforeAddLink(object sender, Lassalle.Flow.BeforeAddLinkEventArgs e)
        {        
            bool cancel = true;
            if (e.Dst is Pin && e.Org is Pin)
            {
                Pin pinDst = (Pin)e.Dst;
                Pin pinOrg = (Pin)e.Org;
                if (pinDst.Node != pinOrg.Node)
                    cancel = false;
            }
            e.Cancel.Cancel = cancel;
        }
        
        void CreateMyComplexNode(Node node)
        {
            // Avoid too small nodes
            if (node.Rect.Width < 50) 
                node.Size = new SizeF(50, node.Rect.Height);
            if (node.Rect.Height < 50) 
                node.Size = new SizeF(node.Rect.Width, 50);

            // Make node transparent
            node.DrawColor = Color.Transparent;
            node.Transparent = true;

            // Create "child" nodes
            CreateLabelNode(node);
            CreatePins(node);
        }

        void CreateLabelNode(Node node)
        {
            float x = node.Rect.Left + pinDistance + pinWidth;
            float y = node.Rect.Top + pinDistance + pinHeight;
            float cx = node.Rect.Width - 2 * (pinDistance + pinWidth);
            float cy = node.Rect.Height - 2 * (pinDistance + pinHeight);

            // Create the label node...
            Node labelNode = addFlow1.Nodes.Add(x, y, cx, cy);
            labelNode.Selectable = false;
            labelNode.Logical = false;
            labelNode.Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle;
            labelNode.Parent = node;

            // Memorize a reference to the label node in the main node
            node.Tag = labelNode;
        }

        void CreatePins(Node node)
        {
            Node labelNode = (Node)node.Tag;
            for (int i = 1; i <= 4; i++)
            {
                Pin pin = new Pin(node, i);
                addFlow1.Nodes.Add(pin);
                pin.Parent = labelNode;
            }
        }

        void UpdateNodePosition(Node node)
        {
            float x = node.Rect.Left + pinDistance + pinWidth;
            float y = node.Rect.Top + pinDistance + pinHeight;
            float cx = node.Rect.Width - 2 * (pinDistance + pinWidth);
            float cy = node.Rect.Height - 2 * (pinDistance + pinHeight);

            Node labelNode = (Node)node.Tag;
            labelNode.Rect = new RectangleF(x, y, cx, cy);
            
            foreach (Node child in labelNode.Children)
            {
                if (child is Pin)
                {
                    Pin pin = (Pin)child;
                    pin.SetPinLocation();
                }
            }
        }
    }

    public class Pin : Node
    {
        int m_number = 0;
        Node m_node = null;

        public Pin(Node node, int number)
        {
            m_node = node;
            m_number = number;

            SetPinLocation();
            Size = new SizeF(Pins3.pinWidth, Pins3.pinHeight);
            FillColor = Color.Red;
            XMoveable = false;
            YMoveable = false;
            XSizeable = false;
            YSizeable = false;
            Logical = false;
            Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle;
        }

        public void SetPinLocation()
        {
            float x = Node.Rect.Left + Pins3.pinDistance + Pins3.pinWidth / 2;
            float y = Node.Rect.Top + Pins3.pinDistance + Pins3.pinHeight / 2;
            float cx = Node.Rect.Width - 2 * (Pins3.pinDistance + Pins3.pinWidth / 2);
            float cy = Node.Rect.Height - 2 * (Pins3.pinDistance + Pins3.pinHeight / 2);
            RectangleF rc = new RectangleF(x, y, cx, cy);
            switch(m_number)
            {
                case 1:
                    x = rc.Left - Pins3.pinDistance - Pins3.pinWidth / 2;
                    y = rc.Top + rc.Height / 2 - Pins3.pinHeight / 2;
                    break;
                case 2:
                    x = rc.Left + rc.Width / 2 - Pins3.pinWidth / 2;
                    y = rc.Top - Pins3.pinDistance - Pins3.pinHeight / 2;
                    break;
                case 3:
                    x = rc.Right + Pins3.pinDistance - Pins3.pinWidth / 2;
                    y = rc.Top + rc.Height / 2 - Pins3.pinHeight / 2;
                    break;
                case 4:
                    x = rc.Left + rc.Width / 2 - Pins3.pinWidth / 2;
                    y = rc.Bottom + Pins3.pinDistance - Pins3.pinHeight / 2;
                    break;
            }
            this.Location = new PointF(x, y);
        }
        
        public int Number
        {
            get { return m_number; }
            set { m_number = value; }
        }

        public Node Node
        {
            get { return m_node; }
            set { m_node = value; }
        }
    }
}