//----------------------------------------------------------------------------------------
// AddFlow for WinForms: RouteLink sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using Lassalle.Flow;

namespace RouteLink
{
    public partial class RouteLink : Form
    {
        string strProgName = null;
        string strFileName = null;
        
        Node m_animNode = null;
        bool m_sleep = false;
        float m_grain  = 8;
        float m_minDistance = 16;
        Lassalle.Flow.Router.StartingMethod m_startingMethod = Lassalle.Flow.Router.StartingMethod.AllSides;
        bool m_sendStepEvent = false;

        public RouteLink()
        {
            InitializeComponent();
        }

        private void RouteLink_Load(object sender, EventArgs e)
        {
            StatusBar1.Dock = DockStyle.Bottom;

            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.BackColor = SystemColors.Window;
            AddFlow1.Grid.Draw = true;
            AddFlow1.Grid.Snap = true;
            AddFlow1.Grid.Style = GridStyle.DottedLines;
            AddFlow1.Grid.Color = Color.Silver;
            AddFlow1.Grid.Size = new Size(20, 20);
            AddFlow1.CanFireError = true;
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            AddFlow1.DefNodeProp.Gradient = true;
            AddFlow1.DefNodeProp.GradientColor = Color.LightGray;
            AddFlow1.DefNodeProp.Text = "Obstacle";
            AddFlow1.AutoScroll = true;
            AddFlow1.DefLinkProp.Jump = Jump.None;
            AddFlow1.DefLinkProp.AdjustDst = true;
            AddFlow1.DefLinkProp.AdjustOrg = true;

            this.ToolBarButtonFileNew.Tag = "New";
            this.ToolBarButtonFileOpen.Tag = "Open";
            this.ToolBarButtonFileSave.Tag = "Save";
            this.ToolBarButtonEditCut.Tag = "Cut";
            this.ToolBarButtonEditCopy.Tag = "Copy";
            this.ToolBarButtonEditPaste.Tag = "Paste";
            this.ToolBarButtonEditUndo.Tag = "Undo";
            this.ToolBarButtonEditRedo.Tag = "Redo";

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();

            // Create an unselectable "label" node
            Node label = AddFlow1.Nodes.Add(0, 0, 500, 30);
            label.Transparent = true;
            label.Logical = false;
            label.Selectable = false;
            label.TextColor = Color.Blue;
            label.DrawColor = Color.Transparent;
            label.Text = "Create a link between the origin node and the destination node. It will avoid the obstacle nodes.";

            // Create an origin node and a destination node
            Node org = AddFlow1.Nodes.Add(40, 60, 60, 60);
            org.Text = "origin";
            org.FillColor = Color.Yellow;
            org.Gradient = false;

            Node dst = AddFlow1.Nodes.Add(500, 200, 60, 60);
            dst.Text = "destination";
            dst.FillColor = Color.Yellow;
            dst.Gradient = false;

            // Create 2 obstacle nodes
            AddFlow1.Nodes.Add(240, 60, 60, 60);
            AddFlow1.Nodes.Add(140, 200, 60, 60);
        }

        private void RouteLink_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }        

        private void ToolBar1_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            string key = (string)e.Button.Tag;
            switch (key)
            {
                case "New":
                    MenuItemFileNew_Click(sender, e);
                    break;
                case "Open":
                    MenuItemFileOpen_Click(sender, e);
                    break;
                case "Save":
                    MenuItemFileSave_Click(sender, e);
                    break;
                case "Cut":
                    MenuItemEditCut_Click(sender, e);
                    break;
                case "Copy":
                    MenuItemEditCopy_Click(sender, e);
                    break;
                case "Paste":
                    MenuItemEditPaste_Click(sender, e);
                    break;
                case "Undo":
                    MenuItemEditUndo_Click(sender, e);
                    break;
                case "Redo":
                    MenuItemEditRedo_Click(sender, e);
                    break;
            }
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemRouteParameters_Click(object sender, EventArgs e)
        {
            RouteParameters rp = new RouteParameters();
            rp.SendStepEvent = m_sendStepEvent;
            rp.Grain = m_grain;
            rp.MinDistance = m_minDistance;
            rp.StartingMethod = m_startingMethod;
            if (rp.ShowDialog() == DialogResult.OK) 
            {
                m_sendStepEvent = rp.SendStepEvent;
                m_grain = rp.Grain;
                m_minDistance = rp.MinDistance;
                m_startingMethod = rp.StartingMethod;
            }
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: RouteLink Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }

        private void AddFlow1_AfterAddLink(object sender, AfterAddLinkEventArgs e)
        {      
            Lassalle.Flow.Router.RouteFlow route = new Lassalle.Flow.Router.RouteFlow();
            route.SendStepEvent = m_sendStepEvent;
            route.Grain = m_grain;
            route.MinDistance = m_minDistance;
            route.StartingMethod = m_startingMethod;

           if (route.SendStepEvent)
           {
               // In animation mode, we do not register the actions in the undo/redo buffer
               AddFlow1.SkipUndo = true;

               route.Step += new Lassalle.Flow.Router.RouteFlow.StepEventHandler(this.route_step);
               m_animNode = new Node(0, 0, 3, 3);
               m_animNode.Shape.Style = ShapeStyle.Ellipse;
               m_animNode.Logical = false;
               m_animNode.Selectable = false;
               m_animNode.DrawColor = Color.Red;
               m_animNode.FillColor = Color.Red;
               AddFlow1.Nodes.Add(m_animNode);

               Timer timer = new Timer();
               timer.Interval = 50;
               timer.Tick += new EventHandler(timer_Tick); 
               timer.Start();

               m_sleep = false;

               e.Link.Hidden = true;
               route.FindRoute(e.Link);
               e.Link.Hidden = false;

               timer.Tick -= new EventHandler(timer_Tick); 
               route.Step -= new Lassalle.Flow.Router.RouteFlow.StepEventHandler(this.route_step);

               timer.Stop();
               AddFlow1.Nodes.Remove(m_animNode);

               AddFlow1.SkipUndo = false;
           }
           else
           {
                // So that the user will be able to undo the autorouting in one time
                AddFlow1.BeginAction(1007);

                route.FindRoute(e.Link);

                // Terminate the action for the undo processor
                AddFlow1.EndAction();
           }
        }
        
        private void timer_Tick(object sender, EventArgs e)
        {
            m_sleep = false;
        }

        private void route_step(object sender, Lassalle.Flow.Router.StepEventArgs e)
        {
            do 
            {
                Application.DoEvents();
            } while (m_sleep);
            m_sleep = true;
            m_animNode.Location = e.Position;
        }

        private void AddFlow1_AfterMove(object sender, EventArgs e)
        {
            Lassalle.Flow.Router.RouteFlow route = new Lassalle.Flow.Router.RouteFlow();
            route.SendStepEvent = m_sendStepEvent;
            route.Grain = m_grain;
            route.MinDistance = m_minDistance;
            route.StartingMethod = m_startingMethod;

            foreach (Item item in AddFlow1.SelectedItems)
            {
                if (item is Node)
                {
                    Node node = (Node)item;
                    foreach (Link link in node.OutLinks)
                        route.FindRoute(link);
                }
            }
        }
    }
}