//----------------------------------------------------------------------------------------
// AddFlow for WinForms: Shapes sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Lassalle.Flow;

namespace Shapes
{
    public partial class Shapes : Form
    {
        bool shadow = false;

        public Shapes()
        {
            InitializeComponent();
        }

        private void MenuExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Shapes sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void Shapes_Load(object sender, EventArgs e)
        {
            addFlow1.Dock = DockStyle.Fill;
            addFlow1.AutoScroll = true;
            addFlow1.BackColor = SystemColors.Window;
            addFlow1.PageUnit = GraphicsUnit.Point;

            addFlow1.DefNodeProp.Font = new System.Drawing.Font("Arial", (float)7.0, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, (Byte)0);
            addFlow1.DefNodeProp.FillColor = Color.LightYellow;
            addFlow1.DefNodeProp.Shadow.Color = Color.Silver;

            shadow = false;
            DrawShapes(ShapeOrientation.so_0);
        }

        private void MenuOrientation0_Click(object sender, EventArgs e)
        {
            SubChangeOrientation(ShapeOrientation.so_0);
        }

        private void MenuOrientation90_Click(object sender, EventArgs e)
        {
            SubChangeOrientation(ShapeOrientation.so_90);
        }

        private void MenuOrientation180_Click(object sender, EventArgs e)
        {
            SubChangeOrientation(ShapeOrientation.so_180);
        }

        private void MenuOrientation270_Click(object sender, EventArgs e)
        {
            SubChangeOrientation(ShapeOrientation.so_270);
        }

        private void MenuShadow_Click(object sender, EventArgs e)
        {
            shadow = !shadow;
            MenuShadow.Checked = shadow;
            ShadowStyle shadowStyle = shadow ? ShadowStyle.RightBottom : ShadowStyle.None;
            foreach (Node node in addFlow1.Nodes)
                node.Shadow.Style = shadowStyle;
        }

        private void SubChangeOrientation(ShapeOrientation orientation)
        {
            foreach (Node node in addFlow1.Nodes)
                node.Shape.Orientation = orientation;
        }
        
        private void DrawShapes(ShapeOrientation orientation)
        {
            addFlow1.Nodes.Clear();
            int i = 0;
            foreach (ShapeStyle shapeStyle in System.Enum.GetValues(typeof(ShapeStyle)))
            {
                float x = 5 + (95 * (i % 7));
                float y = 5 + (95 * (int)(i / 7));
                Node node = new Node(x, y, 60, 75, addFlow1.DefNodeProp);
                node.Shape.Style = shapeStyle;
                node.Shape.Orientation = orientation;
                node.Text = shapeStyle.ToString();
                addFlow1.Nodes.Add(node);
                i++;
            }
        }
    }
}