//----------------------------------------------------------------------------------------
// AddFlow for WinForms: Stress sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using Lassalle.Flow;

namespace Stress
{
    public partial class Stress : Form
    {    
        private int DefaultMaxNodes = 5000;
        private int XMax = 3000;
        private int YMax = 3000;
        private int XSize = 40;
        private int YSize = 40;
        private bool FlagGo = false;

        public Stress()
        {
            InitializeComponent();
        }

        private void Exitmenu_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AboutMenu_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Stress sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void Stress_Load(object sender, EventArgs e)
        {
            Panel2.Dock = DockStyle.Bottom;
            
            addFlow1.Dock = DockStyle.Fill;
            addFlow1.BackColor = SystemColors.Window;
            addFlow1.AutoScroll = true;
            addFlow1.CanUndoRedo = false;
            addFlow1.InteractiveEventsOnly = true;
            addFlow1.LinkCreationMode = LinkCreationMode.MiddleHandle;
            addFlow1.CanShowJumps = false;
            addFlow1.PageUnit = GraphicsUnit.Point;
            addFlow1.DefLinkProp.ArrowDst = Arrow.None;
            addFlow1.DefLinkProp.DrawColor = System.Drawing.Color.Blue;
            addFlow1.DefLinkProp.Jump = Jump.None;
            addFlow1.DefNodeProp.DrawColor = System.Drawing.Color.Blue;

            Text1.Text = DefaultMaxNodes.ToString();

            CommandGo.Enabled = true;
            CommandStop.Enabled = false;
        }

        private void CommandStop_Click(object sender, EventArgs e)
        {
            FlagGo = false;
        }

        private void CommandReset_Click(object sender, EventArgs e)
        {
            LabelStatus.Text = "Deletes the diagram.";
            Cursor = System.Windows.Forms.Cursors.WaitCursor;
            LabelStatus.Refresh();
            addFlow1.Nodes.Clear();
            Cursor = System.Windows.Forms.Cursors.Default;
            LabelStatus.Text = "";
            LabelNodeCount.Text = "0";
        }

        private void CommandGo_Click(object sender, EventArgs e)
        {
            bool repaint = (CheckRepaint.CheckState == CheckState.Checked);
            if (!repaint)
                addFlow1.BeginUpdate();             // Increases speed dramatically.

            int maxnodes = int.Parse(Text1.Text);   // Maximum nodes number
            int nodes = addFlow1.Nodes.Count;       // Current number of created nodes

            FlagGo = true;
            Text1.Enabled = false;
            CheckRepaint.Enabled = false;
            CommandGo.Enabled = false;
            CommandStop.Enabled = true;
            CommandReset.Enabled = false;
            LabelStatus.Text = "Creating nodes and links..";

            Random rand = new Random();
            
            // Create the first node at a random position
            Node org = new Node(rand.Next(0, XMax), rand.Next(0, YMax), XSize, YSize, addFlow1.DefNodeProp);
            addFlow1.Nodes.Add(org);

            for(;;)
            {
                // Node count increment and check
                nodes++;
                if (nodes % 100 == 0)
                {
                    LabelNodeCount.Text = nodes.ToString();

                    // Allow user interaction during the process
                    System.Windows.Forms.Application.DoEvents();
                }            
                if (nodes >= maxnodes)
                {
                    LabelStatus.Text = "Maximum reached: items creation stopped.";
                    break;
                }

                // The user may stop the items creation process
                if (!FlagGo)
                {
                    LabelStatus.Text = "Items creation stopped by the user.";
                    break;
                }

                // Create a node at a random position and create a link from previous node
                Node dst = new Node(rand.Next(0, XMax), rand.Next(0, YMax), XSize, YSize, addFlow1.DefNodeProp);
                addFlow1.Nodes.Add(dst);
                Link link = org.OutLinks.Add(dst);

                // The current destination node is the origin node for the next created link.
                org = dst;
            }

            // Repaint the diagram
            if (!repaint)
                addFlow1.EndUpdate();

            FlagGo = false;
            CommandGo.Enabled = true;
            CommandStop.Enabled = false;
            CommandReset.Enabled = true;
            CheckRepaint.Enabled = true;
            Text1.Enabled = true;
        }
    }
}