//----------------------------------------------------------------------------------------
// AddFlow for WinForms: Tables2 sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using Lassalle.Flow;

namespace Tables2
{
    public partial class Tables2 : Form
    {
        const int fieldHeight = 20;

        string strProgName = null;
        string strFileName = null;

        public Tables2()
        {
            InitializeComponent();
        }

        private void Tables2_Load(object sender, EventArgs e)
        {
            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.BackColor = Color.White;
            AddFlow1.AutoScroll = true;
            AddFlow1.PageUnit = GraphicsUnit.Pixel;
            AddFlow1.IsUnselectableItemHitable = true;
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            AddFlow1.DefNodeProp.FillColor = Color.LightYellow;
            AddFlow1.DefNodeProp.Dock = DockStyle.Top;
            AddFlow1.DefNodeProp.InLinkable = false;
            AddFlow1.DefNodeProp.DrawWidth = 2;
            AddFlow1.DefNodeProp.Alignment = Alignment.CenterTOP;
            AddFlow1.DefNodeProp.LabelEdit = false;
            AddFlow1.DefNodeProp.OutLinkable = false;
            AddFlow1.DefNodeProp.HighlightChildren = false;
            AddFlow1.DefLinkProp.Line.Style = LineStyle.Database;

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();

            DisplayDiagramExample();
        }

        private void  DisplayDiagramExample()
        {
            Node tableNode1 = AddTable("Table1", 33, 20, 100, 133);
            Node tableNode2 = AddTable("Table2", 367, 33, 100, 113);
            Node tableNode3 = AddTable("Table3", 200, 167, 100, 133);
            Node tableNode4 = AddTable("Table4", 533, 267, 100, 153);
            Node tableNode5 = AddTable("Table5", 733, 67, 100, 134);

            // Add fields in each table
            AddField(tableNode1, "field11");
            AddField(tableNode1, "field12");
            AddField(tableNode1, "field13");
            AddField(tableNode1, "field14");
            AddField(tableNode1, "field15");

            AddField(tableNode2, "field21");
            AddField(tableNode2, "field22");
            AddField(tableNode2, "field23");
            AddField(tableNode2, "field24");

            AddField(tableNode3, "field31");
            AddField(tableNode3, "field32");
            AddField(tableNode3, "field33");
            AddField(tableNode3, "field34");
            AddField(tableNode3, "field35");

            AddField(tableNode4, "field41");
            AddField(tableNode4, "field42");
            AddField(tableNode4, "field43");
            AddField(tableNode4, "field44");
            AddField(tableNode4, "field45");
            AddField(tableNode4, "field46");

            AddField(tableNode5, "field51");
            AddField(tableNode5, "field52");
            AddField(tableNode5, "field53");
            AddField(tableNode5, "field54");
            AddField(tableNode5, "field55");

            // Create some relations between fields
            Link link = null;

            link = tableNode1.Children[1].OutLinks.Add(tableNode3.Children[4]);
            link.ArrowDst.Style = ArrowStyle.Many;
            link.ArrowDst.Angle = ArrowAngle.deg30;

            link = tableNode1.Children[4].OutLinks.Add(tableNode2.Children[2]);
            link.ArrowDst.Style = ArrowStyle.Many;
            link.ArrowDst.Angle = ArrowAngle.deg30;

            link = tableNode2.Children[4].OutLinks.Add(tableNode4.Children[6]);
            link.ArrowDst.Style = ArrowStyle.None;

            link = tableNode2.Children[3].OutLinks.Add(tableNode3.Children[1]);
            link.ArrowDst.Style = ArrowStyle.Many;
            link.ArrowDst.Angle = ArrowAngle.deg30;

            link = tableNode1.Children[2].OutLinks.Add(tableNode4.Children[4]);
            link.ArrowDst.Style = ArrowStyle.None;

            link = tableNode4.Children[4].OutLinks.Add(tableNode5.Children[1]);
            link.ArrowDst.Style = ArrowStyle.None;

            AddFlow1.ResetUndoRedo();
        }


        private void Tables2_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {
            MenuItemEditUndo.Enabled = AddFlow1.CanUndo;
            MenuItemEditRedo.Enabled = AddFlow1.CanRedo;
            MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count!= 0);
            MenuItemEditCut.Enabled = (AddFlow1.SelectedItems.Count > 0);
            MenuItemEditCopy.Enabled = (AddFlow1.SelectedItems.Count > 0);
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            MenuItemEditDelete.Enabled = (AddFlow1.SelectedItems.Count > 0);
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {        
            // We want to copy the children of each selected node or link
            // Therefore we copy them in an array, then we select make them selectable 
            // and we select them.
            // As we modify the Selectable property of children nodes and we don't want to  
            // register these actions in the undo/redo list, we set the SkipUndo property to true.
            AddFlow1.SkipUndo = true;
            ArrayList aItems = new ArrayList();
            foreach (Item item in AddFlow1.SelectedItems)
                foreach (Node child in item.Children)
                    aItems.Add(child);

            foreach (Item item in aItems)
            {
                item.Selectable = true;
                item.Selected = true;
            }

            // Copy
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
            
            // Deselect the children and make them unselectable again
            foreach (Item item in aItems)
            {
                item.Selected = false;
                if (IsHeader((Node)item))
                    item.Selectable = false;
            }
            AddFlow1.SkipUndo = false;
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();
                
                ArrayList aItems = new ArrayList();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        if (!IsTableNode(node))
                            aItems.Add(node);
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }
                
                foreach (Item item in aItems)
                {
                    item.Selected = false;
                    if (IsHeader((Node)item))
                        item.Selectable = false;
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Tables2 Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }

        private Node AddTable(string header, float x, float y, float cx, float cy )
        {
            Node tableNode = new Node(x, y, cx, cy, AddFlow1.DefNodeProp);
            tableNode.Properties["Type"].Value = "Table";
            AddFlow1.Nodes.Add(tableNode);
            AddHeader(tableNode, header);
            return tableNode;
        }

        private Node AddHeader(Node tableNode, string headerName)
        {
            // The location and width given in the following line are not important
            // since the node will be docked at the top of the parent node.
            Node headerNode = new Node(0, 0, 20, fieldHeight, headerName, AddFlow1.DefNodeProp);
            headerNode.Selectable = false;
            headerNode.Tooltip = headerName;
            headerNode.FillColor = Color.LightGreen;
            headerNode.DrawColor = Color.Transparent;
            headerNode.Logical = false;
            headerNode.LabelEdit = false;
            headerNode.Properties["Type"].Value = "Header";
            AddFlow1.Nodes.Add(headerNode);
            headerNode.Parent = tableNode;
            return headerNode;
        }

        private Node AddField(Node tableNode, string fieldName) 
        {
            Node fieldNode = new Node(0, 0, 20, 20, fieldName, AddFlow1.DefNodeProp);
            fieldNode.DrawColor = Color.Transparent;
            fieldNode.Alignment = Alignment.LeftJustifyMIDDLE; // The field text is left justified
            fieldNode.Logical = false;
            fieldNode.InLinkable = true;
            fieldNode.OutLinkable = true;
            fieldNode.LabelEdit = false;
            fieldNode.Properties["Type"].Value = "Field";
            AddFlow1.Nodes.Add(fieldNode);
            fieldNode.Parent = tableNode;
            return fieldNode;
        }

        private bool IsTableNode(Node node)
        {
            return ((string)(node.Properties["Type"].Value) == "Table");
        }

        private bool IsField(Node node)
        {
            return ((string)(node.Properties["Type"].Value) == "Field");
        }

        private bool IsHeader(Node node) 
        {
            return ((string)(node.Properties["Type"].Value) == "Header");
        }

        private Node GetHeader(Node node)
        {
            if (node.Children.Count > 0 && IsHeader(node.Children[0])) 
                return node.Children[0];
            else
                return null;
        }

        private bool IsSameTable(Node node1, Node node2)
        {
            return (node1.Parent == node2.Parent && node2.Parent != null);
        }

        private void AddFlow1_BeforeAddLink(object sender, BeforeAddLinkEventArgs e)
        {
            // If the origin and the destination nodes belong to the same table then
            // cancel this link creation. 
            if (IsSameTable(e.Org, e.Dst))
                e.Cancel.Cancel = true;
        }

        private void AddFlow1_BeforeAddNode(object sender, BeforeAddNodeEventArgs e)
        {
            e.Cancel.Cancel = true;
            AddFlow1.BeginAction(1005);
            AddTable("", e.Rect.Left, e.Rect.Top, e.Rect.Width, e.Rect.Height);
            AddFlow1.EndAction();
        }

        private void AddFlow1_AfterResize(object sender, EventArgs e)
        {
            Item item = AddFlow1.SelectedItem;
            if (item is Node)
            {
                Node node = (Node)item;
                if (IsTableNode(node))
                {
                    AddFlow1.AddToLastAction();

                    // We are resizing a "table" node (field nodes are not resizable)
                    // First, we ensure that the height of this node is greater
                    // than the sum of the heights of its "field" nodes.
                    if (node.Rect.Height < node.Children.Count * fieldHeight)
                    {
                        RectangleF rect = new RectangleF(node.Rect.Location, node.Rect.Size);
                        rect.Height = node.Children.Count * fieldHeight + fieldHeight / 2;
                        node.Rect = rect;
                    }

                    AddFlow1.EndAction();
                }
            }
        }

        private void AddFlow1_ReadXMLNodeExtraData(object sender, ReadXMLNodeExtraDataEventArgs e)
        {
            if (e.Reader.Name == "Type")
                e.Node.Properties["Type"].Value = e.Reader.ReadElementContentAsString();
        }

        private void AddFlow1_WriteXMLNodeExtraData(object sender, WriteXMLNodeExtraDataEventArgs e)
        {
            e.Writer.WriteElementString("Type", (string)e.Node.Properties["Type"].Value);
        }

        private void AddFlow1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                if (AddFlow1.ContextMenu != null)
                    AddFlow1.ContextMenu.Dispose();
                if (AddFlow1.PointedItem != null)
                {
                    if (AddFlow1.PointedItem is Lassalle.Flow.Node)
                    {
                        Node node = (Node)AddFlow1.PointedItem;
                        if (IsTableNode(node))
                        {
                            AddFlow1.ContextMenu = new ContextMenu();

                            EventHandler eh1 = new EventHandler(this.OnClickHeader);
                            MenuItem menuItemHeader = new MenuItem("Header...", eh1);
                            AddFlow1.ContextMenu.MenuItems.Add(menuItemHeader);

                            EventHandler eh2 = new EventHandler(this.OnClickAddField);
                            MenuItem menuItemAddField = new MenuItem("Add Field...", eh2);
                            AddFlow1.ContextMenu.MenuItems.Add(menuItemAddField);

                            AddFlow1.ContextMenu.Show(this, new System.Drawing.Point(e.X, e.Y));
                        }
                    }
                }
            }
            else
            {
                if (AddFlow1.PointedItem != null)
                {
                    if (AddFlow1.PointedItem is Lassalle.Flow.Node)
                    {
                        Node node = (Node)AddFlow1.PointedItem;
                        // If it is a header node, then it is unselectable. 
                        // However, in such a case, we select its parent Table node.
                        if (IsHeader(node))
                        {
                            Node tableNode = node.Parent as Node;
                            if (tableNode != null)
                                tableNode.Selected = true;
                        }
                    }
                }
            }
        }

        private void OnClickHeader(object sender, System.EventArgs e)
        {
            Header header = new Header();
            if (header.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string headername = header.textBoxHeader.Text;
                Item item = AddFlow1.PointedItem;
                if (item != null && item is Node)
                {
                    Node node = (Node)item;
                    Node headerNode = GetHeader(node);
                    headerNode.Text = headername;
                }
            }
        }

        private void OnClickAddField(object sender, System.EventArgs e)
        {
            Field field = new Field();
            if (field.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string fieldname = field.textBoxField.Text;
                Item item = AddFlow1.PointedItem;
                if (item != null && item is Node)
                {
                    Node node = (Node)item;
                    AddField(node, fieldname);
                }
            }
        }
    }
}