//----------------------------------------------------------------------------------------
// AddFlow for WinForms: TreeEdit sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using Lassalle.Flow;
using Lassalle.Flow.Layout.Tree;

namespace TreeEdit
{
    public partial class TreeEdit : Form
    {
        Lassalle.PrnFlow.PrnFlow prnflow = null;
        string strProgName = null;
        string strFileName = null;

        public TreeEdit()
        {
            InitializeComponent();
        }

        private void TreeEdit_Load(object sender, EventArgs e)
        {
            StatusBar1.Dock = DockStyle.Bottom;

            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.AutoScroll = true;
            AddFlow1.BackColor = SystemColors.Window;
            AddFlow1.CanReflexLink = false;
            AddFlow1.CanStretchLink = false;
            AddFlow1.CanMultiLink = false;
            AddFlow1.CycleMode = CycleMode.NoCycle;
            AddFlow1.Grid.Draw = true;
            AddFlow1.Grid.Snap = true;
            AddFlow1.Grid.Style = GridStyle.DottedLines;
            AddFlow1.Grid.Color = Color.Silver;
            AddFlow1.CanFireError = true;
            AddFlow1.DefLinkProp.Font = new System.Drawing.Font("Verdana", 8.25f, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, (Byte)0);
            AddFlow1.DefNodeProp.Font = new System.Drawing.Font("Verdana", 8.25f, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, (Byte)0);
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            AddFlow1.DefNodeProp.FillColor = Color.LightYellow;
            AddFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom;
            AddFlow1.DefNodeProp.Shadow.Color = Color.Silver;
            AddFlow1.DefLinkProp.Rigid = true;
            AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None;
            AddFlow1.DefLinkProp.EndCap = LineCap.Custom;
            AddFlow1.DefLinkProp.CustomEndCap = new AdjustableArrowCap(3, 6, true);
            AddFlow1.DefLinkProp.DrawColor = Color.Blue;
            AddFlow1.DefLinkProp.Jump = Jump.None;

            prnflow = new Lassalle.PrnFlow.PrnFlow();

            this.ToolBarButtonFileNew.Tag = "New";
            this.ToolBarButtonFileOpen.Tag = "Open";
            this.ToolBarButtonFileSave.Tag = "Save";
            this.ToolBarButtonFilePrint.Tag = "Print";
            this.ToolBarButtonFilePreview.Tag = "Preview";
            this.ToolBarButtonEditCut.Tag = "Cut";
            this.ToolBarButtonEditCopy.Tag = "Copy";
            this.ToolBarButtonEditPaste.Tag = "Paste";
            this.ToolBarButtonEditUndo.Tag = "Undo";
            this.ToolBarButtonEditRedo.Tag = "Redo";
            this.ToolBarButtonNorth.Tag = "North";
            this.ToolBarButtonEast.Tag = "East";
            this.ToolBarButtonSouth.Tag = "South";
            this.ToolBarButtonWest.Tag = "West";
            this.ToolBarButtonRadial.Tag = "Radial";

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
        }

        private void TreeEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void ToolBar1_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            string key = (string)e.Button.Tag;
            switch (key)
            {
                case "New":
                    MenuItemFileNew_Click(sender, e);
                    break;
                case "Open":
                    MenuItemFileOpen_Click(sender, e);
                    break;
                case "Save":
                    MenuItemFileSave_Click(sender, e);
                    break;
                case "Print":
                    MenuItemFilePrint_Click(sender, e);
                    break;
                case "Preview":
                    MenuItemFilePrintPreview_Click(sender, e);
                    break;
                case "Cut":
                    MenuItemEditCut_Click(sender, e);
                    break;
                case "Copy":
                    MenuItemEditCopy_Click(sender, e);
                    break;
                case "Paste":
                    MenuItemEditPaste_Click(sender, e);
                    break;
                case "Undo":
                    MenuItemEditUndo_Click(sender, e);
                    break;
                case "Redo":
                    MenuItemEditRedo_Click(sender, e);
                    break;                    
                case "North":
                    MenuItemNorth_Click(sender, e);
                    break;                    
                case "East":
                    MenuItemEast_Click(sender, e);
                    break;                    
                case "South":
                    MenuItemSouth_Click(sender, e);
                    break;                    
                case "West":
                    MenuItemWest_Click(sender, e);
                    break;                    
                case "Radial":
                    MenuItemRadialTree_Click(sender, e);
                    break;                    
            }
        }

        private void AddFlow1_BeforeAddLink(object sender, BeforeAddLinkEventArgs e)
        {
            if (e.Dst.InLinks.Count > 0)
            {
                e.Cancel = new System.ComponentModel.CancelEventArgs(true);
                MessageBox.Show("The graph must be composed of a tree or several trees!");
            }
        }

        private void AddFlow1_Error(object sender, Lassalle.Flow.ErrorEventArgs e)
        {
            if (e.ErrorEventType == ErrorEventType.CycleError)
                MessageBox.Show("The graph must be composed of a tree or several trees!");
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuFileSafeAsImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Title = "Export as Image";
            sfd.DefaultExt = ".wmf";
            sfd.AddExtension = true;
            sfd.Filter = "Bitmap format(*.bmp)|*.bmp|Enhanced Windows metafile format (*.emf)|*.emf|Exchangeable Image File (*exif)|*.exif|Graphics Interchange Format (*.gif)|*.gif|Windows icon format (*.ico)|*.ico|Joint Photographic Experts Group format(*.jpeg)|*.jpeg|W3C Portable Network Graphics format (*png)|*png|Tag Image File Format (*tiff)|*tiff|Windows metafile format (*.wmf)|*.wmf";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Metafile mf = AddFlow1.ExportMetafile(false, true, true);
                ImageFormat imf = ImageFormat.Bmp;

                switch (sfd.FilterIndex)
                {
                    case 1:
                        imf = ImageFormat.Bmp;
                        break;
                    case 2:
                        imf = ImageFormat.Emf;
                        break;
                    case 3:
                        imf = ImageFormat.Exif;
                        break;
                    case 4:
                        imf = ImageFormat.Gif;
                        break;
                    case 5:
                        imf = ImageFormat.Icon;
                        break;
                    case 6:
                        imf = ImageFormat.Jpeg;
                        break;
                    case 7:
                        imf = ImageFormat.Png;
                        break;
                    case 8:
                        imf = ImageFormat.Tiff;
                        break;
                    case 9:
                        imf = ImageFormat.Wmf;
                        break;
                }
                mf.Save(sfd.FileName, imf);
            }
        }

        private void MenuItemFilePageSetup_Click(object sender, EventArgs e)
        {
            prnflow.PageSetup();
        }

        private void MenuItemFilePrintPreview_Click(object sender, EventArgs e)
        {
            prnflow.Preview(AddFlow1);
        }

        private void MenuItemFilePrint_Click(object sender, EventArgs e)
        {
            prnflow.Print(AddFlow1);
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {       
            bool isSelectedItems = (AddFlow1.SelectedItems.Count > 0);
            MenuItemEditUndo.Enabled = AddFlow1.CanUndo;
            MenuItemEditRedo.Enabled = AddFlow1.CanRedo;
            MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count != 0);
            MenuItemEditCut.Enabled = isSelectedItems;
            MenuItemEditCopy.Enabled = isSelectedItems;
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            MenuItemEditDelete.Enabled = isSelectedItems;
            MenuItemEditExportAsMetafile.Enabled = isSelectedItems;
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter tw = new XmlTextWriter(stream, null);
            Lassalle.XMLFlow.Serial.FlowToXML(tw, AddFlow1, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                XmlTextReader tr = new XmlTextReader(stream);

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);
                Lassalle.XMLFlow.Serial.XMLToFlow(tr, AddFlow1, true, false);
                AddFlow1.EndAction();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditExportAsMetafile_Click(object sender, EventArgs e)
        {
            Metafile mf = AddFlow1.ExportMetafile(true, true, false);
            // The following line does not work. 
            //Clipboard.SetDataObject(mf, true);
            // Therefore we have replaced it by the next which copy the metafile to the
            // clipboard, using the old GDI Metafile format instead of the new GDI + format.
            // As a result, the antialiasing effect is lost.
            ClipboardMetafileHelper.PutEnhMetafileOnClipboard(this.Handle, mf);
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemNorth_Click(object sender, EventArgs e)
        {
            TreeLayout(AddFlow1, DrawingStyle.Layered, Lassalle.Flow.Layout.Tree.Orientation.North);
        }

        private void MenuItemEast_Click(object sender, EventArgs e)
        {
            TreeLayout(AddFlow1, DrawingStyle.Layered, Lassalle.Flow.Layout.Tree.Orientation.East);
        }

        private void MenuItemSouth_Click(object sender, EventArgs e)
        {
            TreeLayout(AddFlow1, DrawingStyle.Layered, Lassalle.Flow.Layout.Tree.Orientation.South);
        }

        private void MenuItemWest_Click(object sender, EventArgs e)
        {
            TreeLayout(AddFlow1, DrawingStyle.Layered, Lassalle.Flow.Layout.Tree.Orientation.West);
        }

        private void MenuItemRadialTree_Click(object sender, EventArgs e)
        {
            TreeLayout(AddFlow1, DrawingStyle.Radial, Lassalle.Flow.Layout.Tree.Orientation.North);
        }

        private void MenuItemOrthogonal_Click(object sender, EventArgs e)
        {
            ChangeLinkStyles(AddFlow1, LineStyle.VHV);
        }

        private void MenuItemDirectLine_Click(object sender, EventArgs e)
        {
            ChangeLinkStyles(AddFlow1, LineStyle.Polyline);
        }

        private void MenuItemCurve_Click(object sender, EventArgs e)
        {
            ChangeLinkStyles(AddFlow1, LineStyle.Bezier);
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: Tree Editor Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }
        
        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            System.IO.FileStream stream = new System.IO.FileStream(strFileName, System.IO.FileMode.Open);
            XmlTextReader tr = new XmlTextReader(stream);
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            Lassalle.XMLFlow.Serial.XMLToFlow(tr, AddFlow1);
            AddFlow1.SetChangedFlag(false);
            stream.Close();
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            System.IO.FileStream stream = new System.IO.FileStream(strFileName, System.IO.FileMode.Create);
            XmlTextWriter tw = new XmlTextWriter(stream, null);
            Lassalle.XMLFlow.Serial.FlowToXML(tw, AddFlow1);
            stream.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }
        
        void ChangeLinkStyles(AddFlow AddFlow1, LineStyle ls)
        {
            // Change the link style of each existing link
            foreach (Item item in AddFlow1.Items)
            {
                if (item is Link)
                {
                    Link link = (Link)item;
                    link.Line.Style = ls;
                    if (ls == LineStyle.Polyline)
                        link.Points.Clear();
                }
            }

            // Change the link style for the next created links
            AddFlow1.DefLinkProp.Line.Style = ls;
        }
        
        void TreeLayout(AddFlow AddFlow, DrawingStyle ds, Lassalle.Flow.Layout.Tree.Orientation o)
        {
            if (AddFlow1.Nodes.Count == 0)
                return;

            // So that the user will be able to undo the layout in one time
            AddFlow1.BeginAction(1005);

            // Obtain the maximum node width and height
            // These values will be used to set the values of the LayerDistance and VertexDistance properties
            float maxNodeWidth = 0;
            float maxNodeHeight = 0;
            foreach (Node node in AddFlow1.Nodes)
            {
                maxNodeWidth = Math.Max(maxNodeWidth, node.Size.Width);
                maxNodeHeight = Math.Max(maxNodeHeight, node.Size.Height);
            }

            // Create a TFlow object and set its properties
            TFlow tflow = new TFlow();
            tflow.DrawingStyle = ds;
            if (tflow.DrawingStyle == DrawingStyle.Radial)
            {
                // (Here, the VerteDistance and Orientation properties are ignored)
                tflow.LayerDistance = 3 * maxNodeHeight;
            }
            else
            {
                if (o == Lassalle.Flow.Layout.Tree.Orientation.North || o == Lassalle.Flow.Layout.Tree.Orientation.South)
                {
                    tflow.LayerDistance = maxNodeHeight;
                    tflow.VertexDistance = maxNodeWidth / 2;
                }
                else
                {
                    tflow.LayerDistance = maxNodeWidth;
                    tflow.VertexDistance = maxNodeHeight / 2;
                }
                tflow.Orientation = o;
            }

            // Execute the layout
            try
            {
                tflow.Layout(AddFlow1);
            }
            catch (TFlowException err)
            {
                MessageBox.Show(err.Message, this.Text);
            }

            // Terminate the action for the undo processor
            AddFlow1.EndAction();
        }
    }
    public class ClipboardMetafileHelper
    {
        [DllImport("user32.dll", EntryPoint = "OpenClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool OpenClipboard(IntPtr hWnd);

        [DllImport("user32.dll", EntryPoint = "EmptyClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool EmptyClipboard();

        [DllImport("user32.dll", EntryPoint = "SetClipboardData",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr SetClipboardData(int uFormat, IntPtr hWnd);

        [DllImport("user32.dll", EntryPoint = "CloseClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool CloseClipboard();

        [DllImport("gdi32.dll", EntryPoint = "CopyEnhMetaFileA",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr CopyEnhMetaFile(IntPtr hemfSrc, IntPtr hNULL);

        [DllImport("gdi32.dll", EntryPoint = "DeleteEnhMetaFile",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool DeleteEnhMetaFile(IntPtr hemfSrc);

        // Metafile mf is set to an invalid state inside this function
        public static bool PutEnhMetafileOnClipboard(IntPtr hWnd, Metafile mf)
        {
            bool bResult = false;
            IntPtr hEMF = mf.GetHenhmetafile(); // invalidates mf
            if (!hEMF.Equals(new IntPtr(0)))
            {
                IntPtr hEMF2 = CopyEnhMetaFile(hEMF, new IntPtr(0));
                if (!hEMF2.Equals(new IntPtr(0)))
                {
                    if (OpenClipboard(hWnd))
                    {
                        if (EmptyClipboard())
                        {
                            IntPtr hRes = SetClipboardData(14, hEMF2); // 14 == CF_ENHMETAFILE()
                            bResult = hRes.Equals(hEMF2);
                            CloseClipboard();
                        }
                    }
                }
                DeleteEnhMetaFile(hEMF);
            }
            return bResult;
        }
    }
}