//----------------------------------------------------------------------------------------
// AddFlow for Winforms: AfEdit sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using Lassalle.Flow;

namespace afEdit
{
    public partial class afEdit : Form
    {
        Lassalle.PrnFlow.PrnFlow prnflow = null;
        string strProgName = null;
        string strFileName = null;
        SizeF pagesize = SizeF.Empty;

        public afEdit()
        {
            InitializeComponent();
        }

        private void afEdit_Load(object sender, EventArgs e)
        {
            StatusBar1.Dock = DockStyle.Bottom;
            SplitContainer1.Dock = DockStyle.Fill;

            addFlow1.Dock = DockStyle.Fill;
            addFlow1.BorderStyle = BorderStyle.FixedSingle;
            addFlow1.BackColor = SystemColors.Window;
            addFlow1.Grid.Draw = true;
            addFlow1.Grid.Snap = true;
            addFlow1.Grid.Style = GridStyle.DottedLines;
            addFlow1.Grid.Color = Color.Silver;
            addFlow1.Grid.Size = new Size(20, 20);
            addFlow1.CanFireError = true;
            addFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle;
            addFlow1.AutoScroll = true;
            addFlow1.DefLinkProp.Line.RoundedCorner = true;
            addFlow1.DefLinkProp.Jump = Jump.Arc;

            // Get the printer page size
            prnflow = new Lassalle.PrnFlow.PrnFlow();
            pagesize = prnflow.GetPageSize(addFlow1);

            // Display a printer page grid
            addFlow1.PageGrid.Size = new Size((int)pagesize.Width, (int)pagesize.Height);
            addFlow1.PageGrid.Color = Color.Chocolate;
            addFlow1.PageGrid.Style = GridStyle.DottedLines;
            addFlow1.PageGrid.Draw = true;

            this.ToolBarButtonFileNew.Tag = "New";
            this.ToolBarButtonFileOpen.Tag = "Open";
            this.ToolBarButtonFileSave.Tag = "Save";
            this.ToolBarButtonFilePrint.Tag = "Print";
            this.ToolBarButtonFilePreview.Tag = "Preview";
            this.ToolBarButtonEditCut.Tag = "Cut";
            this.ToolBarButtonEditCopy.Tag = "Copy";
            this.ToolBarButtonEditPaste.Tag = "Paste";
            this.ToolBarButtonEditUndo.Tag = "Undo";
            this.ToolBarButtonEditRedo.Tag = "Redo";

            PropertyGrid1.SelectedObject = addFlow1;
            PropertyGrid1.CommandsVisibleIfAvailable = true;

            Label1.TextAlign = ContentAlignment.MiddleCenter;
            Label1.Text = "AddFlow control";
            Label1.Font = new System.Drawing.Font("Arial", 12.0f, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (Byte)0);

            MenuItemEditEditionMode.Checked = true;

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
        }

        private void afEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void ToolBar1_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            string key = (string)e.Button.Tag;
            switch(key)
            {
            case "New":
                MenuItemFileNew_Click(sender, e);
                break;
            case "Open":
                MenuItemFileOpen_Click(sender, e);
                break;
            case "Save":
                MenuItemFileSave_Click(sender, e);
                break;
            case "Print":
                MenuItemFilePrint_Click(sender, e);
                break;
            case "Preview":
                MenuItemFilePrintPreview_Click(sender, e);
                break;
            case "Cut":
                MenuItemEditCut_Click(sender, e);
                break;
            case "Copy":
                MenuItemEditCopy_Click(sender, e);
                break;
            case "Paste":
                MenuItemEditPaste_Click(sender, e);
                break;
            case "Undo":
                MenuItemEditUndo_Click(sender, e);
                break;
            case "Redo":
                MenuItemEditRedo_Click(sender, e);
                break;
            }
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            
            bool interactiveEventsOnly = addFlow1.InteractiveEventsOnly;
            bool canUndoRedo = addFlow1.CanUndoRedo;
            bool sendSelectionChangeEvent = addFlow1.SendSelectionChangeEvent;

            addFlow1.InteractiveEventsOnly = true;
            addFlow1.SendSelectionChangeEvent = false;
            addFlow1.CanUndoRedo = false;

            addFlow1.Nodes.Clear();
            addFlow1.ResetUndoRedo();
            addFlow1.SetChangedFlag(false);

            addFlow1.CanUndoRedo = canUndoRedo;
            addFlow1.SendSelectionChangeEvent = sendSelectionChangeEvent;
            addFlow1.InteractiveEventsOnly = interactiveEventsOnly;

            strFileName = "";
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuFileSafeAsImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Title = "Export as Image";
            sfd.DefaultExt = ".wmf";
            sfd.AddExtension = true;
            sfd.Filter = "Bitmap format(*.bmp)|*.bmp|Enhanced Windows metafile format (*.emf)|*.emf|Exchangeable Image File (*exif)|*.exif|Graphics Interchange Format (*.gif)|*.gif|Windows icon format (*.ico)|*.ico|Joint Photographic Experts Group format(*.jpeg)|*.jpeg|W3C Portable Network Graphics format (*png)|*png|Tag Image File Format (*tiff)|*tiff|Windows metafile format (*.wmf)|*.wmf";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Metafile mf = addFlow1.ExportMetafile(false, true, true);
                ImageFormat imf = ImageFormat.Bmp;

                switch(sfd.FilterIndex)
                {
                case 1:
                    imf = ImageFormat.Bmp;
                    break;
                case 2:
                    imf = ImageFormat.Emf;
                    break;
                case 3:
                    imf = ImageFormat.Exif;
                    break;
                case 4:
                    imf = ImageFormat.Gif;
                    break;
                case 5:
                    imf = ImageFormat.Icon;
                    break;
                case 6:
                    imf = ImageFormat.Jpeg;
                    break;
                case 7:
                    imf = ImageFormat.Png;
                    break;
                case 8:
                    imf = ImageFormat.Tiff;
                    break;
                case 9:
                    imf = ImageFormat.Wmf; 
                    break;
                }
                mf.Save(sfd.FileName, imf);
            }
        }

        private void MenuFileSafeAsSVG_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Title = "Export as SVG file";
            sfd.DefaultExt = ".svg";
            sfd.AddExtension = true;
            sfd.Filter = "SVG file(*.svg)|*.svg";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                System.Xml.XmlDocument oDocument = Lassalle.Flow.SVG.SVGFlow.FlowToSVG(addFlow1);
                oDocument.Save(sfd.FileName);
                System.Diagnostics.Process.Start(sfd.FileName);
            }
        }

        private void MenuItemFilePageSetup_Click(object sender, EventArgs e)
        {
            prnflow.PageSetup();
            pagesize = prnflow.GetPageSize(addFlow1);
            addFlow1.Invalidate();
        }

        private void MenuItemFilePrintPreview_Click(object sender, EventArgs e)
        {
            prnflow.Preview(addFlow1);
        }

        private void MenuItemFilePrint_Click(object sender, EventArgs e)
        {
            prnflow.Print(addFlow1);
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        void MenuItemEdit_Popup(object obj, EventArgs ea)
        {
            bool isItems = (addFlow1.Items.Count > 0);
            bool isSelectedItems = (addFlow1.SelectedItems.Count > 0);
            
            Item item = addFlow1.SelectedItem;
            bool isCurrentNode = (item != null && item is Node);
            bool isCurrentLink = (item != null && item is Link);
            MenuItem miEdit = this.Menu.MenuItems[1];
            miEdit.MenuItems[0].Enabled = addFlow1.CanUndo;
            miEdit.MenuItems[1].Enabled = addFlow1.CanRedo;
            if (miEdit.MenuItems[1].Enabled)
                miEdit.MenuItems[1].Text = "Redo " + ActionName(addFlow1.RedoCode);
            if (miEdit.MenuItems[0].Enabled)
                miEdit.MenuItems[0].Text = "Undo " + ActionName(addFlow1.UndoCode);
            miEdit.MenuItems[3].Enabled = isSelectedItems;
            miEdit.MenuItems[4].Enabled = isSelectedItems;
            miEdit.MenuItems[5].Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            miEdit.MenuItems[6].Enabled = isSelectedItems;
            miEdit.MenuItems[8].Enabled = isItems;
            
            MenuItemEditUngroup.Enabled = isCurrentNode;
            MenuItemEditZOrderBack.Enabled = isCurrentNode || isCurrentLink;
            MenuItemEditZOrderFront.Enabled = isCurrentNode || isCurrentLink;
            MenuItemEditExportAsMetafile.Enabled = isSelectedItems;
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            addFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            addFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            addFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                addFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                addFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                addFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = addFlow1.Grid.Size.Width;
                float dy = addFlow1.Grid.Size.Height;
                foreach (Item item in addFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                addFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {    
            addFlow1.SendSelectionChangeEvent = false;
            addFlow1.DeleteSel();
            addFlow1.SendSelectionChangeEvent = true;
            SelectionChangeHandle();
        }

        private void MenuItemEditExportAsMetafile_Click(object sender, EventArgs e)
        {
            Metafile mf = addFlow1.ExportMetafile(true, true, false);
            // The following line does not work. 
            //Clipboard.SetDataObject(mf, true);
            // Therefore we have replaced it by the next which copy the metafile to the
            // clipboard, using the old GDI Metafile format instead of the new GDI + format.
            // As a result, the antialiasing effect is lost.
            ClipboardMetafileHelper.PutEnhMetafileOnClipboard(this.Handle, mf);
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            addFlow1.SendSelectionChangeEvent = false;
            foreach (Item item in addFlow1.Items)
                item.Selected = true;
            addFlow1.SendSelectionChangeEvent = true;
            SelectionChangeHandle();
        }

        private void MenuItemEditEditionMode_Click(object sender, EventArgs e)
        {
            addFlow1.MouseAction = MouseAction.None;
            MenuItemEditEditionMode.Checked = true;
            MenuItemEditSelectMode.Checked = false;
            MenuItemEditPanMode.Checked = false;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            addFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditEditionMode.Checked = false;
            MenuItemEditSelectMode.Checked = true;
            MenuItemEditPanMode.Checked = false;
        }

        private void MenuItemEditPanMode_Click(object sender, EventArgs e)
        {
            addFlow1.MouseAction = MouseAction.Pan;
            MenuItemEditEditionMode.Checked = false;
            MenuItemEditSelectMode.Checked = false;
            MenuItemEditPanMode.Checked = true;
        }

        private void MenuItemEditZOrderBack_Click(object sender, EventArgs e)
        {
            if (addFlow1.SelectedItem != null) 
                addFlow1.SelectedItem.ZOrder = 0;
        }

        private void MenuItemEditZOrderFront_Click(object sender, EventArgs e)
        {
            if (addFlow1.SelectedItem != null) 
                addFlow1.SelectedItem.ZOrder = addFlow1.Items.Count - 1;
        }

        private void MenuItemEditGroup_Click(object sender, EventArgs e)
        {
            float leftMin = 0;
            float topMin = 0;
            float rightMax = 0;
            float bottomMax = 0;
            bool first = false;

            first = true;
            foreach (Item item in addFlow1.SelectedItems)
            {
                if (item is Node)
                {
                    Node node = (Node)item;
                    RectangleF rc = node.Rect;
                    if (first)
                    {
                        first = false;
                        leftMin = rc.Left;
                        topMin = rc.Top;
                        rightMax = rc.Left + rc.Width;
                        bottomMax = rc.Top + rc.Height;
                    }
                    else
                    {
                        if (rc.Left < leftMin) 
                            leftMin = rc.Left;
                        if (rc.Top < topMin)
                            topMin = rc.Top;
                        if (rc.Right > rightMax)
                            rightMax = rc.Right;
                        if (rc.Bottom > bottomMax)
                            bottomMax = rc.Bottom;
                    }
                }
            }

            // We group the nodes only if there are several nodes.
            if (!first)
            {
                // Create the "owner" node. Make it not visible, not resizeable
                Node owner = new Node(leftMin, topMin, rightMax - leftMin, bottomMax - topMin);
                owner.Hidden = true;
                owner.XSizeable = false;
                owner.YSizeable = false;
                owner.RemoveChildren = false;
                owner.Shape.Style = ShapeStyle.Rectangle;
                addFlow1.Nodes.Add(owner);

                // Create a Rigid, Hidden and unselectable link between the owner node and each 
                // selected node.
                foreach (Item item in addFlow1.SelectedItems)
                {
                    if (item is Node) 
                    {
                        Node node = (Node)item;
                        node.Parent = owner;
                    }
                }

                // Select the owner node
                addFlow1.SelectedItem = owner;
            }
        }

        private void MenuItemEditUngroup_Click(object sender, EventArgs e)
        {
            if (addFlow1.SelectedItem is Node)
                addFlow1.Nodes.Remove((Node)addFlow1.SelectedItem);
        }

        private void MenuItemViewZoom100_Click(object sender, EventArgs e)
        {
            addFlow1.Zoom = new Zoom(1, 1);
        }

        private void MenuItemViewZoom150_Click(object sender, EventArgs e)
        {
            addFlow1.Zoom = new Zoom(1.5f, 1.5f);
        }

        private void MenuItemViewZoom75_Click(object sender, EventArgs e)
        {
            addFlow1.Zoom = new Zoom(0.75f, 0.75f);
        }

        private void MenuItemViewZoomRectangle_Click(object sender, EventArgs e)
        {
            if (addFlow1.MouseAction == MouseAction.ZoomIsotropic)
                addFlow1.MouseAction = MouseAction.None;
            else
                addFlow1.MouseAction = MouseAction.ZoomIsotropic;
            MenuItemViewZoomRectangle.Checked = (addFlow1.MouseAction == MouseAction.ZoomIsotropic);
        }

        private void MenuItemPropertiesWindow_Click(object sender, EventArgs e)
        {
            SplitContainer1.Panel2Collapsed = !SplitContainer1.Panel2Collapsed;
            MenuItemPropertiesWindow.Checked = SplitContainer1.Panel2Collapsed;
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms Flowchart Editor Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void addFlow1_SelectionChange(object sender, SelectionChangeArgs e)
        {
            SelectionChangeHandle();
            addFlow1.Refresh();
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1) 
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            addFlow1.ResetUndoRedo();
            addFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            addFlow1.ReadXml(reader);
            reader.Close();
            addFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);        
            writer.Formatting = Formatting.Indented;
            addFlow1.WriteXml(writer);
            writer.Close();
            addFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }
        
        string  FileTitle() 
        {
            if (strFileName != null && strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }
        
        bool OkToTrash()
        {
            if (!addFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch(dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }
        
        string ActionName(Lassalle.Flow.Action code)
        {
            switch(code)
            {
                case Lassalle.Flow.Action.NodeAdd:
                    return "node creation";
                case Lassalle.Flow.Action.NodeResize:
                    return "node resize";
                case Lassalle.Flow.Action.DeleteSel:
                    return "delete selection";
                case Lassalle.Flow.Action.MoveItems:
                    return "move items";
                case Lassalle.Flow.Action.LinkAdd:
                    return "link creation";
                case Lassalle.Flow.Action.LinkStretch:
                    return "link stretch";
                case Lassalle.Flow.Action.ZOrder:
                    return "ZOrder";
                case Lassalle.Flow.Action.Org:
                    return "link origin change";
                case Lassalle.Flow.Action.Dst:
                    return "link destination change";
                case Lassalle.Flow.Action.Text:
                    return "node text change";
                case (Lassalle.Flow.Action)1001:
                    return "node properties change";
                case (Lassalle.Flow.Action)1002:
                    return "link properties change";
                case (Lassalle.Flow.Action)1003:
                    return "paste";
                default:
                    // here you can add your own action code
                    // (see the AddFlow method: BeginLassalle.Flow.Action)
                    return "";
            }
        }

        private void SelectionChangeHandle()
        {
            Item item = addFlow1.SelectedItem;
            if (item != null)
            {
                if (item is Node)
                {
                    Node node = (Node)item;
                    PropertyGrid1.SelectedObject = node;
                    Label1.Text = "Selected Node";
                }
                else if (item is Link)
                {
                    Link link = (Link)item;
                    PropertyGrid1.SelectedObject = link;
                    Label1.Text = "Selected Link";
                }
            }
            else
            {
                PropertyGrid1.SelectedObject = addFlow1;
                Label1.Text = "AddFlow control";
            }
        }
    }

    public class ClipboardMetafileHelper
    {
        [DllImport("user32.dll", EntryPoint = "OpenClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool OpenClipboard(IntPtr hWnd);

        [DllImport("user32.dll", EntryPoint = "EmptyClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool EmptyClipboard();

        [DllImport("user32.dll", EntryPoint = "SetClipboardData",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr SetClipboardData(int uFormat, IntPtr hWnd);

        [DllImport("user32.dll", EntryPoint = "CloseClipboard",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool CloseClipboard();

        [DllImport("gdi32.dll", EntryPoint = "CopyEnhMetaFileA",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr CopyEnhMetaFile(IntPtr hemfSrc, IntPtr hNULL);

        [DllImport("gdi32.dll", EntryPoint = "DeleteEnhMetaFile",
        SetLastError = true, ExactSpelling = true, CallingConvention = CallingConvention.StdCall)]
        public extern static bool DeleteEnhMetaFile(IntPtr hemfSrc);

        // Metafile mf is set to an invalid state inside this function
        public static bool PutEnhMetafileOnClipboard(IntPtr hWnd, Metafile mf) 
        {
            bool bResult = false;
            IntPtr hEMF = mf.GetHenhmetafile(); // invalidates mf
            if (!hEMF.Equals(new IntPtr(0)))
            {
                IntPtr hEMF2 = CopyEnhMetaFile(hEMF, new IntPtr(0));
                if (!hEMF2.Equals(new IntPtr(0)))
                {
                    if (OpenClipboard(hWnd))
                    {
                        if (EmptyClipboard())
                        { 
                            IntPtr hRes = SetClipboardData(14, hEMF2); // 14 == CF_ENHMETAFILE()
                            bResult = hRes.Equals(hEMF2);
                            CloseClipboard();
                        }
                    }
                }
                DeleteEnhMetaFile(hEMF);
            }
            return bResult;
        }
    }
}