//---------------------------------------------------------------------------------------
// DlgFlow component.
// Copyright (c) 2003-2004 Lassalle Technologies. All Rights Reserved.
// http://www.lassalle.com
// Author: Patrick Lassalle
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
// ---------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.ComponentModel.Design;
using Lassalle.Flow;

namespace Lassalle.DlgFlow
{
    /// <summary>
    /// This component provides several dialog boxes frequently used in the samples provided
    /// with AddFlow.
    /// </summary>
    public class DlgFlow : System.ComponentModel.Component
    {
        private System.ComponentModel.Container components = null;

        /// <summary>
        /// Initializes a new instance of the DlgFlow class with default values.
        /// </summary>
        public DlgFlow()
        {
            InitializeComponent();
        }

        #region Component Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            components = new System.ComponentModel.Container();
        }
		#endregion
           
        /// <summary>
        /// Display the dialog box allowing to change the node properties.
        /// </summary>
        /// <param name="addflow">A reference to an AddFlow control</param>
        /// <param name="node">A reference to a Node object</param>
        /// <remarks>If node is null, then we change the properties of the DefNodeProp
        /// object property of the AddFlow control.</remarks>
        public void NodePropertyPage(AddFlow addflow, Node node)
        {
            addflow.BeginAction(1001);
            NodePropertyPage npp = new NodePropertyPage();
            npp.Node = node;
            npp.AddFlow = addflow;
            npp.ShowDialog();
            addflow.EndAction(); 
        }
        
        /// <summary>Display the dialog box allowing to change the link properties</summary>
        /// <param name="addflow">A reference to an AddFlow control</param>
        /// <param name="link">A reference to a Link object</param>
        /// <remarks>If link is null, then we change the properties of the DefLinkProp
        /// object property of the AddFlow control.</remarks>
        public void LinkPropertyPage(AddFlow addflow, Link link)
        {
            addflow.BeginAction(1002);
            LinkPropertyPage lpp = new LinkPropertyPage();
            lpp.Link = link;
            lpp.AddFlow = addflow;
            lpp.ShowDialog();
            addflow.EndAction(); 
        }

        /// <summary>Display the dialog box allowing to change the grid properties of 
        /// an AddFlow control</summary>
        /// <param name="addflow">A reference to an AddFlow control</param>
        public void GridPropertyPage(AddFlow addflow)
        {
            GridPropertyPage gpp = new GridPropertyPage();
            gpp.AddFlow = addflow;
            gpp.ShowDialog();
        }
        
        /// <summary>Display the dialog box allowing to change the capabilities of 
        /// an AddFlow control</summary>
        /// <param name="addflow">A reference to an AddFlow control</param>
        public void AddFlowPropertyPage(AddFlow addflow)
        {
            AddFlowPropertyPage app = new AddFlowPropertyPage();
            app.AddFlow = addflow;
            app.ShowDialog();
        }
    }
}