using System;
using System.Collections;

namespace Lassalle.GraphAlgo
{
	/// <summary>VerticesCollection class.</summary>
	public class VerticesCollection : IEnumerable, ICollection
	{
		private ArrayList arrayList = null;

		private VerticesCollection(){}

		internal VerticesCollection(ArrayList a)
		{
			arrayList = a;
		}
		
		/// <summary>Returns false, indicating that the collection has not a fixed size.</summary>
		public bool IsFixedSize
		{
			get { return false; }
		}

		/// <summary>Returns true, indicating that the collection is not read-only.</summary>
		public bool IsReadOnly
		{
			get { return false; }
		}

		/// <summary>IEnumerable Interface Implementation</summary>
		/// <returns>Returns a type safe enumerator.</returns>
		public IEnumerator GetEnumerator()
		{
			return arrayList.GetEnumerator();
		}

		/// <summary>Implements ICollection.IsSynchronized</summary>
		public bool IsSynchronized
		{
			get { return arrayList.IsSynchronized; }
		}

		/// <summary>Implements ICollection.SyncRoot</summary>
		public object SyncRoot
		{
			get { return arrayList.SyncRoot; }
		}

		/// <summary>Implements ICollection.CopyTo</summary>
		public void CopyTo(Array array, int arrayIndex)
		{
			arrayList.CopyTo(array, arrayIndex);
		}

		/// <summary>Indexer. It returns the vertex from the Vertices collection 
		/// at the specified index.</summary>
		public Vertex this[int index]
		{
			get 
			{ 
				if (index < 0 || index > Count - 1)
					throw new IndexOutOfRangeException();
				return (Vertex)arrayList[index]; 
			}
		}        

		/// <summary>Gets the total number of vertices.</summary>
		public int Count
		{            
			get { return arrayList.Count; }
		}

		/// <summary>Retrieves the index of the specified vertex in 
		/// the Vertices collection.</summary>
		/// <param name="o">The vertex to locate in the collection</param>
		/// <returns>A zero-based value that represents the position of the specified vertex  
		/// in the collection</returns>
		public int IndexOf(object o)
		{
			return arrayList.IndexOf(o);
		}
        
		/// <summary>Determines whether the collection contains a specific value.</summary>
		/// <param name="o">The item object to locate in the collection</param>
		/// <returns></returns>
		bool Contains(object o)
		{
			return arrayList.Contains(o);
		}
	}
}
