'----------------------------------------------------------------------------------------
' AddFlow for WinForms: Demo sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports System.Drawing.Drawing2D

Public Class Demo
    Const linkHorzOffset As Integer = 27
    Const fieldHeight As Integer = 20

    Private Sub Demo_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        FlowchartExample()
        NetworkExample()
        InternetTrafficExample()
        WorkFlowExample()
        StateDiagramExample()
        CauseEffectExample()
        DatabaseExample()
    End Sub

    Private Sub MenuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuExit.Click
        End
    End Sub

    Private Sub MenuItemProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemProperties.Click
        Dim addflow As Lassalle.Flow.AddFlow = TabControl1.SelectedTab.Controls(0)
        Dim p As New Properties
        p.PropertyGrid1.SelectedObject = addflow
        p.ShowDialog()
    End Sub

    Private Sub MenuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuAbout.Click
        MsgBox("AddFlow for WinForms: Demo" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub FlowchartExample()
        With AddFlow1
            .Dock = DockStyle.Fill
            .AutoScroll = True
            .BackColor = SystemColors.Window
            .PageUnit = GraphicsUnit.Point
            .Grid.Draw = True
            .Grid.Snap = True
            .Grid.Style = GridStyle.DottedLines
            .Grid.Color = Color.Silver

            .DefNodeProp.Shadow.Style = ShadowStyle.RightBottom
            .DefNodeProp.Shadow.Color = Color.Silver
            .DefNodeProp.Shadow.Size = New Size(4, 4)

            .DefLinkProp.Line.RoundedCorner = True
            .DefLinkProp.ArrowDst.Style = ArrowStyle.Arrow
            .DefLinkProp.ArrowDst.Angle = ArrowAngle.deg30
            .DefLinkProp.ArrowMid.Angle = ArrowAngle.deg30
            .DefLinkProp.BackMode = BackMode.Opaque
            .DefLinkProp.Shadow.Style = ShadowStyle.RightBottom
            .DefLinkProp.Shadow.Color = Color.Silver
            .DefLinkProp.Shadow.Size = New Size(4, 4)

            .BeginUpdate() ' Optimize speed

            Dim node0 As Node = .Nodes.Add(40, 16, 174, 16, "Encoding Application Flowchart")
            node0.Transparent = True
            node0.DrawColor = Color.Transparent
            node0.TextColor = Color.Blue
            node0.Shape.Style = ShapeStyle.Rectangle
            node0.Selectable = False
            node0.Font = New System.Drawing.Font("Verdana", 9.75!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, CType(0, Byte))

            Dim node1 As Node = .Nodes.Add(60, 48, 112, 32)
            node1.Text = "Timer1:" & vbCrLf & "called(every)" & vbCrLf & "2 seconds)"
            node1.Shape.Style = ShapeStyle.PredefinedProcess
            node1.FillColor = Color.LightYellow

            Dim node2 As Node = .Nodes.Add(76, 112, 80, 32)
            node2.Text = "Timer1" & vbCrLf & "= True ?"
            node2.Shape.Style = ShapeStyle.Losange

            Dim node3 As Node = .Nodes.Add(60, 186, 112, 48)
            node3.Text = "Check if a" & vbCrLf & "previous(encoding)" & vbCrLf & "session failed"
            node3.Shape.Style = ShapeStyle.Losange

            Dim node4 As Node = .Nodes.Add(60, 276, 112, 48)
            node4.Text = "Check if newly" & vbCrLf & "updated files"
            node4.Shape.Style = ShapeStyle.Losange

            Dim node5 As Node = .Nodes.Add(60, 366, 112, 48)
            node5.Text = "Encoding" & vbCrLf & "Step1" & vbCrLf & "..." & vbCrLf & "Step5"
            node5.Shape.Style = ShapeStyle.Rectangle

            Dim node6 As Node = .Nodes.Add(220, 112, 80, 32, "Exit Timer1")
            node6.Shape.Style = ShapeStyle.Termination

            Dim node7 As Node = .Nodes.Add(220, 286, 80, 32, "Exit Timer1")
            node7.Shape.Style = ShapeStyle.Termination

            Dim node8 As Node = .Nodes.Add(74, 436, 80, 32, "Exit Timer1")
            node8.Shape.Style = ShapeStyle.Termination

            node1.OutLinks.Add(node2)

            node2.OutLinks.Add(node3, "no")

            node3.OutLinks.Add(node4, "no")

            node4.OutLinks.Add(node5, "yes")

            node5.OutLinks.Add(node8)

            node2.OutLinks.Add(node6, "yes")

            node4.OutLinks.Add(node7, "no")

            Dim link As Link = node3.OutLinks.Add(node5, "yes")
            link.Line = New Line(LineStyle.HVH, False, True)
            link.Points(1) = New PointF(16, 210)
            link.Points(2) = New PointF(16, 390)

            .EndUpdate()
        End With
    End Sub

    Private Sub NetworkExample()
        AddFlow5.Dock = DockStyle.Fill
        AddFlow5.AutoScroll = True
        AddFlow5.BackColor = Color.AliceBlue

        AddFlow5.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow5.DefNodeProp.Transparent = True
        AddFlow5.DefNodeProp.DrawColor = Color.Transparent
        AddFlow5.DefNodeProp.Alignment = Alignment.CenterBOTTOM
        AddFlow5.DefNodeProp.TextColor = Color.Orange

        AddFlow5.DefLinkProp.Jump = Jump.Arc
        AddFlow5.DefLinkProp.ArrowDst.Style = ArrowStyle.None
        AddFlow5.DefLinkProp.DrawColor = Color.SlateBlue
        AddFlow5.DefLinkProp.Line.DoubleLine = True
        AddFlow5.DefLinkProp.Jump = Jump.None

        AddFlow5.Images.Add(ImageList1.Images(0))
        AddFlow5.Images.Add(ImageList1.Images(1))
        AddFlow5.Images.Add(ImageList1.Images(2))

        AddFlow5.BeginUpdate() ' Optimize speed

        Dim node As Node
        Dim link As Link

        node = AddFlow5.Nodes.Add(20, 90, 40, 50, "John")
        node.ImageIndex = 0

        node = AddFlow5.Nodes.Add(120, 20, 40, 50, "Paul")
        node.ImageIndex = 0

        node = AddFlow5.Nodes.Add(290, 70, 40, 50, "Tony")
        node.ImageIndex = 2
        node.TextColor = Color.Lime

        node = AddFlow5.Nodes.Add(250, 170, 50, 60, "George")
        node.ImageIndex = 0

        node = AddFlow5.Nodes.Add(90, 180, 40, 50, "Lisa")
        node.ImageIndex = 1

        link = AddFlow5.Nodes(0).OutLinks.Add(AddFlow5.Nodes(1))
        link = AddFlow5.Nodes(1).OutLinks.Add(AddFlow5.Nodes(2))
        link = AddFlow5.Nodes(2).OutLinks.Add(AddFlow5.Nodes(3))
        link = AddFlow5.Nodes(3).OutLinks.Add(AddFlow5.Nodes(4))
        link = AddFlow5.Nodes(4).OutLinks.Add(AddFlow5.Nodes(0))

        AddFlow5.EndUpdate()
    End Sub

    Private Sub InternetTrafficExample()
        With AddFlow4
            .Dock = DockStyle.Fill
            .AutoScroll = True
            .BackColor = Color.LightYellow
            .PageUnit = GraphicsUnit.Point
            .Grid.Draw = False

            .DefLinkProp.Jump = Jump.Arc
            .DefLinkProp.ArrowDst.Style = ArrowStyle.Circle
            .DefLinkProp.ArrowDst.Filled = True
            .DefLinkProp.AdjustDst = True
            .DefLinkProp.AdjustOrg = True

            .BeginUpdate() ' Optimize speed

            ' Create the nodes
            Dim node As Node
            node = .Nodes.Add(268, 6, 48, 32, "Stub Network")
            node = .Nodes.Add(150, 6, 48, 32, "Stub Network")
            node = .Nodes.Add(241, 244, 48, 32, "Stub Network")
            node = .Nodes.Add(370, 234, 48, 32, "Stub Network")
            node = .Nodes.Add(300, 260, 48, 32, "Stub Network")
            node = .Nodes.Add(92, 198, 80, 32, "Regional Network")
            node.DrawWidth = 4
            node.DrawColor = Color.Maroon
            node = .Nodes.Add(284, 198, 80, 32, "Regional Network")
            node.DrawWidth = 4
            node.DrawColor = Color.Maroon
            node = .Nodes.Add(156, 118, 144, 48, "Backbone Network")
            node.DrawWidth = 5
            node.DrawColor = Color.DarkBlue
            node = .Nodes.Add(188, 54, 80, 32, "Regional Network")
            node.DrawWidth = 4
            node.DrawColor = Color.Maroon
            node = .Nodes.Add(30, 241, 48, 32, "Stub Network")
            node = .Nodes.Add(98, 263, 48, 32, "Stub Network")
            node = .Nodes.Add(172, 250, 48, 32, "Stub Network")

            ' Create the links
            Dim link As Link

            link = .CreateLink(.Nodes(8), .Nodes(7))
            link.Points(0) = New PointF(228, 86)
            link.Points(1) = New PointF(228, 118)

            link = .CreateLink(.Nodes(6), .Nodes(7))
            link.Points(0) = New PointF(305, 199)
            link.Points(1) = New PointF(257, 163)

            link = .CreateLink(.Nodes(5), .Nodes(7))
            link.Points(0) = New PointF(150, 199)
            link.Points(1) = New PointF(198, 163)

            link = .CreateLink(.Nodes(9), .Nodes(5))
            link.Points(0) = New PointF(69, 245)
            link.Points(1) = New PointF(97, 223)

            link = .CreateLink(.Nodes(10), .Nodes(5))
            link.Points(0) = New PointF(124, 263)
            link.Points(1) = New PointF(129, 230)

            link = .CreateLink(.Nodes(11), .Nodes(5))
            link.Points(0) = New PointF(185, 252)
            link.Points(1) = New PointF(164, 224)

            link = .CreateLink(.Nodes(3), .Nodes(6))
            link.Points(0) = New PointF(375, 240)
            link.Points(1) = New PointF(348, 226)

            link = .CreateLink(.Nodes(4), .Nodes(6))
            link.Points(0) = New PointF(324, 260)
            link.Points(1) = New PointF(324, 229)

            link = .CreateLink(.Nodes(2), .Nodes(6))
            link.Points(0) = New PointF(280, 248)
            link.Points(1) = New PointF(305, 230)

            link = .CreateLink(.Nodes(1), .Nodes(8))
            link.Points(0) = New PointF(188, 35)
            link.Points(1) = New PointF(211, 55)

            link = .CreateLink(.Nodes(0), .Nodes(8))
            link.Points(0) = New PointF(276, 34)
            link.Points(1) = New PointF(247, 55)

            .EndUpdate()
        End With
    End Sub

    Private Sub WorkFlowExample()
        With AddFlow3
            .Dock = DockStyle.Fill
            .AutoScroll = True
            .BackColor = SystemColors.Window
            .PageUnit = GraphicsUnit.Point
            .Grid.Draw = True
            .Grid.Snap = True
            .Grid.Style = GridStyle.DottedLines
            .Grid.Color = Color.Silver

            ' Save the default property values for nodes and links
            Dim dn As Node = .DefNodeProp.Clone()
            Dim dl As Link = .DefLinkProp.Clone()

            .DefNodeProp.Shape.Style = ShapeStyle.RoundRect
            .DefNodeProp.DrawColor = Color.Red

            .DefLinkProp.Jump = Jump.Arc
            .DefLinkProp.ArrowDst.Filled = True

            .Images.Add(ImageList2.Images(0))
            .Images.Add(ImageList2.Images(1))
            .Images.Add(ImageList2.Images(2))

            .BeginUpdate() ' Optimize speed

            Dim node As Node, link As Link

            ' Create nodes
            node = .Nodes.Add(70, 4, 87, 64, "Order Entry")
            node.Shape.Style = ShapeStyle.Ellipse
            node.FillColor = Color.LightYellow
            node.TextMargin = New Size(5, 5)
            node.Alignment = Alignment.CenterBOTTOM
            node.ImageIndex = 1

            node = .Nodes.Add(98, 105, 30, 30, "and")
            node.Shape.Style = ShapeStyle.Ellipse
            node.FillColor = Color.White

            node = .Nodes.Add(95, 321, 28, 32, "and")
            node.Shape.Style = ShapeStyle.Ellipse
            node.FillColor = Color.White

            node = .Nodes.Add(7, 219, 74, 35, "Inventory Check")
            node.FillColor = Color.Gold

            node = .Nodes.Add(138, 180, 85, 31, "Compile Reference")
            node.FillColor = Color.Gold

            node = .Nodes.Add(138, 264, 85, 29, "Evaluate Reference")
            node.FillColor = Color.Gold

            node = .Nodes.Add(164, 320, 32, 32, "or")
            node.Shape.Style = ShapeStyle.Losange
            node.FillColor = Color.LightSkyBlue

            node = .Nodes.Add(92, 426, 32, 32, "or")
            node.Shape.Style = ShapeStyle.Losange
            node.FillColor = Color.LightSkyBlue

            node = .Nodes.Add(70, 375, 77, 35, "Order Evaluation")
            node.FillColor = Color.Gold

            node = .Nodes.Add(2, 485, 91, 50, "Letter of regret")
            node.FillColor = Color.Gold
            node.TextMargin = New Size(5, 5)
            node.Alignment = Alignment.CenterBOTTOM
            node.ImageIndex = 0

            node = .Nodes.Add(134, 495, 79, 26, "Billing")
            node.FillColor = Color.Gold

            node = .Nodes.Add(135, 554, 78, 25, "Shipping")
            node.FillColor = Color.Gold

            node = .Nodes.Add(92, 667, 74, 61, "Archive")
            node.Shape.Style = ShapeStyle.Ellipse
            node.DrawColor = Color.DarkGreen
            node.FillColor = Color.LightYellow
            node.TextMargin = New Size(5, 5)
            node.Alignment = Alignment.CenterBOTTOM
            node.ImageIndex = 2

            .DefNodeProp.DrawColor = Color.Black
            .DefNodeProp.Transparent = True
            .DefNodeProp.Shape.Style = ShapeStyle.Rectangle

            node = .Nodes.Add(10, 80, 70, 40, "Jane" & vbCrLf & "INVENTORY MANAGER")

            node = .Nodes.Add(200, 41, 72, 35, "Paul" & vbCrLf & "SECRETARY")

            node = .Nodes.Add(200, 376, 73, 32, "Lydia" & vbCrLf & "SECRETARY")

            node = .Nodes.Add(367, 518, 40, 36, "Bill")
            node.DashStyle = DashStyle.Dot

            node = .Nodes.Add(7, 602, 63, 40, "Jack" & vbCrLf & "ARCHIVER")

            ' Create links

            .DefLinkProp.DrawColor = Color.DarkGreen

            link = .Nodes(0).OutLinks.Add(.Nodes(1))

            link = .Nodes(1).OutLinks.Add(.Nodes(3))
            link.Points.Add(New PointF(53, 142))

            link = .Nodes(4).OutLinks.Add(.Nodes(5))

            link = .Nodes(3).OutLinks.Add(.Nodes(2))
            link.Points.Add(New PointF(53, 336))

            link = .Nodes(5).OutLinks.Add(.Nodes(6))

            link = .Nodes(6).OutLinks.Add(.Nodes(2), "Yes")

            link = .Nodes(2).OutLinks.Add(.Nodes(8))

            link = .Nodes(8).OutLinks.Add(.Nodes(7))

            link = .Nodes(7).OutLinks.Add(.Nodes(9), "Reject")
            link.Points.Add(New PointF(47, 453))

            link = .Nodes(7).OutLinks.Add(.Nodes(10), "Accept")
            link.Points.Add(New PointF(173, 453))

            link = .Nodes(10).OutLinks.Add(.Nodes(11))

            link = .Nodes(11).OutLinks.Add(.Nodes(12))
            link.DrawColor = Color.DarkGreen

            link = .Nodes(9).OutLinks.Add(.Nodes(12))
            link.DrawColor = Color.DarkGreen

            link = .Nodes(1).OutLinks.Add(.Nodes(4))
            link.Points.Add(New PointF(180, 141))

            link = .Nodes(6).OutLinks.Add(.Nodes(4), "No")
            link.Points.Add(New PointF(268, 336))
            link.Points.Add(New PointF(268, 195))

            ' Default properties for the end
            .DefLinkProp.DrawColor = Color.Black
            .DefLinkProp.ArrowDst.Style = ArrowStyle.Circle
            .DefLinkProp.ArrowDst.Filled = False
            .DefLinkProp.Line.Style = LineStyle.Bezier

            ' Now continue

            link = .Nodes(13).OutLinks.Add(.Nodes(0))

            link = .Nodes(14).OutLinks.Add(.Nodes(4))

            link = .Nodes(14).OutLinks.Add(.Nodes(5))
            link.Points(1) = New PointF(301, 171)
            link.Points(2) = New PointF(280, 200)

            link = .Nodes(13).OutLinks.Add(.Nodes(3))
            link.Points(2) = New PointF(5, 164)

            link = .Nodes(15).OutLinks.Add(.Nodes(8))

            link = .Nodes(16).OutLinks.Add(.Nodes(10))
            link.DashStyle = DashStyle.Dot

            link = .Nodes(16).OutLinks.Add(.Nodes(11))
            link.DashStyle = DashStyle.Dot

            link = .Nodes(17).OutLinks.Add(.Nodes(9))

            link = .Nodes(17).OutLinks.Add(.Nodes(12))

            .EndUpdate()

            ' Restore the default property values for nodes and links
            .DefNodeProp = dn
            .DefLinkProp = dl
        End With
    End Sub

    Private Sub StateDiagramExample()
        With AddFlow2
            .Dock = DockStyle.Fill
            .AutoScroll = True
            .BackColor = SystemColors.Window
            .PageUnit = GraphicsUnit.Point

            .DefNodeProp.Gradient = True
            .DefNodeProp.GradientColor = Color.LightBlue
            .DefNodeProp.GradientMode = LinearGradientMode.Horizontal

            .DefLinkProp.Jump = Jump.Arc
            .DefLinkProp.ArrowDst.Angle = ArrowAngle.deg30
            .DefLinkProp.ArrowDst.Filled = True
            .DefLinkProp.Line.Style = LineStyle.Spline

            .BeginUpdate() ' Optimize speed

            Dim node As Node
            Dim link As Link

            node = .Nodes.Add(24, 166, 71, 32, "Development")
            node = .Nodes.Add(206, 167, 64, 32, "Alpha tests")
            node = .Nodes.Add(134, 41, 54, 43, "Design phase")
            node = .Nodes.Add(207, 288, 75, 33, "Beta tests")
            node = .Nodes.Add(29, 302, 74, 30, "Release")

            link = .Nodes(0).OutLinks.Add(.Nodes(1), "End")
            link.Points.Add(New PointF(118, 152))

            link = .Nodes(2).OutLinks.Add(.Nodes(2), "Test")
            link.Line.Style = LineStyle.Bezier
            link.Points(1) = New PointF(221, 14)
            link.Points(2) = New PointF(130, 5)

            link = .Nodes(1).OutLinks.Add(.Nodes(0), "Problem")
            link.Points.Add(New PointF(142, 201))

            link = .Nodes(1).OutLinks.Add(.Nodes(3), "Stable")
            link.Points.Add(New PointF(234, 223))
            link.Points.Add(New PointF(216, 259))

            link = .Nodes(3).OutLinks.Add(.Nodes(4), "Work")
            link.Points.Add(New PointF(162, 336))

            link = .Nodes(4).OutLinks.Add(.Nodes(0), "New features")
            link.Points.Add(New PointF(54, 257))

            link = .Nodes(4).OutLinks.Add(.Nodes(4), "bug")
            link.Line.Style = LineStyle.Bezier
            link.Points(1) = New PointF(117, 376)
            link.Points(2) = New PointF(39, 391)

            link = .Nodes(2).OutLinks.Add(.Nodes(0), "Development decision")
            link.Points.Add(New PointF(84, 102))

            link = .Nodes(1).OutLinks.Add(.Nodes(1), "bug")
            link.Points(1) = New PointF(300, 148)
            link.Points(2) = New PointF(260, 127)

            link = .Nodes(3).OutLinks.Add(.Nodes(0), "Design problem")
            link.DashStyle = DashStyle.Dot
            link.Points.Add(New PointF(170, 264))
            link.Points.Add(New PointF(113, 238))

            link = .Nodes(3).OutLinks.Add(.Nodes(3), "bug")
            link.Points(1) = New PointF(320, 272)
            link.Points(2) = New PointF(273, 244)

            link = .Nodes(1).OutLinks.Add(.Nodes(0), "New ideas")
            link.Points.Add(New PointF(182, 220))
            link.Points.Add(New PointF(133, 223))

            .EndUpdate() ' Optimize speed
        End With
    End Sub

    Private Sub CauseEffectExample()
        With AddFlow6
            .Dock = DockStyle.Fill
            .AutoScroll = True
            .BackColor = SystemColors.Window
            .PageUnit = GraphicsUnit.Point
            .OwnerDraw = True
            .Grid.Draw = False
            .Grid.Snap = False

            .DefNodeProp.FillColor = Color.LightYellow
            .DefNodeProp.DrawColor = Color.Transparent
            .DefNodeProp.Alignment = Alignment.CenterBOTTOM
            .DefNodeProp.TextMargin = New Size(0, 10)
            .DefNodeProp.Shape.Style = ShapeStyle.Rectangle
            .DefNodeProp.DrawWidth = 1
            .DefNodeProp.Font = New System.Drawing.Font("Verdana", 10.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, CType(0, Byte))
            .DefNodeProp.YMoveable = False
            .DefNodeProp.YSizeable = False
            .DefNodeProp.OutLinkable = False
            .DefNodeProp.InLinkable = False
            .DefNodeProp.LabelEdit = False
            .DefNodeProp.HighlightChildren = False

            .DefLinkProp.ArrowDst.Style = ArrowStyle.None
            .DefLinkProp.Jump = Jump.Arc
            .DefLinkProp.EndCap = LineCap.ArrowAnchor
            .DefLinkProp.DrawWidth = 3
            .DefLinkProp.AdjustOrg = True
            .DefLinkProp.AdjustDst = True

            .BeginUpdate() ' Optimize speed

            Dim node0 As Node = .Nodes.Add(432, 16, 128, 48)
            node0.Properties("swimline").Value = "Financial"
            AddChildren(node0, "Control expenditure growth")

            Dim node1 As Node = .Nodes.Add(224, 16, 128, 48)
            node1.Properties("swimline").Value = "Financial"
            AddChildren(node1, "Improve product contribution")

            Dim node2 As Node = .Nodes.Add(16, 16, 128, 48)
            node2.Properties("swimline").Value = "Financial"
            AddChildren(node2, "Maintain profitable growth")

            Dim node3 As Node = .Nodes.Add(16, 128, 128, 48)
            node3.Properties("swimline").Value = "Customer"
            AddChildren(node3, "Achieve 95% customer satisfaction")

            Dim node4 As Node = .Nodes.Add(16, 352, 128, 48)
            node4.Properties("swimline").Value = "Learning"
            AddChildren(node4, "Recruit Management internally")

            Dim node5 As Node = .Nodes.Add(224, 352, 144, 48)
            node5.Properties("swimline").Value = "Learning"
            AddChildren(node5, "Retain the best people")

            Dim node6 As Node = .Nodes.Add(16, 240, 128, 48)
            node6.Properties("swimline").Value = "Internal"
            AddChildren(node6, "Improve Product Quality")

            Dim node7 As Node = .Nodes.Add(224, 240, 144, 48)
            node7.Properties("swimline").Value = "Internal"
            AddChildren(node7, "Improve sales lead generation")

            Dim node8 As Node = .Nodes.Add(224, 128, 144, 48)
            node8.Properties("swimline").Value = "Customer"
            AddChildren(node8, "Improve customer loyalty")

            Dim link As Link

            link = node4.OutLinks.Add(node5)
            link.Points(0) = New PointF(144, 376)
            link.Points(1) = New PointF(224, 376)

            link = node5.OutLinks.Add(node6)
            link.Points(0) = New PointF(242, 352)
            link.Points(1) = New PointF(128, 288)

            link = node5.OutLinks.Add(node7)
            link.Points(0) = New PointF(314, 352)
            link.Points(1) = New PointF(278, 288)

            link = node6.OutLinks.Add(node3)
            link.Points(0) = New PointF(48, 240)
            link.Points(1) = New PointF(96, 176)

            link = node3.OutLinks.Add(node2)
            link.Points(0) = New PointF(112, 128)
            link.Points(1) = New PointF(64, 64)

            link = node7.OutLinks.Add(node0)
            link.Points(0) = New PointF(350, 240)
            link.Points(1) = New PointF(464, 64)

            link = node3.OutLinks.Add(node8)
            link.Points(0) = New PointF(144, 152)
            link.Points(1) = New PointF(224, 152)

            link = node0.OutLinks.Add(node1)
            link.Points(0) = New PointF(432, 32)
            link.Points(1) = New PointF(352, 32)

            link = node1.OutLinks.Add(node2)
            link.Points(0) = New PointF(224, 32)
            link.Points(1) = New PointF(144, 32)

            link = node0.OutLinks.Add(node2)
            link.Points(0) = New PointF(432, 48)
            link.Points(1) = New PointF(144, 48)
            link.Points.Add(New PointF(272, 96))

            .EndUpdate()

            .CanDrawNode = False
        End With
    End Sub

    Private Sub AddFlow6_DiagramOwnerDraw(ByVal sender As Object, ByVal e As Lassalle.Flow.DiagramOwnerDrawEventArgs) Handles AddFlow6.DiagramOwnerDraw
        Dim grfx As Graphics = e.Graphics

        ' Save the graphics state because it does not belong to us
        ' (although in this case, it not necessary because we do not alter the Graphics state)
        Dim gs As GraphicsState = grfx.Save()

        Dim swimLinesWidth As Single = AddFlow6.ClientRectangle.Size.Width + AddFlow6.Extent.Width
        Dim pen As Pen = New Pen(Color.OrangeRed, 2)
        grfx.DrawLine(pen, 0, 84, swimLinesWidth, 84)
        grfx.DrawLine(pen, 0, 196, swimLinesWidth, 196)
        grfx.DrawLine(pen, 0, 308, swimLinesWidth, 308)
        grfx.DrawLine(pen, 0, 420, swimLinesWidth, 420)

        Dim font As Font = New Font("Times New Roman", 15)
        grfx.DrawString("Financial", font, Brushes.Navy, 470, 69)
        grfx.DrawString("Customer", font, Brushes.Navy, 470, 181)
        grfx.DrawString("Internal", font, Brushes.Navy, 470, 293)
        grfx.DrawString("Learning", font, Brushes.Navy, 470, 405)

        ' Restore the graphics state
        grfx.Restore(gs)
    End Sub

    Private Sub AddChildren(ByRef parent As Node, ByVal text As String)
        ' The location and width given in the following line are not important
        ' since the node will be docked at the top of the parent node.
        Dim node1 As Node = New Node(0, 0, 20, 10, AddFlow6.DefNodeProp)

        Dim s As String = parent.Properties("swimline").Value
        Select Case s
            Case "Financial"
                node1.FillColor = Color.Gold
            Case "Internal"
                node1.FillColor = Color.LimeGreen
            Case "Learning"
                node1.FillColor = Color.SteelBlue
            Case "Customer"
                node1.FillColor = Color.LightSalmon
        End Select
        node1.DrawColor = Color.Transparent
        node1.Selectable = False
        node1.Logical = False
        node1.LabelEdit = False
        node1.Dock = DockStyle.Top
        AddFlow6.Nodes.Add(node1)
        node1.Parent = parent

        Dim node2 As Node = New Node(0, 0, 20, 10, AddFlow6.DefNodeProp)
        node2.FillColor = Color.White
        node2.GradientColor = Color.Silver
        node2.DrawColor = Color.Transparent
        node2.Selectable = False
        node2.Logical = False
        node2.LabelEdit = False
        node2.Dock = DockStyle.Fill
        node2.Gradient = True
        node2.GradientMode = LinearGradientMode.Vertical
        node2.Text = text
        AddFlow6.Nodes.Add(node2)
        node2.Parent = parent
    End Sub

    Private Sub DatabaseExample()
        With AddFlow7
            .Dock = DockStyle.Fill
            .BackColor = Color.White
            .AutoScroll = True
            .CanDrawNode = False
            .DefNodeProp.Shape.Style = ShapeStyle.Rectangle
            .DefNodeProp.FillColor = Color.LightYellow
            .DefNodeProp.Dock = DockStyle.Top
            .DefNodeProp.InLinkable = False
            .DefNodeProp.DrawWidth = 2
            .DefNodeProp.Alignment = Alignment.CenterTOP
            .DefNodeProp.LabelEdit = False
            .DefNodeProp.OutLinkable = False
            .DefNodeProp.HighlightChildren = False
            .DefLinkProp.AdjustOrg = True
            .DefLinkProp.AdjustDst = True

            .BeginUpdate() ' Optimize speed

            Dim tableNode1 As Node = AddTable("Table1", 33, 20, 100, 133)
            Dim tableNode2 As Node = AddTable("Table2", 367, 33, 100, 113)
            Dim tableNode3 As Node = AddTable("Table3", 200, 167, 100, 133)

            ' Add fields in each table
            AddField(tableNode1, "field11")
            AddField(tableNode1, "field12")
            AddField(tableNode1, "field13")
            AddField(tableNode1, "field14")
            AddField(tableNode1, "field15")

            AddField(tableNode2, "field21")
            AddField(tableNode2, "field22")
            AddField(tableNode2, "field23")
            AddField(tableNode2, "field24")

            AddField(tableNode3, "field31")
            AddField(tableNode3, "field32")
            AddField(tableNode3, "field33")
            AddField(tableNode3, "field34")
            AddField(tableNode3, "field35")

            Dim link As Link
            link = tableNode1.Children(1).OutLinks.Add(tableNode3.Children(4))
            link.ArrowDst.Style = ArrowStyle.Many
            link.ArrowDst.Angle = ArrowAngle.deg30
            AdjustLinkSegment(link)

            link = tableNode1.Children(4).OutLinks.Add(tableNode2.Children(2))
            link.ArrowDst.Style = ArrowStyle.Many
            link.ArrowDst.Angle = ArrowAngle.deg30
            AdjustLinkSegment(link)
            link = tableNode2.Children(3).OutLinks.Add(tableNode3.Children(1))
            link.ArrowDst.Style = ArrowStyle.Many
            link.ArrowDst.Angle = ArrowAngle.deg30
            AdjustLinkSegment(link)

            .EndUpdate()
        End With
    End Sub

    Private Sub AddFlow7_AfterAddLink(ByVal sender As System.Object, ByVal e As Lassalle.Flow.AfterAddLinkEventArgs) Handles AddFlow7.AfterAddLink
        Dim link As Link = e.Link
        AdjustLinkSegment(link)
    End Sub

    Private Sub AddFlow7_BeforeAddLink(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeAddLinkEventArgs) Handles AddFlow7.BeforeAddLink
        ' If the origin and the destination nodes belong to the same table then
        ' cancel this link creation. 
        If IsSameTable(e.Org, e.Dst) Then
            e.Cancel.Cancel = True
        End If
    End Sub

    Private Sub AddFlow7_AfterMove(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddFlow7.AfterMove
        Dim item As Item = AddFlow7.SelectedItem
        If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
            Dim node As Node = item
            If IsTableNode(node) Then
                AdjustAllLinkSegments(node)
            End If
        End If
    End Sub

    Private Sub AddFlow7_AfterResize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddFlow7.AfterResize
        Dim item As Item = AddFlow7.SelectedItem
        If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
            Dim node As Node = item
            If IsTableNode(node) Then
                ' We are resizing a "table" node (field nodes are not resizable)
                ' First, we ensure that the height of this node is greater
                ' than the sum of the heights of its "field" nodes.
                If node.Rect.Height < node.Children.Count * fieldHeight Then
                    Dim rect As RectangleF = New RectangleF(node.Rect.Location, node.Rect.Size)
                    rect.Height = node.Children.Count * fieldHeight + fieldHeight / 2
                    node.Rect = rect
                End If
                AdjustAllLinkSegments(node)
            End If
        End If
    End Sub

    Private Function IsSameTable(ByVal node1 As Node, ByVal node2 As Node) As Boolean
        Return (node1.Parent Is node2.Parent And Not (node2.Parent Is Nothing))
    End Function

    Private Function IsTableNode(ByRef node As Node) As Boolean
        Return (node.Properties("Type").Value = "Table")
    End Function

    Private Function AddTable(ByRef header As String, ByRef x As Single, ByRef y As Single, ByRef cx As Single, ByRef cy As Single) As Node
        Dim tableNode As Node = New Node(x, y, cx, cy, AddFlow7.DefNodeProp)
        tableNode.Properties("Type").Value = "Table"
        AddFlow7.Nodes.Add(tableNode)
        AddHeader(tableNode, header)
        Return tableNode
    End Function

    Private Function AddHeader(ByRef tableNode As Node, ByRef headerName As String) As Node
        ' The location and width given in the following line are not important
        ' since the node will be docked at the top of the parent node.
        Dim headerNode As Node = New Node(0, 0, 20, fieldHeight, headerName, AddFlow7.DefNodeProp)
        headerNode.FillColor = Color.LightGreen
        headerNode.DrawColor = Color.Transparent
        headerNode.Selectable = False
        headerNode.Logical = False
        headerNode.LabelEdit = False
        headerNode.Properties("Type").Value = "Header"
        AddFlow7.Nodes.Add(headerNode)
        headerNode.Parent = tableNode
        Return headerNode
    End Function

    Private Function AddField(ByRef tableNode As Node, ByRef fieldName As String) As Node
        Dim fieldNode As Node = New Node(0, 0, 20, 20, fieldName, AddFlow7.DefNodeProp)
        fieldNode.DrawColor = Color.Transparent
        fieldNode.Alignment = Alignment.LeftJustifyMIDDLE ' The field text is left justified
        fieldNode.Logical = False
        fieldNode.InLinkable = True
        fieldNode.OutLinkable = True
        fieldNode.LabelEdit = False
        fieldNode.Properties("Type").Value = "Field"
        AddFlow7.Nodes.Add(fieldNode)
        fieldNode.Parent = tableNode
        Return fieldNode
    End Function

    Private Sub AdjustLinkSegment(ByRef link As Link)
        If link Is Nothing Then Exit Sub

        link.Points.Clear()

        Dim pt1 As PointF = link.Points.Item(0)
        Dim pt2 As PointF = link.Points.Item(1)
        Dim rcOrg As RectangleF = link.Org.Rect
        Dim rcDst As RectangleF = link.Dst.Rect

        Dim x4, x3, y3, y4 As Single
        If rcOrg.Left + rcOrg.Width / 2 < rcDst.Left + rcDst.Width / 2 Then
            pt1.X = rcOrg.Left + rcOrg.Width
            pt1.Y = rcOrg.Top + rcOrg.Height / 2
            pt2.X = rcDst.Left
            pt2.Y = rcDst.Top + rcDst.Height / 2
            x3 = Math.Max(pt1.X + linkHorzOffset, 0)
            y3 = pt1.Y
            x4 = Math.Max(pt2.X - linkHorzOffset, 0)
            y4 = pt2.Y
        Else
            pt1.X = rcOrg.Left
            pt1.Y = rcOrg.Top + rcOrg.Height / 2
            pt2.X = rcDst.Left + rcDst.Width
            pt2.Y = rcDst.Top + rcDst.Height / 2
            x3 = Math.Max(pt1.X - linkHorzOffset, 0)
            y3 = pt1.Y
            x4 = Math.Max(pt2.X + linkHorzOffset, 0)
            y4 = pt2.Y
        End If

        link.Points.Item(0) = pt1
        link.Points.Item(1) = pt2
        link.Points.Add(New PointF(x3, y3))
        link.Points.Add(New PointF(x4, y4))
    End Sub

    Private Sub AdjustAllLinkSegments(ByRef tableNode As Node)
        If tableNode Is Nothing Then Exit Sub
        For Each fieldNode As Node In tableNode.Children
            For Each link As Link In fieldNode.Links
                AdjustLinkSegment(link)
            Next
        Next
    End Sub
End Class
