'----------------------------------------------------------------------------------------
' AddFlow for WinForms: DemoLayout sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports Lassalle.Flow.Layout.Hierarchic
Imports Lassalle.Flow.Layout.Symmetric
Imports Lassalle.Flow.Layout.Tree
Imports Lassalle.Flow.Layout.Orthogonal
Imports Lassalle.Flow.Layout.SP

Public Class DemoLayout
    Dim sflow As Lassalle.Flow.Layout.Symmetric.SFlow
    Dim oflow As Lassalle.Flow.Layout.Orthogonal.OFlow
    Dim hflow As Lassalle.Flow.Layout.Hierarchic.HFlow
    Dim tflow As Lassalle.Flow.Layout.Tree.TFlow
    Dim spflow As Lassalle.Flow.Layout.SP.SPFlow

    Private strProgName As String
    Private strFileName As String

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        StatusBar1.Dock = DockStyle.Bottom

        SplitContainer1.Dock = DockStyle.Fill
        SplitContainer1.FixedPanel = FixedPanel.Panel2
        SplitContainer1.IsSplitterFixed = True

        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.AutoScroll = True
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.Grid.Snap = False
        AddFlow1.Grid.Style = GridStyle.DottedLines
        AddFlow1.Grid.Color = Color.Silver
        AddFlow1.Grid.Size = New Size(20, 20)
        AddFlow1.CanFireError = True
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.Gradient = True
        AddFlow1.DefNodeProp.GradientMode = Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal
        AddFlow1.DefNodeProp.GradientColor = Color.Yellow
        AddFlow1.DefLinkProp.Jump = Jump.Arc

        hflow = New HFlow
        oflow = New OFlow
        sflow = New SFlow
        tflow = New TFlow
        spflow = New SPFlow

        Me.ToolBarButtonFileNew.Tag = "New"
        Me.ToolBarButtonFileOpen.Tag = "Open"
        Me.ToolBarButtonFileSave.Tag = "Save"
        Me.ToolBarButtonEditCut.Tag = "Cut"
        Me.ToolBarButtonEditCopy.Tag = "Copy"
        Me.ToolBarButtonEditPaste.Tag = "Paste"
        Me.ToolBarButtonEditUndo.Tag = "Undo"
        Me.ToolBarButtonEditRedo.Tag = "Redo"
        Me.ToolBarButtonLayoutHierarchic.Tag = "Hierarchic"
        Me.ToolBarButtonLayoutSymmetric.Tag = "Symmetric"
        Me.ToolBarButtonLayoutTree.Tag = "Tree"
        Me.ToolBarButtonLayoutOrthogonal.Tag = "Orthogonal"
        Me.ToolBarButtonLayoutSeriesParallel.Tag = "Series Parallel"

        CheckBoxDisplayJumps.Checked = (AddFlow1.DefLinkProp.Jump <> Jump.None)
        CheckBoxDirectedGraph.Checked = (AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.Arrow)

        ' HFlow parameters
        ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.North.ToString())
        ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.East.ToString())
        ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.South.ToString())
        ComboBoxHFlowOrientation.Items.Add(Lassalle.Flow.Layout.Hierarchic.Orientation.West.ToString())
        TextBoxHFlowVertexDistance.Text = hflow.VertexDistance.ToString
        TextBoxHFlowLayerDistance.Text = hflow.LayerDistance.ToString
        TextBoxHFlowLayerWidth.Text = hflow.LayerWidth.ToString()
        TextBoxHFlowXMargin.Text = hflow.MarginSize.Width.ToString
        TextBoxHFlowYMargin.Text = hflow.MarginSize.Height.ToString
        ComboBoxHFlowOrientation.SelectedItem = hflow.Orientation.ToString()

        ' OFlow parameters
        ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.North.ToString())
        ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.East.ToString())
        ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.South.ToString())
        ComboBoxOFlowOrientation.Items.Add(Lassalle.Flow.Layout.Orthogonal.Orientation.West.ToString())
        TextBoxOFlowXMargin.Text = oflow.MarginSize.Width.ToString
        TextBoxOFlowYMargin.Text = oflow.MarginSize.Height.ToString
        TextBoxOFlowNodeSizeRatio.Text = oflow.NodeSizeRatio.ToString
        ComboBoxOFlowOrientation.SelectedItem = oflow.Orientation.ToString()
        TextBoxOFlowXGrid.Text = oflow.GridSize.Width.ToString
        TextBoxOFlowYGrid.Text = oflow.GridSize.Height.ToString

        ' SFlow parameters
        TextBoxSFlowXMargin.Text = sflow.MarginSize.Width.ToString
        TextBoxSFlowYMargin.Text = sflow.MarginSize.Height.ToString
        TextBoxSFlowVertexDistance.Text = sflow.VertexDistance.ToString
        CheckBoxSFlowSendStepEvent.Checked = sflow.SendStepEvent
        CheckBoxSFlowUnmoveableNodesAccepted.Checked = sflow.UnmoveableNodesAccepted
        CheckBoxSFlowRandomStart.Checked = sflow.RandomStart
        CheckBoxSFlowAnimation.Checked = sflow.Animation

        ' TFlow parameters
        ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.North.ToString())
        ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.East.ToString())
        ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.South.ToString())
        ComboBoxTFlowOrientation.Items.Add(Lassalle.Flow.Layout.Tree.Orientation.West.ToString())
        ComboBoxTFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.Tree.DrawingStyle.Layered.ToString())
        ComboBoxTFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.Tree.DrawingStyle.Radial.ToString())
        TextBoxTFlowVertexDistance.Text = tflow.VertexDistance.ToString
        TextBoxTFlowLayerDistance.Text = tflow.LayerDistance.ToString
        TextBoxTFlowXMargin.Text = tflow.MarginSize.Width.ToString
        TextBoxTFlowYMargin.Text = tflow.MarginSize.Height.ToString
        ComboBoxTFlowOrientation.SelectedItem = tflow.Orientation.ToString()
        ComboBoxTFlowDrawingStyle.SelectedItem = tflow.DrawingStyle.ToString()

        ' SPFlow parameters
        ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.North.ToString())
        ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.East.ToString())
        ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.South.ToString())
        ComboBoxSPFlowOrientation.Items.Add(Lassalle.Flow.Layout.SP.Orientation.West.ToString())
        ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.BusOrthogonalDrawing.ToString())
        ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.StraightLine.ToString())
        ComboBoxSPFlowDrawingStyle.Items.Add(Lassalle.Flow.Layout.SP.DrawingStyle.VisibilityDrawing.ToString())
        TextBoxSPFlowVertexDistance.Text = spflow.VertexDistance.ToString
        TextBoxSPFlowLayerDistance.Text = spflow.LayerDistance.ToString
        TextBoxSPFlowXMargin.Text = spflow.MarginSize.Width.ToString
        TextBoxSPFlowYMargin.Text = spflow.MarginSize.Height.ToString
        TextBoxSPFlowVertexWidth.Text = spflow.VertexSize.Width.ToString
        TextBoxSPFlowVertexHeight.Text = spflow.VertexSize.Height.ToString
        ComboBoxSPFlowOrientation.SelectedItem = spflow.Orientation.ToString()
        ComboBoxSPFlowDrawingStyle.SelectedItem = spflow.DrawingStyle.ToString()

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub ToolBar1_ButtonClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolBarButtonClickEventArgs) Handles ToolBar1.ButtonClick
        Dim key As String = e.Button.Tag
        Select Case key
            Case "New"
                MenuItemFileNew_Click(sender, e)
            Case "Open"
                MenuItemFileOpen_Click(sender, e)
            Case "Save"
                MenuItemFileSave_Click(sender, e)
            Case "Cut"
                MenuItemEditCut_Click(sender, e)
            Case "Copy"
                MenuItemEditCopy_Click(sender, e)
            Case "Paste"
                MenuItemEditPaste_Click(sender, e)
            Case "Undo"
                MenuItemEditUndo_Click(sender, e)
            Case "Redo"
                MenuItemEditRedo_Click(sender, e)
            Case "Hierarchic"
                MenuItemLayoutHierarchic_Click(sender, e)
            Case "Symmetric"
                MenuItemLayoutSymmetric_Click(sender, e)
            Case "Tree"
                MenuItemLayoutTree_Click(sender, e)
            Case "Orthogonal"
                MenuItemLayoutOrthogonal_Click(sender, e)
            Case "Series Parallel"
                MenuItemLayoutSeriesParallel_Click(sender, e)
        End Select
    End Sub

    Private Sub AddFlow1_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles AddFlow1.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Right Then
            Dim item As Item = AddFlow1.PointedItem
            If Not (item Is Nothing) Then
                Dim dlgflow As Lassalle.DlgFlow.DlgFlow = New Lassalle.DlgFlow.DlgFlow
                If TypeOf (item) Is Node Then
                    Dim node As Node = CType(item, Node)
                    dlgflow.NodePropertyPage(AddFlow1, node)
                ElseIf TypeOf (item) Is Link Then
                    Dim link As Link = CType(item, Link)
                    dlgflow.LinkPropertyPage(AddFlow1, link)
                End If
            End If
        End If
    End Sub

    Private Sub AddFlow1_BeforeAddNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeAddNodeEventArgs)
        e.Size = New SizeF(AddFlow1.Grid.Size.Width, AddFlow1.Grid.Size.Height)
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub
        strFileName = ""
        AddFlow1.Nodes.Clear()
        AddFlow1.ResetUndoRedo()
        AddFlow1.SetChangedFlag(False)
        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        With AddFlow1
            MenuItemEditUndo.Enabled = .CanUndo
            MenuItemEditRedo.Enabled = .CanRedo
            If MenuItemEditRedo.Enabled Then
                MenuItemEditRedo.Text = "Redo " & ActionName(.RedoCode)
            End If
            If MenuItemEditUndo.Enabled Then
                MenuItemEditUndo.Text = "Undo " & ActionName(.UndoCode)
            End If
            MenuItemEditSelectAll.Enabled = (.Nodes.Count <> 0)
            MenuItemEditCut.Enabled = (.SelectedItems.Count > 0)
            MenuItemEditCopy.Enabled = (.SelectedItems.Count > 0)
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
            MenuItemEditDelete.Enabled = (.SelectedItems.Count > 0)
        End With
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        For Each item As Item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemViewZoomNormal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoomNormal.Click
        MenuItemViewZoomNormal.Checked = True
        MenuItemViewZoomFit.Checked = False
        AddFlow1.Zoom = New Zoom(1, 1)
    End Sub

    Private Sub MenuItemViewZoomFit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoomFit.Click
        MenuItemViewZoomNormal.Checked = False
        MenuItemViewZoomFit.Checked = True
        Dim rc As RectangleF = New RectangleF(New PointF(0, 0), AddFlow1.Extent)
        AddFlow1.ZoomRectangle(rc, ZoomType.Isotropic)
    End Sub

    Private Sub MenuItemViewGrid_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewGrid.Click
        Dim dlgflow As Lassalle.DlgFlow.DlgFlow = New Lassalle.DlgFlow.DlgFlow
        dlgflow.GridPropertyPage(AddFlow1)
    End Sub

    Private Sub MenuItemViewUserInterfaceOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewUserInterfaceOptions.Click
        Dim dlgflow As Lassalle.DlgFlow.DlgFlow = New Lassalle.DlgFlow.DlgFlow
        dlgflow.AddFlowPropertyPage(AddFlow1)
    End Sub

    Private Sub MenuItemDefaultNodeProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDefaultNodeProperties.Click
        Dim dlgflow As Lassalle.DlgFlow.DlgFlow = New Lassalle.DlgFlow.DlgFlow
        dlgflow.NodePropertyPage(AddFlow1, Nothing)
    End Sub

    Private Sub MenuItemDefaultLinkProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDefaultLinkProperties.Click
        Dim dlgflow As Lassalle.DlgFlow.DlgFlow = New Lassalle.DlgFlow.DlgFlow
        dlgflow.LinkPropertyPage(AddFlow1, Nothing)
    End Sub

    Private Sub MenuItemLayoutHierarchic_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemLayoutHierarchic.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1004)

        ' Updates the properties
        hflow.MarginSize = New SizeF(Convert.ToSingle(TextBoxHFlowXMargin.Text), Convert.ToSingle(TextBoxHFlowYMargin.Text))
        hflow.Orientation = CType(ComboBoxHFlowOrientation.SelectedIndex, Lassalle.Flow.Layout.Hierarchic.Orientation)
        hflow.VertexDistance = Convert.ToInt32(TextBoxHFlowVertexDistance.Text)
        hflow.LayerDistance = Convert.ToInt32(TextBoxHFlowLayerDistance.Text)
        hflow.LayerWidth = Convert.ToInt32(TextBoxHFlowLayerWidth.Text)

        ' Execute the layout
        hflow.Layout(AddFlow1)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub MenuItemLayoutTree_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemLayoutTree.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1005)

        ' Updates the properties
        tflow.MarginSize = New SizeF(Convert.ToSingle(TextBoxTFlowXMargin.Text), Convert.ToSingle(TextBoxTFlowYMargin.Text))
        tflow.Orientation = CType(ComboBoxTFlowOrientation.SelectedIndex, Lassalle.Flow.Layout.Tree.Orientation)
        tflow.VertexDistance = Convert.ToInt32(TextBoxTFlowVertexDistance.Text)
        tflow.LayerDistance = Convert.ToInt32(TextBoxTFlowLayerDistance.Text)
        tflow.DrawingStyle = CType(ComboBoxTFlowDrawingStyle.SelectedIndex, Lassalle.Flow.Layout.Tree.DrawingStyle)

        ' Execute the layout
        On Error Resume Next
        tflow.Layout(AddFlow1)
        If Err.Number <> 0 Then MessageBox.Show(Err.Description)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub MenuItemLayoutSymmetric_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemLayoutSymmetric.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1006)

        ' Updates the properties
        sflow.MarginSize = New SizeF(Convert.ToSingle(TextBoxSFlowXMargin.Text), Convert.ToSingle(TextBoxSFlowYMargin.Text))
        sflow.VertexDistance = Convert.ToInt32(TextBoxSFlowVertexDistance.Text)
        sflow.SendStepEvent = CheckBoxSFlowSendStepEvent.Checked
        sflow.UnmoveableNodesAccepted = CheckBoxSFlowUnmoveableNodesAccepted.Checked
        sflow.RandomStart = CheckBoxSFlowRandomStart.Checked
        sflow.Animation = CheckBoxSFlowAnimation.Checked

        ' Execute the layout
        sflow.Layout(AddFlow1)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub MenuItemLayoutOrthogonal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemLayoutOrthogonal.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1007)

        ' Updates the properties
        oflow.MarginSize = New SizeF(Convert.ToSingle(TextBoxOFlowXMargin.Text), Convert.ToSingle(TextBoxOFlowYMargin.Text))
        oflow.Orientation = CType(ComboBoxOFlowOrientation.SelectedIndex, Lassalle.Flow.Layout.Orthogonal.Orientation)
        oflow.GridSize = New SizeF(Convert.ToInt32(TextBoxOFlowXGrid.Text), Convert.ToInt32(TextBoxOFlowYGrid.Text))
        oflow.NodeSizeRatio = Convert.ToInt32(TextBoxOFlowNodeSizeRatio.Text)

        ' Execute the layout
        oflow.Layout(AddFlow1)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub MenuItemlayoutSeriesParallel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemlayoutSeriesParallel.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1011)

        ' Updates the properties
        spflow.MarginSize = New SizeF(Convert.ToSingle(TextBoxSPFlowXMargin.Text), Convert.ToSingle(TextBoxSPFlowYMargin.Text))
        spflow.VertexSize = New SizeF(Convert.ToSingle(TextBoxSPFlowVertexWidth.Text), Convert.ToSingle(TextBoxSPFlowVertexHeight.Text))
        spflow.Orientation = CType(ComboBoxSPFlowOrientation.SelectedIndex, Lassalle.Flow.Layout.SP.Orientation)
        spflow.VertexDistance = Convert.ToInt32(TextBoxSPFlowVertexDistance.Text)
        spflow.LayerDistance = Convert.ToInt32(TextBoxSPFlowLayerDistance.Text)
        spflow.DrawingStyle = CType(ComboBoxSPFlowDrawingStyle.SelectedIndex, Lassalle.Flow.Layout.SP.DrawingStyle)

        ' Execute the layout
        On Error Resume Next
        spflow.Layout(AddFlow1)
        If Err.Number <> 0 Then MessageBox.Show(Err.Description)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("DemoLayout Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub CheckBoxDisplayJumps_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxDisplayJumps.CheckedChanged
        If CheckBoxDisplayJumps.Checked Then
            AddFlow1.DefLinkProp.Jump = Jump.Arc
        Else
            AddFlow1.DefLinkProp.Jump = Jump.None
        End If

        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1009)

        ' Change the jump style of each existing link
        For Each node As Node In AddFlow1.Nodes
            For Each link As Link In node.OutLinks
                link.Jump = AddFlow1.DefLinkProp.Jump
            Next
        Next

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub CheckBoxDirectedGraph_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBoxDirectedGraph.CheckedChanged
        If CheckBoxDirectedGraph.Checked Then
            AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.Arrow
        Else
            AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None
        End If

        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1008)

        ' Change the arrow style of each existing link
        For Each node As Node In AddFlow1.Nodes
            For Each link As Link In node.OutLinks
                link.ArrowDst.Style = AddFlow1.DefLinkProp.ArrowDst.Style
            Next
        Next

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub ButtonNormalizeNodes_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonNormalizeNodes.Click
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1010)

        For Each node As Node In AddFlow1.Nodes
            node.Size = New SizeF(AddFlow1.Grid.Size.Width, AddFlow1.Grid.Size.Height)
            node.Shape.Style = ShapeStyle.Rectangle
            node.Gradient = True
            node.GradientMode = Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal
            node.GradientColor = Color.Yellow
        Next

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()

        CheckBoxDisplayJumps.Checked = (AddFlow1.DefLinkProp.Jump <> Jump.None)
        CheckBoxDirectedGraph.Checked = (AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.Arrow)

        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.Gradient = True
        AddFlow1.DefNodeProp.GradientMode = Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal
        AddFlow1.DefNodeProp.GradientColor = Color.Yellow
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = System.IO.Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Function ActionName(ByVal code As Integer) As String
        Select Case code
            Case Action.NodeAdd
                ActionName = "node creation"
            Case Action.NodeResize
                ActionName = "node resize"
            Case Action.DeleteSel
                ActionName = "delete selection"
            Case Action.MoveItems
                ActionName = "move items"
            Case Action.LinkAdd
                ActionName = "link creation"
            Case Action.LinkStretch
                ActionName = "link stretch"
            Case Action.Org
                ActionName = "link origin change"
            Case Action.Dst
                ActionName = "link destination change"
            Case Action.Text
                ActionName = "node text change"
            Case 1001
                ActionName = "node properties change"
            Case 1002
                ActionName = "link properties change"
            Case 1003
                ActionName = "paste"
            Case 1004
                ActionName = "hierarchical layout"
            Case 1005
                ActionName = "tree layout"
            Case 1006
                ActionName = "symmetric layout"
            Case 1007
                ActionName = "orthogonal layout"
            Case 1011
                ActionName = "series parallel layout"
            Case 1008
                ActionName = "change of the directed status of links"
            Case 1009
                ActionName = "change of the jump style of links"
            Case 1010
                ActionName = "normalize nodes"
            Case Else
                ActionName = ""
                ' here you can add your own action code
                ' (see the AddFlow method: BeginAction)
        End Select
    End Function
End Class
