'---------------------------------------------------------------------------------------
' AddFlow for WinForms: DeriveNode sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'---------------------------------------------------------------------------------------

Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports Lassalle.Flow

Public Class DeriveNode
    Private strProgName As String
    Private strFileName As String

    Private Sub DeriveNode_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = Color.White
        AddFlow1.AutoScroll = True
        AddFlow1.DefNodeProp.FillColor = Color.LightYellow
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Document

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub
        strFileName = ""
        AddFlow1.Nodes.Clear()
        AddFlow1.ResetUndoRedo()
        AddFlow1.SetChangedFlag(False)
        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub ExitMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        Dim isSelectedItems As Boolean = (AddFlow1.SelectedItems.Count > 0)

        MenuItemEditUndo.Enabled = AddFlow1.CanUndo
        MenuItemEditRedo.Enabled = AddFlow1.CanRedo
        MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count <> 0)
        MenuItemEditCut.Enabled = isSelectedItems
        MenuItemEditCopy.Enabled = isSelectedItems
        MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
        MenuItemEditDelete.Enabled = isSelectedItems
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            Dim item As Item
            For Each item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        Dim item As Item
        For Each item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemData_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemData.Popup
        MenuItemDataNodeCustomData.Enabled = TypeOf (AddFlow1.SelectedItem) Is Node
    End Sub

    Private Sub MenuItemDataNodeCustomData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDataNodeCustomData.Click
        EnterCustomData()
    End Sub

    Private Sub AboutMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        MsgBox("AddFlow for WinForms: DeriveNode sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_BeforeAddNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeAddNodeEventArgs) Handles AddFlow1.BeforeAddNode
        e.Cancel.Cancel = True
        Dim mynode As MyNode = New MyNode(AddFlow1.DefNodeProp, Nothing, Nothing)
        mynode.Location = e.Location
        mynode.Size = e.Size
        AddFlow1.Nodes.Add(mynode)
        mynode.Selected = True
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddFlow1.DoubleClick
        EnterCustomData()
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_BeforeReadXMLNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeReadXMLNodeEventArgs) Handles AddFlow1.BeforeReadXMLNode
        If (e.Custom = "MyNode") Then
            Dim mynode As MyNode = New MyNode(e.DefNode, Nothing, Nothing)
            e.Node = mynode
        End If
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_ReadXMLNodeExtraData(ByVal sender As System.Object, ByVal e As Lassalle.Flow.ReadXMLNodeExtraDataEventArgs) Handles AddFlow1.ReadXMLNodeExtraData
        If (TypeOf (e.Node) Is MyNode) Then
            Dim mynode As MyNode = e.Node
            If (e.Reader.Name = "Author") Then
                mynode.Author = e.Reader.ReadElementContentAsString()
            End If
            If (e.Reader.Name = "Comment") Then
                mynode.Comment = e.Reader.ReadElementContentAsString()
            End If
        End If
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_WriteXMLNodeExtraData(ByVal sender As System.Object, ByVal e As Lassalle.Flow.WriteXMLNodeExtraDataEventArgs) Handles AddFlow1.WriteXMLNodeExtraData
        If (TypeOf (e.Node) Is MyNode) Then
            Dim mynode As MyNode = e.Node
            If Not (mynode.Author = Nothing) Then
                If mynode.Author.Length > 0 Then
                    e.Writer.WriteElementString("Author", mynode.Author)
                End If
            End If
            If Not (mynode.Comment = Nothing) Then
                If mynode.Comment.Length > 0 Then
                    e.Writer.WriteElementString("Comment", mynode.Comment)
                End If
            End If
        End If
    End Sub

    ' See the tutorial (paragraph "Derivation of Node and Link classes")for explanations
    Private Sub AddFlow1_BeforeWriteXMLNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeWriteXMLNodeEventArgs) Handles AddFlow1.BeforeWriteXMLNode
        If (TypeOf (e.Node) Is MyNode) Then e.Custom = "MyNode"
    End Sub

    Private Sub EnterCustomData()
        If TypeOf (AddFlow1.SelectedItem) Is Node Then
            Dim node As Node = AddFlow1.SelectedItem
            If (TypeOf (node) Is MyNode) Then
                Dim mynode As MyNode = node
                Dim nd As NodeData = New NodeData
                nd.MyNode = mynode
                If nd.ShowDialog() = Windows.Forms.DialogResult.OK Then
                    AddFlow1.SubmitTask(New MyTask(mynode))
                    mynode.Author = nd.TextBoxAuthor.Text
                    mynode.Comment = nd.textBoxComment.Text
                End If
            End If
        End If
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function
End Class

' Create a new node class, derived from the Node class of AddFlow.
' This new class has two new properties: Author and Comment.
Public Class MyNode
    Inherits Node

    Private m_author As String = Nothing
    Private m_comment As String = Nothing

    Public Sub New(ByVal defnode As Node, ByVal author As String, ByVal comment As String)
        MyBase.New(defnode)
        m_author = author
        m_comment = comment
    End Sub

    Public Property Author() As String
        Get
            Return m_author
        End Get
        Set(ByVal Value As String)
            m_author = Value
        End Set
    End Property

    Public Property Comment() As String
        Get
            Return m_comment
        End Get
        Set(ByVal Value As String)
            m_comment = Value
        End Set
    End Property
End Class

Public Class MyTask
    Inherits Task

    Dim m_mynode As MyNode
    Dim m_oldAuthor As String
    Dim m_oldComment As String

    Public Sub New(ByVal mynode As MyNode)
        MyBase.New()
        Me.Code = 1001
        m_mynode = mynode
        m_oldAuthor = m_mynode.Author
        m_oldComment = m_mynode.Comment
    End Sub

    Public Overrides Sub Redo()
        Undo()
    End Sub

    Public Overrides Sub Undo()
        Dim author As String = m_mynode.Author
        Dim comment As String = m_mynode.Comment
        m_mynode.Author = m_oldAuthor
        m_mynode.Comment = m_oldComment
        m_oldAuthor = author
        m_oldComment = comment
    End Sub
End Class