'----------------------------------------------------------------------------------------
' AddFlow for WinForms: GraphTest sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports System.IO
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms
Imports Lassalle.Flow
Imports Lassalle.GraphAlgo

Public Class GraphTest
    Private strProgName As String
    Private strFileName As String
    Private aColor() As Color = {Color.Yellow, Color.LightSkyBlue, Color.Coral, Color.GreenYellow, Color.LightPink, Color.Wheat}

    Private Sub GraphTest_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.Grid.Draw = False
        AddFlow1.Grid.Snap = True
        AddFlow1.Grid.Size = New Size(12, 12)
        AddFlow1.CanFireError = True
        AddFlow1.CanLabelEdit = False
        AddFlow1.CanSizeNode = False
        AddFlow1.CanReflexLink = False
        AddFlow1.CanMultiLink = False
        AddFlow1.AutoScroll = True
        AddFlow1.PageUnit = GraphicsUnit.Pixel
        AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None
        AddFlow1.DefLinkProp.BackMode = BackMode.Opaque
        AddFlow1.DefNodeProp.Font = New Font("verdana", 8)

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()

        ' Create a diagram programmaticaly
        AddFlow1.Nodes.Add(New Node(24, 84, 24, 24))
        AddFlow1.Nodes.Add(New Node(96, 12, 24, 24))
        AddFlow1.Nodes.Add(New Node(96, 84, 24, 24))
        AddFlow1.Nodes.Add(New Node(96, 156, 24, 24))
        AddFlow1.Nodes.Add(New Node(168, 84, 24, 24))
        AddFlow1.Nodes.Add(New Node(240, 36, 24, 24))
        AddFlow1.Nodes.Add(New Node(240, 132, 24, 24))
        AddFlow1.Nodes.Add(New Node(312, 84, 24, 24))
        AddFlow1.Nodes.Add(New Node(384, 36, 24, 24))
        AddFlow1.Nodes.Add(New Node(384, 132, 24, 24))
        AddFlow1.Nodes.Add(New Node(456, 84, 24, 24))
        AddFlow1.Nodes.Add(New Node(96, 228, 24, 24))
        AddFlow1.Nodes.Add(New Node(192, 228, 24, 24))
        AddFlow1.Nodes.Add(New Node(192, 300, 24, 24))
        AddFlow1.Nodes.Add(New Node(264, 300, 24, 24))
        AddFlow1.Nodes(1).OutLinks.Add(AddFlow1.Nodes(0))
        AddFlow1.Nodes(2).OutLinks.Add(AddFlow1.Nodes(0))
        AddFlow1.Nodes(0).OutLinks.Add(AddFlow1.Nodes(3))
        AddFlow1.Nodes(4).OutLinks.Add(AddFlow1.Nodes(1))
        AddFlow1.Nodes(4).OutLinks.Add(AddFlow1.Nodes(2))
        AddFlow1.Nodes(3).OutLinks.Add(AddFlow1.Nodes(4))
        AddFlow1.Nodes(4).OutLinks.Add(AddFlow1.Nodes(5))
        AddFlow1.Nodes(4).OutLinks.Add(AddFlow1.Nodes(6))
        AddFlow1.Nodes(5).OutLinks.Add(AddFlow1.Nodes(7))
        AddFlow1.Nodes(6).OutLinks.Add(AddFlow1.Nodes(7))
        AddFlow1.Nodes(7).OutLinks.Add(AddFlow1.Nodes(8))
        AddFlow1.Nodes(7).OutLinks.Add(AddFlow1.Nodes(9))
        AddFlow1.Nodes(10).OutLinks.Add(AddFlow1.Nodes(8))
        AddFlow1.Nodes(9).OutLinks.Add(AddFlow1.Nodes(10))
        AddFlow1.Nodes(11).OutLinks.Add(AddFlow1.Nodes(12))
        AddFlow1.Nodes(11).OutLinks.Add(AddFlow1.Nodes(13))
        AddFlow1.Nodes(13).OutLinks.Add(AddFlow1.Nodes(12))
        AddFlow1.Nodes(14).OutLinks.Add(AddFlow1.Nodes(13))
    End Sub

    Private Sub GraphTest_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub AddFlow1_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddFlow1.DoubleClick
        Dim item As Item = AddFlow1.PointedItem
        If (TypeOf (item) Is Link) Then
            Dim cost As Cost = New Cost
            cost.Link = item
            cost.ShowDialog()
        End If
    End Sub

    Private Sub AddFlow1_BeforeAddNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeAddNodeEventArgs) Handles AddFlow1.BeforeAddNode
        e.Size = New SizeF(2 * AddFlow1.Grid.Size.Width, 2 * AddFlow1.Grid.Size.Height)
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub

        strFileName = ""
        With AddFlow1
            .Nodes.Clear()
            .ResetUndoRedo()
            .SetChangedFlag(False)
        End With

        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        With AddFlow1
            Dim isSelectedItems As Boolean = (.SelectedItems.Count > 0)
            MenuItemEditUndo.Enabled = .CanUndo
            MenuItemEditRedo.Enabled = .CanRedo
            MenuItemEditSelectAll.Enabled = (.Nodes.Count <> 0)
            MenuItemEditCut.Enabled = isSelectedItems
            MenuItemEditCopy.Enabled = isSelectedItems
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
            MenuItemEditDelete.Enabled = isSelectedItems
        End With
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        For Each item As Item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub menuItemConnectedParts_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemConnectedParts.Click
        ResetItems(AddFlow1)
        Dim g As Graph = New Graph()
        OpenGraph(AddFlow1, g)

        ' Find the connected parts of the graph
        g.SeparateConnectedParts()

        ' Display each connected part
        For Each node As Node In AddFlow1.Nodes
            Dim v As Vertex = node.Tag
            node.Text = v.ConnectedPart.ToString()
            If v.ConnectedPart < aColor.Length Then
                node.FillColor = aColor(v.ConnectedPart)
            Else
                node.FillColor = Color.Silver
            End If
        Next

        CloseGraph(AddFlow1)
    End Sub

    Private Sub menuItemArticulationPoints_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemArticulationPoints.Click
        ResetItems(AddFlow1)
        Dim g As Graph = New Graph()
        OpenGraph(AddFlow1, g)

        ' Find the articulation points of the graph
        g.ArticulationPoints()

        ' Display each connected part
        For Each node As Node In AddFlow1.Nodes
            Dim v As Vertex = node.Tag
            If v.IsArticulation Then
                node.Text = "A"
                node.FillColor = Color.Yellow
            End If
        Next

        CloseGraph(AddFlow1)
    End Sub

    Private Sub menuItemBiconnectedParts_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemBiconnectedParts.Click
        ResetItems(AddFlow1)
        Dim g As Graph = New Graph()
        OpenGraph(AddFlow1, g)

        ' Find the biconnected parts of the graph
        g.ArticulationPoints()
        g.SeparateBiconnectedParts()

        ' Display the biconnected parts
        For Each node As Node In AddFlow1.Nodes
            Dim v As Vertex = node.Tag
            node.Text = v.BiconnectedPart.ToString
            If v.BiconnectedPart < aColor.Length Then
                node.FillColor = aColor(v.BiconnectedPart)
            Else
                node.FillColor = Color.Silver
            End If
        Next

        CloseGraph(AddFlow1)
    End Sub

    Private Sub menuItemSpanningTree_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemSpanningTree.Click
        If AddFlow1.SelectedItem Is Nothing Or TypeOf (AddFlow1.SelectedItem) Is Link Then
            MessageBox.Show("You have first to select a node with the mouse", strProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        ResetItems(AddFlow1)
        Dim g As Graph = New Graph()
        OpenGraph(AddFlow1, g)

        Dim selnode As Node = AddFlow1.SelectedItem
        g.FindSpanningTree(selnode.Tag)

        For Each node As Node In AddFlow1.Nodes
            For Each link As Link In node.OutLinks
                Dim edge As Edge = link.Tag
                If edge.InSpanningTree Then
                    link.DrawColor = Color.Red
                End If
            Next
        Next

        CloseGraph(AddFlow1)
    End Sub

    Private Sub menuItemShortedPath_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemShortedPath.Click
        If AddFlow1.SelectedItem Is Nothing Or TypeOf (AddFlow1.SelectedItem) Is Link Then
            MessageBox.Show("You have first to select a node with the mouse", strProgName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Sub
        End If

        ResetItems(AddFlow1)
        Dim g As Graph = New Graph()
        OpenGraph(AddFlow1, g)

        Dim selnode As Node = AddFlow1.SelectedItem
        g.FindShortestPath(selnode.Tag)

        For Each node As Node In AddFlow1.Nodes
            For Each link As Link In node.OutLinks
                Dim edge As Edge = link.Tag
                If edge.InPathTree Then
                    link.DrawColor = Color.Red
                End If
            Next
        Next

        CloseGraph(AddFlow1)
    End Sub

    Private Sub menuItemReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItemReset.Click
        ResetItems(AddFlow1)
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms: GraphTest Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Private Sub ResetItems(ByVal addflow As AddFlow)
        Dim node As Node, link As Link

        For Each node In addflow.Nodes
            ' Restore colors and text
            node.FillColor = addflow.DefNodeProp.FillColor
            node.Text = ""
        Next
        For Each node In addflow.Nodes
            For Each link In node.OutLinks
                link.DrawColor = addflow.DefLinkProp.DrawColor
            Next
        Next
    End Sub

    Private Sub OpenGraph(ByVal addflow As AddFlow, ByVal g As Graph)
        Dim node As Node, link As Link

        For Each node In addflow.Nodes
            ' Create a vertex corresponding to this node
            Dim v As Vertex = g.AddVertex(node)
            ' Save the node Tag (in case we use it) because we will use it for another purpose
            v.Tag = node.Tag
            ' Use the node Tag to save the corresponding vertex
            node.Tag = v
        Next

        For Each node In addflow.Nodes
            For Each link In node.OutLinks
                ' Create an edge corresponding to this link
                Dim org As Vertex = link.Org.Tag
                Dim dst As Vertex = link.Dst.Tag
                Dim cost As Integer = 0
                If Not (link.Text Is Nothing) Then
                    If link.Text.Length > 0 Then cost = Int32.Parse(link.Text)
                End If
                Dim e As Edge = g.AddEdge(org, dst, link, cost)
                e.Tag = link.Tag
                link.Tag = e
            Next
        Next
    End Sub

    Private Sub CloseGraph(ByVal addflow As AddFlow)
        ' Restore the initial node Tags
        Dim node As Node
        For Each node In addflow.Nodes
            Dim v As Vertex = node.Tag
            node.Tag = v.Tag
        Next
    End Sub
End Class
