'----------------------------------------------------------------------------------------
' AddFlow for WinForms: Lists sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow

Public Class Lists
    Private hideControls As Boolean
    Private currentList As ListView

    Private Sub Lists_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        hideControls = False
        currentList = Nothing

        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.AutoScroll = True
        AddFlow1.PageUnit = GraphicsUnit.Point
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.LabelEdit = False
        AddFlow1.DefNodeProp.FillColor = Color.LightYellow
        AddFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom
        AddFlow1.DefNodeProp.Shadow.Color = Color.Silver
        AddFlow1.DefNodeProp.Alignment = Alignment.CenterTOP
        AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None
        AddFlow1.DefLinkProp.EndCap = System.Drawing.Drawing2D.LineCap.Custom
        AddFlow1.DefLinkProp.CustomEndCap = New System.Drawing.Drawing2D.AdjustableArrowCap(3, 6, True)
        AddFlow1.DefLinkProp.DrawColor = Color.Blue
        AddFlow1.DefLinkProp.Jump = Jump.None

        Dim mynode1 As MyNode = New MyNode(AddFlow1, New PointF(33, 20), New SizeF(120, 100))
        Dim mynode2 As MyNode = New MyNode(AddFlow1, New PointF(367, 33), New SizeF(120, 100))

        Dim lv1 As ListView = mynode1.List
        If Not (lv1 Is Nothing) Then

            Dim lvi11 As ListViewItem = New ListViewItem("field11")
            lvi11.SubItems.Add("subfield 1")
            Dim lvi12 As ListViewItem = New ListViewItem("field12")
            lvi12.SubItems.Add("subfield 2")
            Dim lvi13 As ListViewItem = New ListViewItem("field13")
            lvi13.SubItems.Add("subfield 3")
            Dim lvi14 As ListViewItem = New ListViewItem("field14")
            lvi14.SubItems.Add("subfield 4")
            Dim lvi15 As ListViewItem = New ListViewItem("field15")
            lvi15.SubItems.Add("subfield 5")
            Dim lvi16 As ListViewItem = New ListViewItem("field16")
            lvi16.SubItems.Add("subfield 6")

            lv1.Items.AddRange(New ListViewItem() {lvi11, lvi12, lvi13, lvi14, lvi15, lvi16})
        End If

        Dim lv2 As ListView = mynode2.List
        If Not (lv2 Is Nothing) Then
            Dim lvi21 As ListViewItem = New ListViewItem("field11")
            lvi21.SubItems.Add("subfield 1")

            Dim lvi22 As ListViewItem = New ListViewItem("field12")
            lvi22.SubItems.Add("subfield 2")

            Dim lvi23 As ListViewItem = New ListViewItem("field13")
            lvi23.SubItems.Add("subfield 3")

            Dim lvi24 As ListViewItem = New ListViewItem("field14")
            lvi24.SubItems.Add("subfield 4")

            lv2.Items.AddRange(New ListViewItem() {lvi21, lvi22, lvi23, lvi24})
        End If

        ' Create a link
        AddFlow1.Nodes(0).OutLinks.Add(AddFlow1.Nodes(1))
    End Sub

    Private Sub MenuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuAbout.Click
        MsgBox("AddFlow for WinForms: Lists Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub MenuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuExit.Click
        End
    End Sub

    Private Sub AddFlow1_BeforeAddNode(ByVal sender As Object, ByVal e As Lassalle.Flow.BeforeAddNodeEventArgs) Handles AddFlow1.BeforeAddNode
        ' Cancel the node creation. Instead we create a MyNode object at the same place.
        e.Cancel.Cancel = True
        Dim mynode As MyNode = New MyNode(AddFlow1, e.Location, e.Size)
    End Sub

    Private Sub AddFlow1_AfterMove(ByVal sender As Object, ByVal e As System.EventArgs) Handles AddFlow1.AfterMove
        Dim item As Item
        For Each item In AddFlow1.SelectedItems
            If TypeOf (item) Is Node Then
                Dim mynode As MyNode = item
                mynode.UpdateListPositionAndSize()
            End If
        Next
    End Sub

    Private Sub AddFlow1_AfterResize(ByVal sender As Object, ByVal e As System.EventArgs) Handles AddFlow1.AfterResize
        If AddFlow1.SelectedItem Is Nothing Then Exit Sub
        Dim mynode As MyNode = AddFlow1.SelectedItem
        mynode.UpdateListPositionAndSize()
    End Sub

    Private Sub AddFlow1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles AddFlow1.MouseDown
        If Not (currentList Is Nothing) Then
            currentList.Enabled = False
        End If
        If AddFlow1.InteractiveAction = InteractiveAction.Size Or AddFlow1.InteractiveAction = InteractiveAction.Drag Then
            hideControls = True
            Dim item As Item
            For Each item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim mynode As MyNode = item
                    If Not (mynode.List Is Nothing) Then
                        mynode.List.Visible = False
                    End If
                End If
            Next
        Else
            Dim item As Item = AddFlow1.SelectedItem
            If TypeOf (item) Is Node Then
                Dim mynode As MyNode = item
                currentList = mynode.List
                currentList.Enabled = True
            End If
        End If
    End Sub

    Private Sub AddFlow1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles AddFlow1.MouseUp
        If hideControls Then
            hideControls = False
            Dim item As Item
            For Each item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim mynode As MyNode = item
                    If Not (mynode.List Is Nothing) Then
                        mynode.List.Visible = True
                    End If
                End If
            Next
        End If
    End Sub
End Class

' Create a new node class, derived from the Node class of AddFlow.
Public Class MyNode
    Inherits Node

    Const xMargin As Integer = 10
    Const yMargin As Integer = 10

    Private m_list As ListView

    Public Sub New(ByVal AddFlow1 As AddFlow, ByVal location As PointF, ByVal size As SizeF)
        MyBase.New(AddFlow1.DefNodeProp)

        Me.Location = location
        Me.Size = size
        AddFlow1.Nodes.Add(CType(Me, Lassalle.Flow.Node))

        ' Create the associated control 
        m_list = New ListView
        m_list.Parent = AddFlow1
        m_list.BorderStyle = BorderStyle.FixedSingle
        m_list.LabelEdit = True
        m_list.GridLines = True
        m_list.View = View.Details
        m_list.FullRowSelect = True
        m_list.Enabled = False

        m_list.Columns.Add("Col 1", 60, HorizontalAlignment.Left)
        m_list.Columns.Add("Col 2", 60, HorizontalAlignment.Left)

        UpdateListPositionAndSize()
    End Sub

    Public ReadOnly Property List() As ListView
        Get
            Return m_list
        End Get
    End Property

    Public Sub UpdateListPositionAndSize()
        If m_list Is Nothing Then Exit Sub
        Dim pt1 As PointF = Me.AddFlow.PointFToDevice(Location)
        Dim pt2 As PointF = Me.AddFlow.PointFToDevice(New PointF(Location.X + Size.Width, Location.Y + Size.Height))
        m_list.SetBounds(pt1.X + xMargin, pt1.Y + yMargin, pt2.X - pt1.X - 2 * xMargin, pt2.Y - pt1.Y - 2 * yMargin)
    End Sub
End Class
