'---------------------------------------------------------------------------------------
' AddFlow for WinForms: Navig sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'---------------------------------------------------------------------------------------

Imports Lassalle.Flow

Public Class Navig
    Private Sub Navig_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim node As Node, link As Link

        With AddFlow1
            .LinkCreationMode = LinkCreationMode.MiddleHandle
            .CanDragScroll = False
            .PageUnit = GraphicsUnit.Point
            .DefNodeProp.Gradient = True
            .DefNodeProp.GradientMode = Drawing.Drawing2D.LinearGradientMode.ForwardDiagonal
            .DefNodeProp.GradientColor = Color.LightBlue

            node = .Nodes.Add(67, 77, 26, 30, "1")
            node.Shape.Style = ShapeStyle.Rectangle
            node.FillColor = Color.Yellow
            node = .Nodes.Add(7, 52, 30, 25, "2")
            node = .Nodes.Add(68, 24, 30, 30, "3")
            node.Selected = True
            node = .Nodes.Add(150, 68, 30, 30, "4")
            node.Selected = True
            node = .Nodes.Add(7, 126, 41, 38, "5: owned by 1")
            node = .Nodes.Add(21, 10, 25, 25, "6")
            node = .Nodes.Add(28, 215, 30, 30, "7")
            node = .Nodes.Add(100, 150, 41, 38, "8: owned by 1")
            node = .Nodes.Add(150, 200, 41, 38, "9: owned by 8")
            node = .Nodes.Add(235, 75, 30, 30, "10")
            node = .Nodes.Add(185, 10, 30, 30, "11")

            link = .Nodes(0).OutLinks.Add(.Nodes(1))
            link = .Nodes(0).OutLinks.Add(.Nodes(2))
            link = .Nodes(0).OutLinks.Add(.Nodes(5))
            link = .Nodes(3).OutLinks.Add(.Nodes(0))
            link = .Nodes(6).OutLinks.Add(.Nodes(0))
            link = .Nodes(4).OutLinks.Add(.Nodes(6))
            link.Selected = True
            link = .Nodes(7).OutLinks.Add(.Nodes(0))
            link.Selected = True
            link = .Nodes(8).OutLinks.Add(.Nodes(7))
            link = .Nodes(9).OutLinks.Add(.Nodes(10))

            ' Links that allow ownership between nodes
            link = .Nodes(0).OutLinks.Add(.Nodes(4))
            link.Hidden = True
            link.Rigid = True
            link.Selectable = False
            link.Logical = False
            link = .Nodes(0).OutLinks.Add(.Nodes(7))
            link.Hidden = True
            link.Rigid = True
            link.Selectable = False
            link.Logical = False
            link = .Nodes(7).OutLinks.Add(.Nodes(8))
            link.Hidden = True
            link.Rigid = True
            link.Selectable = False
            link.Logical = False
        End With
    End Sub

    Private Sub ExitMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitMenu.Click
        End
    End Sub

    Private Sub AboutMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutMenu.Click
        MsgBox("AddFlow for WinForms: Navig sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub Button0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button0.Click
        Dim node As Node, link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).Links
            If Not link.Rigid Then
                node = AddFlow1.Nodes(0).GetLinkedNode(link)
                node.DrawColor = Color.Red
            End If
        Next
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).OutLinks
            If Not link.Rigid Then link.Dst.DrawColor = Color.Red
        Next

        'Other method 
        'Dim node As Node = AddFlow1.Nodes(0)
        'Dim i As Integer
        'For i = 0 To node.OutLinks.Count - 1
        '    Dim link As Link
        '    link = node.OutLinks(i)
        '    link.DrawColor = Color.Red
        'Next
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).InLinks
            If Not link.Rigid Then link.Org.DrawColor = Color.Red
        Next
    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).Links
            If Not link.Rigid Then link.DrawColor = Color.Red
        Next
    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).InLinks
            If Not link.Rigid Then link.DrawColor = Color.Red
        Next
    End Sub

    Private Sub Button5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).OutLinks
            If Not link.Rigid Then link.DrawColor = Color.Red
        Next
    End Sub

    ' All nodes
    Private Sub Button6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button6.Click
        Dim node As Node

        ResetColors()
        For Each node In AddFlow1.Nodes
            node.DrawColor = Color.Red
        Next
    End Sub

    ' All links
    Private Sub Button7_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button7.Click
        Dim node As Node, link As Link

        ResetColors()
        For Each node In AddFlow1.Nodes
            For Each link In node.OutLinks
                link.DrawColor = Color.Red
            Next
        Next
    End Sub

    ' All nodes and links
    Private Sub Button8_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button8.Click
        Dim node As Node, link As Link

        ResetColors()
        For Each node In AddFlow1.Nodes
            node.DrawColor = Color.Red
            For Each link In node.OutLinks
                link.DrawColor = Color.Red
            Next
        Next
    End Sub

    ' Selected nodes
    Private Sub Button9_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button9.Click
        Dim item As Item

        ResetColors()
        For Each item In AddFlow1.SelectedItems
            If TypeOf (item) Is Lassalle.Flow.Node Then item.DrawColor = Color.Red
        Next
    End Sub

    ' Owned nodes    
    Private Sub Button10_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button10.Click
        Dim link As Link

        ResetColors()
        For Each link In AddFlow1.Nodes(0).OutLinks
            If link.Rigid Then link.Dst.DrawColor = Color.Red
        Next
    End Sub

    ' Selected links
    Private Sub Button11_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button11.Click
        Dim item As Item

        ResetColors()
        For Each item In AddFlow1.SelectedItems
            If TypeOf (item) Is Lassalle.Flow.Link Then item.DrawColor = Color.Red
        Next
    End Sub

    ' Connected nodes    
    Private Sub Button12_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button12.Click
        ResetColors()

        Dim item As Item
        For Each item In AddFlow1.Nodes(0).ConnectedItems
            If TypeOf (item) Is Node Then item.DrawColor = Color.Red
        Next
    End Sub

    ' Reset draw color of every items (nodes and links)
    Private Sub ResetColors()
        Dim node As Node, link As Link

        For Each node In AddFlow1.Nodes
            node.DrawColor = Color.Black
            For Each link In node.OutLinks
                link.DrawColor = Color.Black
            Next
        Next
    End Sub
End Class
