'----------------------------------------------------------------------------------------
' AddFlow for WinForms: OwnerDraw sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports Lassalle.Flow

Public Class OwnerDraw

    Private Sub OwnerDraw_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.AutoScroll = True
        AddFlow1.OwnerDraw = True
        AddFlow1.PageUnit = GraphicsUnit.Point
        AddFlow1.DefNodeProp.OwnerDraw = True
        AddFlow1.DefLinkProp.OwnerDraw = True
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom

        ' Create 2 nodes and a link
        Dim node1 As Node = New Node(30, 30, 50, 50, "node1", AddFlow1.DefNodeProp)
        Dim node2 As Node = New Node(330, 30, 50, 50, "node2", AddFlow1.DefNodeProp)
        Dim link As Link = New Link("link 12", AddFlow1.DefLinkProp)
        link.DrawColor = Color.Red

        ' Insert them in the graph
        AddFlow1.Nodes.Add(node1)
        AddFlow1.Nodes.Add(node2)
        node1.OutLinks.Add(link, node2)

        ' Add link points
        link.Points.Add(New PointF(55, 160))
        link.Points.Add(New PointF(355, 160))
    End Sub

    Private Sub ExitMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitMenu.Click
        End
    End Sub

    Private Sub AboutMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutMenu.Click
        MsgBox("AddFlow for WinForms: OwnerDraw sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub AddFlow1_DiagramOwnerDraw(ByVal sender As System.Object, ByVal e As Lassalle.Flow.DiagramOwnerDrawEventArgs) Handles AddFlow1.DiagramOwnerDraw
        Dim grfx As Graphics = e.Graphics

        ' Save the graphics state because it does not belong to us
        ' (although in this case, it not necessary because we do not alter the Graphics state)
        Dim gs As GraphicsState = grfx.Save()

        Dim swimLinesHeight As Single = 50
        Dim w As Single = AddFlow1.ClientRectangle.Size.Width + AddFlow1.Extent.Width
        Dim h As Single = AddFlow1.ClientRectangle.Size.Height + AddFlow1.Extent.Height
        Dim swimlines As Integer = h / swimLinesHeight
        Dim i As Integer
        Dim rect As Rectangle
        For i = 0 To swimlines
            rect = New Rectangle(0, i * swimLinesHeight, w, swimLinesHeight)
            If i Mod 2 = 0 Then
                grfx.FillRectangle(Brushes.LightYellow, rect)
            Else
                grfx.FillRectangle(Brushes.LightBlue, rect)
            End If
        Next

        ' Restore the graphics state
        grfx.Restore(gs)
    End Sub

    ' We fill the node with a hatch brush
    Private Sub AddFlow1_NodeOwnerDraw(ByVal sender As System.Object, ByVal e As Lassalle.Flow.NodeOwnerDrawEventArgs) Handles AddFlow1.NodeOwnerDraw
        Dim grfx As Graphics = e.Graphics
        Dim node As Node = e.Node

        ' Save the graphics state because it does not belong to us
        ' (although in this case, it not necessary because we do not alter the Graphics state)
        Dim gs As GraphicsState = grfx.Save()

        Dim hBrush1 As HatchBrush = New HatchBrush(HatchStyle.BackwardDiagonal, Color.White, Color.Silver)
        grfx.FillRectangle(hBrush1, node.Rect)

        ' Restore the graphics state
        grfx.Restore(gs)

        ' Tell AddFlow not to fill the node (because this is already done)
        e.Flags = Not NodeDrawFlags.Fill
    End Sub

    ' If the link has more than one segment, we decide to draw the link text at each segment.
    ' (By default, the link text is displayed only at the middle segment)
    Private Sub AddFlow1_LinkOwnerDraw(ByVal sender As System.Object, ByVal e As Lassalle.Flow.LinkOwnerDrawEventArgs) Handles AddFlow1.LinkOwnerDraw
        Dim grfx As Graphics = e.Graphics
        Dim link As Link = e.Link

        If link.Points.Count > 2 Then
            ' Save the graphics state because it does not belong to us
            ' (although in this case, it not necessary because we do not alter the Graphics state)
            Dim gs As GraphicsState = grfx.Save()

            Dim degrees As Integer = 0
            Dim brush As Brush = New SolidBrush(link.TextColor)
            Dim i As Integer
            For i = 0 To link.Points.Count - 2
                Dim x1 As Single = link.Points(i).X
                Dim y1 As Single = link.Points(i).Y
                Dim x2 As Single = link.Points(i + 1).X
                Dim y2 As Single = link.Points(i + 1).Y
                Dim x As Single = x2 - x1
                Dim y As Single = -(y2 - y1)  ' ... since the y axis is bottom directed 
                Dim d As Single = Math.Sqrt(x * x + y * y)
                Dim cosA As Single = x / d
                Dim sinA As Single = y / d
                If d = 0 Then
                    degrees = 0
                Else
                    degrees = CType((180.0 / 3.14159 * Math.Acos(cosA)), Integer)
                    If (sinA < 0) Then degrees = -degrees
                    If (cosA < 0) Then degrees += 180
                End If
                Dim pt As PointF = New PointF((link.Points(i).X + link.Points(i + 1).X) / 2, (link.Points(i).Y + link.Points(i + 1).Y) / 2)
                grfx.TranslateTransform(pt.X, pt.Y)
                If degrees <> 0 Then grfx.RotateTransform(-degrees)
                grfx.DrawString(link.Text, link.Font, brush, New PointF(0, 0))
                If degrees <> 0 Then grfx.RotateTransform(degrees)
                grfx.TranslateTransform(-pt.X, -pt.Y)
            Next

            ' Restore the graphics state
            grfx.Restore(gs)

            ' Tell AddFlow not to display the text (because this is already done)
            ' However, other parts of the link (line, arrow) will be displayed by AddFlow.
            e.Flags = Not LinkDrawFlags.Text
        End If
    End Sub
End Class
