'---------------------------------------------------------------------------------------
' AddFlow for WinForms: Pins sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'---------------------------------------------------------------------------------------

Imports Lassalle.Flow

Public Class Pins
    Public Shared pinDistance As Integer = 5
    Public Shared pinWidth As Integer = 10
    Public Shared pinHeight As Integer = 10

    Private Sub Pins_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        With AddFlow1
            .Dock = DockStyle.Bottom
            .PageUnit = GraphicsUnit.Point
            .DefNodeProp.Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle
            .AutoScroll = True
            .CanReflexLink = False
            .CanLabelEdit = False
            .BackColor = SystemColors.Window
        End With

        ' Some explanations
        Dim s As String = Nothing
        s = s & "This sample shows how to implement pins. Each created node has 4 pins."
        s = s & "To create a link, bring the mouse cursor into one pin of the origin node, "
        s = s & "press the left button, move the mouse towards the other node. When the "
        s = s & "mouse cursor is into a pin of the other node, release the left button."
        Label1.Text = s
    End Sub

    Private Sub Pins_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        AddFlow1.Location = New Point(0, Label1.Size.Height)
        AddFlow1.Size = New Size(Me.ClientSize.Width, Me.ClientSize.Height - Label1.Size.Height)
    End Sub

    Private Sub ExitMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitMenu.Click
        End
    End Sub

    Private Sub MenuDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub AboutMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutMenu.Click
        MsgBox("AddFlow for WinForms: Pins sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub AddFlow1_AfterAddNode(ByVal sender As System.Object, ByVal e As Lassalle.Flow.AfterAddNodeEventArgs) Handles AddFlow1.AfterAddNode
        CreateMyComplexNode(e.Node)
    End Sub

    Private Sub AddFlow1_BeforeAddLink(ByVal sender As System.Object, ByVal e As Lassalle.Flow.BeforeAddLinkEventArgs) Handles AddFlow1.BeforeAddLink
        Dim bCancel As Boolean = True
        If TypeOf (e.Dst) Is Pin And TypeOf (e.Org) Is Pin Then
            Dim pinDst As Pin = CType(e.Dst, Pin)
            Dim pinOrg As Pin = CType(e.Org, Pin)
            If Not (pinDst.Node Is pinOrg.Node) Then
                bCancel = False
            End If
        End If
        e.Cancel.Cancel = bCancel
    End Sub

    Private Sub AddFlow1_AfterResize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddFlow1.AfterResize
        Dim item As Item = AddFlow1.SelectedItem
        If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
            Dim node As Node = CType(item, Node)
            UpdateNodePosition(node)
        End If
    End Sub

    Private Sub AddFlow1_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles AddFlow1.MouseMove
        Dim bCanDrawLink As Boolean = False
        Dim item As Item = AddFlow1.PointedItem
        If Not (item Is Nothing) Then
            If TypeOf (item) Is Pin Then bCanDrawLink = True
        End If
        If bCanDrawLink Then
            AddFlow1.Cursor = Cursors.UpArrow
        Else
            AddFlow1.Cursor = Cursors.Default
        End If
    End Sub

    Private Sub CreateMyComplexNode(ByVal node As Node)
        ' Avoid too small nodes
        If node.Rect.Width < 50 Then node.Size = New SizeF(50, node.Rect.Height)
        If node.Rect.Height < 50 Then node.Size = New SizeF(node.Rect.Width, 50)

        ' Make node transparent
        node.DrawColor = Color.Transparent
        node.Transparent = True
        node.OutLinkable = false
        node.InLinkable = false

        ' Create "child" nodes
        CreateLabelNode(node)
        CreatePins(node)
    End Sub

    Private Sub CreateLabelNode(ByVal node As Node)
        Dim x As Single = node.Rect.Left + pinDistance + pinWidth
        Dim y As Single = node.Rect.Top + pinDistance + pinHeight
        Dim cx As Single = node.Rect.Width - 2 * (pinDistance + pinWidth)
        Dim cy As Single = node.Rect.Height - 2 * (pinDistance + pinHeight)

        ' Create the label node...
        Dim labelNode As Node = AddFlow1.Nodes.Add(x, y, cx, cy)
        labelNode.Selectable = False
        labelNode.Logical = False
        labelNode.OutLinkable = false
        labelNode.InLinkable = false
        labelNode.Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle
        labelNode.Parent = node

        ' Memorize a reference to the label node in the main node
        node.Tag = labelNode
    End Sub

    Private Sub CreatePins(ByVal node As Node)
        Dim labelNode As Node = node.Tag
        For i As Integer = 1 To 4
            Dim pin As Pin = New Pin(node, i)
            AddFlow1.Nodes.Add(pin)
            pin.Parent = labelNode
        Next
    End Sub

    Private Sub UpdateNodePosition(ByVal node As Node)
        Dim x As Single = node.Rect.Left + pinDistance + pinWidth
        Dim y As Single = node.Rect.Top + pinDistance + pinHeight
        Dim cx As Single = node.Rect.Width - 2 * (pinDistance + pinWidth)
        Dim cy As Single = node.Rect.Height - 2 * (pinDistance + pinHeight)

        Dim labelNode As Node = node.Tag
        labelNode.Rect = New RectangleF(x, y, cx, cy)

        For Each child As Node In labelNode.Children
            If TypeOf (child) Is Pin Then
                Dim pin As Pin = CType(child, Pin)
                pin.SetPinLocation()
            End If
        Next
    End Sub

    Public Class Pin
        Inherits Node

        Dim m_number As Integer
        Dim m_node As Node

        Public Sub New(ByVal node As Node, ByVal n As Integer)
            m_node = node
            m_number = n

            Me.SetPinLocation()
            Me.Size = New SizeF(pinWidth, pinHeight)
            Me.FillColor = Color.Red
            Me.XMoveable = False
            Me.YMoveable = False
            Me.XSizeable = False
            Me.YSizeable = False
            Me.Logical = False
            Me.Shape.Style = Lassalle.Flow.ShapeStyle.Rectangle
        End Sub

        Public Sub SetPinLocation()
            Dim x As Single = Node.Rect.Left + pinDistance + pinWidth / 2
            Dim y As Single = Node.Rect.Top + pinDistance + pinHeight / 2
            Dim cx As Single = Node.Rect.Width - 2 * (pinDistance + pinWidth / 2)
            Dim cy As Single = Node.Rect.Height - 2 * (pinDistance + pinHeight / 2)
            Dim rc As RectangleF = New RectangleF(x, y, cx, cy)
            Select Case m_number
                Case 1
                    x = rc.Left - pinDistance - pinWidth / 2
                    y = rc.Top + rc.Height / 2 - pinHeight / 2
                Case 2
                    x = rc.Left + rc.Width / 2 - pinWidth / 2
                    y = rc.Top - pinDistance - pinHeight / 2
                Case 3
                    x = rc.Right + pinDistance - pinWidth / 2
                    y = rc.Top + rc.Height / 2 - pinHeight / 2
                Case 4
                    x = rc.Left + rc.Width / 2 - pinWidth / 2
                    y = rc.Bottom + pinDistance - pinHeight / 2
            End Select
            Me.Location = New PointF(x, y)
        End Sub

        Public Property Number() As Integer
            Get
                Return m_number
            End Get
            Set(ByVal Value As Integer)
                m_number = Value
            End Set
        End Property

        Public Property Node() As Node
            Get
                Return m_node
            End Get
            Set(ByVal Value As Node)
                m_node = Value
            End Set
        End Property
    End Class
End Class
