'---------------------------------------------------------------------------------------
' AddFlow for WinForms: PropertyBag sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'---------------------------------------------------------------------------------------

Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports Lassalle.Flow

Public Class PropertyBag
    Private strProgName As String
    Private strFileName As String

    Private Sub PropertyBag_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub PropertyBag_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = Color.White
        AddFlow1.AutoScroll = True

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub

        strFileName = ""
        With AddFlow1
            .Nodes.Clear()
            .ResetUndoRedo()
            .SetChangedFlag(False)
        End With

        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        Dim isSelectedItems As Boolean = (AddFlow1.SelectedItems.Count > 0)

        MenuItemEditUndo.Enabled = AddFlow1.CanUndo
        MenuItemEditRedo.Enabled = AddFlow1.CanRedo
        MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count <> 0)
        MenuItemEditCut.Enabled = isSelectedItems
        MenuItemEditCopy.Enabled = isSelectedItems
        MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
        MenuItemEditDelete.Enabled = isSelectedItems
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            Dim item As Item
            For Each item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        Dim item As Item
        For Each item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemData_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemData.Popup
        MenuItemDataNodeCustomData.Enabled = Not (AddFlow1.SelectedItem Is Nothing) And TypeOf (AddFlow1.SelectedItem) Is Node
    End Sub

    Private Sub MenuItemDataNodeCustomData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDataNodeCustomData.Click
        EnterCustomData()
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms: PropertyBag sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub AddFlow1_AfterAddNode(ByVal sender As Object, ByVal e As Lassalle.Flow.AfterAddNodeEventArgs) Handles AddFlow1.AfterAddNode
        e.Node.Properties("Comment").Value = "Comment" & e.Node.Index.ToString()
        e.Node.Properties("Author").Value = "Author..."
        AddHandler e.Node.Properties("Comment").Updating, AddressOf Comment_Updating
        AddHandler e.Node.Properties("Author").Updating, AddressOf Author_Updating
    End Sub

    ' The WriteMLNodeExtraData event is fired when serializing a node to allow the 
    ' serialization of custom data. 
    Private Sub AddFlow1_WriteXMLNodeExtraData(ByVal sender As Object, ByVal e As Lassalle.Flow.WriteXMLNodeExtraDataEventArgs) Handles AddFlow1.WriteXMLNodeExtraData
        Dim comment As String = e.Node.Properties("Comment").Value
        Dim author As String = e.Node.Properties("Author").Value
        If Not (comment Is Nothing) Then
            If comment.Length > 0 Then
                e.Writer.WriteElementString("Comment", comment)
            End If
        End If
        If Not (author Is Nothing) Then
            If author.Length > 0 Then
                e.Writer.WriteElementString("Author", author)
            End If
        End If
    End Sub

    ' The ReadXMLNodeExtraData event is fired when de-serializing a node and each time 
    ' AddFlow encounters a custom XML element. In this example, there are two possible
    ' custom elements: "Author" and "Comment".
    ' Notice that each time something is read, the IsRead property is set to true to
    ' inform AddFlow that it does not need to read the data itself.
    ' If you do not do that, the node de-serialization will not be successfull.
    Private Sub AddFlow1_ReadXMLNodeExtraData(ByVal sender As Object, ByVal e As Lassalle.Flow.ReadXMLNodeExtraDataEventArgs) Handles AddFlow1.ReadXMLNodeExtraData
        If (e.Reader.Name = "Comment") Then
            e.Node.Properties("Comment").Value = e.Reader.ReadElementContentAsString()
        ElseIf (e.Reader.Name = "Author") Then
            e.Node.Properties("Author").Value = e.Reader.ReadElementContentAsString()
        End If
    End Sub

    Private Sub Comment_Updating(ByVal sender As Object, ByVal e As UpdatingEventArgs)
        If e.NewValue.ToString().Length = 0 Then
            e.Cancel.Cancel = True
        End If
    End Sub

    Private Sub Author_Updating(ByVal sender As Object, ByVal e As UpdatingEventArgs)
        If e.NewValue.ToString().Length = 0 Then
            e.Cancel.Cancel = True
        End If
    End Sub

    Private Sub AddFlow1_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles AddFlow1.DoubleClick
        EnterCustomData
    End Sub

    Private Sub EnterCustomData()
        If Not (AddFlow1.SelectedItem Is Nothing) And TypeOf (AddFlow1.SelectedItem) Is Node Then
            Dim node As Node = AddFlow1.SelectedItem
            Dim nd As NodeData = New NodeData
            nd.Node = node
            If nd.ShowDialog() = Windows.Forms.DialogResult.OK Then
                AddFlow1.SubmitTask(New MyTask(node))
                node.Properties("Author").Value = nd.TextBoxAuthor.Text
                node.Properties("Comment").Value = nd.textBoxComment.Text
            End If
        End If
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function
End Class

Public Class MyTask
    Inherits Task

    Dim m_node As Node
    Dim m_oldAuthor As String
    Dim m_oldComment As String

    Public Sub New(ByVal node As Node)
        MyBase.New()
        Me.Code = 1001
        m_node = node
        m_oldAuthor = node.Properties("Author").Value
        m_oldComment = node.Properties("Comment").Value
    End Sub

    Public Overrides Sub Redo()
        Undo()
    End Sub

    Public Overrides Sub Undo()
        Dim author As String = m_node.Properties("Author").Value
        Dim comment As String = m_node.Properties("Comment").Value
        m_node.Properties("Author").Value = m_oldAuthor
        m_node.Properties("Comment").Value = m_oldComment
        m_oldAuthor = author
        m_oldComment = comment
    End Sub
End Class
