'----------------------------------------------------------------------------------------
' AddFlow for WinForms: RouteLink sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D

Public Class RouteLink
    Private strProgName As String
    Private strFileName As String
    Private m_animNode As Node
    Private m_sleep As Boolean
    Private m_grain As Single = 8
    Private m_minDistance As Single = 16
    Private m_startingMethod As Lassalle.Flow.Router.StartingMethod = Lassalle.Flow.Router.StartingMethod.AllSides
    Private m_sendStepEvent As Boolean = False

    Private Sub RouteLink_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        StatusBar1.Dock = DockStyle.Bottom

        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.Grid.Draw = True
        AddFlow1.Grid.Snap = True
        AddFlow1.Grid.Style = GridStyle.DottedLines
        AddFlow1.Grid.Color = Color.Silver
        AddFlow1.Grid.Size = New Size(20, 20)
        AddFlow1.CanFireError = True
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.Gradient = True
        AddFlow1.DefNodeProp.GradientColor = Color.LightGray
        AddFlow1.DefNodeProp.Text = "Obstacle"
        AddFlow1.AutoScroll = True
        AddFlow1.DefLinkProp.Jump = Jump.None
        AddFlow1.DefLinkProp.AdjustDst = True
        AddFlow1.DefLinkProp.AdjustOrg = True

        Me.ToolBarButtonFileNew.Tag = "New"
        Me.ToolBarButtonFileOpen.Tag = "Open"
        Me.ToolBarButtonFileSave.Tag = "Save"
        Me.ToolBarButtonEditCut.Tag = "Cut"
        Me.ToolBarButtonEditCopy.Tag = "Copy"
        Me.ToolBarButtonEditPaste.Tag = "Paste"
        Me.ToolBarButtonEditUndo.Tag = "Undo"
        Me.ToolBarButtonEditRedo.Tag = "Redo"

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()

        ' Create an unselectable "label" node
        Dim label As Node = AddFlow1.Nodes.Add(0, 0, 500, 30)
        label.Transparent = True
        label.Logical = False
        label.Selectable = False
        label.TextColor = Color.Blue
        label.DrawColor = Color.Transparent
        label.Text = "Create a link between the origin node and the destination node. It will avoid the obstacle nodes."

        ' Create an origin node and a destination node
        Dim org As Node = AddFlow1.Nodes.Add(40, 60, 60, 60)
        org.Text = "origin"
        org.FillColor = Color.Yellow
        org.Gradient = False

        Dim dst As Node = AddFlow1.Nodes.Add(500, 200, 60, 60)
        dst.Text = "destination"
        dst.FillColor = Color.Yellow
        dst.Gradient = False

        ' Create 2 obstacle nodes
        AddFlow1.Nodes.Add(240, 60, 60, 60)
        AddFlow1.Nodes.Add(140, 200, 60, 60)
    End Sub

    Private Sub RouteLink_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub ToolBar1_ButtonClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolBarButtonClickEventArgs) Handles ToolBar1.ButtonClick
        Dim key As String = e.Button.Tag
        Select Case key
            Case "New"
                MenuItemFileNew_Click(sender, e)
            Case "Open"
                MenuItemFileOpen_Click(sender, e)
            Case "Save"
                MenuItemFileSave_Click(sender, e)
            Case "Cut"
                MenuItemEditCut_Click(sender, e)
            Case "Copy"
                MenuItemEditCopy_Click(sender, e)
            Case "Paste"
                MenuItemEditPaste_Click(sender, e)
            Case "Undo"
                MenuItemEditUndo_Click(sender, e)
            Case "Redo"
                MenuItemEditRedo_Click(sender, e)
        End Select
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub
        strFileName = ""
        With AddFlow1
            .Nodes.Clear()
            .ResetUndoRedo()
            .SetChangedFlag(False)
        End With
        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        If OkToTrash() Then End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        Dim isSelectedItems As Boolean

        With AddFlow1
            isSelectedItems = (.SelectedItems.Count > 0)
            MenuItemEditUndo.Enabled = .CanUndo
            MenuItemEditRedo.Enabled = .CanRedo
            If MenuItemEditRedo.Enabled Then
                MenuItemEditRedo.Text = "Redo " & ActionName(.RedoCode)
            End If
            If MenuItemEditUndo.Enabled Then
                MenuItemEditUndo.Text = "Undo " & ActionName(.UndoCode)
            End If
            MenuItemEditSelectAll.Enabled = (.Nodes.Count <> 0)
            MenuItemEditCut.Enabled = isSelectedItems
            MenuItemEditCopy.Enabled = isSelectedItems
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
            MenuItemEditDelete.Enabled = isSelectedItems
        End With
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            Dim tr As New Xml.XmlTextReader(stream)

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        For Each item As Item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms RouteLink Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Function ActionName(ByVal code As Integer) As String
        Select Case code
            Case Action.NodeAdd
                ActionName = "node creation"
            Case Action.NodeResize
                ActionName = "node resize"
            Case Action.DeleteSel
                ActionName = "delete selection"
            Case Action.MoveItems
                ActionName = "move items"
            Case Action.LinkAdd
                ActionName = "link creation"
            Case Action.LinkStretch
                ActionName = "link stretch"
            Case Action.Org
                ActionName = "link origin change"
            Case Action.Dst
                ActionName = "link destination change"
            Case Action.Text
                ActionName = "node text change"
            Case 1003
                ActionName = "paste"
            Case 1007
                ActionName = "link autorouting"
            Case Else
                ActionName = ""
        End Select
    End Function

    Private Sub AddFlow1_AfterAddLink(ByVal sender As Object, ByVal e As Lassalle.Flow.AfterAddLinkEventArgs) Handles AddFlow1.AfterAddLink
        Dim route As Lassalle.Flow.Router.RouteFlow = New Lassalle.Flow.Router.RouteFlow
        route.SendStepEvent = m_sendStepEvent
        route.Grain = m_grain
        route.MinDistance = m_minDistance
        route.StartingMethod = m_startingMethod

        If route.SendStepEvent Then
            ' In animation mode, we do not register the actions in the undo/redo buffer
            AddFlow1.SkipUndo = True

            AddHandler route.Step, AddressOf route_Step
            m_animNode = New Node(0, 0, 3, 3)
            m_animNode.Shape.Style = ShapeStyle.Ellipse
            m_animNode.Logical = False
            m_animNode.Selectable = False
            m_animNode.DrawColor = Color.Red
            m_animNode.FillColor = Color.Red
            AddFlow1.Nodes.Add(m_animNode)

            Dim timer As Timer = New Timer
            timer.Interval = 50
            AddHandler timer.Tick, AddressOf timer_Tick
            timer.Start()

            m_sleep = False

            e.Link.Hidden = True
            route.FindRoute(e.Link)
            e.Link.Hidden = False

            RemoveHandler timer.Tick, AddressOf timer_Tick
            RemoveHandler route.Step, AddressOf route_Step
            timer.Stop()
            AddFlow1.Nodes.Remove(m_animNode)

            AddFlow1.SkipUndo = False
        Else
            ' So that the user will be able to undo the autorouting in one time
            AddFlow1.BeginAction(1007)

            route.FindRoute(e.Link)

            ' Terminate the action for the undo processor
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub timer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        m_sleep = False
    End Sub

    Private Sub route_Step(ByVal sender As Object, ByVal e As Lassalle.Flow.Router.StepEventArgs)
        Do While (m_sleep)
            Application.DoEvents()
        Loop
        m_sleep = True
        m_animNode.Location = e.Position
    End Sub

    Private Sub AddFlow1_AfterMove(ByVal sender As Object, ByVal e As System.EventArgs) Handles AddFlow1.AfterMove
        Dim route As Lassalle.Flow.Router.RouteFlow = New Lassalle.Flow.Router.RouteFlow
        route.SendStepEvent = m_sendStepEvent
        route.Grain = m_grain
        route.MinDistance = m_minDistance
        route.StartingMethod = m_startingMethod

        For Each item As Item In AddFlow1.SelectedItems
            If TypeOf (item) Is Node Then
                Dim node As Node = item
                Dim link As Link
                For Each link In node.OutLinks
                    route.FindRoute(link)
                Next
            End If
        Next
    End Sub

    Private Sub MenuItemRouteParameters_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemRouteParameters.Click
        Dim rp As RouteParameters = New RouteParameters
        rp.SendStepEvent = m_sendStepEvent
        rp.Grain = m_grain
        rp.MinDistance = m_minDistance
        rp.StartingMethod = m_startingMethod
        If rp.ShowDialog() = Windows.Forms.DialogResult.OK Then
            m_sendStepEvent = rp.SendStepEvent
            m_grain = rp.Grain
            m_minDistance = rp.MinDistance
            m_startingMethod = rp.StartingMethod
        End If
    End Sub
End Class
