'----------------------------------------------------------------------------------------
' AddFlow for WinForms: Shapes sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports System
Imports System.Collections

Public Class Shapes
    Dim shadow As Boolean

    Private Sub Shapes_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.AutoScroll = True
        AddFlow1.PageUnit = GraphicsUnit.Point
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.DefNodeProp.Font = New System.Drawing.Font("Arial", 7.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, CType(0, Byte))
        AddFlow1.DefNodeProp.FillColor = Color.LightYellow
        AddFlow1.DefNodeProp.Shadow.Color = Color.Silver

        shadow = False
        DrawShapes(ShapeOrientation.so_0)
    End Sub

    Private Sub MenuExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuExit.Click
        End
    End Sub

    Private Sub MenuShadow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuShadow.Click
        shadow = Not shadow
        MenuShadow.Checked = shadow

        Dim shadowStyle As ShadowStyle
        If shadow Then
            shadowStyle = shadowStyle.RightBottom
        Else
            shadowStyle = shadowStyle.None
        End If

        For Each node As Node In AddFlow1.Nodes
            node.Shadow.Style = shadowStyle
        Next
    End Sub

    Private Sub MenuOrientation0_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuOrientation0.Click
        SubChangeOrientation(ShapeOrientation.so_0)
    End Sub

    Private Sub MenuOrientation90_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuOrientation90.Click
        SubChangeOrientation(ShapeOrientation.so_90)
    End Sub

    Private Sub MenuOrientation180_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuOrientation180.Click
        SubChangeOrientation(ShapeOrientation.so_180)
    End Sub

    Private Sub MenuOrientation270_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuOrientation270.Click
        SubChangeOrientation(ShapeOrientation.so_270)
    End Sub

    Private Sub MenuAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuAbout.Click
        MsgBox("AddFlow for WinForms Shapes Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub DrawShapes(ByVal orientation As ShapeOrientation)
        AddFlow1.Nodes.Clear()
        Dim i As Integer = 0
        For Each shapeStyle As ShapeStyle In System.Enum.GetValues(shapeStyle.GetType())
            Dim x As Single = 5 + (95 * (i Mod 7))
            Dim y As Single = 5 + (95 * Int(i / 7))
            Dim node As Node = New Node(x, y, 60, 75, AddFlow1.DefNodeProp)
            node.Shape.Style = shapeStyle
            node.Shape.Orientation = orientation
            node.Text = shapeStyle.ToString
            AddFlow1.Nodes.Add(node)
            i = i + 1
        Next
    End Sub

    Private Sub SubChangeOrientation(ByVal orientation As ShapeOrientation)
        For Each node As Node In AddFlow1.Nodes
            node.Shape.Orientation = orientation
        Next
    End Sub
End Class
