'----------------------------------------------------------------------------------------
' AddFlow for WinForms: Stress sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports System.IO

Public Class Stress
    Const DefaultMaxNodes As Integer = 5000
    Const XMax As Single = 3000
    Const YMax As Single = 3000

    Dim FlagGo As Boolean

    Private Sub Stress_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        With AddFlow1
            .PageUnit = GraphicsUnit.Point
            .Dock = DockStyle.Fill
            .BackColor = SystemColors.Window
            .AutoScroll = True
            .CanUndoRedo = False
            .InteractiveEventsOnly = True
            .LinkCreationMode = LinkCreationMode.MiddleHandle
            .DefLinkProp.ArrowDst = Arrow.None
            .DefLinkProp.DrawColor = System.Drawing.Color.Blue
            .DefLinkProp.Jump = Jump.None
            .DefNodeProp.DrawColor = System.Drawing.Color.Blue
        End With

        Panel2.Dock = DockStyle.Bottom

        Randomize()
        Text1.Text = DefaultMaxNodes
        FlagGo = False

        CommandGo.Enabled = True
        CommandStop.Enabled = False
    End Sub

    Private Sub Exitmenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Exitmenu.Click
        End
    End Sub

    Private Sub AboutMenu_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AboutMenu.Click
        MsgBox("AddFlow for WinForms: Stress sample" & vbCrLf & "Copyright (c) 2003-2011 Lassalle Technologies")
    End Sub

    Private Sub CommandGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CommandGo.Click
        Dim repaint As Boolean = CheckRepaint.CheckState
        If Not repaint Then
            AddFlow1.BeginUpdate() ' Increases speed dramatically.
        End If

        Dim iMaxNode As Integer = Val(Text1.Text)        ' Maximum nodes number
        Dim iCountNode As Integer = AddFlow1.Nodes.Count ' Current number of created nodes

        FlagGo = True
        Text1.Enabled = False
        CheckRepaint.Enabled = False
        CommandGo.Enabled = False
        CommandStop.Enabled = True
        CommandReset.Enabled = False
        LabelStatus.Text = "Creating nodes and links.."

        ' Create the first node at a random position
        Dim org As Node = New Node(Int(XMax * Rnd()), Int(YMax * Rnd()), 40, 40, AddFlow1.DefNodeProp)
        AddFlow1.Nodes.Add(org)

        Do
            If iCountNode >= iMaxNode Then
                LabelStatus.Text = "Maximum reached: items creation stopped."
                Exit Do
            End If

            If Not FlagGo Then
                LabelStatus.Text = "Items creation stopped by the user."
                Exit Do
            End If

            ' Create a node at a random position
            Dim dst As Node = New Node(Int(XMax * Rnd()), Int(YMax * Rnd()), 40, 40, AddFlow1.DefNodeProp)
            AddFlow1.Nodes.Add(dst)

            ' Create a link 
            Dim link As Link = org.OutLinks.Add(dst)

            ' Node count
            iCountNode = iCountNode + 1
            If iCountNode Mod 100 = 0 Then
                LabelNodeCount.Text = Str(iCountNode)
                ' Allow user interaction during the process
                System.Windows.Forms.Application.DoEvents()
            End If

            ' The current destination node will be the origin for the next created link.
            org = dst
        Loop

        ' Repaint the diagram
        If Not repaint Then
            AddFlow1.EndUpdate()
        End If

        FlagGo = False
        CommandGo.Enabled = True
        CommandStop.Enabled = False
        CommandReset.Enabled = True
        CheckRepaint.Enabled = True
        Text1.Enabled = True
    End Sub

    Private Sub CommandStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CommandStop.Click
        FlagGo = False
    End Sub

    Private Sub CommandReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CommandReset.Click
        LabelStatus.Text = "Deletes the diagram."
        Cursor = System.Windows.Forms.Cursors.WaitCursor
        LabelStatus.Refresh()
        AddFlow1.Nodes.Clear()
        Cursor = System.Windows.Forms.Cursors.Default
        LabelStatus.Text = ""
        LabelNodeCount.Text = "0"
    End Sub
End Class
