'---------------------------------------------------------------------------------------
' AddFlow for WinForms: Tables2 sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'---------------------------------------------------------------------------------------

Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms
Imports Lassalle.Flow

Public Class Tables2
    Const fieldHeight As Integer = 20

    Private strProgName As String
    Private strFileName As String

    Private Sub Tables2_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BackColor = Color.White
        AddFlow1.AutoScroll = True
        AddFlow1.PageUnit = GraphicsUnit.Pixel
        AddFlow1.IsUnselectableItemHitable = True
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.FillColor = Color.LightYellow
        AddFlow1.DefNodeProp.Dock = DockStyle.Top
        AddFlow1.DefNodeProp.InLinkable = False
        AddFlow1.DefNodeProp.DrawWidth = 2
        AddFlow1.DefNodeProp.Alignment = Alignment.CenterTOP
        AddFlow1.DefNodeProp.LabelEdit = False
        AddFlow1.DefNodeProp.OutLinkable = False
        AddFlow1.DefNodeProp.HighlightChildren = False
        AddFlow1.DefLinkProp.Line.Style = LineStyle.Database

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()

        DisplayDiagramExample()
    End Sub

    Private Sub DisplayDiagramExample()
        Dim tableNode1 As Node = AddTable("Table1", 33, 20, 100, 133)
        Dim tableNode2 As Node = AddTable("Table2", 367, 33, 100, 113)
        Dim tableNode3 As Node = AddTable("Table3", 200, 167, 100, 133)
        Dim tableNode4 As Node = AddTable("Table4", 533, 267, 100, 153)
        Dim tableNode5 As Node = AddTable("Table5", 733, 67, 100, 134)

        ' Add fields in each table
        AddField(tableNode1, "field11")
        AddField(tableNode1, "field12")
        AddField(tableNode1, "field13")
        AddField(tableNode1, "field14")
        AddField(tableNode1, "field15")

        AddField(tableNode2, "field21")
        AddField(tableNode2, "field22")
        AddField(tableNode2, "field23")
        AddField(tableNode2, "field24")

        AddField(tableNode3, "field31")
        AddField(tableNode3, "field32")
        AddField(tableNode3, "field33")
        AddField(tableNode3, "field34")
        AddField(tableNode3, "field35")

        AddField(tableNode4, "field41")
        AddField(tableNode4, "field42")
        AddField(tableNode4, "field43")
        AddField(tableNode4, "field44")
        AddField(tableNode4, "field45")
        AddField(tableNode4, "field46")

        AddField(tableNode5, "field51")
        AddField(tableNode5, "field52")
        AddField(tableNode5, "field53")
        AddField(tableNode5, "field54")
        AddField(tableNode5, "field55")

        ' Create some relations between fields
        Dim link As Link

        link = tableNode1.Children(1).OutLinks.Add(tableNode3.Children(4))
        link.ArrowDst.Style = ArrowStyle.Many
        link.ArrowDst.Angle = ArrowAngle.deg30

        link = tableNode1.Children(4).OutLinks.Add(tableNode2.Children(2))
        link.ArrowDst.Style = ArrowStyle.Many
        link.ArrowDst.Angle = ArrowAngle.deg30

        link = tableNode2.Children(4).OutLinks.Add(tableNode4.Children(6))
        link.ArrowDst.Style = ArrowStyle.None

        link = tableNode2.Children(3).OutLinks.Add(tableNode3.Children(1))
        link.ArrowDst.Style = ArrowStyle.Many
        link.ArrowDst.Angle = ArrowAngle.deg30

        link = tableNode1.Children(2).OutLinks.Add(tableNode4.Children(4))
        link.ArrowDst.Style = ArrowStyle.None

        link = tableNode4.Children(4).OutLinks.Add(tableNode5.Children(1))
        link.ArrowDst.Style = ArrowStyle.None

        AddFlow1.ResetUndoRedo()
    End Sub

    Private Sub Tables2_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub
        strFileName = ""
        AddFlow1.Nodes.Clear()
        AddFlow1.ResetUndoRedo()
        AddFlow1.SetChangedFlag(False)
        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        MenuItemEditUndo.Enabled = AddFlow1.CanUndo
        MenuItemEditRedo.Enabled = AddFlow1.CanRedo
        MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count <> 0)
        MenuItemEditCut.Enabled = (AddFlow1.SelectedItems.Count > 0)
        MenuItemEditCopy.Enabled = (AddFlow1.SelectedItems.Count > 0)
        MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
        MenuItemEditDelete.Enabled = (AddFlow1.SelectedItems.Count > 0)
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        ' We want to copy the children of each selected node or link
        ' Therefore we copy them in an array, then we select make them selectable 
        ' and we select them.
        ' As we modify the Selectable property of children nodes and we don't want to  
        ' register these actions in the undo/redo list, we set the SkipUndo property to true.
        AddFlow1.SkipUndo = True
        Dim aItems As ArrayList = New ArrayList()
        For Each item As Item In AddFlow1.SelectedItems
            For Each child As Node In item.Children
                aItems.Add(child)
            Next
        Next
        For Each item As Item In aItems
            item.Selectable = True
            item.Selected = True
        Next

        ' Copy
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)

        ' Deselect the children and make them unselectable again
        For Each item As Item In aItems
            item.Selected = False
            Dim node As Node = item
            If IsHeader(node) Then
                item.Selectable = False
            End If
        Next
        AddFlow1.SkipUndo = False
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            Dim aItems As ArrayList = New ArrayList()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    If Not IsTableNode(node) Then
                        aItems.Add(node)
                    End If
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next

            For Each item As Item In aItems
                item.Selected = False
                Dim node As Node = item
                If IsHeader(node) Then
                    item.Selectable = False
                End If
            Next

            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        For Each item As Item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms: Tables2 sample" & vbCrLf & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub AddFlow1_BeforeAddLink(ByVal sender As Object, ByVal e As Lassalle.Flow.BeforeAddLinkEventArgs) Handles AddFlow1.BeforeAddLink
        ' If the origin and the destination nodes belong to the same table then
        ' cancel this link creation. 
        If IsSameTable(e.Org, e.Dst) Then
            e.Cancel.Cancel = True
        End If
    End Sub

    Private Sub AddFlow1_BeforeAddNode(ByVal sender As Object, ByVal e As Lassalle.Flow.BeforeAddNodeEventArgs) Handles AddFlow1.BeforeAddNode
        e.Cancel.Cancel = True
        AddFlow1.BeginAction(1005)
        AddTable("", e.Rect.Left, e.Rect.Top, e.Rect.Width, e.Rect.Height)
        AddFlow1.EndAction()
    End Sub

    Private Sub AddFlow1_AfterResize(ByVal sender As Object, ByVal e As System.EventArgs) Handles AddFlow1.AfterResize
        Dim item As Item = AddFlow1.SelectedItem
        If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
            Dim node As Node = item
            If IsTableNode(node) Then
                AddFlow1.AddToLastAction()

                ' We are resizing a "table" node (field nodes are not resizable)
                ' First, we ensure that the height of this node is greater
                ' than the sum of the heights of its "field" nodes.
                If node.Rect.Height < node.Children.Count * fieldHeight Then
                    Dim rect As RectangleF = New RectangleF(node.Rect.Location, node.Rect.Size)
                    rect.Height = node.Children.Count * fieldHeight + fieldHeight / 2
                    node.Rect = rect
                End If

                AddFlow1.EndAction()
            End If
        End If
    End Sub

    Private Sub AddFlow1_ReadXMLNodeExtraData(ByVal sender As Object, ByVal e As Lassalle.Flow.ReadXMLNodeExtraDataEventArgs) Handles AddFlow1.ReadXMLNodeExtraData
        If (e.Reader.Name = "Type") Then
            e.Node.Properties("Type").Value = e.Reader.ReadElementContentAsString()
        End If
    End Sub

    Private Sub AddFlow1_WriteXMLNodeExtraData(ByVal sender As Object, ByVal e As Lassalle.Flow.WriteXMLNodeExtraDataEventArgs) Handles AddFlow1.WriteXMLNodeExtraData
        e.Writer.WriteElementString("Type", e.Node.Properties("Type").Value)
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Private Function AddTable(ByRef header As String, ByRef x As Single, ByRef y As Single, ByRef cx As Single, ByRef cy As Single) As Node
        Dim tableNode As Node = New Node(x, y, cx, cy, AddFlow1.DefNodeProp)
        tableNode.Properties("Type").Value = "Table"
        AddFlow1.Nodes.Add(tableNode)
        AddHeader(tableNode, header)
        Return tableNode
    End Function

    Private Function AddHeader(ByRef tableNode As Node, ByRef headerName As String) As Node
        ' The location and width given in the following line are not important
        ' since the node will be docked at the top of the parent node.
        Dim headerNode As Node = New Node(0, 0, 20, fieldHeight, headerName, AddFlow1.DefNodeProp)
        headerNode.Tooltip = headerName
        headerNode.FillColor = Color.LightGreen
        headerNode.DrawColor = Color.Transparent
        headerNode.Selectable = False
        headerNode.Logical = False
        headerNode.LabelEdit = False
        headerNode.Properties("Type").Value = "Header"
        AddFlow1.Nodes.Add(headerNode)
        headerNode.Parent = tableNode
        Return headerNode
    End Function

    Private Function AddField(ByRef tableNode As Node, ByRef fieldName As String) As Node
        Dim fieldNode As Node = New Node(0, 0, 20, 20, fieldName, AddFlow1.DefNodeProp)
        fieldNode.DrawColor = Color.Transparent
        fieldNode.Alignment = Alignment.LeftJustifyMIDDLE ' The field text is left justified
        fieldNode.Logical = False
        fieldNode.InLinkable = True
        fieldNode.OutLinkable = True
        fieldNode.LabelEdit = False
        fieldNode.Properties("Type").Value = "Field"
        AddFlow1.Nodes.Add(fieldNode)
        fieldNode.Parent = tableNode
        Return fieldNode
    End Function

    Private Function IsTableNode(ByRef node As Node) As Boolean
        Return (node.Properties("Type").Value = "Table")
    End Function

    Private Function IsField(ByRef node As Node) As Boolean
        Return (node.Properties("Type").Value = "Field")
    End Function

    Private Function IsHeader(ByRef node As Node) As Boolean
        Return (node.Properties("Type").Value = "Header")
    End Function

    Private Function GetHeader(ByRef node As Node) As Node
        If node.Children.Count > 0 And IsHeader(node.Children(0)) Then
            Return node.Children(0)
        Else
            Return Nothing
        End If
    End Function

    Private Function IsSameTable(ByVal node1 As Node, ByVal node2 As Node) As Boolean
        Return (node1.Parent Is node2.Parent And Not (node2.Parent Is Nothing))
    End Function

    Private Sub AddFlow1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles AddFlow1.MouseDown
        If e.Button = MouseButtons.Right Then
            If Not AddFlow1.ContextMenu Is Nothing Then
                AddFlow1.ContextMenu.Dispose()
            End If
            If Not AddFlow1.PointedItem Is Nothing Then
                If TypeOf (AddFlow1.PointedItem) Is Lassalle.Flow.Node Then
                    Dim node As Node = AddFlow1.PointedItem
                    If IsTableNode(node) Then
                        AddFlow1.ContextMenu = New ContextMenu()

                        Dim eh1 As EventHandler = New EventHandler(AddressOf OnClickHeader)
                        Dim menuItemHeader As MenuItem = New MenuItem("Header...", eh1)
                        AddFlow1.ContextMenu.MenuItems.Add(menuItemHeader)

                        Dim eh2 As EventHandler = New EventHandler(AddressOf OnClickAddField)
                        Dim menuItemAddField As MenuItem = New MenuItem("Add Field...", eh2)
                        AddFlow1.ContextMenu.MenuItems.Add(menuItemAddField)

                        AddFlow1.ContextMenu.Show(Me, New System.Drawing.Point(e.X, e.Y))
                    End If
                End If
            End If
        Else
            If Not (AddFlow1.PointedItem Is Nothing) Then
                If TypeOf (AddFlow1.PointedItem) Is Lassalle.Flow.Node Then
                    Dim node As Node = AddFlow1.PointedItem
                    ' If it is a header node, then it is unselectable. 
                    ' However, in such a case, we select its parent Table node.
                    If IsHeader(node) Then
                        Dim tableNode As Node = node.Parent
                        If Not (tableNode Is Nothing) Then
                            tableNode.Selected = True
                        End If
                    End If
                End If
            End If
        End If
    End Sub

    Private Sub OnClickHeader(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim header As Header = New Header()
        If header.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
            Dim headername As String = header.textBoxHeader.Text
            Dim item As Item = AddFlow1.PointedItem
            If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
                Dim node As Node = item
                Dim headerNode As Node = GetHeader(node)
                headerNode.Text = headername
            End If
        End If
    End Sub

    Private Sub OnClickAddField(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim field As Field = New Field()
        If field.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
            Dim fieldname As String = field.textBoxField.Text
            Dim item As Item = AddFlow1.PointedItem
            If (Not (item Is Nothing) And TypeOf (item) Is Node) Then
                Dim node As Node = item
                AddField(node, fieldname)
            End If
        End If
    End Sub
End Class
