'----------------------------------------------------------------------------------------
' AddFlow for WinForms: TreeEdit sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports Lassalle.Flow
Imports Lassalle.Flow.Layout.Tree
Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D

Public Class TreeEdit
    Const strFilter As String = "TreeEdit Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
    Const strExt As String = "*.xml"
    Private prnflow As Lassalle.PrnFlow.PrnFlow
    Private strProgName As String
    Private strFileName As String

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        StatusBar1.Dock = DockStyle.Bottom

        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.AutoScroll = True
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.CanReflexLink = False
        AddFlow1.CanStretchLink = False
        AddFlow1.CanMultiLink = False
        AddFlow1.CycleMode = CycleMode.NoCycle
        AddFlow1.Grid.Draw = True
        AddFlow1.Grid.Snap = True
        AddFlow1.Grid.Style = GridStyle.DottedLines
        AddFlow1.Grid.Color = Color.Silver
        AddFlow1.CanFireError = True
        AddFlow1.DefLinkProp.Font = New System.Drawing.Font("Verdana", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, CType(0, Byte))
        AddFlow1.DefNodeProp.Font = New System.Drawing.Font("Verdana", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World, CType(0, Byte))
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.FillColor = Color.LightYellow
        AddFlow1.DefNodeProp.Shadow.Style = ShadowStyle.RightBottom
        AddFlow1.DefNodeProp.Shadow.Color = Color.Silver
        AddFlow1.DefLinkProp.Rigid = True
        AddFlow1.DefLinkProp.ArrowDst.Style = ArrowStyle.None
        AddFlow1.DefLinkProp.EndCap = LineCap.Custom
        AddFlow1.DefLinkProp.CustomEndCap = New AdjustableArrowCap(3, 6, True)
        AddFlow1.DefLinkProp.DrawColor = Color.Blue
        AddFlow1.DefLinkProp.Jump = Jump.None

        prnflow = New Lassalle.PrnFlow.PrnFlow

        Me.ToolBarButtonFileNew.Tag = "New"
        Me.ToolBarButtonFileOpen.Tag = "Open"
        Me.ToolBarButtonFileSave.Tag = "Save"
        Me.ToolBarButtonFilePrint.Tag = "Print"
        Me.ToolBarButtonFilePreview.Tag = "Preview"
        Me.ToolBarButtonEditCut.Tag = "Cut"
        Me.ToolBarButtonEditCopy.Tag = "Copy"
        Me.ToolBarButtonEditPaste.Tag = "Paste"
        Me.ToolBarButtonEditUndo.Tag = "Undo"
        Me.ToolBarButtonEditRedo.Tag = "Redo"
        Me.ToolBarButtonNorth.Tag = "North"
        Me.ToolBarButtonEast.Tag = "East"
        Me.ToolBarButtonSouth.Tag = "South"
        Me.ToolBarButtonWest.Tag = "West"
        Me.ToolBarButtonRadial.Tag = "Radial"

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub AddFlow1_BeforeAddLink(ByVal sender As Object, ByVal e As Lassalle.Flow.BeforeAddLinkEventArgs) Handles AddFlow1.BeforeAddLink
        If e.Dst.InLinks.Count > 0 Then
            e.Cancel = New System.ComponentModel.CancelEventArgs(True)
            MsgBox("The graph must be composed of a tree or several trees!")
        End If
    End Sub

    Private Sub AddFlow1_Error(ByVal sender As Object, ByVal e As Lassalle.Flow.ErrorEventArgs) Handles AddFlow1.Error
        If e.ErrorEventType = ErrorEventType.CycleError Then
            MsgBox("The graph must be composed of a tree or several trees!")
        End If
    End Sub

    Private Sub ToolBar1_ButtonClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolBarButtonClickEventArgs) Handles ToolBar1.ButtonClick
        Dim key As String = e.Button.Tag
        Select Case key
            Case "New"
                MenuItemFileNew_Click(sender, e)
            Case "Open"
                MenuItemFileOpen_Click(sender, e)
            Case "Save"
                MenuItemFileSave_Click(sender, e)
            Case "Print"
                MenuItemFilePrint_Click(sender, e)
            Case "Preview"
                MenuItemFilePrintPreview_Click(sender, e)
            Case "Cut"
                MenuItemEditCut_Click(sender, e)
            Case "Copy"
                MenuItemEditCopy_Click(sender, e)
            Case "Paste"
                MenuItemEditPaste_Click(sender, e)
            Case "Undo"
                MenuItemEditUndo_Click(sender, e)
            Case "Redo"
                MenuItemEditRedo_Click(sender, e)
            Case "North"
                MenuItemNorth_Click(sender, e)
            Case "East"
                MenuItemEast_Click(sender, e)
            Case "South"
                MenuItemSouth_Click(sender, e)
            Case "West"
                MenuItemWest_Click(sender, e)
            Case "Radial"
                MenuItemRadialTree_Click(sender, e)
        End Select
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub

        strFileName = ""
        With AddFlow1
            .Nodes.Clear()
            .ResetUndoRedo()
            .SetChangedFlag(False)
        End With

        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = strFilter
        ofd.FileName = strExt
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)

            ' The custom line cap used for our links is not stored in the XML file.
            ' Therefore, we assign it to each link.
            Dim item As Item
            For Each item In AddFlow1.Items
                If TypeOf (item) Is Link Then
                    Dim link As Link = item
                    If Not (AddFlow1.DefLinkProp.CustomEndCap Is Nothing) Then
                        link.CustomEndCap = CType(AddFlow1.DefLinkProp.CustomEndCap.Clone(), CustomLineCap)
                    End If
                End If
            Next
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuItemFilePrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePrint.Click
        prnflow.Print(AddFlow1)
    End Sub

    Private Sub MenuItemFilePageSetup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePageSetup.Click
        prnflow.PageSetup()
    End Sub

    Private Sub MenuItemFilePrintPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePrintPreview.Click
        prnflow.Preview(AddFlow1)
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        Dim isSelectedItems As Boolean = (AddFlow1.SelectedItems.Count > 0)
        MenuItemEditUndo.Enabled = AddFlow1.CanUndo
        MenuItemEditRedo.Enabled = AddFlow1.CanRedo
        MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count <> 0)
        MenuItemEditCut.Enabled = isSelectedItems
        MenuItemEditCopy.Enabled = isSelectedItems
        MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
        MenuItemEditDelete.Enabled = isSelectedItems
        MenuItemEditExportAsMetafile.Enabled = isSelectedItems
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim tw As Xml.XmlTextWriter
        tw = New Xml.XmlTextWriter(stream, Nothing)
        Lassalle.XMLFlow.Serial.FlowToXML(tw, AddFlow1, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            Dim tr As New Xml.XmlTextReader(stream)

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)
            Lassalle.XMLFlow.Serial.XMLToFlow(tr, AddFlow1, True, False)
            AddFlow1.EndAction()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
        End If
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.DeleteSel()
    End Sub

    Private Sub MenuItemEditExportAsMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditExportAsMetafile.Click
        Dim mf As Metafile = AddFlow1.ExportMetafile(True, False, False)
        ' The following line does not work. 
        ' Clipboard.SetDataObject(mf, True)
        ' Therefore we have replaced it by the next which copy the metafile to the
        ' clipboard, using the old GDI Metafile format instead of the new GDI + format.
        ' As a result, the antialiasing effect is lost.
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, mf)
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        SelectAll(AddFlow1)
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms Tree Editor Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = strExt
        End If
        sfd.Filter = strFilter
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub MenuFileSafeAsImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuFileSafeAsImage.Click
        Dim sfd As SaveFileDialog = New SaveFileDialog
        sfd.Title = "Export as Image"
        sfd.DefaultExt = ".wmf"
        sfd.AddExtension = True
        sfd.Filter = "Bitmap format(*.bmp)|*.bmp|Enhanced Windows metafile format (*.emf)|*.emf|Exchangeable Image File (*exif)|*.exif|Graphics Interchange Format (*.gif)|*.gif|Windows icon format (*.ico)|*.ico|Joint Photographic Experts Group format(*.jpeg)|*.jpeg|W3C Portable Network Graphics format (*png)|*png|Tag Image File Format (*tiff)|*tiff|Windows metafile format (*.wmf)|*.wmf"

        If sfd.ShowDialog = Windows.Forms.DialogResult.OK Then
            Dim mf As Metafile = AddFlow1.ExportMetafile(False, True, False)
            Dim imf As ImageFormat = ImageFormat.Bmp

            Select Case sfd.FilterIndex
                Case 1
                    imf = ImageFormat.Bmp
                Case 2
                    imf = ImageFormat.Emf
                Case 3
                    imf = ImageFormat.Exif
                Case 4
                    imf = ImageFormat.Gif
                Case 5
                    imf = ImageFormat.Icon
                Case 6
                    imf = ImageFormat.Jpeg
                Case 7
                    imf = ImageFormat.Png
                Case 8
                    imf = ImageFormat.Tiff
                Case 9
                    imf = ImageFormat.Wmf
            End Select
            mf.Save(sfd.FileName, imf)
        End If
    End Sub

    Private Sub MenuItemNorth_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemNorth.Click
        TreeLayout(AddFlow1, DrawingStyle.Layered, Orientation.North)
    End Sub

    Private Sub MenuItemEast_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEast.Click
        TreeLayout(AddFlow1, DrawingStyle.Layered, Orientation.East)
    End Sub

    Private Sub MenuItemSouth_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemSouth.Click
        TreeLayout(AddFlow1, DrawingStyle.Layered, Orientation.South)
    End Sub

    Private Sub MenuItemWest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemWest.Click
        TreeLayout(AddFlow1, DrawingStyle.Layered, Orientation.West)
    End Sub

    Private Sub MenuItemRadialTree_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemRadialTree.Click
        TreeLayout(AddFlow1, DrawingStyle.Radial, Orientation.North)
    End Sub

    Private Sub MenuItemOrthogonal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemOrthogonal.Click
        ChangeLinkStyles(AddFlow1, LineStyle.VHV)
    End Sub

    Private Sub MenuItemDirectLine_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemDirectLine.Click
        ChangeLinkStyles(AddFlow1, LineStyle.Polyline)
    End Sub

    Private Sub MenuItemCurve_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemCurve.Click
        ChangeLinkStyles(AddFlow1, LineStyle.Bezier)
    End Sub

    Private Sub ChangeLinkStyles(ByVal AddFlow1 As AddFlow, ByVal ls As LineStyle)
        ' Change the link style of each existing link
        For Each item As Item In AddFlow1.Items
            If TypeOf (item) Is Link Then
                Dim link As Link = item
                link.Line.Style = ls
                If ls = LineStyle.Polyline Then
                    link.Points.Clear()
                End If
            End If
        Next
        ' Change the link style for the next created links
        AddFlow1.DefLinkProp.Line.Style = ls
    End Sub

    Private Sub LoadFile(ByVal strFileName As String)
        Dim stream As New System.IO.FileStream(strFileName, System.IO.FileMode.Open)
        Dim tr As Xml.XmlTextReader
        tr = New Xml.XmlTextReader(stream)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Lassalle.XMLFlow.Serial.XMLToFlow(tr, AddFlow1, False, False, False)
        AddFlow1.SetChangedFlag(False)
        stream.Close()
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Dim stream As New System.IO.FileStream(strFileName, System.IO.FileMode.Create)
        Dim tw As Xml.XmlTextWriter
        tw = New Xml.XmlTextWriter(stream, Nothing)
        Lassalle.XMLFlow.Serial.FlowToXML(tw, AddFlow1)
        stream.Close()
        AddFlow1.SetChangedFlag(False)
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If

        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)

        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Private Sub TreeLayout(ByVal AddFlow1 As AddFlow, ByVal ds As DrawingStyle, ByVal o As Orientation)
        If AddFlow1.Nodes.Count = 0 Then Exit Sub

        ' So that the user will be able to undo the layout in one time
        AddFlow1.BeginAction(1005)

        ' Obtain the maximum node width and height
        ' These values will be used to set the values of the LayerDistance and VertexDistance properties
        Dim node As Node
        Dim maxNodeWidth As Single = 0
        Dim maxNodeHeight As Single = 0
        For Each node In AddFlow1.Nodes
            maxNodeWidth = Math.Max(maxNodeWidth, node.Size.Width)
            maxNodeHeight = Math.Max(maxNodeHeight, node.Size.Height)
        Next

        ' Create a TFlow object and set its properties
        Dim tflow As TFlow = New TFlow
        tflow.DrawingStyle = ds
        If tflow.DrawingStyle = DrawingStyle.Radial Then
            ' (Here, the VerteDistance and Orientation properties are ignored)
            tflow.LayerDistance = 3 * maxNodeHeight
        Else
            If o = Orientation.North Or o = Orientation.South Then
                tflow.LayerDistance = maxNodeHeight
                tflow.VertexDistance = maxNodeWidth / 2
            Else
                tflow.LayerDistance = maxNodeWidth
                tflow.VertexDistance = maxNodeHeight / 2
            End If
            tflow.Orientation = o
        End If

        ' Execute the layout
        On Error Resume Next
        tflow.Layout(AddFlow1)
        If Err.Number <> 0 Then MessageBox.Show(Err.Description)

        ' Terminate the action for the undo processor
        AddFlow1.EndAction()
    End Sub

    Private Sub SelectAll(ByVal AddFlow1 As AddFlow)
        Dim item As Item
        For Each item In AddFlow1.Items
            item.Selected = True
        Next
    End Sub
End Class

Public Class ClipboardMetafileHelper
    <DllImport("user32.dll", EntryPoint:="OpenClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function OpenClipboard(ByVal hWnd As IntPtr) As Boolean
    End Function
    <DllImport("user32.dll", EntryPoint:="EmptyClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function EmptyClipboard() As Boolean
    End Function
    <DllImport("user32.dll", EntryPoint:="SetClipboardData", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function SetClipboardData(ByVal uFormat As Integer, ByVal hWnd As IntPtr) As IntPtr
    End Function
    <DllImport("user32.dll", EntryPoint:="CloseClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function CloseClipboard() As Boolean
    End Function
    <DllImport("gdi32.dll", EntryPoint:="CopyEnhMetaFileA", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function CopyEnhMetaFile(ByVal hemfSrc As IntPtr, ByVal hNULL As IntPtr) As IntPtr
    End Function
    <DllImport("gdi32.dll", EntryPoint:="DeleteEnhMetaFile", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function DeleteEnhMetaFile(ByVal hemfSrc As IntPtr) As Boolean
    End Function

    ' Metafile mf is set to an invalid state inside this function
    Public Shared Function PutEnhMetafileOnClipboard(ByVal hWnd As IntPtr, ByVal mf As Metafile) As Boolean
        Dim bResult As New Boolean
        bResult = False
        Dim hEMF, hEMF2 As IntPtr
        hEMF = mf.GetHenhmetafile() ' invalidates mf
        If Not hEMF.Equals(New IntPtr(0)) Then
            hEMF2 = CopyEnhMetaFile(hEMF, New IntPtr(0))
            If Not hEMF2.Equals(New IntPtr(0)) Then
                If OpenClipboard(hWnd) Then
                    If EmptyClipboard() Then
                        Dim hRes As IntPtr
                        hRes = SetClipboardData(14, hEMF2) ' 14 == CF_ENHMETAFILE()
                        bResult = hRes.Equals(hEMF2)
                        CloseClipboard()
                    End If
                End If
            End If
            DeleteEnhMetaFile(hEMF)
        End If
        Return bResult
    End Function
End Class
