'----------------------------------------------------------------------------------------
' AddFlow for WinForms: AfEdit sample
' Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
' http://www.lassalle.com
' Author: Patrick Lassalle    
' THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
'----------------------------------------------------------------------------------------

Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports System.Drawing.Drawing2D
Imports Lassalle.Flow

Public Class afEdit
    Private prnflow As Lassalle.PrnFlow.PrnFlow
    Private strProgName As String
    Private strFileName As String
    Private pagesize As SizeF

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        StatusBar1.Dock = DockStyle.Bottom
        SplitContainer1.Dock = DockStyle.Fill

        AddFlow1.Dock = DockStyle.Fill
        AddFlow1.BorderStyle = BorderStyle.FixedSingle
        AddFlow1.BackColor = SystemColors.Window
        AddFlow1.Grid.Draw = True
        AddFlow1.Grid.Snap = True
        AddFlow1.Grid.Style = GridStyle.DottedLines
        AddFlow1.Grid.Size = New Size(20, 20)
        AddFlow1.Grid.Color = Color.Silver
        AddFlow1.CanFireError = True
        AddFlow1.InteractiveEventsOnly = False
        AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Rectangle
        AddFlow1.DefNodeProp.Font = New Font("Arial", 8, GraphicsUnit.Pixel)
        AddFlow1.AutoScroll = True
        AddFlow1.DefLinkProp.Line.RoundedCorner = True
        AddFlow1.DefLinkProp.Jump = Jump.Arc

        ' Get the printer page size
        prnflow = New Lassalle.PrnFlow.PrnFlow
        pagesize = prnflow.GetPageSize(AddFlow1)

        ' Display a printer page grid
        AddFlow1.PageGrid.Size = New Size(pagesize.Width, pagesize.Height)
        AddFlow1.PageGrid.Color = Color.Chocolate
        AddFlow1.PageGrid.Style = GridStyle.DottedLines
        AddFlow1.PageGrid.Draw = True

        Me.ToolBarButtonFileNew.Tag = "New"
        Me.ToolBarButtonFileOpen.Tag = "Open"
        Me.ToolBarButtonFileSave.Tag = "Save"
        Me.ToolBarButtonFilePrint.Tag = "Print"
        Me.ToolBarButtonFilePreview.Tag = "Preview"
        Me.ToolBarButtonEditCut.Tag = "Cut"
        Me.ToolBarButtonEditCopy.Tag = "Copy"
        Me.ToolBarButtonEditPaste.Tag = "Paste"
        Me.ToolBarButtonEditUndo.Tag = "Undo"
        Me.ToolBarButtonEditRedo.Tag = "Redo"

        PropertyGrid1.SelectedObject = AddFlow1
        PropertyGrid1.CommandsVisibleIfAvailable = True

        Label1.TextAlign = ContentAlignment.MiddleCenter
        Label1.Text = "AddFlow control"
        Label1.Font = New System.Drawing.Font("Arial", 12.0!, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, CType(0, Byte))

        strProgName = Me.Text
        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        If Not OkToTrash() Then e.Cancel = True
    End Sub

    Private Sub ToolBar1_ButtonClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolBarButtonClickEventArgs) Handles ToolBar1.ButtonClick
        Dim key As String = e.Button.Tag
        Select Case key
            Case "New"
                MenuItemFileNew_Click(sender, e)
            Case "Open"
                MenuItemFileOpen_Click(sender, e)
            Case "Save"
                MenuItemFileSave_Click(sender, e)
            Case "Print"
                MenuItemFilePrint_Click(sender, e)
            Case "Preview"
                MenuItemFilePrintPreview_Click(sender, e)
            Case "Cut"
                MenuItemEditCut_Click(sender, e)
            Case "Copy"
                MenuItemEditCopy_Click(sender, e)
            Case "Paste"
                MenuItemEditPaste_Click(sender, e)
            Case "Undo"
                MenuItemEditUndo_Click(sender, e)
            Case "Redo"
                MenuItemEditRedo_Click(sender, e)
        End Select
    End Sub

    Private Sub MenuItemFileNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileNew.Click
        If Not OkToTrash() Then Exit Sub

        Dim interactiveEventsOnly As Boolean = AddFlow1.InteractiveEventsOnly
        Dim canUndoRedo As Boolean = AddFlow1.CanUndoRedo
        Dim sendSelectionChangeEvent As Boolean = AddFlow1.SendSelectionChangeEvent

        AddFlow1.InteractiveEventsOnly = True
        AddFlow1.SendSelectionChangeEvent = False
        AddFlow1.CanUndoRedo = False

        AddFlow1.Nodes.Clear()
        AddFlow1.ResetUndoRedo()
        AddFlow1.SetChangedFlag(False)

        AddFlow1.CanUndoRedo = canUndoRedo
        AddFlow1.SendSelectionChangeEvent = sendSelectionChangeEvent
        AddFlow1.InteractiveEventsOnly = interactiveEventsOnly

        strFileName = ""
        MakeCaption()
    End Sub

    Private Sub MenuItemFileOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileOpen.Click
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = Application.StartupPath
        ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        ofd.FileName = "*.xml"
        If ofd.ShowDialog = Windows.Forms.DialogResult.OK Then
            LoadFile(ofd.FileName)
        End If
    End Sub

    Private Sub MenuItemFileSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSave.Click
        If strFileName.Length = 0 Then
            SaveFileDlg()
        Else
            SaveFile()
        End If
    End Sub

    Private Sub MenuItemFileSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileSaveAs.Click
        SaveFileDlg()
    End Sub

    Private Sub MenuFileSafeAsImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuFileSafeAsImage.Click
        Dim sfd As SaveFileDialog = New SaveFileDialog
        sfd.Title = "Export as Image"
        sfd.DefaultExt = ".wmf"
        sfd.AddExtension = True
        sfd.Filter = "Bitmap format(*.bmp)|*.bmp|Enhanced Windows metafile format (*.emf)|*.emf|Exchangeable Image File (*exif)|*.exif|Graphics Interchange Format (*.gif)|*.gif|Windows icon format (*.ico)|*.ico|Joint Photographic Experts Group format(*.jpeg)|*.jpeg|W3C Portable Network Graphics format (*png)|*png|Tag Image File Format (*tiff)|*tiff|Windows metafile format (*.wmf)|*.wmf"

        If sfd.ShowDialog = Windows.Forms.DialogResult.OK Then
            Dim mf As Metafile = AddFlow1.ExportMetafile(False, True, True)
            Dim imf As ImageFormat = ImageFormat.Bmp

            Select Case sfd.FilterIndex
                Case 1
                    imf = ImageFormat.Bmp
                Case 2
                    imf = ImageFormat.Emf
                Case 3
                    imf = ImageFormat.Exif
                Case 4
                    imf = ImageFormat.Gif
                Case 5
                    imf = ImageFormat.Icon
                Case 6
                    imf = ImageFormat.Jpeg
                Case 7
                    imf = ImageFormat.Png
                Case 8
                    imf = ImageFormat.Tiff
                Case 9
                    imf = ImageFormat.Wmf
            End Select
            mf.Save(sfd.FileName, imf)
        End If
    End Sub

    Private Sub MenuFileSafeAsSVG_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuFileSafeAsSVG.Click
        Dim sfd As SaveFileDialog = New SaveFileDialog
        sfd.Title = "Export as SVG file"
        sfd.DefaultExt = ".svg"
        sfd.AddExtension = True
        sfd.Filter = "SVG file(*.svg)|*.svg"

        If sfd.ShowDialog = Windows.Forms.DialogResult.OK Then
            Dim oDocument As System.Xml.XmlDocument = Lassalle.Flow.SVG.SVGFlow.FlowToSVG(AddFlow1)
            oDocument.Save(sfd.FileName)
            System.Diagnostics.Process.Start(sfd.FileName)
        End If
    End Sub

    Private Sub MenuItemFilePrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePrint.Click
        prnflow.Print(AddFlow1)
    End Sub

    Private Sub MenuItemFilePageSetup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePageSetup.Click
        prnflow.PageSetup()
        pagesize = prnflow.GetPageSize(AddFlow1)
        AddFlow1.Invalidate()
    End Sub

    Private Sub MenuItemFilePrintPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFilePrintPreview.Click
        prnflow.Preview(AddFlow1)
    End Sub

    Private Sub MenuItemFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemFileExit.Click
        End
    End Sub

    Private Sub MenuItemEdit_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEdit.Popup
        Dim isSelectedItems As Boolean = (AddFlow1.SelectedItems.Count > 0)
        Dim item As Item = AddFlow1.SelectedItem
        Dim isCurrentNode As Boolean = Not (item Is Nothing) And TypeOf (item) Is Node
        Dim isCurrentLink As Boolean = Not (item Is Nothing) And TypeOf (item) Is Link

        MenuItemEditUndo.Enabled = AddFlow1.CanUndo
        MenuItemEditRedo.Enabled = AddFlow1.CanRedo
        If MenuItemEditRedo.Enabled Then
            MenuItemEditRedo.Text = "Redo " & ActionName(AddFlow1.RedoCode)
        End If
        If MenuItemEditUndo.Enabled Then
            MenuItemEditUndo.Text = "Undo " & ActionName(AddFlow1.UndoCode)
        End If
        MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count <> 0)
        MenuItemEditCut.Enabled = isSelectedItems
        MenuItemEditCopy.Enabled = isSelectedItems
        MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat")
        MenuItemEditDelete.Enabled = isSelectedItems
        MenuItemEditGroup.Enabled = isSelectedItems
        MenuItemEditUngroup.Enabled = isCurrentNode
        MenuItemEditZOrderBack.Enabled = isCurrentNode Or isCurrentLink
        MenuItemEditZOrderFront.Enabled = isCurrentNode Or isCurrentLink
        MenuItemEditExportAsMetafile.Enabled = isSelectedItems
    End Sub

    Private Sub MenuItemEditUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUndo.Click
        AddFlow1.Undo()
    End Sub

    Private Sub MenuItemEditRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditRedo.Click
        AddFlow1.Redo()
    End Sub

    Private Sub MenuItemEditCopy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCopy.Click
        Dim stream As New System.IO.MemoryStream
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(stream, Nothing)
        AddFlow1.WriteXml(writer, True, False)
        Dim data As New DataObject
        data.SetData("AddFlow.XMLFormat", stream)
        Clipboard.SetDataObject(data, True)
    End Sub

    Private Sub MenuItemEditPaste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditPaste.Click
        Dim data As IDataObject = Clipboard.GetDataObject()
        If data.GetDataPresent("AddFlow.XMLFormat") Then
            ' We first unselect the selected items
            AddFlow1.SelectedItems.Clear()

            ' Get the data from the clipboard
            Dim stream As New System.IO.MemoryStream
            stream = data.GetData("AddFlow.XMLFormat")

            ' We paste the portion of the diagram in our AddFlow control.
            ' We group all the load actions in only one action so that we could undo
            ' the paste action in one time.
            AddFlow1.BeginAction(1003)

            Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(stream)
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
            AddFlow1.ReadXml(reader, True, False, False)
            reader.Close()

            ' We move a little each pasted node and link so that they do not recover
            ' the original items.
            Dim dx As Single = AddFlow1.Grid.Size.Width
            Dim dy As Single = AddFlow1.Grid.Size.Height
            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf (item) Is Node Then
                    Dim node As Node = item
                    node.Location = New PointF(node.Location.X + dx, node.Location.Y + dy)
                Else
                    Dim pt As PointF
                    Dim k As Integer
                    Dim link As Link = item

                    If link.AdjustOrg Then
                        pt = link.Points(0)
                        link.Points(0) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                    For k = 1 To link.Points.Count - 2
                        pt = link.Points(k)
                        link.Points(k) = New PointF(pt.X + dx, pt.Y + dy)
                    Next
                    If link.AdjustDst Then
                        pt = link.Points(link.Points.Count - 1)
                        link.Points(link.Points.Count - 1) = New PointF(pt.X + dx, pt.Y + dy)
                    End If
                End If
            Next
            AddFlow1.EndAction()
        End If
    End Sub

    Private Sub MenuItemEditCut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditCut.Click
        MenuItemEditCopy_Click(sender, e)
        MenuItemEditDelete_Click(sender, e)
    End Sub

    Private Sub MenuItemEditDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditDelete.Click
        AddFlow1.SendSelectionChangeEvent = False
        AddFlow1.DeleteSel()
        AddFlow1.SendSelectionChangeEvent = True
        SelectionChangeHandle()
    End Sub

    Private Sub MenuItemEditExportAsMetafile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditExportAsMetafile.Click
        Dim mf As Metafile = AddFlow1.ExportMetafile(True, True, False)
        ' The following line does not work. 
        'Clipboard.SetDataObject(mf, True)
        ' Therefore we have replaced it by the next which copy the metafile to the
        ' clipboard, using the old GDI Metafile format instead of the new GDI + format.
        ' As a result, the antialiasing effect is lost.
        ClipboardMetafileHelper.PutEnhMetafileOnClipboard(Me.Handle, mf)
    End Sub

    Private Sub MenuItemEditSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectAll.Click
        AddFlow1.SendSelectionChangeEvent = False
        For Each item As Item In AddFlow1.Items
            item.Selected = True
        Next
        AddFlow1.SendSelectionChangeEvent = True
        SelectionChangeHandle()
    End Sub

    Private Sub MenuItemEditSelectMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditSelectMode.Click
        If AddFlow1.MouseAction = MouseAction.Selection Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.Selection
        End If
        MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction = MouseAction.Selection)
    End Sub

    Private Sub MenuItemEditZOrderBack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditZOrderBack.Click
        If Not (AddFlow1.SelectedItem Is Nothing) Then
            AddFlow1.SelectedItem.ZOrder = 0
        End If
    End Sub

    Private Sub MenuItemEditZOrderFront_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditZOrderFront.Click
        If Not (AddFlow1.SelectedItem Is Nothing) Then
            AddFlow1.SelectedItem.ZOrder = AddFlow1.Items.Count - 1
        End If
    End Sub

    Private Sub MenuItemEditGroup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditGroup.Click
        Dim leftMin As Single, topMin As Single
        Dim rightMax As Single, bottomMax As Single
        Dim first As Boolean

        first = True
        For Each item As Item In AddFlow1.SelectedItems
            If TypeOf item Is Node Then
                Dim node As Node = item
                Dim rc As RectangleF = node.Rect
                If first Then
                    first = False
                    leftMin = rc.Left
                    topMin = rc.Top
                    rightMax = rc.Left + rc.Width
                    bottomMax = rc.Top + rc.Height
                Else
                    If rc.Left < leftMin Then leftMin = rc.Left
                    If rc.Top < topMin Then topMin = rc.Top
                    If rc.Right > rightMax Then
                        rightMax = rc.Right
                    End If
                    If rc.Bottom > bottomMax Then
                        bottomMax = rc.Bottom
                    End If
                End If
            End If
        Next

        ' We group the nodes only if there are several nodes.
        If Not first Then
            ' Create the "owner" node. Make it not visible, not resizeable
            Dim owner As New Node(leftMin, topMin, rightMax - leftMin, bottomMax - topMin)
            owner.Hidden = True
            owner.XSizeable = False
            owner.YSizeable = False
            owner.Shape.Style = ShapeStyle.Rectangle
            owner.RemoveChildren = False
            AddFlow1.Nodes.Add(owner)

            For Each item As Item In AddFlow1.SelectedItems
                If TypeOf item Is Node Then
                    Dim node As Node = item
                    node.Parent = owner
                End If
            Next

            ' Select the owner node
            AddFlow1.SelectedItem = owner
        End If
    End Sub

    Private Sub MenuItemEditUngroup_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemEditUngroup.Click
        AddFlow1.Nodes.Remove(AddFlow1.SelectedItem)
    End Sub

    Private Sub MenuItemViewZoom100_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoom100.Click
        AddFlow1.Zoom = New Zoom(1, 1)
    End Sub

    Private Sub MenuItemViewZoom150_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoom150.Click
        AddFlow1.Zoom = New Zoom(1.5, 1.5)
    End Sub

    Private Sub MenuItemViewZoom75_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoom75.Click
        AddFlow1.Zoom = New Zoom(0.75, 0.75)
    End Sub

    Private Sub MenuItemViewZoomRectangle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemViewZoomRectangle.Click
        If AddFlow1.MouseAction = MouseAction.ZoomIsotropic Then
            AddFlow1.MouseAction = MouseAction.None
        Else
            AddFlow1.MouseAction = MouseAction.ZoomIsotropic
        End If
        MenuItemViewZoomRectangle.Checked = (AddFlow1.MouseAction = MouseAction.ZoomIsotropic)
    End Sub

    Private Sub MenuItemPropertiesWindow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemPropertiesWindow.Click
        SplitContainer1.Panel2Collapsed = Not SplitContainer1.Panel2Collapsed
        MenuItemPropertiesWindow.Checked = SplitContainer1.Panel2Collapsed
    End Sub

    Private Sub MenuItemHelpAbout_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MenuItemHelpAbout.Click
        MsgBox("AddFlow for WinForms Flowchart Editor Sample" & vbCr & "Copyright  2003-2011 Lassalle Technologies")
    End Sub

    Private Sub AddFlow1_SelectionChange(ByVal sender As Object, ByVal e As SelectionChangeArgs) Handles AddFlow1.SelectionChange
        SelectionChangeHandle()
    End Sub

    Private Sub SelectionChangeHandle()
        Dim item As Item = AddFlow1.SelectedItem
        If Not (item Is Nothing) Then
            If TypeOf (item) Is Node Then
                Dim node As Node = CType(item, Node)
                PropertyGrid1.SelectedObject = node
                Label1.Text = "Selected Node"
            ElseIf TypeOf (item) Is Link Then
                Dim link As Link = CType(item, Link)
                PropertyGrid1.SelectedObject = link
                Label1.Text = "Selected Link"
            End If
        Else
            PropertyGrid1.SelectedObject = AddFlow1
            Label1.Text = "AddFlow control"
        End If
    End Sub

    Private Function SaveFileDlg() As Boolean
        Dim sfd As SaveFileDialog = New SaveFileDialog
        If (strFileName.Length > 1) Then
            sfd.FileName = strFileName
        Else
            sfd.FileName = "*.xml"
        End If
        sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*"
        sfd.InitialDirectory = Application.StartupPath
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            strFileName = sfd.FileName
            SaveFile()
            MakeCaption()
            SaveFileDlg = True
        Else
            SaveFileDlg = False
        End If
    End Function

    Private Sub LoadFile(ByVal strFileName As String)
        AddFlow1.ResetUndoRedo()
        AddFlow1.Nodes.Clear()
        Dim reader As System.Xml.XmlTextReader = New System.Xml.XmlTextReader(strFileName)
        reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None
        AddFlow1.ReadXml(reader)
        reader.Close()
        AddFlow1.SetChangedFlag(False)
        Me.strFileName = strFileName
        MakeCaption()
    End Sub

    Private Sub SaveFile()
        Cursor = Cursors.WaitCursor
        Dim writer As System.Xml.XmlTextWriter = New System.Xml.XmlTextWriter(strFileName, Nothing)
        writer.Formatting = System.Xml.Formatting.Indented
        AddFlow1.WriteXml(writer)
        writer.Close()
        AddFlow1.SetChangedFlag(False)
        Cursor = Cursors.Default
    End Sub

    Private Sub MakeCaption()
        Text = strProgName & " - " & FileTitle()
    End Sub

    Private Function FileTitle() As String
        If strFileName.Length > 1 Then
            FileTitle = Path.GetFileName(strFileName)
        Else
            FileTitle = "Untitled"
        End If
    End Function

    Private Function OkToTrash() As Boolean
        If Not AddFlow1.IsChanged Then
            OkToTrash = True
            Exit Function
        End If
        Dim dr As DialogResult = MessageBox.Show("The diagram in the " + FileTitle() & " file has changed." & vbCrLf & vbCrLf & "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation)
        Select Case dr
            Case Windows.Forms.DialogResult.Yes
                OkToTrash = SaveFileDlg()
            Case Windows.Forms.DialogResult.No
                OkToTrash = True
            Case Windows.Forms.DialogResult.Cancel
                OkToTrash = False
        End Select
    End Function

    Function ActionName(ByVal code As Integer) As String
        ActionName = ""
        Select Case code
            Case Action.NodeAdd
                ActionName = "node creation"
            Case Action.NodeResize
                ActionName = "node resize"
            Case Action.DeleteSel
                ActionName = "delete selection"
            Case Action.MoveItems
                ActionName = "move items"
            Case Action.LinkAdd
                ActionName = "link creation"
            Case Action.LinkStretch
                ActionName = "link stretch"
            Case Action.ZOrder
                ActionName = "ZOrder"
            Case Action.Org
                ActionName = "link origin change"
            Case Action.Dst
                ActionName = "link destination change"
            Case Action.Text
                ActionName = "node text change"
            Case 1001
                ActionName = "node properties change"
            Case 1002
                ActionName = "link properties change"
            Case 1003
                ActionName = "paste"
            Case Else
                ' here you can add your own action code
                ' (see the AddFlow method: BeginAction)
        End Select
    End Function
End Class

Public Class ClipboardMetafileHelper
    <DllImport("user32.dll", EntryPoint:="OpenClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function OpenClipboard(ByVal hWnd As IntPtr) As Boolean
    End Function
    <DllImport("user32.dll", EntryPoint:="EmptyClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function EmptyClipboard() As Boolean
    End Function
    <DllImport("user32.dll", EntryPoint:="SetClipboardData", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function SetClipboardData(ByVal uFormat As Integer, ByVal hWnd As IntPtr) As IntPtr
    End Function
    <DllImport("user32.dll", EntryPoint:="CloseClipboard", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function CloseClipboard() As Boolean
    End Function
    <DllImport("gdi32.dll", EntryPoint:="CopyEnhMetaFileA", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function CopyEnhMetaFile(ByVal hemfSrc As IntPtr, ByVal hNULL As IntPtr) As IntPtr
    End Function
    <DllImport("gdi32.dll", EntryPoint:="DeleteEnhMetaFile", _
    SetLastError:=True, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Function DeleteEnhMetaFile(ByVal hemfSrc As IntPtr) As Boolean
    End Function

    ' Metafile mf is set to an invalid state inside this function
    Public Shared Function PutEnhMetafileOnClipboard(ByVal hWnd As IntPtr, ByVal mf As Metafile) As Boolean
        Dim bResult As New Boolean
        bResult = False
        Dim hEMF, hEMF2 As IntPtr
        hEMF = mf.GetHenhmetafile() ' invalidates mf
        If Not hEMF.Equals(New IntPtr(0)) Then
            hEMF2 = CopyEnhMetaFile(hEMF, New IntPtr(0))
            If Not hEMF2.Equals(New IntPtr(0)) Then
                If OpenClipboard(hWnd) Then
                    If EmptyClipboard() Then
                        Dim hRes As IntPtr
                        hRes = SetClipboardData(14, hEMF2) ' 14 == CF_ENHMETAFILE()
                        bResult = hRes.Equals(hEMF2)
                        CloseClipboard()
                    End If
                End If
            End If
            DeleteEnhMetaFile(hEMF)
        End If
        Return bResult
    End Function
End Class
